function [DCgain,TOTgain,TOTphase]= bodemode(sys,w,plotflag)
% LTI/BODEMODE
%
% [DCgain,TOTgain,TOTphase] = BODEMODE(SYS,W,PLOTFLAG) computes Bode Gain/Phase based on Modal form
% and a Bode magnitude plot with each individual modal contributions.
% 
% Inputs:
%        SYS: LTI system          
%          W: freq. axis in rad/sec (optional input)
%   PLOTFLAG: optional input, GFACTOR plot = 1, Modal Bode = 2. Both = 3 or no outputs.
%
% Outputs: 
%        TOTgain (abs)
%        TOTphase (deg)
%        DCgain: GFACTOR(SYS)
%
% 
% See also DCGAINMR, GFACTOR, REDUCE.

% R. Y. Chiang & M. G. Safonov 7/2006
% Copyright 1988-2006 The MathWorks, Inc. 
%       $Revision: 1.1.8.1 $
% All Rights Reserved.

if length(sys.d) > 1
    error('This function only works for SISO system. For MIMO try ''gfactor.m''');
end

% Bookkeeping I/O's

if nargin == 2
   plotflag = 0;
end

if nargin == 1 
   f = logspace(-3,3,1000);
   w = f*2*pi;
   plotflag = 3;
end

if nargin == 1 & nargout ~= 0
    plotflag = 0;
end

% if nargout == 0
%     plotflag = 3;
% end

% Start algorithm

f = w/2/pi;

cut = find(abs(eig(sys)) < 1e-5);
cut = length(cut);
d = sys.d;
if cut == 0
    [sys1,sysm] = modreal(sys,length(sys.a));
    [am,bm,cm,dm] = ssdata(sys1);
    dm = sys.d;
else
    [sys1,sysm] = modreal(sys,cut);
    [am,bm,cm,dm] = ssdata(sysm);
    dm = sys.d;
end

% Rigid body dynamics
[a1,b1,c1,d1] = ssdata(sys1);

ga1 = [];
gai = [];
totgain = [];
totphase = [];
dcga = [];

if cut ~= 0 % rigid body dynamics
    ga1 = ltifr(a1,b1,i*w); 
    mag1 = c1*ga1 + diag(d)*ones(size(c1,1),length(w));
    if plotflag == 1 | plotflag == 3
        semilogx(f,20*log10(abs(mag1)),'k-')
        title('Plant Open Loop and Individual Mode Contributions')
        xlabel('Hz');ylabel('dB')
        hold on
        modeid = 1:cut;
        text(f(10),20*log10(abs(mag1(1,1))),num2str(modeid));
    end
    dcga = inf*ones(cut,1);
end

accum = 0;
totclx = ga1;

while accum < length(am)
     
    if (accum+1) == length(am) % if last mode is real !
            a = am(accum+1,accum+1);
            b = bm(accum+1,:);
            c = cm(:,accum+1);
            d = dm;
            omega2 = abs(a);
            mode = 'real';
            modeid = accum+1:accum+1;
    else
        if abs(am(accum+1,accum+2)) < 1e-8 % real mode, off-diagonal is zero
            a = am(accum+1,accum+1);
            b = bm(accum+1,:);
            c = cm(:,accum+1);
            d = dm;
            omega2 = abs(a);
            mode = 'real';
            modeid = accum+1:accum+1;
        else                            % complex mode, 2X2
            a = am(accum+1:accum+2,accum+1:accum+2);
            b = bm(accum+1:accum+2,:);
            c = cm(:,accum+1:accum+2);
            d = dm;
            lami = eig(a);
            omega2 = abs(lami(1)); % formerly: omega2 = lami(1)*lami(2);
            mode = 'comp';
            modeid = accum+1:accum+2;
        end
    end
    
    gai = ltifr(a,b,i*w);    
    magi = c*gai + diag(d)*ones(size(c,1),length(w));
    
    if plotflag == 1 | plotflag == 3
        figure(gcf)
        semilogx(f,20*log10(abs(magi)),'k-');
        text(f(10),20*log10(abs(magi(1,1))),num2str(modeid));
        hold on
    end

    dcga0 = norm(b,2)*norm(c,2)/omega2/2; % formerly     dcga0 = norm(b,2)*norm(c,2)/omega2;
    
    if mode == 'comp'
        dcga0 = [dcga0;dcga0];
    end
    
    dcga = [dcga;dcga0];
    
    totclx = [totclx;gai];
    
    accum = accum + length(a);
    
end

systm = sys1+sysm;
systm.d = sys.d;

clxfq = systm.c*totclx + diag(sys.d)*ones(size(cm,1),length(w));
clxfq = clxfq'; 
totgain = abs(clxfq);

if plotflag >=2 
    totphase = (180./pi)*unwrap(angle(clxfq));
    totphase = 360-totphase;
end

if plotflag == 1 | plotflag == 3
    figure(gcf)
    semilogx(w/2/pi,20*log10(totgain),'k.-')
    figure(gcf+1)
    bar(sort(dcga,'descend')) % bar(dcga)
    title('DC Gain Magnitude')
    xlabel(['Modes: 1 --> ' num2str(length(sys.a))])
end

if plotflag >=2
    if plotflag == 3
        figure(gcf+1)
    end
    subplot(211)
    semilogx(w/2/pi,20*log10(totgain));
    title('Modal Bode Plot')
    ylabel('dB')
    grid on
    subplot(212)
    semilogx(w/2/pi,totphase)
    ylabel('deg')
    xlabel('Hz') 
    grid on    
end

if nargout ~= 0
    DCgain = dcga;
    TOTgain = totgain;
    TOTphase = totphase;
end

% 
% ----- End of bodemode.m % RYC %