function [lb,Delta,bout,wout] = mmupiter(M,index,bstart,wstart,opt)
%   Power method for Mixed-Mu, with BLK representing an upper-loop
%   LFT on M.  

% Argument check
if nargin == 2
    bstart = [];
    wstart = [];
    opt = [];
elseif nargin == 4
    opt = [];
elseif nargin ~= 5
    error('Invalid argument list');
end

% Iteration options
CNTMAX = 50;
stol=1e-7;          % XXX MINOR: Possibly 1e-5*norm(M)? or 1e-5?
step = length(M)/4; % XXX MINOR: FROM RUBLOW.M    
if ~isempty(opt)
    % XXX Need to update if we want to pass in other options
    CNTMAX = opt.CNTMAX;
end


% Initialize vectors
[Mr,Mc]=size(M);
if isempty(bstart)
    [u,s,v] = svd(M);    
    bstart = v(:,1);
    wstart = bstart;    
end    
z = zeros(Mr,1);
a = z;
newa = a; newz = z;
b = bstart;
w = wstart;
newb = b; neww = w;

% Get indices for real scalars, complex scalars, and complex full blocks
nblk = size(index.simpleblk,1);
compr = index.allcomp.allrows;
compc = index.allcomp.allcols;

nreal = index.allreal.num;
realr = index.allreal.allrows; 
realc = index.allreal.allcols; 

nrepc = index.repcomp.num;
repcr = index.repcomp.allrows; 
repcc = index.repcomp.allcols; 

nfull = index.full.num;
fullr = index.full.allrows;
fullc = index.full.allcols;

% Get masks used to vectorize code
real_mask = index.masks.real_mask;
repc_mask = index.masks.repc_mask;
Dfull_maskr = index.masks.Dfull_maskr;
Dfull_maskc = index.masks.Dfull_maskc;
DeltaFull_mask = index.masks.DeltaFull_mask;


% Set up for loop
go = 1;
cnt = 0;
qb = ones(nreal,1);
lb= 0;

% Begin Loop
while go==1 & cnt<CNTMAX
    cnt = cnt + 1;
  
    newa = M*b;
    beta1 = norm(newa);  
    if beta1 < 100*eps 
        a = newa;
        break;
    end
    newa = newa/beta1;  

    if cnt==1 
        qb=real(real_mask*( conj(b(realc)).*newa(realr) ));
    end

    % real blocks
    if nreal>0
        wreal = w(realc);
        areal = newa(realr);
        breal = b(realc);
        norma = sqrt(real_mask*( conj(areal).*areal ));
        normb = sqrt(real_mask*( conj(breal).*breal ));
        wa = real_mask*( conj(wreal).*areal );
        norma = norma+(norma<=100*eps*normb).*(normb+100*eps);
        qz = sign(qb).*normb./norma+step*real(wa);    
        qz = qz./max(abs(qz),1);  
        newz(realr) = (real_mask'*qz).*wreal;
    end
    
    % repeated complex scalar blocks
    if nrepc>0
        wrepc = w(repcc);
        arepc = newa(repcr);
        wa = repc_mask*( conj(wrepc).*arepc );
        abs_wa = abs(wa);
        wa = (abs_wa>100*eps).*( wa./ max(abs_wa,100*eps) );    
        newz(repcr) = (repc_mask'*wa).*wrepc;
    end
    
    % full blocks
    if nfull>0
        wfull = w(fullc);
        afull = newa(fullr);
        norma = sqrt(Dfull_maskr*( conj(afull).*afull ));
        normw = sqrt(Dfull_maskc*( conj(wfull).*wfull ));
        tmp=zeros(nfull,1);
        idx = find(norma>100*eps*normw);
        tmp(idx) = normw(idx)./norma(idx);
        newz(fullr) = (Dfull_maskr'*tmp).*afull;
    end

    % Update
    neww = M'*newz;
    beta2 = norm(neww); 
    if beta2 < 100*eps 
        break;
    end
    neww = neww/beta2;
    
    % real blocks
    if nreal>0
        wreal = neww(realc);
        areal = newa(realr);
        breal = b(realc);
        norma = sqrt(real_mask*( conj(areal).*areal ));
        normb = sqrt(real_mask*( conj(breal).*breal ));
        wa = real_mask*( conj(wreal).*areal );
        norma = norma+(norma<=100*eps*normb).*(normb+100*eps);
        qb = sign(qz).*normb./norma+step*real(wa);    
        qb = qb./max(abs(qb),1);  
        newb(realc) = (real_mask'*qb).*areal;
    end
    
    % repeated complex scalar blocks
    if nrepc>0
        wrepc = neww(repcc);
        arepc = newa(repcr);
        aw = repc_mask*( conj(arepc).*wrepc );
        abs_aw = abs(aw);
        aw = (abs_aw>100*eps).*( aw./ max(abs_aw,100*eps) );    
        newb(repcc) = (repc_mask'*aw).*arepc;
    end
    
    % full blocks
    if nfull>0
        wfull = neww(fullc);
        afull = newa(fullr);
        norma = sqrt(Dfull_maskr*( conj(afull).*afull ));
        normw = sqrt(Dfull_maskc*( conj(wfull).*wfull ));    
        tmp=zeros(nfull,1);
        idx = find(normw>100*eps*norma);
        tmp(idx) = norma(idx)./normw(idx);
        newb(fullc) = (Dfull_maskc'*tmp).*wfull;
    end
      
    newlb = max(beta1,beta2);
    if abs( newlb-lb ) < stol
        chng = [newb; newa; newz; neww]-[b; a; z; w];
        if max(abs(chng)) < stol
            go=0;
        end
    end
        
    % Update
    lb = newlb;
    a = newa;
    b = newb;
    z = newz;
    w = neww;
    
    %fprintf('cnt=%d \t beta1=%6.4f \t beta2=%6.4f\n',cnt,beta1,beta2);  
end

% Store output vectors ---XXX MINOR
bout = b;
wout = w;

% Compute a valid lower bound and perturbation
lvec = cell(nblk,1);
rvec = cell(nblk,1);
if nrepc==0 & nfull ==0 % pure real
  
  % XXX MINOR: If no convergence, can we go back
  % and construct a valid pert?
  % Since iteration typically fails to converge for
  % pure real, probably should warn or error out above
  if go==0 & lb>100*eps
    Delta(realc,realr) = diag(real_mask'*qb);
    lb=max(real(eig(M*Delta)));
    Delta = Delta/lb;
  else
    lb=0;
  end
  
elseif nreal==0 % pure complex

  % Construct a unitary perturbation which, if the 
  % iteration converged, is a local maximum of 
  %      max_{Delta} rho[M*Delta]
  Q=mkcpert(b,a,index,'unitary');
  [evec,eval] = eig(Q*M);
  [lb,ind] = max(abs(diag(eval)));   
  
  if lb > 100*eps  
    % Given: a,b unit norm with lb*a = M*b   
    % Construct a Delta with rank 1 full blocks that
    % takes (lb*a) to b with ||Delta|| = 1/lb 
    b = evec(:,ind(1));
    alb = M*b;
    [Delta,lvec,rvec]=mkcpert(b,alb,index,'dyad');
  end
    
else % mixed real/complex

  % Construct pert with unitary complex blocks
  Q=mkcpert(b,a,index,'unitary');  
  
  if lb <= 100*eps
    [evec,eval] = eig(Q*M);
    [lb,ind] = max(abs(diag(eval)));
    if lb>100*eps
      b = evec(:,ind(1));
      alb = M*b;
      [Delta,lvec,rvec]=mkcpert(b,alb,index,'dyad');
    end     
  else   
    % Construct pert with real blocks.
    lbr = lb;    
    Deltar =  diag(real_mask'*qb)/lbr;
    
    %rdim = index.realidx.realdim(1);    
    Mrr = M(realr,realc);
    rdim = size(Mrr,1);
    if rcond(eye(rdim)-Mrr*Deltar)<100*eps
      Delta = zeros(size(Q));
      Delta(realc,realr) = Deltar;
      lb=lbr;   
    else      
      % Absorb reals and then construct complex pert
      Mrc = M(realr,compc);
      Mcr = M(compr,realc);
      Mcc = M(compr,compc);    
      Mloop = Mcc+Mcr*Deltar*inv(eye(rdim)-Mrr*Deltar)*Mrc;
      [evec,eval] = eig(Q(compc,compr)*Mloop);
      [lbc,ind] = max(abs(diag(eval)));
      b = evec(:,ind(1));
      lb = min(lbr,lbc);
      if lb>100*eps 
        alb = Mloop*b;
        [Delta,lvec,rvec]=mkcpert(b,alb,index,'dyad');
        Delta(realc,realr) = Deltar;
        lb = 1/norm(Delta); 
      end 
      
      % XXX MINOR: If lbr & lbc are different 
      % RUBLOW scales up the real blocks and
      % tries to reform the complex blocks.
    end    
  end  
  
end

if lb <= 100*eps
    % lb<=100*eps ==> Delta is large    
    Delta = zeros(Mc,Mr);
    Delta(realc,realr) = diag(1e50*real_mask'*ones(nreal,1));
    Delta(repcc,repcr) =  diag(1e50*repc_mask'*ones(nrepc,1));
    Delta(fullc,fullr) =  1e50*DeltaFull_mask;
end
