% function [ub,Dr,Dc,Gcr] = mudescentub(M,index,DGinit,opt);
%
% 11/1/2005 PJS
% Based on dang.m by Young/Newlin
% Does not work for rep full although the mods should be straightforward
%
% Notation:  M'*Dr*M+j*(Gcr*M-M'*Gcr') - ub^2*Dc <= 0
% ub is an upper bound for mu

function [ub,Dr,Dc,Gcr] = mudescentub(M,index,DGinit,opt);

% Argument Checks
if nargin==2
    DGinit = [];
    opt = [];
elseif nargin==3
    opt = [];
elseif nargin~=4
    error('[ub,Dr,Dc,Gcr] = mudescentub(M,index,DGinit,opt);');
end

% Initialize Scalings
[Mr,Mc] = size(M);  % M should be 2-D at this point
if isempty(DGinit) | rcond(DGinit.Dc)<eps
    Dr = eye(Mr);
    Dc = eye(Mc);
    Gcr = zeros(Mc,Mr);
    ub = max(svd(M));
else
    Dr = DGinit.Dr;
    Dc = DGinit.Dc;
    Gcr = DGinit.Gcr;
    ub = DGinit.ub;
end
DrOld = Dr;
DcOld = Dc;
GcrOld = Gcr;
ubsqOld = ub^2;

% Iteration options
if isempty(opt)
    stoptol=1+2e-3; 
    descent_steps = 30;   
    mincodel_steps = 30;  
else
    stoptol = opt.stoptol;
    descent_steps = opt.descent_steps;
    mincodel_steps = opt.mincodel_steps;
end

% Get indices for real, repeated scalar, and full blocks
nreal = index.allreal.num;
realrows = index.allreal.allrows; 
realcols = index.allreal.allcols; 

nrep = index.allrep.num;
reprows = index.allrep.allrows; 
repcols = index.allrep.allcols; 

nfull = index.full.num;
fullrows = index.full.allrows;
fullcols = index.full.allcols;

% Get masks used to vectorize code
Drep_mask = index.masks.Drep_mask;
Dfull_maskr = index.masks.Dfull_maskr;
Dfull_maskc = index.masks.Dfull_maskc;
G_mask = index.masks.G_mask;


% Start Descent
GcrM = zeros(Mc);
GcrM(realcols,:) = Gcr(realcols,realrows)*M(realrows,:); 
NL	= M'*Dr*M + j*(GcrM-GcrM');
for i1 = 1:descent_steps    
    % Find (clustered) max generalized eigenvals of:
    %    [M'*Dr*M+j*(Gcr*M-M'*Gcr')]*U = ub^2*Dc*U
    % ub is an upper bound and U is used to define Del
    tmp = Dc\NL;  % Dc should be invertible at this point.
    [U,evals] = eig(tmp);
    evals = real(diag(evals));
    ubsq = max(evals);
    eidx = find(evals>=0.975*ubsq); % find evals clustered near max
    U = U(:,eidx);
    V = M*U;
    evals = evals(eidx);

    % Stop and return if upper bound has not significantly improved
    if ubsq<10*eps 
        ub = sqrt( max(0,ubsq) );
        return;
    elseif (ubsq<ubsqOld/stoptol | i1==1)
        DrOld = Dr;
        DcOld = Dc;
        GcrOld = Gcr;
        ubsqOld = ubsq;
    elseif (ubsq<ubsqOld)
        ub = sqrt(ubsq);        
        return;
    else
        Dr = DrOld;
        Dc = DcOld;
        Gcr = GcrOld;
        ub = sqrt(ubsqOld);
        return;
    end

    %----------------------------------------------------------------
    % Find min co(Del) which yields a descent direction.
    % Note: Del is a subset of a real inner product space which
    % is defined using a "compressed" form of the D/G matrices. 
    % See p.70 of Young's thesis for the definition of this space.    
    %----------------------------------------------------------------

    % First: Pick an initial update direction in co(Del):
    %         Dhat_0:=(Dhatrep,Dhatfull,Ghat)
    % Dhatrep: blks related to repeated scalar (real & complex) D-scales
    % Dhatfull: blks related to complex full D-scales
    % Ghat: blks related to real G-scales
    eta_dim = size(U,2);
    eta = ones( eta_dim ,1);
    eta = eta/norm(eta);
    Ve = V*eta;
    Ue = U*eta; 

    Dhatrep = Ve(reprows)*Ve(reprows)'-ubsq*(Ue(repcols)*Ue(repcols)');
    Dhatrep = Dhatrep.*Drep_mask;    % zero out off-diagonal blocks

    Dhatfull = Dfull_maskr*real(Ve(fullrows).*conj(Ve(fullrows)));
    Dhatfull = Dhatfull-ubsq*Dfull_maskc*real(Ue(fullcols).*conj(Ue(fullcols)));

    VeUe = Ve(realrows)*Ue(realcols)';
    Ghat = j*(VeUe-VeUe'); 
    Ghat = Ghat.*G_mask;  % zero out off-diagonal blocks  
 
    % Second: Iterate towards min co(Del) using algorithm by [GIL]    
    % If eta_dim=1 then no iteration is needed since Del is a single vector
    for i2 = 1:( mincodel_steps*(eta_dim>1) )
        % Given Dhat_i compute:
        %   P_i = arg min_{P \in Del} <Dhat_i,P>
        % where P_i:=(Prep,Pfull,H)
        Drstep = zeros(Mr);
        Drstep(reprows,reprows) = Dhatrep;
        Drstep(fullrows,fullrows) = diag(Dhatfull'*Dfull_maskr);
        Dcstep = zeros(Mc);
        Dcstep(repcols,repcols) = Dhatrep;
        Dcstep(fullcols,fullcols) = diag(Dhatfull'*Dfull_maskc);

        uGMu = U(realcols,:)'*Ghat*V(realrows,:); 
        [eta,evals] = eig(V'*Drstep*V + j*(uGMu - uGMu') - ubsq*(U'*Dcstep*U));
        [mineval,minidx]=min(real(diag(evals)));
        eta = eta(:,minidx);
        eta = eta/norm(eta);

        Ve = V*eta;
        Ue = U*eta;

        Prep = Ve(reprows)*Ve(reprows)'-ubsq*Ue(repcols)*Ue(repcols)';
        Prep = Prep.*Drep_mask;    % zero out off-diagonal blocks

        Pfull = Dfull_maskr*real(Ve(fullrows).*conj(Ve(fullrows)));
        Pfull = Pfull-ubsq*Dfull_maskc*real(Ue(fullcols).*conj(Ue(fullcols)));

        VeUe = Ve(realrows)*Ue(realcols)';
        H = j*(VeUe-VeUe'); 
        H = H.*G_mask;  % zero out off-diagonal blocks  

        % Take a step:
        %   Dhat_{i+1} = min co{Dhat_i,P_i}
        %              = Dhat_i + t*(P_i-Dhat_i)
        % where t = -<Dhat_i,P_i-Dhat_i> / <P_i-Dhat_i,P_i-Dhat_i>
        Deltarep = Prep-Dhatrep;
        Deltafull = Pfull - Dhatfull;
        DeltaG = H-Ghat;
        n=0;
        d=0;
        if nrep>0;         
            n = n + sum(sum(Dhatrep.*(Deltarep.')));
            d = d + sum(sum(Deltarep.*(Deltarep.')));
        end
        if nreal>0
            n = n + sum(sum( Ghat.*(DeltaG')));
            d = d + sum(sum(DeltaG.*(DeltaG')));
        end
        if nfull>0            
            n = n + Dhatfull'*Deltafull;
            d = d + Deltafull'*Deltafull;
        end
        
        if d<=1e-6
            t = (n<0); % XXX MINOR:  Just set t=0?
        else
            t = min([max([-real(n/d) 0]) 1]);
        end                    
        Dhatrep = Dhatrep + t*Deltarep;
        Dhatfull = Dhatfull + t*Deltafull;
        Ghat = Ghat + t*DeltaG;        
    end % for loop for [GIL]'s algorithm   
    
    % Third: Convert into full D/G matrices.  This is a descent direction.
    Drstep = zeros(Mr);
    Drstep(reprows,reprows) = Dhatrep;
    Drstep(fullrows,fullrows) = diag(Dhatfull'*Dfull_maskr);
    Dcstep = zeros(Mc);
    Dcstep(repcols,repcols) = Dhatrep;
    Dcstep(fullcols,fullcols) = diag(Dhatfull'*Dfull_maskc);
    Gcrstep = zeros(Mc,Mr);
    Gcrstep(realcols,realrows) = Ghat;

    % Fourth: Take a step in the descent direction
    GcrstepM = zeros(Mc);
    GcrstepM(realcols,:) = Gcrstep(realcols,realrows)*M(realrows,:);             
    NLstep	= M'*Drstep*M + j*(GcrstepM-GcrstepM');
    
    % For positive t, ubsq(t) initially decreases and then may 
    % increase. The smallest value of t for which ubsq(t)=
    % ubsq(0) is given by the smallest strictly positive 
    % generalized eigval of (NL-ubsq*Dc,NLstep-ubsq*Dcstep). 
    % Choose the step length to be half this value.
    if rcond(NLstep-ubsq*Dcstep)<eps
        tmpeig=eig(NL-ubsq*Dc,NLstep-ubsq*Dcstep);
        tmpeig=sort( real(tmpeig) );
    else
        % This generally seems to be faster than using eig
        % to solve for generalized eigenvalues
        tmp = (NL-ubsq*Dc)/(NLstep-ubsq*Dcstep);
        tmpeig=sort(real(eig(tmp)));
    end
    [mineig,idx]=min(abs(tmpeig));
    if idx(end)+1<=length(tmpeig) && mineig<1e-6
        t = tmpeig(idx(end)+1)/2;
    else
        t = inf;
    end    

    % Choose tmax to ensure Dr-t*Drstep>0 for t<tmax
    if rcond(Dcstep)<eps
        tmpeig=real(eig(Dcstep,Dc));
    else
        tmp = Dc\Dcstep;
        tmpeig = real(eig(tmp));
    end
    tmax = 1/max( [tmpeig; 1e-9] );

    %fprintf('mudes: i1=%d \t t=%0.3f \t tmax=%0.3f \t ubsq=%0.5f \n',i1,t,tmax,ubsq);
    
    % Take step 
    t = min([t 0.95*tmax]);
    NL = NL-t*NLstep;
    Gcr = Gcr-t*Gcrstep;
    Dr = Dr-t*Drstep;
    Dc = Dc-t*Dcstep;

    % WLOG we can scale max entry of Dc to be one.
    dmax = max(max(Dc));
    NL = NL/dmax;
    Gcr = Gcr/dmax;
    Dr = Dr/dmax;
    Dc = Dc/dmax;
        
    if rcond(Dc)<eps 
        Dr = DrOld;
        Dc = DcOld;
        Gcr = GcrOld;
        ub = sqrt(ubsqOld);
        return;
    end    
end % while loop for gradient descent

% Check the results of the last iteration
tmp = Dc\NL;  % Dc should be invertible at this point.
ubsq = max(real(eig(tmp)));
if ubsq<ubsqOld
    ub = sqrt(ubsq);
else    
    Dr = DrOld;
    Dc = DcOld;
    Gcr = GcrOld;
    ub = sqrt(ubsqOld);
    return;
end
