% function [ub,Dr,Dc,Gcr] = mulmiub(M,index,DGinit);
%  This solves the SDP upper bound for mu:
%    min ub 
%    subject to:   0 < Dc 
%                  M'*Dr*M + j*(Gcr*M-M'*Gcr')  <= ub^2*Dc
%                  

function [ub,Dr,Dc,Gcr] = mulmiub(M,index,DGinit);

% Argument Checks
if nargin==2
    DGinit = [];
elseif nargin~=3
    error('[ub,Dr,Dc,Gcr] = mulmiub(M,index,DGinit);');
end

% Problem data
[Mr,Mc] = size(M);  % M should be 2-D at this point
nreal = index.allreal.num;
realr = index.allreal.allrows;
realc = index.allreal.allcols;

% Dpdidx is used to define a compressed version of D-scales
Dpdidx = [];
for i1 = 1:index.full.num
    tmp = index.full.cols{i1};
    Dpdidx = [Dpdidx tmp(2:end)];
end

% Initialize Scalings
if isempty(DGinit)
    Dr_init = [];
    Dc_init = [];
    Gcr_init = [];
    ub_init = [];
    Dpd_init = [];
else
    % Get Hermitian part of initial conditions
    Dr_init = (DGinit.Dr+DGinit.Dr')/2;
    Dc_init = (DGinit.Dc+DGinit.Dc')/2;
    Dpd_init = Dc_init;
    Gcr_init = DGinit.Gcr;
    Gcr_init(realc,realr) = (Gcr_init(realc,realr)+Gcr_init(realc,realr)')/2;
    Dr_init = [real(Dr_init) imag(Dr_init); -imag(Dr_init) real(Dr_init)]; 
    Dc_init = [real(Dc_init) imag(Dc_init); -imag(Dc_init) real(Dc_init)]; 
    Gcr_init = [real(Gcr_init) imag(Gcr_init); -imag(Gcr_init) real(Gcr_init)]; 
    
    %Dr_init = [real(DGinit.Dr) imag(DGinit.Dr); -imag(DGinit.Dr) real(DGinit.Dr)]; 
    %Dc_init = [real(DGinit.Dc) imag(DGinit.Dc); -imag(DGinit.Dc) real(DGinit.Dc)]; 
    %Gcr_init = [real(DGinit.Gcr) imag(DGinit.Gcr); -imag(DGinit.Gcr) real(DGinit.Gcr)]; 
    ub_init = DGinit.ub;    
    
    % Dpd is a 'compressed' version of the D-scales
    % used to enforce the pos. def. constraint
    %DGinit.Dc = (DGinit.Dc+DGinit.Dc')/2;
    %Dpdreal_init = real(DGinit.Dc);
    Dpdreal_init = real(Dpd_init);
    Dpdreal_init(Dpdidx,:) = [];
    Dpdreal_init(:,Dpdidx) = [];
    %Dpdimag_init = imag(DGinit.Dc);
    Dpdimag_init = imag(Dpd_init);
    Dpdimag_init(Dpdidx,:) = [];
    Dpdimag_init(:,Dpdidx) = [];
    Dpd_init = [Dpdreal_init Dpdimag_init; -Dpdimag_init Dpdreal_init];    
end

% Set up LMIs:  Most of the LMI setup does not depend on M.
% Set up this part first or pass in from a previous set up.
setlmis([]);

% Initialize D-scale and G-scale variables
Drreal = zeros(Mr);
Drimag = Drreal;
Dcreal = zeros(Mc);
Dcimag = Dcreal;
Gcrreal = zeros(Mc,Mr);
Gcrimag = Gcrreal;
Dpdreal = [];

% Create D-scale and G-scale variables
blk=index.simpleblk;
vars=0;
ridxm = index.ridxm; % block row indices for M
cidxm = index.cidxm; % block col indices for M
for i1=1:size(blk,1);        
    ridx = ridxm(i1): (ridxm(i1+1)-1);
    cidx = cidxm(i1): (cidxm(i1+1)-1);

    if blk(i1,2) == 0 % repeated real or complex scalar
        blksz = abs(blk(i1,1));
        symblk  = symdec(blksz,vars);
        vars = vars+blksz*(blksz+1)/2;
        Drreal(ridx,ridx) = symblk;
        Dcreal(cidx,cidx) = symblk;

        skewblk  = skewdec(abs(blk(i1,1)),vars);
        vars = vars+blksz*(blksz-1)/2;
        Drimag(ridx,ridx) = skewblk;
        Dcimag(cidx,cidx) = skewblk;

        if blk(i1,1)<0 % repeated real scalar
            symblk  = symdec(abs(blk(i1,1)),vars);
            vars = vars+blksz*(blksz+1)/2;
            Gcrreal(cidx,ridx) = symblk;

            skewblk  = skewdec(abs(blk(i1,1)),vars);
            vars = vars+blksz*(blksz-1)/2;
            Gcrimag(cidx,ridx) = skewblk;
        end

    elseif blk(i1,2)>0 % complex full block
        vars = vars+1;
        Drreal(ridx,ridx) = vars*eye(blk(i1,2));
        Dcreal(cidx,cidx) = vars*eye(blk(i1,1));
    end

end

% Create D-scale variables
Dc=lmivar(3,[Dcreal Dcimag; -Dcimag Dcreal]);
Dr=lmivar(3,[Drreal Drimag; -Drimag Drreal]);

% Dpd is a 'compressed' version of the D-scales
% used to enforce the pos. def. constraint
Dpdreal = Dcreal;
Dpdimag = Dcimag;
Dpdreal(Dpdidx,:) = [];
Dpdreal(:,Dpdidx) = [];
Dpdimag(Dpdidx,:) = [];
Dpdimag(:,Dpdidx) = [];

if any(Dpdimag(:))
    Dpd=lmivar(3,[Dpdreal Dpdimag; -Dpdimag Dpdreal]);
    szDpd = 2*size(Dpdreal,1);            
else
    Dpd=lmivar(3,Dpdreal);
    szDpd = size(Dpdreal,1);

    if ~isempty(Dpd_init)
        Dpd_init = Dpd_init(1:szDpd,1:szDpd);
    end
end

% Create G-scale variables
if nreal>0
    Gcr=lmivar(3,[Gcrreal Gcrimag;-Gcrimag Gcrreal]);
end

% Set D-scale constraints
% XXX Minor: D>I or D>c*I or D>0?
%   Also, D<I? Probably not since gevp already bounds x'*x<R^2
%lmiterm([-1 1 1 Dpd],1,1);      % D>I
%lmiterm([1 1 1 0],eye(szDpd));   % D>I
%lmiterm([-1 1 1 Dpd],1,1);      % D>0
lmiterm([1 1 1 0],1e-5);        % D>c*I
lmiterm([-1 1 1 Dpd],1,1);      % D>c*I

% Set RHS of: M'*Dr*M+j*(Gcr*M-M'*Gcr')  <= ub^2*Dc
lmiterm([-2 1 1 Dc],1,1);

% Set LHS of: M'*Dr*M+j*(Gcr*M-M'*Gcr')  <= ub^2*Dc
Mreal=real(M);   
Mimag=imag(M);
M1 = [Mreal Mimag;-Mimag Mreal];
lmiterm([2 1 1 Dr],M1',M1);
if (nreal>0)
    jM2 = [-Mimag Mreal;-Mreal -Mimag];
    lmiterm([2 1 1 Gcr],1,jM2,'s');
end

% Solve generalized eigenvalue problem
lmisys=getlmis;
if isempty(ub_init)
    tinit = [];
    xinit = [];
else
    tinit = (1.01*ub_init)^2;   % XXX Minor
    if nreal>0
        xinit = mat2dec(lmisys,Dc_init,Dr_init,Dpd_init,Gcr_init);
    else
        xinit = mat2dec(lmisys,Dc_init,Dr_init,Dpd_init);
    end
end

% These are the opts used by mubnd.m
opts = [1e-2;0;1e5;5;1]; 
%opts = [1e-2;0;1e5;5;0]; % Uncomment to view LMILab output
nlfc=1;
[ubsq,xopt] = gevp(lmisys,nlfc,opts,tinit,xinit);

% Form output
if isempty(xopt)
    %warning('Unable to find feasible point');
    ub = inf;
    Dr = eye(Mr);
    Dc = eye(Mc);
    Gcr = zeros(Mc,Mr);
else    
    ub=sqrt(max(0,ubsq));

    if nargout>1,        
        Dc=dec2mat(lmisys,xopt,Dc);   
        Dc=Dc(1:Mc,1:Mc)+j*Dc(1:Mc,Mc+1:2*Mc);
        Dr=dec2mat(lmisys,xopt,Dr);   
        Dr=Dr(1:Mr,1:Mr)+j*Dr(1:Mr,Mr+1:2*Mr);
        
        if (nreal>0),
            Gcr=dec2mat(lmisys,xopt,Gcr);  
            Gcr=Gcr(1:Mc,1:Mr)+j*Gcr(1:Mc,Mr+1:2*Mr);
        else 
            Gcr=zeros(Mc,Mr); 
        end
    end
        
  %psdami = ub^2*Dc-M'*Dr*M-j*(Gcr*M-M'*Gcr');
  %fprintf(' ami_mineig=%6.4f\n',min(real(eig(psdami))));
end

