function [bnds,info] = munogrid(sys,blk,opt,omeg)
% Given an system, sys, and block structure, blk, this function
% computes upper and lower bounds on mu without frequency gridding.
% The function pulls out the frequency variable and treats it
% as an additional, repeated real uMcertainty.

%--------------------------------------------------
% Options Set-up
%--------------------------------------------------
if nargin == 2
    opt = '';
    omeg = [];
elseif nargin == 3
    omeg = [];
end

if isempty(opt)
   opt = '';
end
if isempty(omeg)
   omeg = [0 inf];
end

% Map old options into new options
ostruc.displaywarnings = any(opt=='d');
if ~ostruc.displaywarnings
   SW = warning('off','all');
end


% Set # of tries lower bound
wcglbopt.Ntry = goptvl(opt,'q',1,1); 
wcglbopt.Ntry = 20;

% XXX Set upper bound option
% Currently using diag D/G scales for freq. block to prevent
% explosion of terms when sys has large state dimension.

%--------------------------------------------------
% Check blk structure 
%--------------------------------------------------

% blk must be Nx2 matrix of integers
if size(blk,2)~=2 | any(floor(blk(:))~=ceil(blk(:)))
    error('blk is invalid. blk must be Nx2 matrix of integers');
end

nblk = size(blk,1);
for ii=1:nblk
   if all( blk(ii,:) == [ 1 0] )
      blk(ii,:)  = [ 1 1] ;
   end
   if all( blk(ii,:) == [-1 1] )
      blk(ii,:)  = [-1 0] ;
   end
end
for ii=1:nblk
   if all(blk(ii,:) == [-1 -1])
      blk(ii,:) = [-1 0];
   end
end

% Grab block indices for original block structure
index = muindex(blk);

%--------------------------------------------------
% Check sys input and dimensions
%--------------------------------------------------
if isa(sys,'tf') || isa(sys,'zpk')
    sys = ss(sys);
end
if isa(sys,'ss') 
    A = sys.a;    
    B = sys.b;
    C = sys.c;
    D = sys.d;    
else
    error('sys must be a ss, tf, or zpk')
end
ns = size(A,1);

% Check compatibility of block and sys dimensions
[sysr,sysc]=size(sys);
if index.rdimm~=sysr || index.cdimm~=sysc
    error('Dimensions of sys are incompatible with the blk dimensions.')
end

%--------------------------------------------------
% Initialize outputs
%--------------------------------------------------

bnds = [0 0];
info.bnds = bnds;
info.frequency = 0;
info.Dr = zeros(sysr+ns);
info.Dc = zeros(sysc+ns);
info.Gcr = zeros(sysc+ns,sysr+ns);
info.Delta = zeros(sysc,sysr);
info.blk = blk; 

%--------------------------------------------------
% Eliminate frequency gridding:
%   Treat frequency as a repeated real uncertainty
%--------------------------------------------------

% Add frequency as a repeated real uncertainty and get
% block indices for augmented block structure
wblk = [-ns 0; blk];  
windex = muindex(wblk);
windex.allreal.realidx = muindex(windex.allreal.realblk);
windex.allcomp.compidx = muindex(windex.allcomp.compblk);
ridxm = windex.ridxm; % col2 of tblkp
cidxm = windex.cidxm; % col1 of tblkp

% Check for nominal stability
evl = eig(A);
if max(real(evl))>=0
    % XXX - warn that nominal system is unstable?
    info.bnds = [inf inf];
    
    if ~ostruc.displaywarnings
        warning(SW);
    end
    return
end

% Pull out frequency as a repeated real uncertainty in the first block
Ainv = inv(A);
M = [j*Ainv Ainv*B; -j*C*Ainv -C*Ainv*B+D];
[Mr,Mc] = size(M);

% Normalize frequency uncertainty
wmin = min(omeg);
wmax = max(omeg);
if (wmin>=0) && (wmax>wmin)
    if isinf(wmax)
        C = wmin+1;
        %Nw = [eye(ns) sqrt(2)*eye(ns); sqrt(2)*eye(ns) C*eye(ns)];
        Nw = [1 sqrt(2); sqrt(2) C];
    else
        R = (wmax-wmin)/2;
        C = (wmax+wmin)/2;
        %Nw = [zeros(ns) sqrt(R)*eye(ns); sqrt(R)*eye(ns) C*eye(ns)];
        Nw = [0 sqrt(R); sqrt(R) C];
    end    
    M = lft( kron(Nw,eye(ns)) ,M,ns,ns);
else
    error('Entries of omeg must satisfy 0<=omeg(1)<omeg(2)');
end

%--------------------------------------------------
% Pre-scale: Osborne's
%--------------------------------------------------
Dr = eye(Mr); 
Dc = eye(Mc); 
Gcr = zeros(Mc,Mr);
[dMd,Dr_os,Dci_os] = osbal(M,windex,'d');
Dc_os = inv(Dci_os);

%--------------------------------------------------
% Scale Data 
%--------------------------------------------------
%scale = max(max(abs(dMd)))/5;
%if scale<10*eps,
    scale = 1;
%else
%    dMd=dMd/scale;
%end
    
%--------------------------------------------------
% LMI Upper Bound 
%--------------------------------------------------
[ub,Dr,Dc,Gcr] = munogridub(dMd,windex);

%--------------------------------------------------
% Gain-based Lower Bound
%--------------------------------------------------
lb = 0;
wcfreq = 0;
Delta = zeros(Mc,Mr);
if  ub>100*eps
    [lb,Delta,wcfreq] = munogridlb(dMd,windex,[],[ub lb],wcglbopt);
end
 
%--------------------------------------------------
% Unscale and prepare outputs:
%--------------------------------------------------
if ub<10*eps 
    % Need ub>0 b/c mussv outputs D/G scales in the balaMced
    % form (ami2yMrow will cause problems if ub==0)
    ub = 10*eps; 
end          
ub = ub*scale;
lb = lb*scale;
Delta = Delta/scale;     
wcfreq = wcfreq/scale;     

%Dr = Dr_os'*Dr*Dr_os;
%Dc = Dc_os'*Dc*Dc_os;
%Gcr = scale*(Dc_os'*Gcr*Dr_os);

% Uncomment to test results: 
% psdami should be psd (within numerical errors)
%psdami = ub^2*Dc-M'*Dr*M-j*(Gcr*M-M'*Gcr');
%min(real(eig(psdami)))

% Scale the upper bound by a small amount due to numerical rounding
ub = (1+5e-7)*ub;

% Un-normalize frequency and store outputs
bnds = [ub lb];
info.frequency = lft(wcfreq,Nw);
info.bnds = bnds;
info.Dr = Dr;
info.Dc = Dc;
info.Gcr = Gcr;
info.Delta = Delta;


if ~ostruc.displaywarnings
   warning(SW);
end
