function   [lb,Delta,wcfreq] = munogridlb(M,index,deltareal0,bnds,opt);
% Lower bound without frequency gridding.
% This function assumes the frequency variable is a repeated
% real uncertainty in the first block of the uncertainty. It
% uses a related worst-case gain problem to compute the real blocks
% (including the frequency variable) and the power iteration to compute
% the complex blocks. The algorithm is attempted Ntry times starting 
% from a random initial condition at each try.  
%
% 4/1/06 PJS

% Set number of tries
if nargin < 5
    Ntry = 1;  % Number of tries
else
    % XXX Need to update if we want to pass in other options
    Ntry = opt.Ntry;
end

% Set options  
singtol_real = 1e-7;      % Singularity tolerance for real-mu problems
singtol_comp = 100*eps;   % Singularity tolerance for mixed-mu problems
stoptol = 0.97;           % Algorithm stops if lb/ub>stoptol

% Get size, indices for real/complex blocks, and real mask
[Mr,Mc]=size(M);
nw = abs(index.simpleblk(1,1)); % XXX-assumes freq. is first block

nreal = index.allreal.num;
realidx = index.allreal.realidx;
realr = index.allreal.allrows;
realc = index.allreal.allcols;
szreal = length(realr);

ncomp = index.allcomp.num;
compidx = index.allcomp.compidx;
compr = index.allcomp.allrows;
compc = index.allcomp.allcols;

real_mask = index.masks.real_mask';

% Search for ||Deltar||<=R which makes d->e have large gain and
% then use the power iteration on complex blocks. 
Delta = zeros(Mc-nw,Mr-nw);
wcfreq = 0;

I = eye(szreal);
cnt = 1;
ub = bnds(1,1);
lb = bnds(1,2);
lbfac = 3/4;
while (cnt<=Ntry) && (lb<stoptol*ub) 
        
    % Set upper bound on Delta to achieve a specified lower bound
    lbtry = lb+lbfac*(ub-lb);
    R = 1/lbtry;

    % Scale frequency block and then partition into 
    % 2x2 blocks conformal w/ real/complex blocks
    % (XXX--Assumes that frequency is first block)
    wscl = sqrt(lbtry);
    Mtmp = M;
    Mtmp(1:nw,:) = wscl.*Mtmp(1:nw,:);
    Mtmp(:,1:nw) = wscl.*Mtmp(:,1:nw);    
    Mrr = Mtmp(realr,realc);
    Mrc = Mtmp(realr,compc);
    Mcr = Mtmp(compr,realc);
    Mcc = Mtmp(compr,compc);
    
    % LOCALwclowc finds Deltar which makes d->e have large gain.
    channel = mod(cnt-1,szreal)+1;      % cycle through channels
    if cnt==1 && ~isempty(deltareal0)
        % Use a initial condition on first try if available        
        [wcval,Deltar] = LOCALwclowc(Mrr,realidx,R,channel,deltareal0);
    else
        [wcval,Deltar] = LOCALwclowc(Mrr,realidx,R,channel);
    end
    
    % Check result: 
    %tmp = 1+Mrr(channel,:)*Deltar*inv( eye(szreal)- Mrr*Deltar )*I(:,channel);
    %wcval2 = norm( tmp )
    %[wcval wcval2 abs(wcval2-wcval)]
    
    lb2 = 0;
    if ncomp==0        
        % pure real mu problem
        %if abs(det(eye(szreal) - Mrr*Deltar)) < singtol_real
        if rcond(eye(szreal) - Mrr*Deltar) < singtol_real
            Delta2 = zeros(Mc,Mr);
            Delta2(realc,realr) = Deltar;
            
            freq = Delta2(1,1)/R;
            Delta2 = Delta2(nw+1:end,nw+1:end);            
            lb2 = 1/norm(Delta2);
        end        
    else
        % mixed mu problem
        if abs(det(eye(szreal) - Mrr*Deltar)) < singtol_comp
        %if rcond(eye(szreal) - Mrr*Deltar) < singtol_comp
            Delta2 = zeros(Mc,Mr);
            Delta2(realc,realr) = Deltar;
            
            freq = Delta2(1,1)/R;
            Delta2 = Delta2(nw+1:end,nw+1:end);            
            lb2 = 1/norm(Delta2);
        else
            % wrap in reals
            Mtil = Mcc+Mcr*Deltar*inv(eye(szreal)-Mrr*Deltar)*Mrc;

            % do power iteration on complex blocks
            [lbc,Deltac] = mmupiter(Mtil,compidx);

            % form pert from Deltar and Deltac
            Delta2 = zeros(Mc,Mr);
            Delta2(realc,realr) = Deltar;
            Delta2(compc,compr) = Deltac;
            
            freq = Delta2(1,1)/R;
            Delta2 = Delta2(nw+1:end,nw+1:end);            
            lb2 = 1/norm(Delta2);            
        end        
        
    end
    
    if lb2>lb & lb2<=ub
        % Store Delta if it increased the lower bound
        lb = lb2; 
        Delta = Delta2;         
        wcfreq = freq;
        
        % Reset lbfac
        lbfac = 1/2;        
    else
        % Delta did not increase lb, reduce lbfac for next try
        lbfac = max( 1/32 , lbfac/2 );
    end
    
    cnt = cnt+1;    
end 


%-------------------------------------------------------------------

function [wcval,Delta,deltavals] = LOCALwclowc(M,index,R,channel,deltastart)
% LOCALwclowc solves   max Fu(Delta,L)  subject to ||Delta||<=R
% where L:=[M I(:,channel); M(channel,:) 1] and there is only a
% single disturbance and error.  It is assumed that the block structure
% is purely real ( the code will fail otherwise ).
%
% This is a variant of wclowc with code modified to take
% advantage of the explicit structure of the problem
% created within muwcglb.  Specifically:
%   1) removed code related to complex blocks since muwcglb only
%           calls it with real blocks
%   2) removed code related to multim-dim arrays since muwcglb only
%           calls it with a single matrix (loop on data done in mussvcalc)
%   3) ne=1/nd=1 since muwcglb only adds a disturbance/error on
%           a single channel.  Thus the "22" block of m is a scalar.
%   4) LOCALwclowc has capability to start from a specified initial
%           condition.
%   5) LOCALwclowc combines the code associated with scalar and
%           repeated real perturbations for simplicity/speed.

if nargin<5
    deltastart=[];
end

% Set options 
MXGAIN = 1e12; % Code stops if d->e gain is larger than MXGAIN
MAXCNT = 100; % Number of times to cycle through all reals

% Get sizes and real mask
[Mr,Mc]=size(M);
nreal = index.allreal.num;
real_mask = index.masks.real_mask';

% Insert disturbance and pull off error on a single channel. Also,
% scale L so that code below assumes Delta is in the unit cube.
L = [R*M zeros(Mr,1); M(channel,:) 1]; 
L(channel,end) = R;
ne = 1;
nd = 1;

% Initialize worst-case gain at center of unit cube
wcval = 1;
initcost = wcval;
deltavals_best = zeros(nreal,1);
Lreal = -ones(nreal,1); 
Ureal = ones(nreal,1);

if ~isempty(deltastart)
    % use specified initial condition on first try
    deltavals = deltastart/R;
    deltavals = max( min(1,deltavals) , -1 );
else
    % create random starting point 
    deltavals = Lreal+rand(nreal,1).*(Ureal-Lreal); 
end

% Move to starting point
mloop = zeros(Mr+1);
mloop(end,:) = L(end,:);
deltavec = real_mask*deltavals;
left = [eye(Mr); L(end,1:Mr).*(deltavec.')];
mid =  eye(Mr)-L(1:Mr,1:Mr).*repmat(deltavec.',Mr,1);
right = L(1:Mr,:);
mloop = mloop+ (left/mid)*right;
gain = abs( mloop(end,end) );

% Update bounds to reflect starting point
Lreal = -1-deltavals;
Ureal = 1-deltavals;

% Cycle through reals
mdchange = 1;
cnt = 0;
notinf = 1;
go = 1;
while (cnt<MAXCNT & gain<MXGAIN & go==1) | cnt==0
    cnt = cnt + 1;    
    mdchange = 0;            
    % Create random cycling order each time through
    [dum,oidxreal] = sort(rand(nreal,1));
    gain0 = gain;
    ii = 1;
    while ii<=nreal & gain<MXGAIN
        i = oidxreal(ii);
        blkd = index.allreal.repeated(i);
        ridx = index.allreal.rows{i};
        cidx = index.allreal.cols{i};
        tmpm = mloop([ridx end],[cidx end]);
        [gain,adjustment] = wcslft(tmpm,Lreal(i),Ureal(i),blkd,ne,nd); 
        if adjustment~=0
            deltavals(i) = deltavals(i) + adjustment;
            Lreal(i) = -1 - deltavals(i);
            Ureal(i) = 1 - deltavals(i);
            mdchange = 1;
            if ~isinf(gain)
                mloopnew = mloop;
                mloopnew(ridx,:) = zeros(blkd,Mr+1);
                left = [mloop(1:ridx(1)-1,cidx)*adjustment; ...
                        eye(blkd); mloop(ridx(end)+1:Mr+1,cidx)*adjustment];
                mid = eye(blkd) - mloop(ridx,cidx)*adjustment;
                right = mloop(ridx,:);                    
                mloopnew = mloopnew + (left/mid)*right;
                mloop = mloopnew;                    
            end
        end
        ii = ii + 1;
    end        
    if gain>wcval % XXX shouldn't gain be non-decreasing?
        deltavals_best = deltavals;
        wcval = gain;
    end
    
    %[cnt deltavals_best.' gain] %XXX
    
    if (gain-gain0)<0.001*gain || mdchange ==0
        go = 0; 
    end
end

% Scale Delta
deltavals = deltavals_best*R;
Delta = diag(real_mask*deltavals);