% function [ub,Dr,Dc,Gcr] = munogridub(M,index,opt);
% LMI upper bound without frequency gridding.
% This function assumes the frequency variable is a repeated real
% uncertainty in the first block of the uncertainty. It uses a
% skewed-mu LMI upper bound (cite?) to solve for the upper bound:
%    min ub 
%    subject to:   0 < Dw, 0<Dcu
%    M'*Dr*M + j*(Gcr*M-M'*Gcr') - Dw1 <= ub^2*Dc
% where:
%    Dr = diag(Dw,Dru);
%    Dw1 = diag(Dw,0);
%    Dc = diag(0,Dcu);
%
% A slack variable is introduced to put this in a form compatible
% with that required by gevp:
%    min ub 
%    subject to:   0 < Dw, 0<Dcu
%    M'*Dr*M + j*(Gcr*M-M'*Gcr') - Dw2 <= 0
%    Y <= ub^2 * Dcu
% where:
%    Dr = diag(Dw,Dru);
%    Dw2 = diag(Dw,Y);
%
% Dw is the D-scale for the frequency block and Dru/Dcu are the
% D-scales for the uncertainty blocks associated with the row/column
% dimensions of M.  Gcr contains the G-scales for both the frequency
% and uncertainty blocks.
%
% XXX-The D/G scales associated with the frequency block are restricted
% to be diagonal to reduce the number of variables.  We could add an
% option to allow the user to remove this restriction?
%
% 4/2/06 PJS

function [ub,Dr,Dc,Gcr] = munogridub(M,index,opt);

% Argument Checks
if nargin==2
    opt = '';
elseif nargin~=3
    error('[ub,Dr,Dc,Gcr] = munogridub(M,index);');
end

% Problem data
[Mr,Mc] = size(M);  % M should be 2-D at this point
nreal = index.allreal.num;
realr = index.allreal.allrows;
realc = index.allreal.allcols;
nw = abs(index.simpleblk(1,1)); % XXX-assumes freq. is first block

% Dpdidx is used to define a compressed version of D-scales
Dpdidx = [];
for i1 = 1:index.full.num
    tmp = index.full.cols{i1};
    Dpdidx = [Dpdidx tmp(2:end)];
end

% Set up LMIs:  Most of the LMI setup does not depend on M.
% Set up this part first or pass in from a previous set up.
setlmis([]);

% Initialize D-scale and G-scale variables
Dwreal = zeros(nw); 
Dwimag = Dwreal; 

Drureal = zeros(Mr-nw);
Druimag = Drureal;
Dcureal = zeros(Mc-nw);
Dcuimag = Dcureal;
Gcrreal = zeros(Mc,Mr);
Gcrimag = Gcrreal;
Dpdreal = [];

Yreal = zeros(Mc-nw);
Yimag = Yreal;

% Create D-scale and G-scale variables for frequency block
%  - Assumes frequency uncertainty is the first block
if opt=='f'
    vars = 0;
    
    blksz = nw;
    symblk  = symdec(blksz,vars);
    vars = vars+blksz*(blksz+1)/2;
    Dwreal = symblk;
    
    skewblk  = skewdec(blksz,vars);
    vars = vars+blksz*(blksz-1)/2;
    Dwimag = skewblk;    

    symblk  = symdec(blksz,vars);
    vars = vars+blksz*(blksz+1)/2;
    Gcrreal(1:nw,1:nw) = symblk;

    skewblk  = skewdec(blksz,vars);
    vars = vars+blksz*(blksz-1)/2;
    Gcrimag(1:nw,1:nw) = skewblk;    
else
    %  - Restricts D/G scales for freq. block to be diagonal
    Dwreal = diag(1:nw);
    Gcrreal(1:nw,1:nw) = diag( nw+1 : 2*nw);
    vars = 2*nw;
end

% Create D-scale and G-scale variables for uncertainty blocks
blk=index.simpleblk;
ridxm = index.ridxm; % block row indices for M
cidxm = index.cidxm; % block col indices for M
for i1=2:size(blk,1);        
    ridx = ridxm(i1): (ridxm(i1+1)-1);
    cidx = cidxm(i1): (cidxm(i1+1)-1);

    ridx = ridx-nw;
    cidx = cidx-nw;
    
    if blk(i1,2) == 0 % repeated real or complex scalar
        blksz = abs(blk(i1,1));
        symblk  = symdec(blksz,vars);
        vars = vars+blksz*(blksz+1)/2;
        Drreal(ridx,ridx) = symblk;
        Dcreal(cidx,cidx) = symblk;

        skewblk  = skewdec(abs(blk(i1,1)),vars);
        vars = vars+blksz*(blksz-1)/2;
        Drimag(ridx,ridx) = skewblk;
        Dcimag(cidx,cidx) = skewblk;

        % Create slack variable, Y
        symblk  = symdec(blksz,vars);
        vars = vars+blksz*(blksz+1)/2;
        Yreal(cidx,cidx) = symblk;
        
        skewblk  = skewdec(abs(blk(i1,1)),vars);
        vars = vars+blksz*(blksz-1)/2;
        Yimag(cidx,cidx) = skewblk;
        
        if blk(i1,1)<0 % repeated real scalar
            ridx = ridx+nw;
            cidx = cidx+nw;
                       
            symblk  = symdec(abs(blk(i1,1)),vars);
            vars = vars+blksz*(blksz+1)/2;
            Gcrreal(cidx,ridx) = symblk;

            skewblk  = skewdec(abs(blk(i1,1)),vars);
            vars = vars+blksz*(blksz-1)/2;
            Gcrimag(cidx,ridx) = skewblk;
        end

    elseif blk(i1,2)>0 % complex full block
        vars = vars+1;
        Drreal(ridx,ridx) = vars*eye(blk(i1,2));
        Dcreal(cidx,cidx) = vars*eye(blk(i1,1));
        
        vars=vars+1;
        Yreal(cidx,cidx) = vars*eye(blk(i1,1));
    end

end


% Create D-scale variables
%    Dr = diag(Dw,Dru);
%    Dw2 = diag(Dw,Y);
%    Dpd = diag(Dw,Dcu);
tmpreal = blkdiag(Dwreal,Drreal);
tmpimag = blkdiag(Dwimag,Drimag);
Dr=lmivar(3,[tmpreal tmpimag; -tmpimag tmpreal]);

Dcu=lmivar(3,[Dcreal Dcimag; -Dcimag Dcreal]);

Y=lmivar(3,[Yreal Yimag; -Yimag Yreal]);

Dw = lmivar(3,[Dwreal Dwimag; -Dwimag Dwreal]);

tmpreal = blkdiag(Dwreal,Yreal);
tmpimag = blkdiag(Dwimag,Yimag);
Dw2=lmivar(3,[tmpreal tmpimag; -tmpimag tmpreal]);

if 0%-----
% Dpd is a 'compressed' version of the D-scales
% used to enforce the pos. def. constraint
Dpdreal = blkdiag(Dwreal,Dcreal);
Dpdimag = blkdiag(Dwimag,Dcimag);
Dpdreal(Dpdidx,:) = [];
Dpdreal(:,Dpdidx) = [];
Dpdimag(Dpdidx,:) = [];
Dpdimag(:,Dpdidx) = [];

if any(Dpdimag(:))
    Dpd=lmivar(3,[Dpdreal Dpdimag; -Dpdimag Dpdreal]);
    szDpd = 2*size(Dpdreal,1);            
else
    Dpd=lmivar(3,Dpdreal);
    szDpd = size(Dpdreal,1);
end
end%-----

% Create G-scale variables
if nreal>0
    Gcr=lmivar(3,[Gcrreal Gcrimag;-Gcrimag Gcrreal]);
end

% Set: 
%    M'*Dr*M + j*(Gcr*M-M'*Gcr') - Dw2 <= 0
Mreal=real(M);   
Mimag=imag(M);
M1 = [Mreal Mimag;-Mimag Mreal];
lmiterm([1 1 1 Dr],M1',M1);
lmiterm([-1 1 1 Dw2],1,1);
if (nreal>0)
    jM2 = [-Mimag Mreal;-Mreal -Mimag];
    lmiterm([1 1 1 Gcr],1,jM2,'s');
end

% Set D-scale constraints
lmiterm([2 1 1 0],1e-5);        % D>c*I
lmiterm([-2 1 1 Dw],1,1);      % D>c*I

lmiterm([3 1 1 0],1e-5);        % D>c*I
lmiterm([-3 1 1 Dcu],1,1);      % D>c*I


% Set: Y<ub^2*Dcu
lmiterm([-4 1 1 Dcu],1,1);
lmiterm([4 1 1 Y],1,1);

% Solve generalized eigenvalue problem
lmisys=getlmis;
opts = [1e-2;0;1e5;5;1]; % These are the opts used by mubnd.m
%opts = [1e-2;0;1e5;5;0]; % Uncomment to view LMILab output
nlfc=1;
[ubsq,xopt] = gevp(lmisys,nlfc,opts);

% Form output
if isempty(xopt)
    %warning('Unable to find feasible point');
    ub = inf;
    Dc = zeros(nw);
    Dw = eye(nw);
    Dru = eye(Mr-nw);
    Dcu = eye(Mc-nw);
    Gcr = zeros(Mc,Mr);
else    
    ub=sqrt(max(0,ubsq));
    
    if nargout>1,                
        Dcu=dec2mat(lmisys,xopt,Dcu);   
        tmp = Mc-nw;
        Dcu=Dcu(1:tmp,1:tmp)+j*Dcu(1:tmp,tmp+1:2*tmp);
        Dc = blkdiag(zeros(nw),Dcu);        
        
        Dr=dec2mat(lmisys,xopt,Dr);   
        Dr=Dr(1:Mr,1:Mr)+j*Dr(1:Mr,Mr+1:2*Mr);
        
        Dw2 = dec2mat(lmisys,xopt,Dw2);
        Dw2=Dw2(1:Mc,1:Mc)+j*Dw2(1:Mc,Mc+1:2*Mc);
        Dw1 = Dw2;
        Dw1(nw+1:end,nw+1:end) = 0;
        
        if (nreal>0),
            Gcr=dec2mat(lmisys,xopt,Gcr);  
            Gcr=Gcr(1:Mc,1:Mr)+j*Gcr(1:Mc,Mr+1:2*Mr);
        else 
            Gcr=zeros(Mc,Mr); 
        end

        % Check result (AMI should be psd)
        %    M'*Dr*M + j*(Gcr*M-M'*Gcr') - Dw1 <= ub^2*Dc
        psdami = ub^2*blkdiag(zeros(nw),Dcu)+Dw1-M'*Dr*M-j*(Gcr*M-M'*Gcr');
        fprintf(' ami_mineig=%6.4f\n',min(real(eig(psdami))));
        
    end        
end

