function [B,varargout] = gridureal(A,varargin)
% GRIDUREAL   Grid UREAL atoms over their range
%
%    B = GRIDUREAL(A,N) substitutes N samples of the uncertain
%    real parameters in A returning an array of size [SIZE(A) N].  
%    The N samples are generated by uniformly gridding each UREAL 
%    in A with N points (N=1 uses the nominal of each UREAL) and then
%    evaluating A at the N "diagonal" points in the parameter space.
%    For example, if (a,b,c,d) are four ureals in A then A is
%    evaluated at:
%       (a(1),b(1),c(1),d(1)) ; ...; (a(N),b(N),c(N),d(N))
%    If A includes uncertain objects other than uncertain reals, then 
%    B will be an uncertain object. GRIDUREAL(A) is the same as 
%    GRIDUREAL(A,1). 
%
%    [B,SampleValues] = GRIDUREAL(A,N) additionally returns the specific
%    sampled values (as a STRUCT whose fieldnames are the names of A's
%    uncertain elements) of the uncertain reals.  Hence, B is the
%    same as USUBS(A,SampleValues).
%
%    [B,SampleValues] = GRIDUREAL(A,NAMES,N) samples only the uncertain
%    reals listed in the NAMES variable (CELL, or CHAR array). Any
%    entries of NAMES that are not elements of A are simply ignored. 
%    Note that GRIDUREAL(A,fieldnames(A.Uncertainty),N) is the same 
%    as GRIDUREAL(A,N).
%
%    [B,SampleValues] = GRIDUREAL(A,NAMES1,N1,NAMES2,N2,...) takes N1 
%    samples of the uncertain elements listed in NAMES1, and N2 samples of
%    the uncertain elements listed in NAMES2 and so on.  SIZE(B) will
%    equal [SIZE(A) N1 N2 ...].
% 
%    % EXAMPLE 1: (CUT/PASTE)
%    % Create two UREALS
%    gamma = ureal('gamma',4);
%    tau = ureal('tau',.5,'Percentage',30);
%    % Create Uncertain Plant
%    P = tf(gamma,[tau 1]);
%    % Create Integral Controller based on nominal plant parameters
%    KI = 1/(2*tau.Nominal*gamma.Nominal);
%    C = tf(KI,[1 0]);
%    % Create Uncertain Closed-Loop System
%    CLP = feedback(P*C,1);
%    % Plot (many) Uncertain Plant step responses 
%    subplot(2,1,1); step(gridureal(P,20))
%    % Plot (many) Uncertain Closed-Loop step responses
%    subplot(2,1,2); step(gridureal(CLP,20))
% 
%    % EXAMPLE 2: (CUT/PASTE)
%    % Make 4 UREALs, and a simple uncertain matrix
%    a=ureal('a',1); b=ureal('b',2); c=ureal('c',3); d=ureal('d',4);
%    m = [a b;c d];
%    % Sample the (a,b) space in 5 places, and the (c,d) space in 3
%    % places, and evaluate m at the 15 "grid"-points 
%    m1 = gridureal(m,{'a';'b'},5,{'c';'d'},3);
%    % Sample the (a,b,c,d) space at 15 places, and evaluate m at
%    % these 15 points 
%    m2 = gridureal(m,{'a';'b';'c';'d'},15);
%    % The 2,1 entry of m1 (which is c) only takes on 3 distinct values,...
%    subplot(2,1,1)
%    hist(m1(2,1,:))
%    % while the 2,1 entry of m2 (which is also c) takes
%    % on 15 distinct values uniformly through its range
%    subplot(2,1,2)
%    hist(m2(2,1,:))
%
%    See also USAMPLE, USUBS.

% Copyright 2003-2004 The MathWorks, Inc.
varargout = cell(1,nargout-1);
if isa(A,'atom')
   [B,varargout{:}] = gridureal(umat(A),varargin{:});
else
   estr = 'First argument should be an ATOM.';
   error(estr);
end