function sysout = parallel(s1,s2,varargin)
%PARALLEL  Parallel interconnection of two uncertain models.
%
%                          +------+
%            v1 ---------->|      |----------> z1
%                          |USYS1 |
%                   u1 +-->|      |---+ y1
%                      |   +------+   |
%             u ------>+              O------> y
%                      |   +------+   |
%                   u2 +-->|      |---+ y2
%                          |USYS2 |
%            v2 ---------->|      |----------> z2
%                          +------+
%
%   USYS = PARALLEL(USYS1,USYS2,IN1,IN2,OUT1,OUT2) connects the two 
%   uncertaint models USYS1 and USYS2 in parallel such that the inputs 
%   specified by IN1 and IN2 are connected and the outputs specified
%   by OUT1 and OUT2 are summed.  The resulting USS model USYS maps 
%   [v1;u;v2] to [z1;y;z2].  The vectors IN1 and IN2 contain indexes 
%   into the input vectors of USYS1 and USYS2, respectively, and define 
%   the input channels u1 and u2 in the diagram.  Similarly, the 
%   vectors OUT1 and OUT2 contain indexes into the outputs of these 
%   two systems. 
%
%   If IN1,IN2,OUT1,OUT2 are jointly omitted, PARALLEL forms the 
%   standard parallel interconnection of USYS1 and USYS2 and returns
%          USYS = USYS2 + USYS1 .
%
%   If USYS1 and USYS2 are arrays of USS models, PARALLEL returns an USS
%   array USYS of the same size where 
%      USYS(:,:,k) = PARALLEL(USYS1(:,:,k),USYS2(:,:,k),IN1,...) .
%
%   See also APPEND, SERIES, FEEDBACK, ICONNECT, SYSIC.

% Copyright 2004 The MathWorks, Inc.

sysout = parallel(umat(s1),umat(s2),varargin{:});
