function out = ss(varargin)
%SS  Creates state-space model or converts model to state space.
%       
%   SYS = SS(A,B,C,D) creates a SS object SYS representing the 
%   continuous-time state-space model
%         dx/dt = Ax(t) + Bu(t)
%          y(t) = Cx(t) + Du(t)
%   You can set D=0 to mean the zero matrix of appropriate dimensions.
%   If one or more of the matrices A,B,C,D have uncertainty, SS returns 
%   an uncertain state-space (USS) model (Robust Control Toolbox only).
%
%   SYS = SS(A,B,C,D,Ts) creates a discrete-time state-space model with  
%   sample time Ts (set Ts=-1 if the sample time is undetermined).
%
%   SYS = SS creates an empty SS object.
%   SYS = SS(D) specifies a static gain matrix D.
%
%   In all cases above, the input list can be followed by pairs
%      'PropertyName1', PropertyValue1, ...
%   that set various model properties (see LTIPROPS for details).
%
%   You can create arrays of state-space models by using ND arrays for
%   A,B,C,D.  The first two dimensions of A,B,C,D define the 
%   number of states, inputs, and outputs, while the remaining 
%   dimensions specify the array sizes.  For example, 
%      sys = ss(rand(2,2,3,4),[2;1],[1 1],0)
%   creates a 3x4 array of SISO state-space models.  You can also
%   use indexed assignment and STACK to build SS arrays:
%      sys = ss(zeros(1,1,2))     % create 2x1 array of SISO models
%      sys(:,:,1) = rss(2)        % assign 1st model
%      sys(:,:,2) = ss(-1)        % assign 2nd model
%      sys = stack(1,sys,rss(5))  % add 3rd model to array
%
%   SYS = SS(SYS) converts a system SYS of arbitrary class to state 
%   space, that is, computes a state-space realization of SYS.
%
%   SYS = SS(SYS,'min') computes a minimal realization of SYS.
%
%   See also LTIMODELS, LTIPROPS, DSS, RSS, DRSS, SSDATA, TF, ZPK, FRD.

% Copyright 2003-2005 The MathWorks, Inc.
%   $Revision: 1.1.6.4 $  $Date: 2005/06/27 23:27:32 $

nin = nargin;
% Determine how many Property NameValue pairs (NVP)
% AIN denotes number of leading arguments (not name/value pairs)
nvp = 0;
i = 1;
while i<nin && isa(varargin{nin-i},'char')
   nvp = nvp + 1;
   i = i + 2;
end
ain = nin - 2*nvp;

switch ain
   case 0
      disp('usage: out = ss(a,b,c,d)');
      return
   case 4
      [a,b,c,d] = deal(varargin{1:4});
      Ts = 0;
   case 5
      [a,b,c,d,Ts] = deal(varargin{1:5});
   case 1
      d = varargin{1};
      a = [];
      b = [];
      c = [];
      Ts = 0;
   case 2
      error('Undefined C and D matrices.');
   case 3
      error('Undefined D matrix.');
end

try
   out = ss(umat(a),umat(b),umat(c),umat(d),Ts,varargin{ain+1:end});
   return
catch
   rethrow(lasterror);
end

