function [B,varargout] = usample(A,varargin)
%USAMPLE   Generates random samples of uncertain matrix or system.
%   B = USAMPLE(A,N) picks N random samples of the uncertainty in A and 
%   returns these samples in an array B of size [SIZE(A) N].  A can be 
%   an uncertain element, matrix, or system (see UMAT, USS, UFRD, UREAL,...) 
%   USAMPLE(A) is the same as USAMPLE(A,1).
%
%   [B,SAMPLES] = USAMPLE(A,N) also returns the uncertainty samples in 
%   the N-by-1 structure array SAMPLES whose field names and values are 
%   the names and sample values of the uncertain elements in A.  Note 
%   that B is the same as USUBS(A,SAMPLES).
%
%   [B,SAMPLES] = USAMPLE(A,NAMES,N) samples only the uncertain
%   elements listed in the string or string vector NAMES. The output B 
%   is uncertain unless NAMES includes all uncertain elements.  Any 
%   entry of NAMES that does not appear in A is ignored.  
%
%   [B,SAMPLES] = USAMPLE(A,NAMES1,N1,NAMES2,N2,...) takes N1 samples of
%   the uncertain elements listed in NAMES1, N2 samples of the uncertain
%   elements listed in NAMES2, and so on.  The resulting B has size 
%   [SIZE(A) N1 N2 ...].
%
%   The scalar parameter BW in
%      [B,SAMPLES] = USAMPLE(A,N,BW),
%      [B,SAMPLES] = USAMPLE(A,NAMES,N,BW) and 
%      [B,SAMPLES] = USAMPLE(A,NAMES1,N1,NAMES2,N2,BW)
%   affects how ULTIDYN elements within A are sampled, restricting the
%   poles of the samples.  If A is a continuous-time USS or UFRD, then
%   the poles of sampled GainBounded ULTIDYN elements in SAMPLES will each
%   have magnitude <= BW.  If A is a discrete-time, then sampled GainBounded
%   ULTIDYN elements are obtained by Tustin transformation of a continuous
%   system, again using BW as the (continuous) pole magnitude bound.  
%   If the ULTIDYN type is PositiveReal, then the samples are obtained by
%   bilinearly transforming (see "Normalizing Functions for Uncertain Atoms"
%   in the documentation) across the input-output channels of the GainBounded
%   elements described above.
%
%   USAMPLE is useful for Monte-Carlo analysis of uncertain systems.
%
%   Example 1: 
%      % Create an uncertain model of P(s) = g/(tau*s+1) where
%      % g varies in [3,5] and tau = 0.5 +/- 30%
%      g = ureal('g',4);
%      tau = ureal('tau',.5,'Percentage',30);
%      P = tf(g,[tau 1]);
%      % Create an integral controller based on nominal (g,tau)
%      C = tf(1,[1 0]) / (2*tau.Nominal*g.Nominal)
%      % Plot the plant and closed-loop responses for 20 samples
%      % of (g,tau)
%      subplot(2,1,1); step(usample(P,20))
%      subplot(2,1,2); step(usample(feedback(P*C,1),20))
%
%   Example 2:
%      % Create 4 uncertain parameters a,b,c,d and form M=[a b;c d]
%      a=ureal('a',1); b=ureal('b',2); c=ureal('c',3); d=ureal('d',4);
%      M = [a b;c d];
%      % Pick 5 samples for (a,b), 3 samples for (c,d), and evaluate
%      % M at the corresponding 5x3=15 combinations
%      M1 = usample(M,{'a';'b'},5,{'c';'d'},3)
%      % Evaluate M at 15 samples of (a,b,c,d)
%      M2 = usample(M,{'a';'b';'c';'d'},15)
%
%   See also USUBS, USS, UFRD, UMAT, UREAL, UCOMPLEX, ULTIDYN.

%   Copyright 2003-2006 The MathWorks, Inc.
%   $Revision: 1.1.6.4 $  $Date: 2006/12/27 21:20:25 $

varargout = cell(1,nargout-1);
if isa(A,'atom')
   [B,varargout{:}] = usample(umat(A),varargin{:});
else
   estr = 'First argument should be an ATOM.';
   error(estr);
end
