function sys = uss(varargin)
%USS  Specify uncertain state space.  
%   USS is the constructor for uncertain state space models 
%   (USS objects).  Uncertain state space models are
%   useful for worst-case gain and robustness analysis.
%  
%   There are three ways to construct a USS object.
%   
%   1.  Use SS to create uncertain state-space objects from 
%       uncertain state-space matrix data (See UMAT).
%
%       p = ureal('p',1);
%       A = [0 3*p; -p p^2];
%       B = [0; p];
%       C = ones(2,2);  D=zeros(2,1);
%       usys = ss(A,B,C,D);
%   
%   2.  Combine an SS, TF, or ZPK object with uncertain elements.
%
%       sys = tf(1, [1 1]);
%       k = ureal('k',1);
%       usys= k*sys;
%
%   3.  Recast a double, uncertain element (ATOM object), LTI 
%       or UMAT object as a USS object.
%
%       M = tf(1, [1 1 1]);
%       M = uss(M);
%       Here, M is a 3-state SISO system (USS object) with no uncertainty.
%
%   If M is a DOUBLE, then USYS is a static gain with no uncertain elements.
%   If M is an uncertain atom, then USYS is a static gain.
%   If M is an LTI object, then USYS has no uncertainty.
%   If M is a UMAT, then USYS is a static gain.
%
%   See also: SS, METHODS(USS)

%   Copyright 2003-2004 The MathWorks, Inc.

nin = nargin;
nvp = 0;
i = 1;
while nin>i && isa(varargin{nin-i},'char')
   nvp = nvp + 1;
   i = i + 2;
end
ain = nin - 2*nvp;

if nvp==0
   if ain==1  % USS(ATOM)
      x = varargin{1};
      sx = size(x);
      cx = class(x);
      switch cx
         case {'ureal','ultidyn','ucomplex','ucomplexm','udyn'}
            Xs.UmatData = umat(x);
            Xs.StateName = cell(0,1);
            Xs.Ts = utarbts;
            a4 = dynamicsys(sx(1),sx(2));
            sys = uss(Xs,a4,varargin{2:end});
         otherwise
            error('Invalid ATOM class');
      end
   elseif ain>=4 % USS(A/D,A/D,A/D,A/D; D or SS/TF), at least one atom
      cain = zeros(1,4);
      for i=1:4
         cain(i) = isa(varargin{i},'double') || isa(varargin{i},'ureal') || ...
            isa(varargin{i},'ucomplex') || isa(varargin{i},'ucomplexm') || ...
            isa(varargin{i},'udyn') || isa(varargin{i},'ultidyn');
      end
      if all(cain==1)
         if ain==4
            sys = uss(umat(varargin{1}),umat(varargin{2}),...
               umat(varargin{3}),umat(varargin{4}));
         elseif ain==5 && (isa(varargin{5},'double') || ...
              isa(varargin{5},'ss') || isa(varargin{5},'tf') || ...
              isa(varargin{5},'zpk') || isa(varargin{5},'frd'))
            sys = uss(umat(varargin{1}),umat(varargin{2}),...
              umat(varargin{3}),umat(varargin{4}),varargin{5});
         else
            error('5th argument should be TS or REFSYS');
         end
      else
         badarg = find(cain==0);
         error(['Argument #' int2str(badarg(1)) ' is incorrect class']);
      end
   else 
      error('Invalid argument list')
   end
else
   sys = uss(varargin{1:ain});  % y can be only USS or UFRD
   sys = set(sys,varargin{ain+1:end});
end
