%FITFRD   Fit frequency response data with state-space model.
%   B = FITFRD(A,N) is a state-space object, with state-dimension N,
%   whose frequency response closely matches the frequency response
%   data in A.  A is an FRD object, and N is a nonnegative integer.
%
%   A must have either 1 row or 1 column, though it need not
%   be 1-by-1.  B will be the same size as A.  In all cases, N should
%   be a nonnegative scalar.
%
%   B = FITFRD(A,N,RD) forces the relative degree of B to be RD.  RD
%   should be a nonnegative integer.  The default value for RD is 0.
%   If A is a row (or column) then RD can be a vector of the same size
%   as well, specifying the relative degree of each entry of B.  If
%   RD is a scalar, then it specifies the relative degree for all entries
%   of B.  The default for RD can also be specified by setting RD = [].
%
%   B = FITFRD(A,N,RD,WT) uses the magnitude of WT to weight the
%   optimization fit criteria.  WT may be a DOUBLE, SS or FRD.  If
%   WT is a scalar, then it is used to weight all entries of the
%   error criteria (A-B).  If WT is a vector, it should be the same
%   size as A, and each individual entry of WT acts as a weighting
%   function on the corresponding entry of (A-B).
%
%   See also: INVFREQS, INVFREQZ 

% Copyright 2003-2005 The MathWorks, Inc.
% old help
%   CODE   : 0 - no restriction on pole location of SYS (default)
%            1 - restrict SYS to be stable, minimum-phase, requires
%                   FRDATA to be 1x1 VARYING matrix
%            2 - constrain rational fit so that SYS is stable
%   IDNUM  : number of LS iterations (default = 2)

function [sys,fail] = fitfrd(frdsys,ord,reldeg,weight,code,idnum)

% we should inherit all of the dynamic system properties from FRDATA
if nargin < 2
   error(['minimal usage: sys = fitfrd(frdsys,order)']);
elseif nargin==2
   reldeg = [];
   weight = [];
   code = [];
   idnum = [];
elseif nargin == 3
   weight = [];
   code = [];
   idnum = [];
elseif nargin == 4
   code = [];
   idnum = [];
elseif nargin == 5
   idnum = [];
end
fail = 0;

szf = size(frdsys);
if ~isa(frdsys,'frd') || length(szf)>2 || min(szf)>1
   error('A should be a FRD, with 1 row or column, and no array dimensions.');
end
flag = 0;  % input is a ROW
Ntf = szf(2);
if (szf(1)~=1)
   frdsys = frdsys.';
   flag = 1; % input is a COLUMN
   Ntf = szf(1);
end

if isempty(code)
   code = 0;
end
if isempty(idnum)
   idnum = 6;
end
if isa(code,'double') && ndims(code)==2 && isequal(size(code),[1 1])
   if code~=1 & code~=2
      code = 0;
   end
else
   error('CODE should be a scalar double.');
end
if isa(idnum,'double') && ndims(idnum)==2 && isequal(size(idnum),[1 1])
   if ceil(idnum)~=floor(idnum) || idnum<0
      warning('Resetting IDNUM to default...');
      idnum = 6;
   end
else
   error('IDNUM should be a scalar double.');
end

[respdata,Frequency,Ts] = frdata(frdsys);
Units = get(frdsys,'Units');
nfreq = length(Frequency);

if isequal(Units,'rad/s')
   romega = Frequency(:);
elseif  isequal(Units,'Hz')
   romega = (2*pi)*Frequency(:);
end
if Ts==0
   if romega(1)==0 & romega(2)>0
      w1 = romega(2);
   elseif romega(1)>0
      w1 = romega(1);
   else
      error('Need a nonzero 1st or 2nd frequency');
   end
   if isinf(romega(end)) && ~isinf(romega(end-1))
      w2 = romega(end-1);
   elseif ~isinf(romega(end))
      w2 = romega(end);
   else
      error('Need a non-infinite last (or second to last) frequency');
   end
   [alpha,wm,domeorig,L,P,ejtheta] = hp2dmax(w1,w2,romega);
else
   thetavec = romega*abs(Ts);
   keep = find(thetavec>=0 & thetavec<=pi);
   if length(keep)~=nfreq
      error('Frequency data (in rad/s) should be in [0 pi/Ts]');
   else 
      if thetavec(1)==0 & thetavec(2)>0
         t1 = thetavec(2);
      elseif thetavec(1)>0
         t1 = thetavec(1);
      else
         error('Need a nonzero 1st or 2nd frequency');
      end
      if thetavec(end)==pi & thetavec(end-1)<pi
         t2 = thetavec(end-1);
      elseif thetavec(end)<pi
         t2 = thetavec(end);
      else
         error('Need a non-pi last (or second to last) frequency');
      end
      [b,tm,domeorig,L,P,ejpsi] = d2dmax(t1,t2,thetavec);
   end
end


szw = size(weight);
if isempty(weight)
   weight = ones(nfreq,Ntf);
elseif isa(weight,'frd')
   [wresp,wromega,wTs] = frdata(weight);
   wUnits = get(weight,'Units');
   if isequal(wUnits,'rad/s')
      wromega = wromega(:);
   elseif  isequal(wUnits,'Hz')
      wromega = (2*pi)*wromega(:);
   end
   
	if ~isequal(wromega,romega)
	   error('Weight and FRDATA have incompatible frequency data')
   elseif wTs~=-1 && wTs~= Ts
	   error('Weight and FRDATA have incompatible sample times')
	elseif szw(1)~=1 & szw(2)~=1
	   error('Weight must be Column or Row');
	elseif szw(1)==1 & szw(2)==Ntf
	   weight = abs(permute(wresp,[3 2 1]));
	elseif szw(1)==Ntf & szw(2)==1
	   weight = abs(permute(wresp,[3 1 2]));
	elseif (szw(1)==1 & szw(2)==1)
	   weight = repmat(abs(wresp(:)),[1 Ntf]);
	else
	   error('Incompatible Dimensions between WT and FRDATA ')
	end
elseif isa(weight,'double')
   if szw(1)==1 & szw(2)==1
      weight = ones(nfreq,Ntf);
   elseif szw(1)~=1 & szw(2)~=1
      error('Weight must be Column or Row');
   elseif max(szw) == Ntf
      weight = weight(:)';
      weight = repmat(abs(weight),nfreq,1);
   else
      error('Incompatible Dimensions between WT and FRDATA ')
   end
elseif isa(weight,'ss') || isa(weight,'tf') || isa(weight,'zpk')
   % do we need to check Ts? 
	wTs = get(weight,'Ts');
	if wTs~=-1 && wTs~= Ts
	   error('Weight and FRDATA have different sample times')
	elseif szw(1)==1 & szw(2)==1
	   weight = frd(weight,romega);   % Use ROMEGA in doing the FRD calc 
      wresp = frdata(weight);
      weight = repmat(abs(wresp(:)),[1 Ntf]);
	elseif szw(1)~=1 & szw(2)~=1
	   error('Weight must be Column or Row');
	elseif szw(1)==1 & szw(2)==Ntf
      weight = frd(weight,romega);  % Use ROMEGA in doing the FRD calc 
      weight = abs(permute(frdata(weight),[3 2 1]));
   elseif szw(1)==Ntf & szw(2)==1
      weight = frd(weight,romega);  % Use ROMEGA in doing the FRD calc 
      weight = abs(permute(frdata(weight),[3 1 2]));
	else
	   error('Incompatible Dimensions between WT and FRDATA ')
	end
else
	error('WT is not valid');
end
% at this point, weight is NFREQ x NTF double



% XXX Check reldeg>=0 && reldeg <= ord?
if isempty(reldeg)
   reldeg = zeros(1,Ntf);
elseif length(reldeg)==1
   reldeg = reldeg*ones(1,Ntf);
end

if length(reldeg)==Ntf & all(reldeg>=0) & all(reldeg<=ord)
   reldeg = reldeg(:)';
else
   error('The size of RELDEG is incompatible with FRDATA.');
end
   
respdata = permute(respdata,[3 2 1]);
if ord>0   
   zz = exp(sqrt(-1)*domeorig);
	if Ts==0
		tmp = repmat(zz+1,[1 Ntf]).^repmat(reldeg,[nfreq,1]);
   else
		tmp = repmat(zz-1/b,[1 Ntf]).^repmat(reldeg,[nfreq,1]);
	end
   weight = weight.*tmp;
	respdata = respdata./tmp;

	[Ncell,den] = fitsyseng(respdata,domeorig,ord,ord-reldeg(:),weight,idnum);
   
%  This is the hack-CODE for enforcing stable -- reflect the poles, and
%  then holding the denominator fixed, refit the numerator with 1
%  iteration.
   if code==2
      rs = roots(den);
      uloc = find(abs(rs)>1);
      if length(uloc)>0
         rs(uloc) = rs(uloc)./(abs(rs(uloc)).^2);
         sden = real(poly(rs).');
         [Ncell,Dummyden] = fitsyseng(respdata,domeorig,ord,ord-reldeg(:),weight,1,sden);
         den = sden;
      end
   end
	for i=1:Ntf
       Ncell{i} = Ncell{i}';  % make the individual N's row vectors
	end
	den = den(:).'; % make D a row vector

% GJB 15Oct04: Adding MINREAL to remoce P/Z cancellations
%    sysC = ss(tf(Ncell,den,Ts)); % SYSS will DIM-by-1, regardless
% 	[Amat,Bmat,Cmat,Dmat] = ssdata(sysC);
   sysC = ss(tf(Ncell,den,-1)); % SYSS will DIM-by-1, regardless
   sysC = minreal(sysC);
	[Amat,Bmat,Cmat,Dmat] = ssdata(sysC);
   ord = size(Amat,1);
%
   pssmat = [Amat Bmat;Cmat Dmat];
   try
      sys = lft(kron(L,eye(ord)),pssmat,ord,ord);
   catch
      fail = 1;
      [Ncell,den] = fitsyseng(respdata,domeorig,ord-1,ord-1-reldeg(:),weight,idnum);
      for i=1:Ntf
          Ncell{i} = Ncell{i}';  % make the individual N's row vectors
      end
      den = den(:).'; % make D a row vector
      sysC = ss(tf(Ncell,den,-1)); % SYSS will DIM-by-1, regardless
      sysC = minreal(sysC);
      [Amat,Bmat,Cmat,Dmat] = ssdata(sysC);
      ord = size(Amat,1);
      pssmat = [Amat Bmat;Cmat Dmat];
      try
         sys = lft(kron(L,eye(ord)),pssmat,ord,ord);
      catch
         error('Forming disk-to-disk transformation fails') 
      end
   end
   Amat = sys(1:ord,1:ord);
   Bmat = sys(1:ord,ord+1:end);
   Cmat = sys(ord+1:end,1:ord);
   Dmat = sys(ord+1:end,ord+1:end);
   if Ts==0	
   	for i = 1:Ntf
         Cmat(i,:) = Cmat(i,:)*(-2*alpha)^reldeg(i);  
         Dmat(i,:) = Dmat(i,:)*(-2*alpha)^reldeg(i);  
         [Cmat(i,:),Dmat(i,:)] = sisormzero(Amat,Bmat,Cmat(i,:),Dmat(i,:),alpha,reldeg(i));
      end
      sys = ss(Amat,Bmat,Cmat,Dmat);
   else
      if b~=0
         gam = b^2/(b^2-1);
         for i = 1:Ntf
            Cmat(i,:) = Cmat(i,:)/(gam)^reldeg(i);  
            Dmat(i,:) = Dmat(i,:)/(gam)^reldeg(i);  
            [Cmat(i,:),Dmat(i,:)] = sisormzero(Amat,Bmat,Cmat(i,:),Dmat(i,:),-1/b,reldeg(i));
         end
      end
      sys = ss(Amat,Bmat,Cmat,Dmat,Ts);
   end
else
   sys = zeros(1,Ntf);
	for kk=1:Ntf
		amat = [weight(:,kk);zeros(nfreq,1)];
		bmat = [weight(:,kk).*real(respdata(1:nfreq,kk)); ...
		        weight(:,kk).*imag(respdata(1:nfreq,kk))];
		sys(kk) = amat\bmat;
	end
   sys = ss([],[],[],sys,Ts);
   if flag==1
      sys = sys.';
   end
end

if code == 1 & Ntf == 1
   if Ts==0
      sys = xextsmp(sys);
   else
      %warning('XXX still need to fix this in FITFRD');
      if ~isstatic(sys)
         Ts = sys.Ts;
         sys = c2d(xextsmp(d2c(sys,'prewarp',1)),Ts,'prewarp',1);
      end
   end
end


if flag==0
   sys = sys.';
end
% sys = strans(sys);
% if code == 1 &  Ntf == 1
%    if Ts==0
%       sys = extsmp(sys);
%    else
%       sys = co2di(extsmp(di2co(sys,1)),1);
%    end
% end
