%FITMAGFRD   Fit frequency response magnitude data with stable,
% minimum-phase state-space model using a version of log-Chebychev
% magnitude design.
%   B = FITMAGFRD(A,N) is a stable, minimum-phase SS object, with
%   state-dimension N, whose frequency response magnitude closely
%   matches the magnitude data in A.  A is a 1-by-1 FRD object,
%   and N is a nonnegative integer.
%
%   B = FITMAGFRD(A,N,RD) forces the relative degree of B to be RD.  RD
%   should be a nonnegative integer.  The default value for RD is 0.
%   The default for RD can also be specified by setting RD = [].
%
%   B = FITMAGFRD(A,N,RD,WT) uses the magnitude of WT to weight the
%   optimization fit criteria.  WT may be a DOUBLE, SS or FRD.  If
%   WT is a scalar, then it is used to weight all entries of the
%   error criteria (A-B).  If WT is a vector, it should be the same
%   size as A, and each individual entry of WT acts as a weighting
%   function on the corresponding entry of (A-B).  The default value
%   for WT is 1, and can also be specified by setting WT = [].
%
%   B = FITMAGFRD(A,N,RD,WT,C) enforces additional magnitude constraints
%   on B, relative to A as:
%      if C(w) == -1, then enforce abs(B(w)) <= abs(A(w))
%      if C(w) ==  1, then enforce abs(B(w)) >= abs(A(w))
%      if C(w) ==  0, then no additional constraint
%   C should be a scalar DOUBLE, or FRD, with C.Frequency equal to A.Frequency.
%   
%   FITMAGFRD uses a version of log-Chebychev magnitude design, solving 
%      min f     subject to (at every frequency point in A):  
%              |d|^2 /(1+ f/WT) < |n|^2/A^2 < |d|^2*(1 + f/WT)
%                   additional constraints imposed with C
%   where B = n/d.  n and d have orders (N-RD) and N, respectively.
%   The problem is solved using bisection on f and linear programming 
%   for fixed f.
%
%   Finally, an alternate approximate method, which cannot enforce the
%   constraints defined by C is B = FITFRD(GENPHASE(A),N,RD,WT).
%
%   See also: FITFRD, GENPHASE.

% Copyright 2003-2005 The MathWorks, Inc.

%    OPTS       - (optional, default=[*** 0.01]) 
%         1 x 2 vector of options = [tmax ttol]. As a default,
%         tmax is a crude upper bound on problem.
%
% Outputs:
%    SYS      - stable, min-phase system which fits magnitude data
%    TMIN     - minimum achieved value of t 
% Coding:  8/29/2001   PJS      Based on magfit.m 
%         10/11/2001   PJS      Map jw-axis to unit disk for better
%                               numerical conditioning
%         10/17/2001   PJS      Improved cutting plane constraints
%         10/24/2001   PJS      Added code for strictly proper fitting
%         08/02/2003   PJS      Final improvements

function [sys] = fitmagfrd(data,ord,reldeg,weight,constraint,opts)
%XXXMAJOR error messages are not clear regarding that A and WT and
% CONSRAINT should all be 1-by-1.
%XXXMINOR lots of the code has NTF sprinkled throughout, since
% it comes from FITFRD.  Here, Ntf must be 1, so some clarity
% could be achieved through simplification.
% XXXMINOR we should inherit all of the dynamic system properties from FRDATA
if nargin < 2
   error(['Not enough input arguments.']);
elseif nargin==2
   reldeg = [];
   weight = [];
   constraint = [];
   opts = [];
elseif nargin == 3
   weight = [];
   constraint = [];
   opts = [];
elseif nargin == 4
   constraint = [];
   opts = [];
elseif nargin == 5
   opts = [];
end

szf = size(data);
if ~isa(data,'frd') || length(szf)>2 || max(szf)>1
   error('A should be a 1x1 FRD with no array dimensions.');
end
data = abs(data);  % fitmagfrd fits the mag data
Ntf = 1;

% Data
[ResponseData,Frequency,Ts] = frdata(data);
nfreq = length(Frequency);
Units = get(data,'Units');
%Ts = get(data,'Ts');
%nfreq = length(data.Frequency);
if isequal(Units,'rad/s')
   romega = Frequency(:);
elseif  isequal(Units,'Hz')
   romega = (2*pi)*Frequency(:);
end
if Ts==0
   if romega(1)==0 & romega(2)>0
      w1 = romega(2);
   elseif romega(1)>0
      w1 = romega(1);
   else
      error('Need a nonzero 1st or 2nd frequency');
   end
   if isinf(romega(end)) && ~isinf(romega(end-1))
      w2 = romega(end-1);
   elseif ~isinf(romega(end))
      w2 = romega(end);
   else
      error('Need a non-infinite last (or second to last) frequency');
   end
   [alpha,wm,domeorig,L,P,ejtheta] = hp2dmax(w1,w2,romega);
else
   thetavec = romega*abs(Ts);
   keep = find(thetavec>=0 & thetavec<=pi);
   if length(keep)~=nfreq
      error('Frequency data (in rad/s) should be in [0 pi/Ts]');
   else 
      if thetavec(1)==0 & thetavec(2)>0
         t1 = thetavec(2);
      elseif thetavec(1)>0
         t1 = thetavec(1);
      else
         error('Need a nonzero 1st or 2nd frequency');
      end
      if thetavec(end)==pi & thetavec(end-1)<pi
         t2 = thetavec(end-1);
      elseif thetavec(end)<pi
         t2 = thetavec(end);
      else
         error('Need a non-pi last (or second to last) frequency');
      end
      [b,tm,domeorig,L,P,ejpsi] = d2dmax(t1,t2,thetavec);
   end
end

szw = size(weight);
if isempty(weight)
   weight = ones(nfreq,Ntf);
elseif isa(weight,'frd')
   % Data
   [wResponseData,wFrequency,wTs] = frdata(weight);
   wUnits = get(weight,'Units');
   if isequal(wUnits,'rad/s') 
      wromega = wFrequency(:);
   elseif  isequal(wUnits,'Hz')
      wromega = (2*pi)*wFrequency(:);
   end
   
	if ~isequal(wromega,romega)
	   error('Weight and FRDATA have incompatible frequency data')
   elseif wTs~=-1 && wTs~= Ts
	   error('Weight and FRDATA have incompatible sample times')
	elseif szw(1)~=1 & szw(2)~=1
	   error('Weight must be Column or Row');
	elseif szw(1)==1 & szw(2)==Ntf
	   weight = abs(permute(wResponseData,[3 2 1]));
	elseif szw(1)==Ntf & szw(2)==1
	   weight = abs(permute(wResponseData,[3 1 2]));
	elseif (szw(1)==1 & szw(2)==1)
	   weight = repmat(abs(wResponseData(:)),[1 Ntf]);
	else
	   error('Incompatible Dimensions between WT and FRDATA ')
	end
elseif isa(weight,'double')
   if szw(1)==1 & szw(2)==1
      weight = ones(nfreq,Ntf);
   elseif szw(1)~=1 & szw(2)~=1
      error('Weight must be Column or Row');
   elseif max(szw) == Ntf
      weight = weight(:)';
      weight = repmat(abs(weight),nfreq,1);
   else
      error('Incompatible Dimensions between WT and FRDATA ')
   end
elseif isa(weight,'ss') || isa(weight,'tf') || isa(weight,'zpk')
   % do we need to check Ts? 
   % Data
   [wResponseData,wFrequency,wTs] = frdata(weight);
   wUnits = get(weight,'Units');
	if wTs~=-1 && wTs~= Ts
	   error('Weight and FRDATA have different sample times')
	elseif szw(1)==1 & szw(2)==1
	   weight = frd(weight,romega);   % Use ROMEGA in doing the FRD calc 
      weight = repmat(abs(wResponseData(:)),[1 Ntf]);
	elseif szw(1)~=1 & szw(2)~=1
	   error('Weight must be Column or Row');
	elseif szw(1)==1 & szw(2)==Ntf
      weight = frd(weight,romega);  % Use ROMEGA in doing the FRD calc 
      weight = abs(permute(wResponseData,[3 2 1]));
   elseif szw(1)==Ntf & szw(2)==1
      weight = frd(weight,romega);  % Use ROMEGA in doing the FRD calc 
      weight = abs(permute(wResponseData,[3 1 2]));
	else
	   error('Incompatible Dimensions between WT and FRDATA ')
	end
else
	error('WT is not valid');
end
% at this point, weight is NFREQ x NTF double

constrflg = true;
szc = size(constraint);
if isempty(constraint)
   constrflg = false;
   constraint = zeros(nfreq,Ntf);
elseif isa(constraint,'frd')
   % Data
   [cResponseData,cFrequency,cTs] = frdata(constraint);
       %cUnits = get(weight,'Units');
       cUnits = get(constraint,'Units');
       if isequal(cUnits,'rad/s') 
          cromega = cFrequency(:);
       elseif  isequal(cUnits,'Hz')
          cromega = (2*pi)*cFrequency(:);
       end
       
    if ~isequal(cromega,romega)
        error('Constraint and FRDATA have incompatible frequency data')
    elseif cTs~=-1 && cTs~= Ts
        error('Constraint and FRDATA have incompatible sample times')
    elseif szc(1)~=1 & szc(2)~=1
        error('Constraint must be Column or Row');
    elseif szc(1)==1 & szc(2)==Ntf
        %constraint = abs(permute(cResponseData,[3 2 1]));
        constraint = permute(cResponseData,[3 2 1]);
    elseif szc(1)==Ntf & szc(2)==1
        %constraint = abs(permute(cResponseData,[3 1 2]));
        constraint = permute(cResponseData,[3 1 2]);
    elseif (szc(1)==1 & szc(2)==1)
        %constraint = repmat(abs(cResponseData(:)),[1 Ntf]);
        constraint = repmat(cResponseData(:),[1 Ntf]);
    else
        error('Incompatible Dimensions between Constraint and FRDATA ')
    end
elseif isa(constraint,'double')
   if szc(1)==1 & szc(2)==1
      constraint = constraint*ones(nfreq,Ntf);
   elseif szc(1)~=1 & szc(2)~=1
      error('Constraint must be Column or Row');       
   elseif max(szc) == Ntf
      constraint = constraint(:)';
      constraint = repmat(abs(constraint),nfreq,1);
   elseif max(szc) ~=nfreq
      error('Incompatible Dimensions between Constraint and FRDATA ')
   end
else
	error('Constraint is not valid');
end
constraint = sign(real(constraint));  
constraint = constraint(:);

% Error checking on opts
respdata = permute(ResponseData,[3 2 1]);
if ~isempty(opts) & (length(opts)>=2)
	if opts(1)>0
     tmax = opts(1);
	end
	if opts(2)>0
     ttol = opts(2);
	end
else
	ttol = 0.01;   
   % GJB: Pete change based on errors 2Nov04
	% tmax= (1+ttol)*max(weight)*( ( max(respdata)/min(respdata) )^2-1 );
   if constrflg
      tmax= (1+ttol)*max(weight)*( ( max(respdata)/min(respdata) )^2-1 );
   else
      tmax= (1+ttol)*max(weight)*( max(respdata)/min(respdata) - 1);
   end
end

% XXX Check reldeg>=0 && reldeg <= ord?
if isempty(reldeg)
   reldeg = zeros(1,Ntf);
elseif length(reldeg)==1
   reldeg = reldeg*ones(1,Ntf);
end

if length(reldeg)==Ntf & all(reldeg>=0) & all(reldeg<=ord)
   reldeg = reldeg(:)';
else
   error('The size of RD is incompatible with A.');
end
   
zz = exp(sqrt(-1)*domeorig);
if Ts==0
	tmp = repmat(zz+1,[1 Ntf]).^repmat(reldeg,[nfreq,1]);
else
	tmp = repmat(zz-1/b,[1 Ntf]).^repmat(reldeg,[nfreq,1]);
end
weight = weight.*tmp;
respdata = respdata./tmp;

[num,den] = fitmagfrdeng(respdata,domeorig,ord,ord-reldeg(:),...
                           weight,constraint,ttol,tmax);
sysC = ss(tf(num,den,Ts)); % SYSS will DIM-by-1, regardless
[Amat,Bmat,Cmat,Dmat] = ssdata(sysC);
pssmat = [Amat Bmat;Cmat Dmat];
sys = lft(kron(L,eye(ord)),pssmat,ord,ord);
Amat = sys(1:ord,1:ord);
Bmat = sys(1:ord,ord+1:end);
Cmat = sys(ord+1:end,1:ord);
Dmat = sys(ord+1:end,ord+1:end);
if Ts==0	
	for i = 1:Ntf
      Cmat(i,:) = Cmat(i,:)*(-2*alpha)^reldeg(i);  
      Dmat(i,:) = Dmat(i,:)*(-2*alpha)^reldeg(i);  
      [Cmat(i,:),Dmat(i,:)] = ...
         sisormzero(Amat,Bmat,Cmat(i,:),Dmat(i,:),alpha,reldeg(i));
   end
   sys = ss(Amat,Bmat,Cmat,Dmat);
else
   if b~=0
      gam = b^2/(b^2-1);
      for i = 1:Ntf
         Cmat(i,:) = Cmat(i,:)/(gam)^reldeg(i);  
         Dmat(i,:) = Dmat(i,:)/(gam)^reldeg(i);  
         [Cmat(i,:),Dmat(i,:)] = sisormzero(Amat,Bmat,Cmat(i,:),Dmat(i,:),-1/b,reldeg(i));
      end
   end
   sys = ss(Amat,Bmat,Cmat,Dmat,Ts);
end

