function resp = genphase(d)
%GENPHASE  Generate PHASE associated with stable, minimum phase
%   system from the frequency response magnitude.
%
%   COMPLEXRESP = GENPHASE(MAGRESP) uses the complex-cepstrum
%   (Oppenheim & Schafer, Digital Signal Processing, pg 501) to
%   generate a complex frequency response, COMPLEXRESP, whose
%   magnitude is equal to the real, positive response MAGRESP,
%   but whose phase corresponds to a stable, minimum phase
%   function.  Both MAGRESP and COMPLEXRESP are FRD.
%
%   See also: FITFRD, FITMAGFRD

% Copyright 2003-2005 The MathWorks, Inc.
if nargin == 0
   %disp(['usage: complexresp = genphase(magresp)']);
   error('GENPHASE requires 1 input');
   return
end

% Get data
[ResponseData,Frequency,Ts] = frdata(d);
Units = get(d,'Units');

szd = size(d);
if isequal([1 1],szd)
   magorig = ResponseData(:).';  % row vector
   nfreq = length(Frequency);
   if nfreq<3
      error('Need at least 3 points in Frequency Data')
   end
   Ts = get(d,'Ts');

   if isequal(Units,'rad/s')
      romega = Frequency(:).';
   elseif  isequal(Units,'Hz')
      romega = (2*pi)*Frequency(:).';
   end

   if Ts==0
      if romega(1)==0 & romega(2)>0
         w1 = romega(2);
      elseif romega(1)>0
         w1 = romega(1);
      else
         error('Need a nonzero 1st or 2nd frequency');
      end
      if isinf(romega(end)) && ~isinf(romega(end-1))
         w2 = romega(end-1);
      elseif ~isinf(romega(end))
         w2 = romega(end);
      else
         error('Need a non-infinite last (or second to last) frequency');
      end
      [alpha,wm,domeorig,L,P,ejtheta] = hp2dmax(w1,w2,romega);
   else
      thetavec = romega*abs(Ts);
      keep = find(thetavec>=0 & thetavec<=pi);
      if length(keep)~=nfreq
         error('Frequency data (in rad/s) should be in [0 pi/Ts]');
      else 
         if thetavec(1)==0 & thetavec(2)>0
            t1 = thetavec(2);
         elseif thetavec(1)>0
            t1 = thetavec(1);
         else
            error('Need a nonzero 1st or 2nd frequency');
         end
         %XXX change problems with round off errors...
         if (thetavec(end)==pi || thetavec(end)>pi) & thetavec(end-1)<pi
            t2 = thetavec(end-1);
         elseif thetavec(end)<pi
            t2 = thetavec(end);
         else
            error('Need a non-pi last (or second to last) frequency');
         end
%        disp('thetavec and pi')
%        [thetavec(end)-pi t2-pi thetavec(end) t2]
         %XXX Problem that t2 maybe > pi
         [b,tm,domeorig,L,P,ejpsi] = d2dmax(t1,t2,thetavec);
      end
   end   
	lowf = domeorig(1);
	highf = domeorig(end);
	hNdef = 2^12;
	Ndef = 2*hNdef;
	smgap = min([lowf pi-highf]);
	if smgap==0
      npts = Ndef;
      hnpts = hNdef;
	else
      nn = ceil( (log(2*pi/smgap))/log(2) );
      npts = 2*(2^nn);
      hnpts = 2^nn;
      if npts < Ndef
         npts = Ndef;
         hnpts = hNdef;
      end
      if npts > 1000000
         warning(['GENPHASE: large frequency range - calculation may be slow!'])
      end
	end
	[compd] = enggenphase(domeorig,magorig,hnpts,npts);
	resp = frd(reshape(compd,1,1,nfreq),Frequency,d);
else
   error('D must be 1x1 FRD');
end


function compd = enggenphase(domeorig,magorig,hnpts,npts)
% interpolate the frequency and magnitude data to
% HNPTS points linearly spaced around top half of disk
[lindome,linmag] = LOCALterpol(domeorig,magorig,hnpts);
% duplicate data around disk
dome = [lindome (2*pi)-fliplr(lindome)];          %all disk
mag = [linmag  fliplr(linmag)];                   %all disk
% complex cepstrum to get min-phase
ymag = log( mag .^2 );
ycc = ifft(ymag);                              % 2^N
nptso2 = npts/2;                               % 2^(N-1)
xcc = ycc(1:nptso2);                           % 2^(N-1)
xcc(1) = xcc(1)/2;                             % halve it at 0
xhat = exp(fft(xcc));                          % 2^(N-1)
domeg = dome(1:2:nptso2-1);                    % 2^(N-2)
xhat = xhat(1:nptso2/2);                       % 2^(N-2)
% interpolate back to original frequency data
[compd] = LOCALterpolb(domeg,xhat,domeorig);


function [freq,mag] = LOCALterpol(freqin,magin,npts)
%   This function interpolates the irregularly spaced
%   frequency/magnitude data FREQIN and MAGIN,
%   in the following way.  First, it assumes that
%   FREQIN is a row vector of nonnegative frequencies,
%   between 0 and pi:
%   foreach 1 <= i <= NPTS
%      if      pi*(i-1)/NPTS  <  FREQIN(1),
%        then    FREQ(i)  :=  MAG(1)
%      if      FREQIN(length(FREQIN))  <=  pi*(i-1)/NPTS
%        then    MAGOUT((i) =  MAGIN(length(MAGIN))
%      for other values of FREQIN(i), the program does
%      linear interpolation of LOG(MAG) between data points
lmagin = log(magin);
freq = (pi/npts)*(0:npts-1);
lmag = zeros(1,npts);
topval = length(freqin);

plow = find(freq<freqin(1));
lmag(plow) = lmagin(1);

phigh = find(freqin(topval)<=freq);
lmag(phigh) = lmagin(topval);

% The loop is much faster than the INTERP1, for linear interp
% lmag(mstart:mstop) = interp1(freqin,lmagin,freq(mstart:mstop));
for i=2:topval
    p = ceil(npts*freqin(i-1)/pi+1):(ceil(npts*freqin(i)/pi+1)-1);
    rat = freq(p) - freqin(i-1)*ones(1,length(p));
    rat = (1/(freqin(i)-freqin(i-1)))*rat;
    lmag(p)=(ones(1,length(p))-rat)*lmagin(i-1)+rat*lmagin(i);
end
mag = exp(lmag);


function [mag] = LOCALterpolb(fastfre,fastmag,slowfre)
%   Interpolates back after complex cepstrum calculation,
%   from the interpolated, regularly spaced data, to the original
%   frequency response data.
nptsslo = length(slowfre);
mag = zeros(1,nptsslo);
nptsfst = length(fastfre);
strt = 1;
stop = nptsslo;
lowstuff = find(slowfre<=fastfre(1));
if ~isempty(lowstuff)
    mag(lowstuff) = fastmag(1);
    strt = length(lowstuff) + 1;
end
highstuff = find(slowfre>=fastfre(nptsfst));
if ~isempty(highstuff)
    mag(highstuff) = fastmag(nptsfst);
    stop = nptsslo - length(highstuff);
end
mag(strt:stop) = interp1(fastfre,fastmag,slowfre(strt:stop));