function [info,sntisnto] = loopsens(P,C)
%LOOPSENS  Sensitivity functions of plant-controller feedback loop.
%
%LOOPTRANSFER = LOOPSENS(P,C) analyzes the multivariable feedback 
%loop consisting of C in negative feedback with P.  C should only be the 
%compensator in the feedback path, without reference channels, if  it is a 
%"2-dof" architecture.
%
%LOOPTRANSFER is a structure with the following fields:
%      Poles: closed-loop poles
%     Stable: 1 if nominal closed loop is stable, 0 otherwise. NaN for FRDs.
%         Si: input-to-plant sensitivity function
%         Ti: input-to-plant complementary sensitivity function
%         Li: input-to-plant loop transfer function
%         So: output-of-plant sensitivity function
%         To: output-of-plant complementary sensitivity function
%         Lo: output-of-plant loop transfer function
%        PSi: plant times input-to-plant sensitivity function
%        CSo: compensator times output-of-plant sensitivity function
% 
%See also LOOPMARGIN, ROBUSTSTAB, ROBUSTPERF, WCSENS, WCMARGIN.

% Copyright 2004 The MathWorks, Inc.

szp = size(P);
szc = size(C);
if szc(1)==szp(2) & szp(1)==szc(2)
   ny = szp(1);
   nu = szp(2);
   [sntisnto,li,lo] = LOCALallst(P,C);
   szl = size(li);
   info.Poles = nan([1 1 szl(3:end)]);
   info.Stable = nan([1 1 szl(3:end)]);
   %info.Si = parenget(sntisnto,{1:nu,1:nu});
   L.type = '()';
   L.subs = {1:nu,1:nu};
   info.Si = subsref(sntisnto,L);
   %info.Ti = -parenget(sntisnto,{nu+1:2*nu,1:nu});
   L.subs = {nu+1:2*nu,1:nu};
   info.Ti = subsref(sntisnto,L);
   info.Li = li;
   %info.So =  parenget(sntisnto,{2*nu+1:2*nu+ny  , nu+1:nu+ny});
   L.subs = {2*nu+1:2*nu+ny  , nu+1:nu+ny};
   info.So = subsref(sntisnto,L);
   %info.To = -parenget(sntisnto,{2*nu+ny+1:2*nu+2*ny, nu+1:nu+ny});
   L.subs = {2*nu+ny+1:2*nu+2*ny, nu+1:nu+ny};
   info.To = subsref(sntisnto,L);
   info.Lo = lo;
   %info.PSi = parenget(sntisnto,{2*nu+1:2*nu+ny,1:nu});
   L.subs = {2*nu+1:2*nu+ny,1:nu};
   info.PSi = subsref(sntisnto,L);
   %info.CSo = -parenget(sntisnto,{1:nu,nu+1:nu+ny});
   L.subs = {1:nu,nu+1:nu+ny};
   info.CSo = subsref(sntisnto,L);
else
   error('Invalid Dimension')
end


function [SNTISNTO,Li,Lo] = LOCALallst(P,C)

szp = size(P);
szc = size(C);
if szc(1)==szp(2) & szp(1)==szc(2)
   %e1 = icsignal(szc(1));
   %e2 = icsignal(szc(1));
   %d1 = icsignal(szc(1));
   %e3 = icsignal(szc(2));
   %e4 = icsignal(szc(2));
   %d2 = icsignal(szc(2));
   %m = ic;
   %m.eq{1} = e4 == P*e1;
   %m.eq{2} = e2 == C*(-e3);
   %m.eq{3} = e1 == e2 + d1;
   %m.eq{4} = e3 == e4 + d2;
   %m.Input = [d1;d2];
   %m.Output = [e1;e2;e3;e4];
   %SNTISNTO2 = m.System;
   % H = [Si -CSo;-Ti -CSo;PSi So;PSi -To];
   ny = szp(1);
   nu = szp(2);
   tmp1 = [eye(nu) zeros(nu,ny) eye(nu);zeros(nu,nu+ny) eye(nu);...
          zeros(ny,nu) eye(ny) zeros(ny,nu);zeros(ny,nu+ny+nu);...
          zeros(ny,nu) -eye(ny) zeros(ny,nu)];
   tmp2 = [zeros(2*nu,ny);eye(ny);eye(ny);-eye(ny)];
   tmp3 = [eye(nu) zeros(nu,ny) eye(nu)];
   % This works fine whether P is a SS or uss
   SNTISNTO = lft(tmp1+tmp2*P*tmp3,C);
   Li = C*P;
   Lo = P*C;
else
  error('Invalid dimensions');
end
