function sys = augw(varargin)
%AUGW  Augments plant with LTI weights for mixed-sensitivity control design.
%
% P = AUGW(G,W1,W2,W3) augments plant with LTI weights
%   for use in weighted mixed-sensitivity control system design.  For
%   use with HINFSYN or H2SYN.  All inputs must be proper.
% 
%   Input:
%         G = LTI plant of dimension [NY,NU]=size(G).
%         W1= LTI sensitivity weight
%         W2= LTI control signal weight
%         W3= LTI complementary sensitivity weight
% 
%   Output: augmented plant, with partitioning
% 
%         P = MKTITO(SYS,NY,NU)
% 
%         where
% 
%             SYS  = [W1     , -W1*G
%                     0      ,  W2
%                     0      ,  W3*G
%                     I      , -G       ];
%  
%   For dimensional compatibility, each of the three weights W1,W2,
%   and W3 must be either empty, a scalar (SISO) or have respective
%   input dimensions NY,NU and NY. 
% 
%   If one of the weightings is not needed, simply assign an empty matrix
%   []; e.g., P = AUGW(G,W1,[],W3) is SYS but without the
%   second row (without the row containing W2).
% 
%   See also:  H2SYN, HINFSYN, MIXSYN, MKTITO 

% Copyright 2003-2005 The MathWorks, Inc.

%  M. G. Safonov 12/01/02


if nargin==0,
    disp('Usage P=AUGW(G,W1,W2,W3)')
elseif nargin==1,
    error('Too few input arguments')
elseif nargin>4,
    error('Too many input arguments')
end

inarg={[],[],[],[]};
inarg(1:nargin)=varargin(1:nargin);

% make sure input is proper or, if double, convert to ss
for i=1:4,
    if isa(inarg{i},'double')
        inarg{i}=ss(inarg{i});
    elseif ~isa(inarg{i},'lti')
        error(['Input argument number ', num2str(i), ' is an ', upper(class(inarg{i})) ' object; it should be an LTI.'])            
    end
    if ~isproper(inarg{i})
        error('All inputs G,W1,W2,W3 must be proper')
    end
    inarg{i}=ss(inarg{i}); % convert a input to ss format
end

g=inarg{1}; 
[nyg,nug]=size(g);

nny=[nyg,nug,nyg,nyg];  % lengths of outputs [y1a y1b y1c y2]
py2=cumsum(nny);  %   end index pointers for [y1a y1b y1c y2]
py1=py2-nny+1;    % start index pointers for [y1a y1b y1c y2]

for i=2:4,
    ni=nny(i-1);
    if issiso(inarg{i}), % Make SISO weights MIMO, if needed
        temp=inarg{i};
        for j=1:ni-1,
            inarg{i}=append(inarg{i},temp);
        end
    elseif isempty(inarg{i}),
        % Fix sizes of empty weights, if needed
        inarg{i}=ss(zeros(0,ni));
    elseif ~isequal(size(inarg{i},2),ni),
        error(['MIMO Weight W', num2str(i-1), ' must have', num2str(ni), 'inputs.']) 
    end
end

% 
%             SYS  = [W1     , -W1*G
%                     0      ,  W2
%                     0      ,  W3*G
%                     I      , -G       ];

ey=eye(nyg);
eu=eye(nug);

% form augmented plant  sys=[sysdot1,sysdot2]
	% sysdot1=[I; 0; 0; I];
	sysdot1= [ey;  zeros(sum(nny(2:3)),nyg);  ey ];
         
	% sysdot2 = [-I;0;I;-I]*G + [0; I; 0; 0]
	sysdot2= [-ey; zeros(nug,nyg); ey; -ey ]*g;
   sysdot2 = subsasgn(sysdot2,substruct('()',{py1(2):py2(2),':'}),eye(nug)); % sysdot2(py1(2):py2(2),:)=eye(nug);
	sys=[sysdot1,sysdot2];


% append weights: sys=append(W1,W2,W3,I)*sys
w=append(inarg{2:4});
if isempty(w),
    error('At least one of the weights W1, W1, W3, must be non-empty')
end
sys=append(w,ey)*sys;


% TITO partitioning
sys=mktito(sys,nyg,nug);

%%%%%%%% END LTI/AUGW.M RYC/MGS 12/01/02 %%%%%%%%%%%
