function [Gred,info] = bstmr(G,varargin)

%BSTMR Balanced stochastic truncation (BST) model reduction for continuous/discrete & 
% stable/unstable plant.
%
% [GRED,REDINFO] = BSTMR(G,ORDER,KEYWORD,VALUE) performs a relative error 
% Schur model reduction on LTI object G. The infinity-norm of the relative error 
% is bounded as(Zhou, 1995):
%                 -1                          n
%            |GRED (s)(GRED(s)-G1(s))|    <= prod (1+2*si*(sqrt(1+si^2)+si))-1
%                                     inf    k+1
% where si denotes the i-th Hankel singular value of the all-pass "phase 
% matrix" of G(s). The algorithm is based on the Balanced Stochastic Truncation 
% (BST) theory with Relative Error Method (REM).
% For unstable G the algorithm works by first splitting G into a sum of stable 
% and antistable part, reduces the stable part, then add the unstable part back 
% for the final output G. The unstable part is kept as default.
%
% Inputs:
%    G  - LTI system to be reduced
%
% Optional inputs:
%    ORDER - an integer array with desired order of reduced model. 
%		 A batch run of [m:n] can be specified for a pack of reduced 
% 		 order models to be generated. The unstable part is kept as
% 		 default.
%
%   KEY       |VALUE        | MEANING
%  -----------------------------------------------------------------------
%  'MaxError' |real no. or  | Reduce to achieve H-infinity error
%             |vector       | If present, 'MaxError' overrides ORDER input.
%  ------------------------------------------------------------------------
%  'Display'  |'off' or 'on'| Display HANKELSV plots (default 'off')
%
%  Outputs:
%    Gred    - LTI reduced order system
%    REDINFO - a struct of REDINFO.ErrorBound, REDINFO.StabSV,
%    REDINFO.Unstab.SV
%
% See also	REDUCE, BALANCMR, SCHURMR, NCFMR, HANKELMR.

% R. Y. Chiang & M. G. Safonov 8/2002
% Copyright 1988-2005 The MathWorks, Inc.
%       $Revision: 1.1.6.7 $

if nargin == 1
   Type = 3;
end

if hasInternalDelay(G)
    error('Non-rational system is not supported.')
end

try 
    G = ss(G);
catch
    error('First input argument should be a SS object !');
end

G = balreal(G);

if ndims(G) > 2
    error('Model reduction routines only deal with single LTI model, no LTI arrays !')
end

% Get the correct algorithm:

D = get(G,'d');
[md,nd]=size(D);
if md > nd
    algo = 'bstschml';
else
    algo = 'bstschmr';
end

% Handle discrete case:

paaz = 2^-4*sqrt(eps);
Ts0 = getTs(G);
Ts = abs(Ts0); % take care of the inherited case (Ts = -1)

if Ts,
    a = get(G,'a');
    nx = size(a,1);
    if (max(abs(eig(a)+1))>paaz)  % if no poles near z=-1
        paaz = 0;
    end
    G = bilin(G,-1,'S_Tust',[Ts, 1-paaz]);
end

Gorig = G;
[a,b,c,d] = ssdata(G);

% Find no. of stable poles + jw-axis poles
lamda = eig(a);
ra = length(lamda);
mjws = length(find(real(lamda) < sqrt(eps)));
no_unstable = ra-mjws;

% Find no. of jw-axis poles
no_jw_pole = 0;
Gjw = ss([],[],[],zeros(size(d)));
Gu =  ss([],[],[],zeros(size(d)));
no_jw_pole = length((find(abs(real(pole(G))) < sqrt(eps))));

if no_jw_pole ~= 0
    gsjw = G;
    if no_unstable ~= 0
        [Gu,gsjw] = modreal(G, no_unstable ,'real');
        [aa,bb,cc,dd] = ssdata(Gu);
        Gu = ss(aa,bb,cc,zeros(size(dd)));
    end
    [Gjw,Gs] = modreal(gsjw, no_jw_pole, 'real');
    G = Gu+Gs;
    [aa,bb,cc,dd] = ssdata(Gjw);
    Gjw = ss(aa,bb,cc,zeros(size(dd)));
end

% Check stability of G
 
if no_unstable ~=0
    [G,P,m] = stabproj(G);
else
    P = ss([],[],[],zeros(size(get(G,'d'))));
    m = size(G,'o');
end

% Processing input arguments

vvaagg = varargin;

% Defaults:

Type = 3;   % reduced style
Typeoriginal = 1;
redval = {[],[]};
dispval = 'off';
displayflag = 0;

% Checking the "order" case (2nd argument is not char string)

if ~isempty(vvaagg)
   if isnumeric(vvaagg{1,1})
      redval = vvaagg{1,1}-no_unstable-no_jw_pole; % Reserve unstable part in reduced model
      Type = 1;
      vvaagg = varargin(2:end);
   end
end 

if ~isempty(vvaagg)
   if mod(length(vvaagg),2) 
     error('Arguments should be in pairs !')
   end
end

% Loop through varargin:

if ~isempty(vvaagg)
for i = 1:2:length(vvaagg)-1
    curarg0 = vvaagg{1,i};
    curarg = lower(curarg0(1:3));
    if ~isstr(curarg)
        error('Key value must be a string !')
    end
    
    switch curarg
    
    % Checking 'order'
    case 'ord'
         Type = 1;
         Typeoriginal = Type;
         redval = vvaagg{1,i+1}-no_unstable-no_jw_pole; % Reserve unstable part in reduced model
         
    % Checking 'MaxError'
    case 'max' 
        
        Type = 2;
        Typeoriginal = Type;
        redval = vvaagg{1,i+1};   % overwrites the "order" value
                
    % Checking 'Display'
    case 'dis' 
        dispval = vvaagg{1,i+1};
        if strcmp(dispval(1:2),'on')
           Type = 3;
           displayflag = 1;
        %   redval = [];
        end
    otherwise
       error(['''' curarg0 '''', ' is not a valid input (try ''MaxError'',''Display'' or ''Order'').'])   
    end
end % for loop
end % if vvaagg is not empty

% ----- Start execution:

if ~iscell(redval)
   redval = {redval};
end

if displayflag == 1 |Type == 3
   hankelsv(Gorig,'mult')
   if isempty(redval{1,1})
     ord = input(['Please enter the desired order: (>=' num2str(no_unstable+no_jw_pole) ') ']);  
     if isempty(ord)
           ord = length(ssdata(Gorig));
           disp('Original model returned.');
     end
     close(gcf)
     redval{1,1} = ord-no_unstable-no_jw_pole;
   end
   Type = Typeoriginal;
end

stabexe = length(redval{1,1});
Ebound = zeros(stabexe,1);
Gred = [];
for i = 1:stabexe
    if redval{1,1}(i) < 0
       no_ns = no_unstable+no_jw_pole;
       warning('Requested order %d is less than the number of unstable poles (%d). \nUsing order %d instead.',...
          redval{1,1}(i)+no_ns,no_ns,no_ns)
       redval{1,1}(i) = 0;
    end
    f = str2func(algo); % using function handle per Pascal suggestion
    [gr,totbnd,hhsv] = f(G,Type,redval{1,1}(i));
    gr = gr+P+Gjw; % add back the unstable & integral part of G 
    if Ts, % discrete case
       gr = bilin(gr,1,'S_Tust',[Ts,1-paaz]);
       gr = setTs(gr,Ts0);
    end  
    set(gr,'d',D)    % put back original D matrix
    Gred = subsasgn(Gred,substruct('()',{':' ':' i}),gr);  % Gred(:,:,i) = gr; 
    Ebound(i) = totbnd;
    info.ErrorBound = Ebound;
    [h_stab,h_unstab]=hankelsv(Gorig,'mult');
    info.StabSV = h_stab;
    info.UnstabSV = h_unstab;
end
  
% ---------- End of BSTMR.M % 10/26/02 
