%GAPMETRIC  Computes the gap and the Vinnicombe gap metric.
%
%    [GAP,NUGAP] = GAPMETRIC(P1,P2) calculates the gap metric and
%    the Vinnicombe variation of the gap metric, called the "nugap"
%    metric between systems P1 and P2. 
%
%    [GAP,NUGAP] = GAPMETRIC(P1,P2,TOL)  specifies a relative accuracy
%    TOL for calculating the gap metric and Vinnicombe gap metric. TOL
%    must be a positive scalar between 1e-5 and .1. (TOL=1e-3 by default).
%
%      GAP-TOL <   GAP(P1,P2) <=  GAP
%    NUGAP-TOL < NUGAP(P1,P2) <= NUGAP
%
%   See also CPMARGIN, LOOPMARGIN, MUSSV, NCFMR, NORM, GAPMETRIC, WCMARGIN.

%   Copyright 1991-2004 MUSYN Inc. and The MathWorks, Inc.
% $Revision: 1.1.6.3 $ $Date: 2004/11/18 23:41:13 $

function [gap,nugap] = gapmetric(sys1,sys2,ttol)

if nargin < 2,
  disp('usage: [gap,nugap] = gapmetric(sys1,sys2,ttol)');
  return;
end;
if nargin==2,
   ttol=0.001;
end;
szt = size(ttol);
if isa(ttol,'double') && szt(1)==szt(2) && szt(1)==1 && ttol>=1e-5 ...
      && ttol<=0.1
   ttol = ttol;
else
   warning('TTOL is being set to 1e-3'); 
   ttol = 0.001;
end
gap = [];
nugap = [];

if isa(sys1,'ss') || isa(sys1,'zpk') || isa(sys1,'tf') || isa(sys1,'double')
   if isa(sys2,'ss') || isa(sys2,'zpk') || isa(sys2,'tf') || isa(sys2,'double')
      sys1 = minreal(ss(sys1));
      sys2 = minreal(ss(sys2));
      szs1 = size(sys1);
      szs2 = size(sys2);
      if szs1(1)~=szs2(1) || szs1(2)~=szs2(2)
         disp('P1 and P2 must have the same # of inputs and outputs')
         return
      end
   else
      error('Invalid class of P2. P2 must be SS/TF/ZPK or double')
   end
else
    error('Invalid class of P1. P1 must be SS/TF/ZPK or double') 
end

% handle the extra dimension information
exd1 = szs1(3:end);
exd2 = szs2(3:end);
gap = zeros([1 1 exd1]);
nugap = zeros([1 1 exd1]);
if isempty(exd1) && isempty(exd2) %no extra dimensions
   gap = LOCALgap(sys1,sys2,ttol);
   nugap = LOCALnugap(sys1,sys2,ttol);
elseif isempty(exd1) % P1 has no EXD but P2 has EXD
   gap = zeros([1 1 exd2]);
   nugap = zeros([1 1 exd2]);
   for i=1:prod(exd2)
      gap(1,1,i) = LOCALgap(sys1,sys2(:,:,i),ttol);
      nugap(1,1,i) = LOCALnugap(sys1,sys2(:,:,i),ttol);
   end
elseif isempty(exd2) % P2 has no EXD but P1 has EXD
  gap = zeros([1 1 exd1]);
  nugap = zeros([1 1 exd1]);
  for i=1:prod(exd1)
      gap(1,1,i) = LOCALgap(sys1(:,:,i),sys2,ttol);
      nugap(1,1,i) = LOCALnugap(sys1(:,:,i),sys2,ttol);
  end
elseif all(exd1==exd2)   % P1 and P2 have same EXD or no EXD
   gap = zeros([1 1 exd1]);
   nugap = zeros([1 1 exd1]);
   for i=1:prod(exd1)
      gap(1,1,i) = LOCALgap(sys1(:,:,i),sys2(:,:,i),ttol);
      nugap(1,1,i) = LOCALnugap(sys1(:,:,i),sys2(:,:,i),ttol);
   end
else
    error('Inconsistent array dimensions between P1 and P2')
end

% gap = LOCALgap(sys1,sys2,ttol);
% nugap = LOCALnugap(sys1,sys2,ttol);

 
function gap = LOCALnugap(sys1,sys2,ttol)

szs1 = size(sys1);
p1 = szs1(1);
m1 = szs1(2);
n1 = length(get(sys1,'StateName'));
n2 = length(get(sys2,'StateName'));
szs2 = size(sys2);

if ~isstatic(sys1) && ~isstatic(sys2)
  [a1,b1,c1,d1] = ssdata(sys1);
  [a2,b2,c2,d2] = ssdata(sys2);
  w = inv(eye(m1)+d2'*d1);
  y = inv(eye(p1)+d2*d1');
  Agp = [-(a1-b1*w*d2'*c1)', c1'*y*c2; b2*w'*b1', a2-b2*d1'*y*c2];
  evals = eig(Agp);
  poseig = sum(real(evals)>0);
  if poseig~=n1,
     gap=1;
  else
     sysnlcf1 = xsncfbal(sys1);
     sysnrcf2 = xsncfbal(sys2.');
     sysnrcf2 = sysnrcf2.';
     nugap = ...
        norm(sysnlcf1*[zeros(m1,p1) -eye(m1);eye(p1) zeros(p1,m1)]*sysnrcf2,inf,ttol);
     gap = nugap(1);
   end; % if poseig~=n1
 end; %if sys1 and sys2 are SS

if ~isstatic(sys1) && isstatic(sys2) % exchange sys1 and sys2
   sys3 = sys1; 
   sys1 = sys2; 
   sys2 = sys3;
   n3 = n1;
   n1 = n2;
   n2 = n3;
end; %if sys1 SS, sys2 static

if isstatic(sys1) && ~isstatic(sys2)
  d1 = get(sys1,'d');
  [a2,b2,c2,d2] = ssdata(sys2);
  w = inv(eye(m1)+d2'*d1);
  y = inv(eye(p1)+d2*d1');
  Agp = [ a2-b2*d1'*y*c2];
  evals = eig(Agp);
  poseig = sum(real(evals)>0);
  if poseig~=0,
     gap = 1;
   else
     [sysnlcf1,tmp] = qr([d1'; eye(p1)]);
     sysnlcf1 = sysnlcf1(:,1:p1)';
     sysnrcf2 = xsncfbal(sys2.');
     sysnrcf2 = sysnrcf2.';
     nugap = ...
        norm(sysnlcf1*[zeros(m1,p1) -eye(m1);eye(p1) zeros(p1,m1)]*sysnrcf2,inf,ttol);
     gap = nugap(1);
    end; % if poseig~=n1
 end; %if sys1 static, sys2 SS

if isstatic(sys1) && isstatic(sys2)
   d1 = get(sys1,'d'); 
   d2 = get(sys2,'d');
   [sysnlcf1,tmp] = qr([d1'; eye(p1)]);
   sysnlcf1 = sysnlcf1(:,1:p1)';
   [sysnrcf2,tmp] = qr([d2; eye(m1)]);
   sysnrcf2 = sysnrcf2(:,1:m1);
   gap = norm(sysnlcf1*[zeros(m1,p1) -eye(m1);eye(p1) zeros(p1,m1)]*sysnrcf2);
 end; %isys1 and sys2 are constant

%---------- LOCALGAP -----------------%
function delta = LOCALgap(sys1,sys2,ttol)

epr = 1e-8;
hanktol = 1e-8;
szs1 = size(sys1);
szs2 = size(sys2);
p = szs1(1); 
m = szs1(2); 
pm = p + m;

% Compute the normalized rcf's, i.e. the graph symbols for SYS1 and SYS2.
if ~isstatic(sys1)   % not a static gain
  [sysnlcf1,sig1,grph1] = xsncfbal(sys1,2,hanktol);
else                 % pure static gain
  dmat = get(sys1,'d');
  grph1 = [ dmat; eye(m) ]*inv( sqrtm(eye(m)+dmat'*dmat));
end
if ~isstatic(sys2)   % not a static gain
  [sysnlcf2,sig2,grph2] = xsncfbal(sys2,2,hanktol);
else                 % pure static gain
  dmat = get(sys2,'d');
  grph2 = [ dmat; eye(m) ]*inv( sqrtm(eye(m)+dmat'*dmat));
end

% Form the pieces of a Hamiltonian matrix to be used in the tests below.
P = [grph2 grph1];
P = ss(P);
[aP,bP,cP,dP] = ssdata(P);
[sizeaPc,sizeaPr] = size(aP);
A = [aP zeros(sizeaPc); -cP'*cP -aP'];
B = [bP;-cP'*dP]; Br=[dP'*cP bP'];
DD = dP'*dP;
eye12 = [zeros(m) zeros(m); zeros(m) eye(m)];
eye21 = [eye(m) zeros(m); zeros(m) zeros(m)];

% Check if, after the reduction in xsncfbal, both systems are constant.
 if isstatic(P)
   delta = sqrt(1-(min(svd(DD(1:m,m+1:2*m))))^2);
   return
 end

% Compare the values of the two directed gaps to 1-ttol.
d = 1-ttol;
dsquare = d*d;
%
H = A-B*inv(DD-dsquare*eye12)*Br;
[x1,x2,fail1] = ric_schr(H,epr);
if  min(real(eig(x2/x1)))<-epr
  fail1 = fail1+1;
end
%
H = A-B*inv(DD-dsquare*eye21)*Br;
[x1,x2,fail2] = ric_schr(H,epr);
if min(real(eig(x2/x1)))<-epr
  fail2 = fail2+1;
end
%
if ( fail1 + fail2 ~= 0),
  delta = 1;
  return
end
dmax = 1-ttol;

% Compute a lower bound for the gap.
dmin = sqrt(1-(min(svd(DD(1:m,m+1:2*m))))^2);

% Use bisection.
r = dmax-dmin;
n = ceil(log(r/ttol)/log(2));
for k=1:n,
  d = dmin;
  dmax = d+r;
  r = r/2;
  d = d+r;
  H = A-B*inv(DD-d*d*eye12)*Br;
  [x1,x2,fail] = ric_schr(H,epr);
  if fail~=0
    dmin = d;
  elseif min(real(eig(x2/x1)))<-epr
    dmin = d;
  end
end
delta = dmin+r;
