function [k,varargout]=hinfsyn(varargin)
% LTI/HINFSYN
% [K,CL,GAM,INFO] = HINFSYN(P,NMEAS,NCON) or 
% [K,CL,GAM,INFO] = HINFSYN(P,NMEAS,NCON,KEY1,VALUE1,KEY2,VALUE2,...)
%    computes H-infinity controller K for partitioned plant P via the
%    gamma-iteration, computing the minimal cost GAM in [GMIN,GMAX] for
%    which the closed-loop system CL= LFT(P,K) satisfies 
%                 HINFNORM(CL) < GAM.
%    NMEAS is the number of measurement outputs from the plant and NCON is
%    the number of control inputs to the plant; these may be omitted if  
%    P=MKTITO(P,NMEAS,NCON) or P=AUGW(SYS,W1,W2,W3). 
% 
%       KEY     |VALUE   | MEANING
%     ----------------------------------------------------------------
%      'GMAX'   | real   | initial upper bound on GAM (Inf default)
%      'GMIN'   | real   | initial lower bound on GAM (0 default)
%      'TOLGAM' | real   | relative error tolerance for GAM (.01 default)
%      'METHOD' |        | H-infinity solution method: 
%   	        |'ric'   | - (default) standard 2-Riccati solution, DGKF1989
% 	            |'lmi'   | - LMI solution, Packard 1992, Gahinet 1994
%               |'maxe'  | - maximum entropy, HINFSYNE, Glover-Doyle 1988
%      'S0'     | real   |  (default=Inf) frequency S0 at which entropy is
%               |        |      evaluated, only applies to METHOD 'maxe'   	
%      'DISPLAY'|'on/off'| display synthesis information to screen,
%               |        |  (default = 'off')   
%        ----------------------------------------------------------------
%      outputs:
%        K     -  H-infinity controller
%        CL    -  lft(P,K) (closed-loop system)
%        GAM   -  H-infinity cost
%        INFO  -  Structure array containing possible additional information  
%                 depending on 'METHOD':
%                 INFO.AS  - all solutions controller, LTI two-port LFT
%                 INFO.KFI  - full information gain matrix (constant
%                             feedback U2 = KFI*[X; U1] )
%                 INFO.KFC  - full control gain matrix (constant 
%                             output-injection; KFC is the dual of KFI)
%                 INFO.GAMFI - H-infinity cost for full information KFI
%                 INFO.GAMFC - H-infinity cost for full control KFC
% 
%     See also: H2SYN HINFNORM MKTITO

% Copyright 2003-2005 The MathWorks, Inc.


% M. G. Safonov 12/28/2002
nag1=nargin;
nag2=nargout;

% initialize
DocKeys={'GMAX ','GMIN ','TOLGAM ','METHOD ','S0 ','DISPLAY '};
UndocKeys={'EPR ','EPP ','SO ','LMIOPTIONS ','OPTIONS '};
% varargout=cell(1,nargout);
nout = max(nargout,1)-1;
varargout=cell(1,nout);
gmax=Inf;
gmin=1e-6;
tol=.01; %tolgam
method='ric';
s0=Inf;
k=ss;
cl=ss;
gam=[];
gopt=[];
info=struct;
quiet=1;  % no printing from hinfsyne
ricmethd = 2; % default is to use mutools ric_schr
%ricmethd = 0; % try pascal's ric_gcare
epr = 1e-10;  % default tol for when real(eig) of Hamiltonian is zero
epp = 1e-6;   % default tol for positive definiteness determination for Riccati solution
lmiopt=[0 0 0 1];
ka=[];
cl=[];
k=[];
kfi=[];
kfc=[];
gamfi=[];
gamfc=[];
paa =2^-8*sqrt(eps);
paaz=2^-4*sqrt(eps);

%  no input --> show usage
if ~nag1,  
    disp('Usage [K,CL,GAM,INFO] = hinfsyn(P,NMEAS,NCON,KEY1,VALUE1,KEY2,VALUE2,...)')
    return
end

% get plant
plant=varargin{1};
% Make sure both systems are state-space models
try
   plant=ss(plant);
catch
   error('HINFSYN cannot handle improper models.')
end

% save warning state as WSTATE; turn off 'MATLAB:nearlySingularMatrix'
wstate=warning('off','MATLAB:nearlySingularMatrix');
try % try hinfsyn
    Ts=get(plant,'Ts');
    if Ts,  % Discrete-time 
        [a,b,c,d]=ssdata(plant);
        nx=size(a,1);
        if (max(abs(eig(a)+1))>paaz), % if no poles near z=-1
            paaz=0;
        end
        % plant=bilinz2s(plant);
        % The transformation below doesn't seem correct for all sample times.
        % shouldn't [2, 1-paaz] be [Ts, 1-paaz]?
        plant = bilin(plant,-1,'S_Tust',[2,1-paaz]); 
        % plant = d2c(plant,'tustin'); % d2c doesn't work - causes hinfsyn to
                                       % bomb on discrete plants

        if nag2>3, % if more than 3 input arguments
            [k,cl,gam,info]=hinfsyn(plant,varargin{2:end});
            if isfield(info,'AS') & isa(info.AS,'lti')
                AS=info.AS;
                AS=bilin(AS,1,'S_Tust',[2,1-paaz]); set(AS,'Ts',Ts);
                % AS = c2d(AS,Ts,'tustin');  % c2d (unshifted tustin) doesn't work for plants
                                             % with pole or zeros near origin
                [ncon,nmeas]=size(k); 
                AS=mktito(AS,nmeas,ncon);    % add tito partition to AS
                info.AS=AS;
            end
            if isfield(info,'KFC')
                info.KFC='KFC not available for discrete-time plants'; % need to create discrete hinffi
            end
            if isfield(info,'KFI'),
                info.KFC='KFI not available for discrete-time plants'; % need to create discrete hinffi
            end
        else
            [k,cl,gam]=hinfsyn(plant,varargin{2:end});
        end
        % k=bilins2z(k);
        % cl=bilins2z(cl);
        if ~isempty(k)
           k  = bilin(k,1,'S_Tust',[2,1-paaz]);
           cl = bilin(cl,1,'S_Tust',[2,1-paaz]);
           set(k,'Ts',Ts);
           set(cl,'Ts',Ts);
           % k  = c2d(k,Ts,'tustin');  % c2d doesn't correctly handle the
                                       % "1-paaz" shift term
           % cl = c2d(cl,Ts,'tustin'); % c2d doesn't correctly handle the
                                       % "1-paaz" shift term
        else
           cl = [];
           gam = [];
        end
    else % Continous Time Case:
       % if NMEAS and NCONT not specified, then extract from plant using ISTITO
       % and insert in VARARGIN array
       [ny,nu]=size(plant);
        if isequal(nag1,1) | isstr(varargin{2}) % pop nmeas and ncon from tito
          [tito,U1,U2,Y1,Y2]=istito(plant);
          if ~tito, 
              error('You must specify NMEAS and NCON'), 
          end
          ncon=length(U2);
          nmeas=length(Y2);
          % push nmeas and ncon into varargin
          temp=[varargin(1), {nmeas}, {ncon}, varargin(2:end)];
          varargin=temp;
        else
          nmeas=varargin{2};
          ncon=varargin{3};
          [ny,nu]=size(plant);
          U1=[1:nu-ncon];
          U2=[nu-ncon+1:nu];
          Y1=[1:ny-nmeas];
          Y2=[ny-nmeas+1:ny];
        end
        ny1=length(Y1);
       nu1=length(U1);

       % if input 4 is not a string, try old non-LTI syntax but with lti and return
        if nag1>3 & ~isstr(varargin{4})
          [varargin{:}]=lti2mat(varargin{:});
          try  % Obsolete mutools hinfsyn command syntax
             [k,varargout{:}]=hinfsyn(varargin{:});
             [k,varargout{:}]=mat2lti(k,varargout{:});
             % warning('Obsolete mutools HINFSYN syntax, with LTI plant');
          catch
             error('usage [K,CL,GAM,INFO] = hinfsyn(P,NMEAS,NCON,KEY1,VALUE1,KEY2,VALUE2,...)')
          end
          return
        end

        % get (KEY,OPTION) pairs 
        key=varargin(4:2:end);
        opt=varargin(5:2:end);
        nopt=length(key);
        nkey=length(opt);
        if ~isequal(nkey,nopt),
           error('Number of keys and options must be equal')
        end

        % replace defaults values with option values
        for i=1:nkey
          keyi=keymatch(key{i},DocKeys,UndocKeys);
          switch keyi
             case 'GMAX '
                gmax=opt{i};
             case 'GMIN '
                gmin=opt{i};
             case 'TOLGAM '
                tol=opt{i};
             case 'METHOD '
                method=lower(opt{i});
             case {'S0 ','SO '}
                s0=opt{i};
             case 'DISPLAY '
                verbose=opt{i};
                if isstr(verbose),
                   if strcmpi(verbose(1:2),'on')
                      lmiopt(4) = 1;
                      quiet = -1; % -1 --> full verbosity for hinfsyne
                      if length(verbose)>2 && strcmpi(verbose(1:3),'onh')
                         quiet = 0; % 0 -->  verbosity with NO HEADER for hinfsyne
                      end
                   else
                      lmiopt(4) = 1;
                      quiet = 1;
                   end
    %               if isempty(verbose)
    %                  error([ '''' opt{i} '''is not a valid option for KEY ' key{i} ])
    %               end
    %            elseif ~isequal(quiet,1) & ~isequal(quiet,0)
    %               error(['Invalid option for key ' key{i} ])
                end 
    %            quiet = (~verbose);
    %            if ~quiet, 
    %                quiet=-1; % -1 --> full verbosity for hinfsyn3
    %            end
             case 'EPP '
                epp=opt{i};
             case 'EPR '
                epr=opt{i};
             case {'LMIOPTIONS ','OPTIONS '}
                lmiopt=opt{i};
          end
        end
       if gmin>gmax,
          error('Cannot use GMIN > GMAX')
       end

        % Now determine METHOD
        AllMeths={'RIC ','MAXE ','LMI '};
        imeth=strmatch(upper(method),AllMeths);
        switch length(imeth)
          case 0 
             display(['Method ''' methi ''' is invalid.'])
             error(['Use one of the following: ' horzcat(DocKeys{:})])
          case 1
             methi=upper(AllMeths{imeth});
          otherwise
             display(['Method ''' methi ''' is ambiguous.'])
             error(['Matches all of the following: ' horzcat(AllKeys{imeth})])
        end
        gmax=min(gmax,1/sqrt(eps));  % Avoid gam=Inf singularity
        gmax1=gmax;                  % initial estimate of gam for hinfsyn
       % gam=gmax;                  % initial estimate of gam

       % Improve initial GAM using H2SYN: 
       % with estimate gam=normhinf(cl+plant.d(Y1,U1)) where [k,cl]=h2syn(plant2);
       plant2=plant;
       dd=get(plant2,'D');
       dd11=dd(Y1,U1);
       dd(Y1,U1)=0*dd11; % set plant2 D11-matrix to zero
       set(plant2,'D',dd);  
       dd12=dd(Y1,U2);
       dd21=dd(Y2,U1);
       if gmin==gmax
          gmax1 = gmax;
       elseif length(Y1)>=length(U2) & length(Y2)<=length(U1) & min(min(svd(dd12)),min(svd(dd21)))>sqrt(eps), 
%           [k,cl]=h2syn(plant2,nmeas,ncon);
%           gam=norm(cl+ss(dd11),Inf);
%           if nag2<4 & gam<=gmin  % DONE 
%              methi='DONE ';
%           end
%           gmax1=min(gam,gmax);  % revised initial gmax for hinfsyn 
         try
             [k,cl,H2gam,H2info]=h2syn(plant2,nmeas,ncon);
             if H2info.ReportX2 >= sqrt(eps) || H2info.ReportY2 >= sqrt(eps)
                gmax1 = gmax;
                if nag2<4 & gam<=gmin  % DONE 
                   methi='DONE ';
                end
             else
                gam = norm(cl+ss(dd11),Inf);
                if nag2<4 & gam<=gmin  % DONE 
                   methi='DONE ';
                end
                gmax1 = min(gam,gmax);  % revised initial gmax for hinfsyn 
                gmax1 = 1.05*gmax1;
             end
         catch
            gmax1 = gmax;
         end
       end

       %%%% Now compute the controller, depending on METHOD
       switch methi
         case {'DONE '} % do nothing already have gam<=gmin, k & cl
         case {'RIC ','MAXE '}  % 'ric' and 'maxe'
            [a,b,c,d]=ssdata(plant);
            d12=d(Y1,U2);
            d21=d(Y2,U1);
            % If D12,D12 full rank, try HINFSYNE without regularizing perturbations
            if rank(d12)==ncon & rank(d21)==nmeas,
               [ka,cla,gam] = hinfsyne(plant,nmeas,ncon,gmin,gmax1,tol,-1,quiet,ricmethd,epr,epp);
            end % if ineeded, regularize plant to compute ka, gam
            if ~isempty(ka),  % HINFSYNE was successful 
               %XXX SHOULD THIS BE "P"?? or "PLANT"? GJB 17sept03
               augp=plant; 
            else  %  HINFSYNE failed
               % 
               % augment plant with NCON rows and NMEAS columns of zeros
               %             x     U1aug U1pert U2aug
               %   xdot    [ A   | B1    0     | B2 
               %            --------------------------
               %   Y1aug     C1  | D11   0     | D12
               %   Y1pert    0   | 0     0     | 0
               %            --------------------------
               %   Y2aug     0   | D21  0      | D22   ]           
               nu1=length(U1);
               ny1=length(Y1);
               U1aug=U1;
               U1pert=nu1+1:nu1+nmeas;
               U2aug=U2+nmeas;
               Y1aug=Y1;
               Y1pert=ny1+1:ny1+ncon;
               Y2aug=Y2+ncon;
               augplant=ss(zeros(ny+ncon,nu+nmeas));
               % augplant([Y1 Y2aug],[U1 U2aug])=plant;
               augplant = subsasgn(ss(zeros(ny+ncon,nu+nmeas)),...
                  substruct('()',{[Y1 Y2aug],[U1 U2aug]}),plant);
               [aa,bb,cc,dd] = ssdata(augplant);
                nx=size(aa,1);
               % now make AUGPLANT regular via perturbation iteratively 
               % decrease the perturbations until gam stops decreasing
               count=0;
               pert=1e-3; % initial guess for perturbations to D12 & D21
               % loop until ( isempty(ka)  | 4*(gam-gmax1-sqrt(eps))>tol*gam )
               while logical(1)
                  count=count+1;
                  % augment plant d-matrix
                  dd(Y1pert,U2aug)=pert*eye(ncon);   % D12aug = [D12; pert*I] 
                  dd(Y2aug,U1pert)=pert*eye(nmeas);  % D21aug = [D21, pert*I] 
                  % shift poles to make jw-axis poles slightly unstable
                  aapert=aa+paa*eye(nx); %aapert=aa; 
                  augp1=ss(aapert,bb,cc,dd);  
                  %         [ A   | B1   0    | B2 
                  %           -----------------------
                  %           C1  | D11  0    | D12
                  %           0   | 0    0    | eps*I
                  %           -----------------------
                  %           0   | D21 eps*I | D22   ]
                  % Don't display results from this regulization process
                  % warning('off')
                  [ka1,cla1pert,gmax1] = ...
                     hinfsyne(augp1,nmeas,ncon,gmin,gmax1,tol,-1,1,ricmethd,epr,epp);
                  % warning('on')
                  if ~isempty(ka1)  % HINFSYNE success
                     % if first GAM or if GAM decreased
                     if isempty(gam) | (4*(gam-gmax1-sqrt(eps)) > tol*gam) 
                        gam=gmax1;
                        ka=ka1;
                        clapert=cla1pert;
                        augp=augp1;
                        % unshift poles
                        [akap,bka,cka,dka]=ssdata(ka);
                        aka=akap-paa*eye(nx); % aka=akap; %
                        ka=ss(akap,bka,cka,dka);
                     else % reduce PERT and loop again
                        if pert<sqrt(eps),  % except if pert is nearly zero
                           break
                        else
                           pert=pert/8;
                        end
                     end
                  else % unsuccessful
                     break
                  end
               end  % end while logical(1)
               if exist('augp','var')
                  ka1 = hinfsyne(augp,nmeas,ncon,gmin,gam,tol,-1,quiet,ricmethd,epr,epp);
               end
            end % HINFSYNE failed

            if isempty(ka), 
               estrg = ['Ill-conditioned problem, HINFSYN cannot compute '];
               estrg = [estrg 'controller with method RIC'];
               %error(estrg)
               k = [];
               return
            end

            % if option 'maxe', compute max entropy controller
            if isequal(method(1),'m') &  s0<Inf, % 'maxe' method with finite s0>0
    %           cla=slft(p,ka,ncon,nmeas); %SHOULD THIS BE "PLANT"?
               cla = lft(plant,ka,ncon,nmeas);
               s0=abs(s0);
                [a,b,c,d]=ssdata(cla);
               if ~isempty(d)
                  szcla = size(cla);
                  nout = szcla(1);
                  nin = szcla(2);
                  nstates = length(get(cla,'StateName'));
                  Phi=( d(nout-nmeas+1:nout,nin-ncon+1:nin) + ...
                        (c(nout-nmeas+1:nout,:)/(eye(nstates)*s0-a) )*...
                      b(:,nin-ncon+1:nin))'*gam^2;
                  k = lft(ka,Phi);
               end
            else % 'ric' method central controller (also, maxe when s0==Inf)
               k  = subsref(ka,substruct('()',{1:ncon,1:nmeas}));  % ka(1:ncon,1:nmeas)
            end
            % compute cl
            if nag2>1 | s0<Inf,
               cl = lft(plant,k,ncon,nmeas);
            end

            % if nargin>3, compute INFO 
            if nag2>3,
               % if possible, compute kfi full-info and kfc full-control
               % [k,g,gfin,ax,hamx] = hinffi(p,ncon,gmin,gmax,tol,ricmethd,epr,epp)
               [npy,npu] = size(augp);
               try
                  pp = subsref(augp,substruct('()',{1:npy-nmeas,':'}));  % augp(1:end-nmeas,:)
                  [kfi,clfi,gamfi] = hinffi(lti2mat(pp),ncon,gmin,gam,tol,2,epr,epp);
               end
               augpt = augp.';
               try
                  pp = subsref(augpt,substruct('()',{1:npu-ncon,':'}));  % augpt(1:end-ncon,:)
                  [kfct,clfct,gamfc] = hinffi(lti2mat(pp),nmeas,gmin,gam,tol,2,epr,epp);
                  kfc = kfct';  % untranspose full-control result
               end
               ka=mktito(ka,nmeas,ncon);
               info.AS=ka;
               info.KFI=kfi; % full-state feedback
               info.KFC=kfc; % full-control (output injection) feedback
               info.GAMFC=gamfc;
               info.GAMFI=gamfi;
            end
         case {'LMI '}
            % [gopt,Kcen,x1,x2,y1,y2] = hinflmi(P,r,gmin,tol,lmiopt)
            % plant=lti2mat(plant);
            % options=[0 0 1e-3 1]; % default
            % tol=.01 % default
            [gopt,k,x1,x2,y1,y2] = hinflmi(lti2mat(plant),[nmeas,ncon],max([gmin,gopt]),tol,lmiopt);
            k=mat2lti(k);
            if nag2>1,
               cl=lft(plant,k);
            end
            if gopt<=gmax, 
                gam=max(gmin,gopt); 
            end
            if nag2>3,
               info.AS=['AS not available with HINFSYN method ''LMI''.      '
                        'Use option METHOD ''RIC'' to obtain all solutions. '];
               info.KFI  =['KFI not available with HINFSYN method ''LMI''.  '];
               info.KFC  =['KFC not available with HINFSYN method ''LMI''.  '];
               info.GAMFI=['GAMFC not available with HINFSYN method ''LMI''.'];
               info.GAMFC=['GAMFI not available with HINFSYN method ''LMI''.'];
            end
        end  % end switch methi
    end % end if discrete else continous
catch
    warning(wstate);  % restore prior warning state stored in variable WSTATE
    rethrow(lasterror)  % then do error  
end % try hinfsyn
warning(wstate);  % restore warning state to WSTATE
temp={cl,gam,info};
varargout=temp(1:nout);
% 
% ----- End of LTI/HINFSYN .M ---- 17-Sep-2003 %
