function varargout = loopmargin(varargin)
%LOOPMARGIN  Comprehensive stability margin analysis of LTI and
%            Simulink feedback loops.
%
% LTI Feedback Loops:
% 
% [CM,DM,MM] = LOOPMARGIN(L) analyzes the N-channel multivariable 
% feedback loop:
% 
%         u --->O---->[ L ]----+---> y
%             - |              |         
%               +<-------------+
% 
% The N-by-N loop transfer matrix L can be any LTI model.  Negative
% feedback is assumed in the feedback path (specify -L otherwise).
% 
% LOOPMARGIN computes three types of stability margins:
% 
% * The classical gain and phase margins CM for each individual feedback
%   channel (loop-at-a-time margins) with all other loops closed (see 
%   ALLMARGIN for details on the fields of CM).
% 
% * The disk margin DM for each individual channel with all other loops
%   closed.  See DMPLOT for details.  DM is a N-by-1 structure with fields:
%      GainMargin: Disk gain margin (absolute value)
%     PhaseMargin: Disk phase margin (degrees)
%       Frequency: Frequency (in rad/sec) at which the margin disk
%                  touches the Nyquist plot of the loopgain
% 
% * The multi-loop disk margin, MM, indicates how much simultaneous 
%   (across channels), independent gain and phase variations can be
%   tolerated in each feedback channel while still maintaining stability
%   of the closed-loop system (note that for single-loop control systems, 
%   this is same as DM).  MM is a scalar structure with fields:
%      GainMargin: Guaranteed bound on simultaneous, independent, gain 
%                  variations allowed in all feedback channels.
%     PhaseMargin: Guaranteed bound on simultaneous, independent, phase 
%                  variations allowed in all feedback channels (degrees).
%       Frequency: Associated with GainMargin/PhaseMargin fields (rad/sec).
% 
% Unless L is FRD, the frequency range and number of points used to
% calculate DM and MM margins are chosen automatically. 
% 
% Output arguments can be limited to only those requested using an
% optional character string argument.  [M1,M2] = LOOPMARGIN(L,'m,c')
% returns the multi-loop diskmargin ('m') in M1, and the classical
% margins ('c') in M2.  Use 'd' to specify the disk margin.  This
% optional 2nd argument may be any combination, in any order, of the
% 3 characters 'c', 'd' and 'm'.
% 
% L can be an LTI array, and the margin computation is simply repeated
% for all array dimensions.  If SIZE(L) equals [N N N1 N2 ...], then
% SIZE(CM) will equal [N N1 N2 ...], SIZE(DM) will equal [N N1 N2 ...],
% and SIZE(MM) will equal [1 N1 N2 ...].
% 
% [CMI,DMI,MMI,CMO,DMO,MMO,MMIO] = LOOPMARGIN(P,C) computes the stability
% margins of the multivariable feedback loop consisting of the controller
% C in negative feedback with the plant P.  
% 
%         u --->O--->[ C ]-->[ P ]---+---> y
%             - |                    |         
%               +<-------------------+
% 
% * The CMI, DMI and MMI structures give the loop-at-a-time gain and 
%   phase margins, disk margins. and independent channel margins at 
%   the plant inputs, respectively (L = C*P)
% * The CMO, DMO and MMO structures give the corresponding margins at
%   the plant outputs (L = P*C)
% * The structure MMIO gives the disk margins for simultaneous, independent
%   variations in all input and output channels of the feedback loop.
% 
% Output arguments can be limited to only those requested using a
% character string option argument.  [M1,M2,M3] = LOOPMARGIN(P,C,'mo,ci,mm')
% returns the multi-loop diskmargin at plant output ('mo') in M1,
% the classical margins at the plant input ('ci') in M2, and the disk
% margins for simultaneous, independent variations in all input and
% output channels ('mm') in M3.  This optional 3rd argument may be any
% combination, in any order, of the 7 character pairs 'ci', 'di',
% 'mi', 'co', 'do', 'mo' and 'mm'.
% 
% If either P or C are LTI arrays, then the dimensioning rules described
% above for L also apply, resulting in multidimensional struct arrays for
% the output arguments.
% 
% Example:
%   a = [-0.2 10;-10 -0.2];
%   b = eye(2);
%   c = [1 8;-10 1];
%   d = zeros(2,2);
%   sat = ss(a,b,c,d);
%   k = [1 -2;0 1];
%   [cmi,dmi,mmi,cmo,dmo,mmo,mmio] = loopmargin(sat,k)
%   [dmi,dmo,mmio] = loopmargin(sat,k,'di,do,mm')
%
% SIMULINK Models:
% 
% [CM,DM,MM] = LOOPMARGIN(MODEL,BLOCKS,PORTS) does a multi-loop 
% stabilily margin analysis using the Simulink  Control Design (SCD). 
% MODEL specifies the name of the Simulink diagram for analysis. The 
% margin analysis points are defined at the output ports (PORTS) of
% blocks (BLOCKS) within the model. BLOCKS is a cell array of full block
% path names and PORTS is a vector of the same dimension as BLOCKS.
%
% Three types of stability margins are computed: loop-at-a-time classical 
% gain and phase margins (CM), loop-at-a-time disk margins (DM) and a 
% multi-loop disk margin (MM).
% 
% [CM,DM,MM] = LOOPMARGIN(MODEL,BLOCKS,PORTS,OP) uses the operating point 
% object, OP to create linearized systems from the Simulink MODEL.  
% 
% [CM,DM,MM,INFO] = LOOPMARGIN(MODEL,BLOCKS,PORTS,OP) returns INFO in
% addition to the margins. INFO is a structure with fields OperatingPoint,
% LinearizationIO and SignalNames corresponding to the analysis. Additional
% input arguments for LOOPMARGIN are OPTIONS for linearization and
% SNAPSHOTTIMES for linearization times. See LINEARIZE for more
% information.
%
% Margin output arguments can be limited to only those requested using an
% optional character string argument.  INFO is always the last output. 
% For example,  [MM,CM,INFO] = LOOPMARGIN(MODEL,BLOCKS,PORTS,'m,c') returns 
% the multi-loop diskmargin ('m') in MM, the classical margins ('c') in CM, 
% and the INFO structure.
% 
% See also ALLMARGIN, DMPLOT, LMIO, LOOPSENS, OPERPOINT, ROBUSTSTAB,
%         WCMARGIN,  WCGAIN.

%   Copyright 2003-2004 The MathWorks, Inc.
%   $Revision: 1.1.6.12 $ $Date: 2008/01/15 18:55:40 $

P = varargin{1};
szp = size(P);
ni = nargin;
nout = nargout;
calccode = {'ci' 'di' 'mi' 'co' 'do' 'mo' 'mm'};
validcode = calccode;
varargout = cell(1,nout);

if ni==1
   if szp(1)~=szp(2)
      error('Loop Transfer Matrix must be square')
   end
   C = eye(szp(1));
   if nout>3
      error('If specifying the loop gain, L, 3 or less output arguments are allowed.');
   end
   calccode = calccode(1:nout);
elseif ni==2
   if isa(varargin{2},'char')
      if szp(1)~=szp(2)
         error('Loop Transfer Matrix must be square')
      end
      userin = lower(varargin{2}(find(varargin{2}~=','&varargin{2}~=' ')));
      C = eye(szp(1));
      if nout>3
         error('If specifying the loop gain, L, 3 or less output arguments are allowed.');
      end
      if length(userin)<nout
         error('Output argument/option specifier mismatch.');
      end
      calccode = calccode(1:nout);
      for i=1:nout
         calccode{i} = [userin(i) 'i'];
      end
   else
      if nout>7
         error('If specifying P and C, 7 or less output arguments are allowed.');
      end
      C = varargin{2};
      calccode = calccode(1:nout);
   end   
elseif ni==3
   C = varargin{2};
   userin = lower(varargin{3}(find(varargin{3}~=','&varargin{3}~=' ')));
   if length(varargin{3})<2*nout
      error('Output argument/option specifier mismatch, need 2 char for every output.');
   end
   calccode = calccode(1:nout);
   for i=1:nout
      calccode{i} = userin(2*(i-1)+1:2*i);
   end
else
   error('Invalid number of input arguments.')
end
szc = size(C);

if (isa(P,'lti') && hasdelay(P)) || (isa(C,'lti') && hasdelay(C))
   error('LOOPMARGIN does not handle systems with delays')
end

strdiff = setdiff(calccode,validcode);
if ~isempty(strdiff)
   error(['''' strdiff{1} '''' ' is an invalid output specifier']);
end
if length(unique(calccode))<length(calccode)
   error(['Output specifiers should not be repeated']);
end

if szc(1)==szp(2) && szp(1)==szc(2)
   ny = szp(1);
   nu = szp(2);
   [info,sntisnto] = loopsens(P,C);  % need to run this regardless of what we calc

   L.type = '()';
   L.subs = {1:2*nu,1:nu};
   mtmp = subsref(sntisnto,L);
   SpTi = [eye(nu) eye(nu)]*mtmp;
   L.type = '()';
   L.subs = {2*nu+1:2*nu+2*ny,nu+1:nu+ny};
   mtmp = subsref(sntisnto,L);
   SpTo = [eye(ny) eye(ny)]*mtmp;
   
   sz = size(sntisnto);
   exd = sz(3:end);
   pexd = prod(exd);
   opt = wcgopt;
   opt.ArrayDimPtWise = 1:length(exd);
   
   ciRef = strmatch('ci',calccode);
   diRef = strmatch('di',calccode);
   miRef = strmatch('mi',calccode);
   coRef = strmatch('co',calccode);
   doRef = strmatch('do',calccode);
   moRef = strmatch('mo',calccode);
   mmRef = strmatch('mm',calccode);
   if ~isempty(ciRef)
      varargout{ciRef} = LOCALClassicalInput(nu,sntisnto,exd,pexd);
   end
   if ~isempty(diRef)
      % XXX LOCALDiskInput will bomb if LTI array contains systems of varying
      % state dimensions.
      % Convert to FRD so that works for different state dim LTI arrays 
      if isa(SpTi,'ss') || isa(SpTi,'zpk') || isa(SpTi,'tf')
         %XXXMINOR genfresp doesn't work with LTI arrays
         [mag,ph,w,FocusInfo] = genfresp(subsref(SpTi,substruct('()',{':',':',1})),4,[]);
         if length(w)>15
            w = w(4:end-4);
         end
         SpTi = frd(SpTi,w);
      end
      varargout{diRef} = LOCALDiskInput(nu,SpTi,opt,info.Stable,exd,pexd);
   end
   if ~isempty(coRef)
      varargout{coRef} = LOCALClassicalOutput(ny,nu,sntisnto,exd,pexd);
   end
   if ~isempty(doRef)
      % XXX LOCALDiskOutput will bomb if LTI array contains systems of varying
      % state dimensions.
      % Convert to FRD so that works for different state dim LTI arrays 
      if isa(SpTo,'ss') || isa(SpTo,'zpk') || isa(SpTo,'tf')
         %XXXMINOR genfresp doesn't work with LTI arrays
         [mag,ph,w,FocusInfo] = genfresp(subsref(SpTo,substruct('()',{':',':',1})),4,[]);
         if length(w)>15
            w = w(4:end-4);
         end
         SpTo = frd(SpTo,w);
      end
      varargout{doRef} = LOCALDiskOutput(ny,SpTo,opt,info.Stable,exd,pexd);
   end
   
   if ~isempty(miRef) || ~isempty(moRef) || ~isempty(mmRef)
      stabindx = [];
      ustabindx = [];
      if isa(sntisnto,'ss') || isa(sntisnto,'zpk') || isa(sntisnto,'tf')
         %XXXMINOR genfresp doesn't work with LTI arrays
         [mag,ph,w,FocusInfo] = genfresp(subsref(sntisnto,substruct('()',{':',':',1})),4,[]);
         if length(w)>15
            w = w(4:end-4);
         end
         sntisntog = frd(sntisnto,w);
         sntisntog = reshape(ufrd(sntisntog),[pexd 1]); %reshape to index correctly, currently no FRD RESHAPE
         sntisntog = depromote(sntisntog);
   %-------------new code for checking stability ----------------------%
         ustabindx = [];
         %Need to determine if the closed-loop systems were stable
         ustabindx = find(info.Stable==0);
         stabindx = 1:pexd;
         stabindx(ustabindx) = [];
         if ~isempty(stabindx)
            sntisntog = subsref(sntisntog,substruct('()',{':',':',stabindx})); %sntisntog(:,:,stabindx);
         else
            sntisntog = [];
            stabindx = 1:pexd;
         end
   %------------- code for checking stability ----------------------%
      else
         sntisntog = sntisnto;
         stabindx = 1:pexd;
      end
      if ~isempty(sntisntog) % there are nominally stable interconnections
         if ~isempty(miRef)
            varargout{miRef} = LOCALMultiInput(sntisntog,ny,nu,stabindx,exd,pexd,ustabindx);
         end
         if ~isempty(moRef)
            varargout{moRef} = LOCALMultiOutput(sntisntog,ny,nu,stabindx,exd,pexd,ustabindx);
         end
         if ~isempty(mmRef)
            varargout{mmRef} = LOCALMultiMMIO(sntisntog,ny,nu,stabindx,exd,pexd,ustabindx);
         end
      else
         if ~isempty(miRef)
            varargout{miRef} = struct('GainMargin',[],'PhaseMargin',[],...
               'Frequency',[]);
         end
         if ~isempty(moRef)
            varargout{moRef} = struct('GainMargin',[],'PhaseMargin',[],...
               'Frequency',[]);
         end
         if ~isempty(mmRef)
            varargout{mmRef} = struct('GainMargin',[],'PhaseMargin',[],...
               'Frequency',[]);
         end
      end
   end
else
   error('Invalid P and C dimensions.')
end

function [GM,PM] = LOCALminmg(mg,Bnd);
% MAXGAIN output from WCGAIN for an FRD/LTI is a double
idx = find(mg<=Bnd);
PM = zeros(size(mg));
GM = zeros(size(mg));
if ~isempty(idx)
   GM(idx) = inf;
   PM(idx) = 90;
   gidx = 1:prod(size(mg));
   gidx(idx) = [];
   if ~isempty(gidx)
      GM(gidx) = (1+1./mg(gidx))./(1-1./mg(gidx));
      PM(gidx) = (180/pi)*2*atan(1./mg(gidx));
   end
else
   GM = (1+1./mg)./(1-1./mg);
   PM = (180/pi)*2*atan(1./mg);
end

function Si = LOCALClassicalInput(nu,sntisnto,exd,pexd)
allmi = cell(nu,1);
for i=1:nu
   SingleloopLii = lft(subsref(sntisnto,substruct('()',{nu+i,i})),[-1 -1;1 0]);
   allmi{i} = allmargin(SingleloopLii);
end
for i=1:nu
   Si(i,:) = reshape(allmi{i},[1 pexd]);
end
if nu==1 && ~isempty(exd)
%   Si = reshape(Si,[exd]);
   Si = reshape(Si,[1 exd]);
elseif ~isempty(exd)
   Si = reshape(Si,[nu exd]);
end

function So = LOCALClassicalOutput(ny,nu,sntisnto,exd,pexd)
allmo = cell(ny,1);
for i=1:ny
   SingleloopLoi = lft(subsref(sntisnto,substruct('()',{2*nu+ny+i,nu+i})),[-1 -1;1 0]);
   allmo{i} = allmargin(SingleloopLoi);
end
for i=1:ny
   So(i,:) = reshape(allmo{i},[1 pexd]);
end
if ny==1 && ~isempty(exd)
%   So = reshape(So,exd);
   So = reshape(So,[1 exd]);
elseif ~isempty(exd)
   So = reshape(So,[ny exd]);
end

function Diski = LOCALDiskInput(nu,SpTi,opt,StableInfo,exd,pexd)
blightI = cell(nu,1);
for i=1:nu
   nomgain = wcgain(subsref(SpTi,substruct('()',{i,i})),opt,[],[],StableInfo);
   sWi = nomgain.CriticalFrequency;
   [GM,PM] = LOCALminmg(nomgain.LowerBound,1.001);
   GM = [1./GM(:), GM(:)];
   PM = [-PM(:), PM(:)];
   Wi = sWi(:);
   %GM = cat(2,1./GM,GM);
   %GM = reshape(GM,[2 prod(exd)]).';
   %PM = cat(2,-PM,PM);
   %PM = reshape(PM,[2 prod(exd)]).';
   %Wi = reshape(sWi,[1 prod(exd)]).';
   if isempty(exd)
      GMCell = mat2cell(GM,ones(prod(exd),1),2);
      PMCell = mat2cell(PM,ones(prod(exd),1),2);
      WCell  = mat2cell(Wi,ones(prod(exd),1),1);
   else
      GMCell = reshape(mat2cell(GM,ones(prod(exd),1),2),exd);
      PMCell = reshape(mat2cell(PM,ones(prod(exd),1),2),exd);
      WCell  = reshape(mat2cell(Wi,ones(prod(exd),1),1),exd);
   end
   blightI{i} = struct('GainMargin',GMCell,'PhaseMargin',PMCell,...
                        'Frequency',WCell);
end
for i=1:nu
   Diski(i,:) = reshape(blightI{i},[1 pexd]);
end
if nu==1 && ~isempty(exd)
%   Diski = reshape(Diski,[exd]);
   Diski = reshape(Diski,[1 exd]);
elseif ~isempty(exd)
   Diski = reshape(Diski,[nu exd]);
end


function Disko = LOCALDiskOutput(ny,SpTo,opt,StableInfo,exd,pexd)
blightO = cell(ny,1);
% if isa(SpTo,'ss') || isa(SpTo,'zpk') || isa(SpTo,'tf')
%    %XXXMINOR genfresp doesn't work with LTI arrays
%    [mag,ph,w,FocusInfo] = genfresp(subsref(SpTo,substruct('()',{':',':',1})),4,[]);
%    if length(w)>15
%       w = w(4:end-4);
%    end
%    SpTo = frd(sntisnto,w);
% DO I HAVE TO RESHAPE IT HERE?????
%    sntisntog = reshape(ufrd(SpTo),[pexd 1]); %reshape to index correctly, currently no FRD RESHAPE
%    SpTo = depromote(SpTo);
% end
for i=1:ny
   nomgain = wcgain(subsref(SpTo,substruct('()',{i,i})),opt,[],[],StableInfo);
   sWo = nomgain.CriticalFrequency;
   GM = [];
   PM = [];
   [GM,PM] = LOCALminmg(nomgain.LowerBound,1.001);
   GM = cat(2,1./GM,GM);
   GM = reshape(GM,[2 prod(exd)]).';
   PM = cat(2,-PM,PM);
   PM = reshape(PM,[2 prod(exd)]).';
   Wo = reshape(sWo,[1 prod(exd)]).';
   if isempty(exd)
      GMCell = mat2cell(GM,ones(prod(exd),1),2);
      PMCell = mat2cell(PM,ones(prod(exd),1),2);
      WCell  = mat2cell(Wo,ones(prod(exd),1),1);

   else
      GMCell = reshape(mat2cell(GM,ones(prod(exd),1),2),exd);
      PMCell = reshape(mat2cell(PM,ones(prod(exd),1),2),exd);
      WCell  = reshape(mat2cell(Wo,ones(prod(exd),1),1),exd);
   end
   blightO{i} = struct('GainMargin',GMCell,'PhaseMargin',PMCell,...
                        'Frequency',WCell);
end
for i=1:ny
   Disko(i,:) = reshape(blightO{i},[1 pexd]);
end
if ny==1 && ~isempty(exd)
%   Disko = reshape(Disko,exd);
   Disko = reshape(Disko,[1 exd]);
elseif ~isempty(exd)
   Disko = reshape(Disko,[ny exd]);
end

function Mi = LOCALMultiInput(sntisntog,ny,nu,stabindx,exd,pexd,ustabindx)
% MIMO Input margins
SpTi = [eye(nu) eye(nu)]*subsref(sntisntog,substruct('()',...
   {1:2*nu,1:nu}));  % sntisntog(1:2*nu,1:nu);
blksti = ones(nu,2);
bndsi = mussv(SpTi,blksti,'sU'); %SpTi is an FRD
kmi   = zeros([length(stabindx) 1]);
freqi = zeros([length(stabindx) 1]);
for i=1:length(stabindx)
   [kmi(i),freqi(i)] = norm(subsref(bndsi,substruct('()',{1,1,i})),inf);
   kmi(i) = max([1.005 kmi(i)]);
end
GMi = zeros([pexd 1]);
PMi = zeros([pexd 1]);
Wi  = zeros([pexd 1]);
GMi(stabindx) = (1+1./kmi)./(1-1./kmi);
PMi(stabindx) = (180/pi)*2*atan(1./kmi);
GMi(ustabindx) = 1;
PMi(ustabindx) = 0;
PMi = PMi(:);
GMi = reshape(GMi,[1 1 exd]);
GMi = cat(2,1./GMi,GMi);
PMi = reshape(PMi,[1 1 exd]);
PMi = cat(2,-PMi,PMi);
Wi(stabindx) = freqi;
Wi  = reshape(Wi,[1 1 exd]);
GMCell = num2cell(GMi,[1 2]);
PMCell = num2cell(PMi,[1 2]);
WCell  = num2cell(Wi,[1 2]);
if ~isempty(exd)
%    GMCell = reshape(GMCell,exd);
%    PMCell = reshape(PMCell,exd);
%    WCell = reshape(WCell,exd);
   GMCell = reshape(GMCell,[1 exd]);
   PMCell = reshape(PMCell,[1 exd]);
   WCell = reshape(WCell,[1 exd]);
end
Mi = struct('GainMargin',GMCell,'PhaseMargin',PMCell,...
            'Frequency',WCell);

function Mo = LOCALMultiOutput(sntisntog,ny,nu,stabindx,exd,pexd,ustabindx)    
SpTo = [eye(ny) eye(ny)]*subsref(sntisntog,substruct('()',...
   {2*nu+1:2*nu+2*ny,nu+1:nu+ny})); % sntisntog(2*nu+1:2*nu+2*ny,nu+1:nu+ny);
blksto = ones(ny,2);
[bndso,muinfo] = mussv(SpTo,blksto,'sU');
kmo   = zeros([length(stabindx) 1]);
freqo = zeros([length(stabindx) 1]);
for i=1:length(stabindx)
   [kmo(i),freqo(i)] = norm(subsref(bndso,substruct('()',{1,1,i})),inf);
   kmo(i) = max([1.005 kmo(i)]);
end
GMo = zeros([pexd 1]);
PMo = zeros([pexd 1]);
Wo  = zeros([pexd 1]);
GMo(stabindx) = (1+1./kmo)./(1-1./kmo);
PMo(stabindx) = (180/pi)*2*atan(1./kmo);
GMo(ustabindx) = 1;
PMo(ustabindx) = 0;
PMo = PMo(:);
GMo = reshape(GMo,[1 1 exd]);
GMo = cat(2,1./GMo,GMo);
PMo = reshape(PMo,[1 1 exd]);
PMo = cat(2,-PMo,PMo);
Wo(stabindx) = freqo;
Wo  = reshape(Wo,[1 1 exd]);
GMCell = num2cell(GMo,[1 2]);
PMCell = num2cell(PMo,[1 2]);
WCell  = num2cell(Wo,[1 2]);
if ~isempty(exd)
%    GMCell = reshape(GMCell,exd);
%    PMCell = reshape(PMCell,exd);
%    WCell = reshape(WCell,exd);
   GMCell = reshape(GMCell,[1 exd]);
   PMCell = reshape(PMCell,[1 exd]);
   WCell = reshape(WCell,[1 exd]);
end
Mo = struct('GainMargin',GMCell,'PhaseMargin',PMCell,...
            'Frequency',WCell);

function Mio = LOCALMultiMMIO(sntisntog,ny,nu,stabindx,exd,pexd,ustabindx)
STio = [eye(nu) eye(nu) zeros(nu,2*ny);...
    zeros(ny,2*nu) eye(ny) eye(ny)]*sntisntog;
blkstio = ones(ny+nu,2);
bndsio = mussv(STio,blkstio,'sU');
kmio   = zeros([length(stabindx) 1]);
freqio = zeros([length(stabindx) 1]);
for i=1:length(stabindx)
   [kmio(i),freqio(i)] = norm(subsref(bndsio,substruct('()',{1,1,i})),inf);
   kmio(i) = max([1.005 kmio(i)]);
end
GMio = zeros([pexd 1]);
PMio = zeros([pexd 1]);
Wio  = zeros([pexd 1]);
GMio(stabindx) = (1+1./kmio)./(1-1./kmio);
PMio(stabindx) = (180/pi)*2*atan(1./kmio);
GMio(ustabindx) = 1;
PMio(ustabindx) = 0;
PMio = PMio(:);
GMio = reshape(GMio,[1 1 exd]);
GMio = cat(2,1./GMio,GMio);
PMio = reshape(PMio,[1 1 exd]);
PMio = cat(2,-PMio,PMio);
Wio(stabindx) = freqio;  % freqo --> freqio ???
Wio  = reshape(Wio,[1 1 exd]);
GMCell = num2cell(GMio,[1 2]);
PMCell = num2cell(PMio,[1 2]);
WCell  = num2cell(Wio,[1 2]);
if ~isempty(exd)
%    GMCell = reshape(GMCell,exd);
%    PMCell = reshape(PMCell,exd);
%    WCell = reshape(WCell,exd);
   GMCell = reshape(GMCell,[1 exd]);
   PMCell = reshape(PMCell,[1 exd]);
   WCell = reshape(WCell,[1 exd]);
end
Mio = struct('GainMargin',GMCell,'PhaseMargin',PMCell,...
            'Frequency',WCell);
