function [k,cl,gam,info]=loopsyn(G,Gd,circ,circ1)
% [K,CL,GAM,INFO]=LOOPSYN(G,Gd) 
% [K,CL,GAM,INFO]=LOOPSYN(G,Gd,{wmin, wmax})
% [K,CL,GAM,INFO]=LOOPSYN(G,Gd,[wmin, wmax]) computes a stabilizing 
%       H-infinity controller K for plant G to shape the sigma 
%       plot of the loop transfer function G*K to have desired 
%       loopshape Gd with accuracy GAM in the sense that if w0 is 
%       the 0db crossover frequency of sigma(Gd) then, roughly,
%               min(sigma(G*K)) > max(sigma(Gd))/GAM, for wmin<w<w0
%               max(sigma(G*K)) < min(sigma(Gd))*GAM ,for   w0<w<wmax.
%       Also returned is a MIMO stable min-phase W satisfying
%       sigma(Gd)=sigma(G*W).  
%       
%       Inputs:
%       -------
%        G   -  LTI open-loop plant to be controlled
%        Gd  -  LTI desired loop shape, may be either SISO or NxN
%                  where N is the output (row) dimension of the plant.
%                  Gd must be stable and min-phase.
%       Optional Input:
%       -------
%        {wmin, wmax} or [wmin,wmax] 
%                frequency range for loopshaping (default {0,Inf})
%                
%       Outputs:
%       -------
%        K    -  H-infinity loopshaping controller
%        CL   -  =G*K/(I+GK) closed-loop (complementary sensitivity)
%        GAM  -  loop-shaping accuracy (GAM is never less than 1,
%                  GAM=1 is perfect fit)  
%        INFO -  struct array with various information fields:
%                 INFO.W        -  stable min-phase loop-shaping pre-filter
%                 INFO.Gs=Gs;   -  shaped plant Gs=G*W
%                 INFO.Ks=Ks;   -  controller for shaped plant (K=W*Ks)
%                 INFO.range    -  cell array containing, frequency range
%                                  {Wmin,Wmax} over used for loop-shaping; 
%                                  if nargin=2, INFO.range will be 
%                                  automatically determined
% 
% LIMITATION:  G must be stabilizable, detectable, full-rank and 'fat'
%     (i.e., G must have at least as many inputs as outputs).
%       
% SUGGESTED USE:  
%           s= zpk('s'); 
%           K=loopsyn(G,w0/s); 
%      where the w0  is desired closed-loop bandwidth.  The variable 
%      w0 may be as a design knob.   To decrease GAM, try reducing w0.   
%      Values of GAM below about 3 are typically regarded as satisfactory, 
%      with smaller values corresponding to better fit and greater robustness.     
% 
% COMMENTS:  Gs=G*W is called the 'shaped plant' and Ks=minreal(W\K) 
%      simultaneously solves both of the following two H-infinity optimal 
%      control problems
%      1).   GAM = min  || [I; Ks]*inv(I+Gs*Ks)*[I, Gs] ||
%                  Ks                                     Hinf
% 
%      2).   GAM = min  || [I; Gs]*inv(I+Ks*Gs)*[I, Ks] ||
%                  Ks                                     Hinf
% and GAM is the minimal H-infinity COST for both of these problems.    
% 
% See also HINFSYN NCFSYN MIXSYN GAP NUGAP

% Copyright 1988-2005 The MathWorks, Inc.
% $Revision: 1.1.6.5 $ $Date: 2005/11/15 01:22:00 $
% M. G. Safonov 11/21/2001, rev. 12/18/2002.

nag1=nargin;
nag2=nargout;
if nag1==0 & nag2==0,
    disp('Usage [K,CL,GAM,INFO]=loopsyn(G,Gd)')
    return
elseif nag1<2,
    error('Too few input arguments')
end
if nag1<3;
    circ=[0,Inf];
elseif nag1==3 & iscell(circ) & isequal(size(circ),[1 2]) & isscalar(circ{1}) & isscalar(circ{2}),
    circ=[circ{1},circ{2}];
elseif nag1==4 & isscalar(circ),
    circ=[circ,circ1];
end
if ~isa(circ,'double') | ~isequal(size(circ),[1 2]),
    error('RANGE should be a 1x2 cell array containing two positive scalars {wmin,wmax}')
end 
% circ=sort(abs(circ));
if min(circ)<0,
    error('Inputs WMIN and WMAX must be nonnegative')
end
if isequal(min(circ),max(circ)),
    error('Input WMIN must be less than WMAX')
end

circsave=circ;

% initialize
L=ss(Gd);
G=ss(G);
Gorig=G;
[mg,ng]=size(G);
info=struct;
info.W=[];
info.Gs=[];
info.Ks=[];

paaz=2^-4*sqrt(eps);

Ts1=get(G,'Ts');  % Get sampling times
Ts2=get(L,'Ts');

% begin error checking
% Check for compatible dimensions
  if ~issiso(L) & size(L,1)~=size(G,1),
    error('G must have the same number outputs as Gd, or Gd must be SISO')
  end
% Check for compatible Ts
% if both are non-constant systems
  if size(get(G,'a')) & size(get(L,'a')) 
  % if (only one discrete) or ( both discrete  & both positive     &  both not equal    )
    if  xor(Ts1,Ts2)       |  ( all([Ts1,Ts2]) &  min([Ts1,Ts2])>0 & ~isequal(Ts1,Ts2)  ) 
        error('both LTI models must have the same sample time')
    end
  end
% end error checking
 

Ts=max([Ts1,Ts2]); % sampling-time to be returned with all LTI output
if Ts,  % Discrete-time
	if max(abs(pole(L))) >1,
        error('Desired loop-shape Gd is unstable')
    end

    DL=get(L,'d');
    if rank(DL)<min(mg),
        error('Input Gd must be bi-proper (Gd.D must be full rank)')
    end
	if max(abs(zero(L))) >1,
        error('Input Gd is not minimum-phase')
    end 
    T=abs(Ts);
    [a,b,c,d]=ssdata(G);
    nx=size(a,1);
    if (max(abs(eig(a)+1))>paaz), % if no poles near z=-1
        paaz=0;
    end
    % G=bilinz2s(G);
    % L=bilinz2s(L);    
    G=bilin(G,-1,'S_Tust',[T,1-paaz]);
    L=bilin(L,-1,'S_Tust',[T,1-paaz]);
    if nag1<3,
        [k,cl,gam,info]=loopsyn(G,L);
    else
        circ(2)=min(circ(2),pi/T);  % if wmax>pi/T, then set wmax=pi/T
        for i=1:2,
            if circ(i)==pi/T,
                circ(i)=Inf;
            elseif circ(i)>0,
                z=exp(j*circ(i)*T);               % z=exp(jwT)
                circ(i)=(2/T)*imag((z-1)/(z+1)); % tustin z2s jwT=(z-1)/(z+1) <=> w=(2/T)*imag((z-1)/(z+1))
            end
        end
        [k,cl,gam,info]=loopsyn(G,L,circ);
    end
    % k=bilins2z(k);
    k=bilin(k,1,'S_Tust',[T,1-paaz]); set(k,'Ts',Ts);
    % if nag2>1,
    %     % cl=bilins2z(cl);
    %      cl=bilin(cl,1,'S_Tust',[T,1-paaz]); set(cl,'Ts',Ts); 
    % end
    if nag2>3, % if more than 3 input arguments
        % info.W =bilins2z(info.W);
        % info.Gs=bilins2z(info.Gs);
        % info.Ks=bilins2z(info.Ks);
        W= bilin( info.W,1,'S_Tust',[T,1-paaz]); set( W,'Ts',Ts); info.W =W;
        Gs=bilin(info.Gs,1,'S_Tust',[T,1-paaz]); set(Gs,'Ts',Ts); info.Gs=Gs;
        Ks=bilin(info.Ks,1,'S_Tust',[T,1-paaz]); set(Ks,'Ts',Ts); info.Ks=Ks;
        circ=sort(abs(horzcat(info.range{:})));
        for i=1:2
            % circ(i)=T*angle(j*circ(i)+1); % tustin s2z
            if isinf(circ(i)),
                z=-1;
            else
                s=j*circ(i);            % s=jw
                z=(1+T*s/2)/(1-T*s/2);  % tustin s2z
            end
            circ(i)=angle(z)/T; % z=exp(jwT) <=> w=angle(z)/T 
        end
        % info.range=circ;
        info.range={circ(1),circ(2)};  % 1x2 cell array
    end
else % Continuous-time

	if max(real(pole(L))) >0,
        error('Desired loop-shape Gd is unstable')
	end
	if max(real(zero(L))) >0,
        error('Desired loop-shape Gd is not minimum-phase')
	end
	
	% Test dimensional compatibility
	i1=1:mg;
	i2=mg+1:ng;
	
	if mg>ng, 
       error('Ill-posed problem -- plant G cannot have more outputs than inputs'),
	end
    
	if nag1<3,  % look for singularity and attempt to correct automatically
        % detect and correct singularities  (i.e., jw-axis zeros or poles)
        % via bilin pole shifting 
        singular=0; % initially assume nonsingular 
        [a,b,c,d]=ssdata(G);
        zpG=[zero(G);
             pole(G)];
        zpGimag=zpG(find((abs(real(zpG)))<2^-4*sqrt(eps)));
        if length(zpGimag) | rank(d)<mg  
            singular=1;
            zpL=[zero(L); pole(L)];
            zp=[zpGimag;zpL];
            % zpmax = max jw-axis zero
            if rank(d)<mg; % zero at w=Inf
                zp=[zp; Inf];
            end
            wmin=min(abs(zp));       % smallest pole or zero of L or G
            wmax=max(abs(zp));       % greatest finite pole or zero of L or G
            % either G or L is changing for w between wmin and wmax
            cptL=-min(10*wmax , 1/sqrt(sqrt(eps)));
            cptR=-max(wmin/10 , sqrt(sqrt(eps)));
            circ=[cptL,cptR];
        end
    else 
        singular=1;
        circ=sort(-abs(circ)); % freq range for bilin
    end
	if singular,
        G=bilin(G,1,'Sft_jw',circ);  %bilin pole shifting transform
        L=bilin(L,1,'Sft_jw',circ);
	end
    
	% Square-down G*V=[R,0]  via left GCD factorization
	if mg<ng
        [R,U,V]=gcd(G,'left');  % R has form R=ss(A,H+B1,C,D1) where A,C,D same as G
	else
        gcdflag=0;
        R=G;
        V=eye(mg);
        U=V;
	end
	% V is squaring-down compensator R=G*V
    V = subsref(V,substruct('()',{':',1:mg})); %V=V(:,1:mg);
	
	% Inner-outer factorization 
	% where Rpz is stable and min-phase and Thetaz and Thetap are stable-all-pass
	[Thetaz1,junk,Mz]=iofr(R);  %  R=Thetaz*Mz with Thetaz stable-all-pass, Mz min-phase
    if max(real(pole(Thetaz1)))>0,
        error('Numerically ill-conditioned problem  -- cannot achieve loop-shape Gd due to round-off error')
    end
	[Thetaz]=ss(schbal(ssbal(Thetaz1),2,.5)); % discard nonminimal states from all-pass Thetaz
	[Thetap1,junk,Mpz]=iofr(transpose(inv(Mz)));
	[Thetap]=ss(schbal(ssbal(Thetap1),2,.5)); % discard nonminimal states from all-pass Thetap
	Thetap=transpose(Thetap);
	Mpz=transpose(Mpz);
	Gallpass=Thetaz/Thetap;
	
	% We now have G*V=R=Gallpass/Mpz  ==> Gallpass=G*V*Mpz
	% where Gallpass = Thetaz/Thetap
	
	% shaped plant Gs and weight
	Gs=Gallpass*L; % Gs=G*V*Mpz*L
	W1=V*Mpz*L;
	
	% G*W1=Gallpass*L 
	
	% Compute ncf controller Ks and hinf cost gam

    [Ks,junk,gam]=ncfsyn(Gs); % lti/ncfsyn
    if abs(imag(gam))>0 & real(gam)==0,
        gam=Inf;  
    end
	Ks=-Ks; % Change to negative feedback
	k=(W1*Ks);
    % Ls=Gs*Ks;
    % I=eye(mg);
    % S=inv(I+Ls);
	% cl=I-S;  % cl=Ls/(I+Ls)               
    if singular, % if singular
        k=bilin(k,-1,'Sft_jw',circ);        % reverse bilin pole shifting transform
        % cl=bilin(cl,-1,'Sft_jw',circ);      % reverse bilin pole shifting transform
        if nargout>3,
            W1=bilin(W1,-1,'Sft_jw',circ);  % reverse bilin pole shifting transform
            Ks=bilin(Ks,-1,'Sft_jw',circ);  % reverse bilin pole shifting transform
            Gs=bilin(Gs,-1,'Sft_jw',circ);  % reverse bilin pole shifting transform
        end
    end
	info.W=W1;
	info.Ks=Ks;
	info.Gs=Gs;
    range=sort(abs(circ));  % restore negative circ numbers to positive for range
    info.range={range(1),range(2)};    
end % end continuous-time
cl=feedback(Gorig*k,eye(mg));

%%%%%%%%% End LTI/LOOPSYN.M MGS/RYC 01/03/2003 %%%%%%%%
