% LTIARRAY2USS   Computes uncertain system bounding given LTI SS array
%
% USYS = LTIARRAY2USS(P,PARRAY,ORD) calculates an uncertain system
% USYS with nominal value P, and whose range of behavior includes
% the given array of systems, PARRAY.  USYS employs an input 
% multiplicative uncertainty model, of the form
%         USYS = P*(I + WT*ULTIDYN('Name',[size(P,2) size(P,2)]))
% where WT is a stable scalar system, whose magnitude overbounds
% the relative difference, (P-PARRAY)/P.  The state dimension of WT is ORD.
% Both P and PARRAY must be in the classes SS/TF/ZPK/FRD.  If P is an FRD,
% then USYS will be a UFRD object, otherwise USYS will be a USS object.
% The name of the ULTIDYN atom is based on the variable name of PARRAY
% in the calling workspace.
%
% [USYS,WT] = LTIARRAY2USS(P,PARRAY,ORD) returns the weight WT used
% to bound the infinity norm of (P-PARRAY)/P.  
%
% [USYS,WT] = LTIARRAY2USS(P,PARRAY,ORD,'OutputMult') uses multiplicative
% uncertainty at the plant output (as opposed to additive uncertainty).  
% The formula for USYS is 
%   USYS = (I + WT*ULTIDYN('Name',[size(P,1) size(P,1)]))*P
%
% [USYS,WT] = LTIARRAY2USS(P,PARRAY,ORD,'Additive') uses additive
% uncertainty.  This is the default uncertainty type. 
%   USYS = P + WT*ULTIDYN('Name',[size(P,1) size(P,2)])
%
% [USYS,WT] = LTIARRAY2USS(P,PARRAY,ORD,'InputMult') uses multiplicative
% uncertainty at the plant input (this is the default). The formula for USYS is 
%   USYS = P*(I + WT*ULTIDYN('Name',[size(P,2) size(P,2)]))
%
% [USYS,WT,DIFFDATA] = LTIARRAY2USS(P,PARRAY,ORD,TYPE) returns the norm of
% the difference (absolute difference for Additive, and relative difference
% for Multiplicative) between the nominal model P and PARRAY.  WT satisfies
% DIFFDATA(w_i) < |WT(w_i)| at all frequency points.
%
% See also: FITMAGFRD, USS, ULTIDYN

function [Pu,wt,pkdata] = ltiarray2uss(Pnom,Parray,order,type)

%   Copyright 2003-2006 The MathWorks, Inc.
%   $Revision: 1.1.6.3 $ $Date: 2007/09/18 02:22:22 $

if nargin<4
   type = [];
end
if isempty(type)
   type = 'InputMult';
end

szP = size(Pnom);
szPa = size(Parray);
if szP(1)==szPa(1) && szP(2)==szPa(2) && length(szP)==2
   if isa(Pnom,'ss') || isa(Pnom,'tf') || isa(Pnom,'zpk') || isa(Pnom,'double')
      if isa(Parray,'ss') || isa(Parray,'tf') || isa(Parray,'zpk') ...
            || isa(Parray,'double')
         [mag,ph,omega] = genfresp(Pnom,4,[]);
         if length(omega)>15
      omega = elimfreqpoints(omega,get(Pnom,'Ts'));
         end
      elseif isa(Parray,'frd')
         omega = pvget(Parray,'Frequency');
      else
         error(['Parray of class ''' class(Parray) ''' is invalid input.'])
      end
   elseif isa(Pnom,'frd')
      omega = pvget(Pnom,'Frequency');
      if isa(Parray,'frd')
         if ~all(omega == pvget(Parray,'Frequency'))
            error(['FRDs Parray and Pnom must have the same frequency vectors.'])
         end
      elseif ~(isa(Parray,'ss') || isa(Parray,'tf') || isa(Parray,'zpk') ...
                  || isa(Parray,'double'))
         error(['Parray of class ''' class(Parray) ''' is invalid input.'])
      end
   else
      error(['Pnom of class ''' class(Pnom) ''' is invalid input.'])
   end
   if ~isa(Pnom,'frd')
      Pnomg = frd(Pnom,omega);
   else
      Pnomg = Pnom;
   end
   if ~isa(Parray,'frd')
      Parrayg = frd(Parray,omega);
   else
      Parrayg = Parray;
   end

   if isempty(order) || ~isa(order,'double') || floor(order)~=ceil(order) ...
         || order< 0 || imag(order)~=0
      error('ORDER must be a positive integer')
   end

   if strncmpi(type,'ad',2)
      type = 'Additive';
      dsuf = '_Add';
   elseif strncmpi(type,'in',2)
      type = 'InputMult';
      dsuf = '_InputMult';
   elseif strncmpi(type,'out',3)
      type = 'OutputMult';
      dsuf = '_OutputMult';
   else
      error('TYPE is incorrectly specified')
   end
   if isempty(inputname(2))
      DeltaName = [dsuf(2:end) 'Delta' ];
   else
      DeltaName = [inputname(2) dsuf 'Delta' ];
   end

   switch type
      case 'Additive'
         devg = Parrayg(:,:,:) - Pnomg;
         opt = wcgopt;
         opt.FreqPtWise = 1;
         mxgain = wcgain(devg,opt);
         wt = fitmagfrd(mxgain.UpperBound,order,[],[],1);
         Delta = ultidyn(DeltaName,szP(1:2));
         if szP(1)<szP(2)
            Pu = Pnom + wt*Delta;
         else
            Pu = Pnom + Delta*wt;
         end
      case 'InputMult'
         RPnomg = get(Pnomg,'ResponseData');
         rdata = Parrayg(:,:,:);
         RParrayg = get(rdata,'ResponseData');
         sza = size(RParrayg);
         rdevg = zeros([sza(2) sza(2) sza(3)]);
         for i=1:size(RParrayg,3) % loop over frequency
            zz = RParrayg(:,:,i,:);
            for j=1:size(RParrayg,4) % loop over array
              rdevg(:,:,i,j) = RPnomg(:,:,i)\(zz(:,:,j) - RPnomg(:,:,i));
            end
         end
         devg = frd(rdevg,omega,get(Parrayg,'Ts'));
         opt = wcgopt;
         opt.FreqPtWise = 1;
         mxgain = wcgain(devg,opt);
         wt = fitmagfrd(mxgain.UpperBound,order,[],[],1);
         Delta = ultidyn(DeltaName,[szP(2) szP(2)]);
         Pu = Pnom*(eye(szP(2)) + wt*Delta);
      case 'OutputMult'
         RPnomg = get(Pnomg,'ResponseData');
         rdata = Parrayg(:,:,:);
         RParrayg = get(rdata,'ResponseData');
         sza = size(RParrayg);
         rdevg = zeros([sza(1) sza(1) sza(3)]);
         for i=1:size(RParrayg,3) % loop over frequency
            zz = RParrayg(:,:,i,:);
            for j=1:size(RParrayg,4) % loop over array
              rdevg(:,:,i,j) = (zz(:,:,j) - RPnomg(:,:,i))/RPnomg(:,:,i);
            end
         end
         devg = frd(rdevg,omega,get(Parrayg,'Ts'));
         opt = wcgopt;
         opt.FreqPtWise = 1;
         mxgain = wcgain(devg,opt);
         wt = fitmagfrd(mxgain.UpperBound,order,[],[],1);
         Delta = ultidyn(DeltaName,[szP(1) szP(1)]);
         Pu = (eye(szP(1)) + wt*Delta)*Pnom;
      otherwise
   end      
   pkdata = mxgain.UpperBound;
else
   error('P and Parray should have same input/output dimensions.');
end
      