function [k,varargout]=mixsyn(varargin)
% [K,CL,GAM,INFO]=MIXSYN(G,W1,W2,W3) or
% [K,CL,GAM,INFO]=MIXSYN(G,W1,W2,W3,KEY1,VALUE1,KEY2,VALUE2,...) 
% 
% MIXSYN H-infinity mixed-sensitivity synthesis method for robust 
% control design. Controller K stabilizes plant G and minimizes 
% the H-infinity cost function
%                ||   W1*S ||
%                || W2*K*S ||
%                ||   W3*T ||Hinf              
% where 
%     S := inv(I+G*K)         % sensitivity
%     T := I-S = G*K/(I+G*K)  % complementary sensitivity 
%     W1, W2 and  W3 are stable LTI 'weights' 
% 
% Inputs:
%    G         LTI plant
%    W1,W2,W3  LTI weights (either SISO or compatibly dimensioned MIMO)
%              To omit weight, use empty matrix (e.g., W2=[] omits W2)
%    
% Optional Inputs: 
%    KEY1      MIXSYN supports all of the key & value input pairs 
%    VALUE1       available to the function HINFSYN; see the 
%    etc.         HINFSYN documentation for further information
%       
% Outputs:
%    K         H-infinity Controller
%    CL        CL=[W1*S; W2*K*S; W3*T]; weighted closed-loop system
%    GAM       GAM=hinfnorm(CL), closed-loop H-infinity norm
%    INFO      Information STRUCT, see HINFSYN documentation for details
% 
% 
% 
% Example:
%    G=ss(-1,2,3,4);   % plant to be controlled 
%    w0=10;     % desired closed-loop bandwidth
%    A=1/1000;  % desired disturbance attenuation inside bandwidth
%    M=2 ;      % desired bound on hinfnorm(S) & hinfnorm(T)
%    s=tf('s'); % Laplace transform variable 's'
%    W1=(s/M+w0)/(s+w0*A); % Sensitivity weight
%    W2=[];                % Empty control weight
%    W3=(s+w0/M)/(A*s+w0); % Complementary sensitivity weight
%    [K,CL,GAM,INFO]=mixsyn(G,W1,W2,W3);
%
% Plot results of successful design:
%    L=G*K;  % loop transfer function
%    S=inv(1+L); % Sensitivity
%    T=1-S;      % complementary sensitivity
%    sigma(GAM/W1,GAM/W3,S,T);
% 
% See also HINFSYN AUGW

% Copyright 2003-2004 The MathWorks, Inc.

% M. G. Safonov 12/02/02

% Initialize
nag1=nargin;
nag2=nargout;
nout = max(nargout,1)-1;
varargout=cell(1,nout);
nargkey=5; % guess location of first option keyword 
sysargs={[],[],[],[]};  % G,W1,W2,W3 defaults

% give brief help if nargin=0
if ~nag1,
    disp('Usage [K,CL,GAM,INFO]=MIXSYN(G,W1,W2,W3,KEY1,VALUE1,KEY2,VALUE2,...)')
    return
end


% Reset NARGKEY to point to location of first string in VARARGIN
for i=min(5,nag1):-1:1,
    if isstr(varargin{i});
        nargkey=i;
    end
end

% get plant size
g=varargin{1};
[nmeas,ncon]=size(g);

% error checking
if ~isa(g,'lti');
    error('G must be an LTI system')
end
for i=1:nargkey-2,
    Wi=varargin{i+1};
    if isa(Wi,'lti') & ~isstable(Wi),  % Wi=varargin(i+1);
        error(['W' num2str(i) ' must be stable'])
    end
end
    


% collect input arguments for hinfsyn in cell array INARGS
sysargs(1:nargkey-1)=varargin(1:nargkey-1);  
sys=augw(sysargs{:});  % augmented plant sys=augw(G,W1,W2,W3)
if nag1 >= nargkey,
    inargs=[{sys,nmeas,ncon}, varargin(nargkey:end)];
else
    inargs={sys,nmeas,ncon};
end

% call hinfsyn
[k,varargout{:}]=hinfsyn(inargs{:});


%%%%%
function y=isstable(sys)
   y=1;  % stable
   Ts=get(sys,'Ts');
   p=pole(sys);
   if length(p),
       if Ts,
           y=(max(abs(p))<1);
       else
           y=(max(real(p))<0);
       end
   end

%%%%%% End LTI/MIXSYN.M RYC/MGS 12/03/02 %%%%%%%
