%NCFMARGIN  Normalized coprime stability margin of plant-controller 
%           feedback loop.
%
%    [MARG,FREQ] = NCFMARGIN(P,C) calculates the normalized coprime 
%    factor/gap metric robust stability of the multivariable feedback 
%    loop consisting of C in negative feedback with P.  C should only 
%    be the compensator in the feedback path, not any reference channels,
%    if it is a "2-dof" architecture. The output MARG contains upper
%    and lower bound for the normalized coprime factor/gap metric robust 
%    stability margin. FREQ is the frequency associated with the upper
%    bound.
%
%    [MARG,FREQ] = NCFMARGIN(P,C,TOL)  specifies a relative accuracy TOL 
%    for calculating the normalized coprime factor/gap metric robust 
%    stability margin. (TOL=1e-3 by default).
%
%   See also LOOPMARGIN, GAPMETRIC, WCMARGIN.

%   Copyright 1991-2006 MUSYN Inc. and The MathWorks, Inc.
% $Revision: 1.1.6.4 $

function [marg, freq] = ncfmargin(system,controller,ttol)

if (nargin ~=2 & nargin ~= 3),
  disp('usage:  [marg, freq] = ncfmargin(system,controller)')
  return 
end
if nargin == 2
  ttol=1E-3;
end

% original EMARGIN assumed positive feedback.
controller = -controller;

if isa(system,'ss') || isa(system,'zpk') || isa(system,'tf') || ...
        isa(system,'double')
   if isa(controller,'ss') || isa(controller,'zpk') || ...
        isa(controller,'tf') || isa(controller,'double')
      system = ss(system);
      controller = ss(controller);
      if (hasdelay(system) || hasdelay(controller))
         error('NCFMARGIN can not form feedback loop for systems with delays')
      end
      szs = size(system);
      szc = size(controller);
      if szc(1)==szs(2) & szs(1)==szc(2)
         outp = szs(1);
         inp = szs(2);
      else
         error('Invalid Dimension')
      end
   else
      error('Invalid class of C. C must be SS/TF/ZPK or double')
   end
else
    error('Invalid class of P. P must be SS/TF/ZPK or double') 
end

% handle the extra dimension information
exd1 = szs(3:end);
exd2 = szc(3:end);
if length(exd1)==length(exd2) && all(exd1==exd2)   % P and K have same EXD or no EXD
   marg = zeros([1 1 exd1]);
   freq = zeros([1 1 exd1]);
   for i=1:prod(exd1)
      [out1,out2] = ...
          LOCALcpmarg(system(:,:,i),controller(:,:,i),outp,inp,ttol);
      marg(1,1,i) = out1;
      freq(1,1,i) = out2;
   end
elseif isempty(exd1) % P has no EXD but K has EXD
   marg = zeros([1 1 exd2]);
   freq = zeros([1 1 exd2]);
   for i=1:prod(exd2)
      [out1,out2] = ...
          LOCALcpmarg(system,controller(:,:,i),outp,inp,ttol);
      marg(1,1,i) = out1;
      freq(1,1,i) = out2;
   end
elseif isempty(exd2) % K has no EXD but P has EXD
   marg = zeros([1 1 exd1]);
   freq = zeros([1 1 exd1]);
   for i=1:prod(exd1)
      [out1,out2] = ...
          LOCALcpmarg(system(:,:,i),controller,outp,inp,ttol);
      marg(1,1,i) = out1;
      freq(1,1,i) = out2;
   end
else
    error('Inconsistent array dimensions between P and C')
end


function [marg,freq] = LOCALcpmarg(system,controller,outp,inp,ttol);

L.type = '()';
L.subs = {[1:outp 1:outp],[1:inp 1:inp]};
Pnew = subsref(system,L);
%Pnew = system([1:outp 1:outp],[1:inp 1:inp]);
top = [Pnew [zeros(outp,outp); eye(outp,outp)]];
bot = [[eye(inp,inp) eye(inp,inp),zeros(inp,outp)] ];
Pnew = [ top; bot ];
L.subs = {[1:outp 2*outp+1:2*outp+inp outp+1:2*outp],...
              [1:inp 2*inp+1:2*inp+outp inp+1:2*inp]};
Pnew = subsref(Pnew,L);
%Pnew = Pnew([1:outp 2*outp+1:2*outp+inp outp+1:2*outp],...
%              [1:inp 2*inp+1:2*inp+outp inp+1:2*inp]);
          
pp = lft(Pnew,controller);
[marg,freq]  = norm(pp,inf,ttol);
marg = 1/marg;
