function [GRED,info] = ncfmr(G,varargin)

%NCFMR  balanced truncation model reduction for normalized coprime factors of 
%  continuous/discrete & stable/unstable G.
%
%[GRED,REDINFO] = NCFMR(G,ORDER,KEY1,VALUE1,KEY2,VALUE2,...) performs 
%    balanced truncation model reduction on the normalized coprime factors of
%    the LTI system G such that the sigma plot of the error (GR - G) < 2*(the 
%    sum of the discarded states' Hankel singular values). The normalized
%    coprime factors in GL and GR are both balanced realizations 
%    with Hankel singular values given in REDINFO.hsv. The unstable part of G is
%    kept as default.
%
%    GL = [Nl Ml] and Nl Nl~ + Ml Ml~ = I, G = Ml^(-1) Nl.
%    GR = [Nr;Mr] and Nr~ Nr + Mr~ Mr = I, G = Nr Mr^(-1).
%
% Inputs:
%    G  - LTI system to be reduced
%
% Optional inputs:
%    ORDER - an integer array with desired order of reduced model. 
%		 A batch run of [m:n] can be specified for a pack of reduced 
% 		 order models to be generated.    
%
%   KEY       |VALUE        | MEANING
%  -------------------------------------------------------------------------
%  'MaxError' |real no. or  | Reduce to achieve H-infinity error
%             |vector       | If present, 'MaxError' overrides ORDER 
%  -------------------------------------------------------------------------
%  'Display'  |'off' or 'on'| Display HANKELSV plots (default 'off')
%
%  Outputs:
%    GRED    - LTI reduced order system
%    REDINFO - a struct of REDINFO.hsv (Hankel SV's of coprime factors),
%              REDINFO.GL & REDINFO.GR 
%
% See also	REDUCE, SCHURMR, BSTMR, NCFMR, HANKELMR. 

% R. Y. Chiang & M. G. Safonov 4/15/02
% Copyright 1988-2005 The MathWorks, Inc.
% $Revision: 1.1.6.6 $ $Date: 2005/12/22 18:23:21 $
% All Rights Reserved. 

if nargin == 1
   Type = 3;
end

if hasInternalDelay(G)
    error('Non-rational system is not supported.')
end

try 
    G = ss(G);
catch
    error('First input argument should be a SS object !');
end

if ndims(G) > 2
    error('Model reduction routines only deal with single LTI model, no LTI arrays !')
end

G = ssbal(G);

% Handle discrete case:

paaz = 2^-4*sqrt(eps);
Ts0 = getTs(G);
Ts = abs(Ts0); % take care of the inherited case (Ts = -1)

if Ts,
    a = get(G,'a');
    if (max(abs(eig(a)+1))>paaz)  % if no poles near z=-1
        paaz = 0;
    end
    G = bilin(G,-1,'S_Tust',[Ts, 1-paaz]);
end

% Processing input arguments

vvaagg = varargin;

% Defaults:

Type = 3;   % reduced style
Typeoriginal = 1;
redval = {[],[]};
dispval = 'off';
displayflag = 0;

% Checking the "order" case (2nd argument is not char string)

if ~isempty(vvaagg)
   if isnumeric(vvaagg{1,1})
      redval = vvaagg{1,1};
      Type = 1;
      vvaagg = varargin(2:end);
   end 
end

if ~isempty(vvaagg)
   if mod(length(vvaagg),2) 
     error('Arguments should be in pairs !')
   end
end

% Loop through varargin:

if ~isempty(vvaagg)
for i = 1:2:length(vvaagg)-1
    curarg0 = vvaagg{1,i};
    curarg = lower(curarg0(1:3));
    if ~isstr(curarg)
        error('Key value must be a string')
    end
    
    switch curarg
    
    % Checking 'order'
    case 'ord'
         Type = 1;
         Typeoriginal = Type;
         redval = vvaagg{1,i+1}; 
         
    % Checking 'MaxError'
    case 'max' 
        
        Type = 2;
        Typeoriginal = Type;
        redval = vvaagg{1,i+1};   % overwrites the "order" value
        
    case 'dis' 
        dispval = vvaagg{1,i+1};
        if strcmp(dispval(1:2),'on')
           Type = 3;
           displayflag = 1;
          % redval = [];
        end     
    otherwise
        error(['''' curarg0 '''', ' is not a valid input (try ''MaxError'',''Display'' or ''Order'').']) 
    end
end % for loop
end % if vvaagg is not empty

% ----- Start execution:

if ~iscell(redval)
   redval = {redval};
end

[a,b,c,d] = ssdata(G);
[md,nd]= size(d);

if displayflag == 1 | Type == 3
   hankelsv(G,'ncf')
   if isempty(redval{1,1})
     ord = input('Please enter the desired order: ');
     if isempty(ord)
           ord = length(ssdata(G));
           disp('Original model returned.');
     end
     close(gcf)  
     redval{1,1} = ord;
   end
   Type = Typeoriginal;
end

stabexe = length(redval{1,1});
GRED = []; GL = []; GR = [];
for i = 1:stabexe
    [gl,hhsv,gr] = sncfbal(G,Type,redval{1,1}(i));
    if Ts, % discrete case
        gr = bilin(gr,1,'S_Tust',[Ts,1-paaz]);
        gl = bilin(gl,1,'S_Tust',[Ts,1-paaz]);
        gr = setTs(gr,Ts0);
        gl = setTs(gl,Ts0);
    end
    GL = subsasgn(GL,substruct('()',{':' ':' i}),gl);  % GL(:,:,i) = gl
    GR = subsasgn(GR,substruct('()',{':' ':' i}),gr);  % GR(:,:,i) = gr
    info.GL = GL;
    info.GR = GR;
    if ~isempty(get(gl,'a'))
        nl=subsref(gl,substruct('()',{':' 1:nd}));                % gl(:,1:nd);
        ml=subsref(gl,substruct('()',{':' nd+1:size(gl,2)}));   % gl(:,nd+1:end);
        [aa,bn,cc,dn]=ssdata(nl);
        [aa,bm,cc,dm]=ssdata(ml);
        cc=dm\cc;
        dn=dm\dn;
        gred=ss(aa-bm*cc,bn-bm*dn,cc,dn,Ts0);
    else
        gred = ss(get(G,'d'));
    end
    GRED = subsasgn(GRED,substruct('()',{':' ':' i}),gred);  % GRED(:,:,i) = gred = nr/mr; 
end
  
info.hsv = hhsv;

% ---------- End of NCFMR.M % 10/26/02

function [sysnlcf,sig,sysnrcf,Ham]  = sncfbal(sys,Type,arg)

if Type == 1,
    ord = arg;
end

if Type == 2,
    tol = arg;
end
    
   if nargin<1
     disp('usage: [sysnlcf,sig,sysnrcf] = sncfbal(sys,tol)')
     return
   end
   [A,B,C,D]=ssdata(sys);
   [p,m] = size(D);
   n = length(A);
   
 %  Solve the GFARE soln X=S'*S

   Ham = [A' zeros(n); -B*B' -A] - [C'; -B*D']*inv(eye(p)+D*D')*[D*B' C];
   [U,Hamm]=schur(Ham);
   [a,b,c,d] = ssdata(sys);
   if any(imag([a,b;c,d])), 
	[U,Hamm]=ocsf(U,Hamm);
   else 
	[U,Hamm]=orsf(U,Hamm,'s'); 
   end
   [S,SS]=schur((U(1:n,1:n)'*U(n+1:2*n,1:n)+U(n+1:2*n,1:n)'*U(1:n,1:n))/2);
    S=S*sqrt(abs(SS))*S';
   [qdr,Dr]=qr([D;eye(m)]);Dr=inv(Dr(1:m,1:m));
   [qdl,Dl]=qr([D';eye(p)]);Dl=inv(Dl(1:p,1:p)');

% change coordinates using U(1:n,1:n)

   nmuinv= norm(inv(U(1:n,1:n)));
   if nmuinv>1e12, 
     disp('Warning from SNCFBAL: SYS is close to being undetectable.');
     disp('                      Results maybe unreliable.')
     % return
   end %if nmuinv>1e12,
   B=U(1:n,1:n)'*B; %temp variable
   Cl=Dl*(C/U(1:n,1:n)');
   C=C*U(n+1:2*n,1:n); %temp variable
   Al=Hamm(1:n,1:n)';
   Bl=(B-C'*D)*Dr*Dr';        
   Hl=-(C'+B*D') *Dl'*Dl;       

% realization of nlcf [N M] is now pck(Al,[Bl Hl],Cl,[Dl*D;Dl])
% Calculate observability Gramian for nrcf [Nl Ml], R*R'

   nm11=n:-1:1;
   R=sjh6(Al(nm11,nm11),Cl(:,nm11));
   R=R(nm11,nm11)';

% calculate the Hankel-singular values of [Nl Ml]

   [W,T,V] = svd(S*R);
   sig = diag(T);

% balancing coordinates

   T = W'*S;
   Cl = Cl*T'; Al = Al*T'; 
   T = R*V; 
   Bl = T'*Bl; Al = T'*Al; Hl = T'*Hl;

% calculate the truncated dimension nn

 if nargin<3, 
    Type = 2;
	tol=max([sig(1)*1.0E-12,1.0E-16]);
end  
  
   nn = n;
   if Type == 2
    for i=n:-1:1, 
    	if sig(i)<=tol 
		   nn=i-1; 
	    end
    end
   elseif Type == 1
       nn = min([ord,length(sig(find(sig>eps)))]);
%        if ord > nn
%            warning('A minimal order model has replaced your desired model.')
%        end
       arg = nn;
   end
  
   if nn==0, 
	sysnrcf = ss([],[],[],[D*Dr;Dr]);
	sig=[];
    sysnlcf = ss([],[],[],[Dl*D Dl]);
   else   
	sig = sig(1:nn);
	% diagonal scaling  by sig(i)^(-0.5)
	irtsig = sig.^(-0.5);
	onn=1:nn;
	Al=Al(onn,onn).*(irtsig*irtsig');
	Bl=(irtsig*ones(1,m)).*Bl(onn,:);
	Cl=Cl(:,onn).*(ones(p,1)*irtsig');
	Hl=Hl(onn,:).*(irtsig*ones(1,p));
	sysnlcf=ss(Al,[Bl Hl],Cl,[Dl*D Dl]);
   end

%now calculate the right coprime factorization if nargout>2.

 if nargout>2,
  if nn==0, 
	sysnrcf = ss([],[],[],[D*Dr;Dr]);
	sig=[];
    sysnlcf = ss([],[],[],[Dl*D Dl]);
  elseif (sig(1)<0.999)&(nmuinv<=1e12) & sig(1)~= 0
	roms2=sqrt(1-sig.^2);
	sysnrcf=[zeros(nn+p+m,nn+m) nn*eye(nn+p+m,1); zeros(1,nn+m), -inf];
	B=(Bl-Hl*D)*Dr;
    sysnrcf(nn+p+1:nn+m+p,1:nn)=...
		-(Dr*B').*(ones(m,1)*(sig./roms2)')...
		-(D'*Dl'*Cl).*(ones(m,1)*roms2');
	sysnrcf(nn+1:nn+p,1:nn)=...
		(Dl\Cl).*(ones(p,1)*roms2')+D*sysnrcf(nn+p+1:nn+m+p,1:nn);
	sysnrcf(1:nn+m+p,nn+1:nn+m)=[B./(roms2*ones(1,m));D*Dr;Dr];
	sysnrcf(1:nn,1:nn)=(Al).*(roms2*(roms2.^(-1))');
    [Ar,Br,Cr,Dr] = unpck(sysnrcf);
    sysnrcf = ss(Ar,Br,Cr,Dr);
  else
    [a,b,c,d] = ssdata(sys);
    sys = ss(a.',c.',b.',d.');
	sys2 = sncfbal(sys,Type,arg);
    [a,b,c,d] = ssdata(sys2);
    sysnrcf = ss(a.',c.',b.',d.');
  end %if nn==0, sysnrcf
 end, % if nargout>2
    
%*******************
function u = sjh6(a,c)

if isempty(a) | isempty(c)
    u = [];
    return
end
%  check if input has any complex inputs
  if any(any(imag(a)))| any(any(imag(c))) realfl=0;
      else realfl=1; end
  [na,ma]=size(a);
  [mc,nc]=size(c);
  n = na; m = mc;
% perform complex Schur form on a
  if realfl [ua,a] = rsf2csf(eye(n),a); end
% qr decomposition of c
  if realfl c=c*ua;end
  c = triu(qr(c));
% now calculate the solution from equations (5.13) - (5.16).
  u = zeros(n);  id=eye(n); y=zeros(1,n-1);
  for i=1:(n-1)
    f = sqrt(-2*real(a(i,i)));
    u(i,i) = abs(c(1,1))/f;
    e = sign(c(1,1))*f;
    if c(1,1)==0
	e=f;
    end
    u(i,(i+1):n) = (-e'*c(1,2:(n-i+1)) -u(i,i)'*a(i,(i+1):n))/...
        (a(i,i)'*id(1:(n-i),1:(n-i)) + a((i+1):n,(i+1):n));
    y(1,1:(n-i)) = c(1,2:(n-i+1)) - e*u(i,(i+1):n);
    if m>1
	c(1:m,1:(n-i)) = triu(qr([c(2:m,2:(n-i+1));y(1,1:(n-i))]));
    else
	c(1:m,1:(n-i)) = y(1:m,1:(n-i));
    end
  end
  f = sqrt(-2*real(a(n,n)));
  u(n,n) = abs(c(1,1))/f;
% now obtain u in upper triangular form
  if ~realfl u = triu(qr(u));, end;
  if realfl u=u*ua'; u(1:n,1:n) = qrsc(u);end;