function [k,varargout]=ncfsyn(varargin)
%NCFSYN  H-infinity normalized coprime factor controller synthesis.
%
% [K,CL,GAM,INFO]=NCFSYN(G) 
% [K,CL,GAM,INFO]=NCFSYN(G,W1) 
% [K,CL,GAM,INFO]=NCFSYN(G,W1,W2) computes the Glover-McFarlane 
% 	H-infinity normalized coprime factor (NCF) loop-shaping 
% 	controller K for LTI plant G with LTI weights W1 and W2. The
%  closed-loop system returned, CL, represents the transfer matrix
%  from [w1; w2] to [z1; z2].
%
%              ^ z1                          ^ z2
%        +     |   ____      ____      ____  |         ____     
%   w1 ---> + --->| W2 |--->| Ks |--->| W1 |---> + ---| G  |----
%           ^      ----      ----      ----      ^     ----    |
%         + |                                    |             |
%           |                                 w2 |             |
%           |__________________________________________________|
% 
% 	The positive feedback controller Ks robustly stabilizes a family 
% 	of systems given by a ball of uncertainty in the normalized coprime
% 	factors of the shaped plant Gs=W2*G*W1.
%
% [K,CL,GAM,INFO]=NCFSYN(G,W1,W2,'ref') computes the Glover-McFarlane 
%  H-infinity NCF loop-shaping controller K, with a reference command,
%  for LTI plant G, weights W1 and W2 if the 'ref' option is included.
%  The closed-loop system returned, CL, represents the transfer matrix
%  from [w1; w2; ref] to [z1; z2].
%
%              ^ z1                            ^ z2
%        +     |     ____      ____      ____  |         ____     
%   w1 ---> + ----->|    |--->|    |--->|    |---> + ---| G  |----
%           ^       | W2 |    | Ks |    | W1 |     ^     ----     |
%         + |   --->|    |    |    |    |    |     |              |
%           |   ref  ----      ----      ----   w2 |              | 
%           |                                                     |
%           |_____________________________________________________|
%  
%     Input:
%     -------
%      G  -   LTI plant to be controlled, which must be square (NxN) 
%
%     Optional Input:
%     -------
%      W1,W2  - stable min-phase LTI weights, either SISO or square (NxN).
%               Default is W1=W2=eye(N)
%      'ref'  - includes a reference input to the controller.
%
%     Output:
%     -------
%      K      -  K=W1*Ks*W2, H-infinity optimal loopshaping controller 
%      CL     -  CL = [I; K]*inv(I-G*K)*[I, G] , closed-loop 
%      GAM    -  the H-infinity optimal cost 
%      INFO   -  struct array with various information fields, such as
%                INFO.emax   -  the nugap robustness, EMAX=1/GAM 
%                INFO.Gs     -  the 'shaped plant' Gs=W2*G*W1
%                INFO.Ks     -  Ks = NCFSYN(Gs) = NCFSYN(Gs,I,I)
% 
% 	THEORY:  Gs = W2*G*W1 is called the 'shaped plant'. The controller
% 	  Ks simultaneously solves both of the following two H-infinity 
%     optimal control problems for the shaped plant Gs:
%          1).    min  || [I; Ks]*inv(I-Gs*Ks)*[I, Gs] ||
%                  K                                     Hinf
% 	
%          2).    min  || [I; Gs]*inv(I-Ks*Gs)*[I, Ks] ||
%                  Ks                                    Hinf
% 	and GAM is the minimal H-infinity COST for both problems.   
% 	
%   See also: gapmetric,hinfsyn,loopsyn,ncfmargin

% Copyright 2003-2006 The MathWorks, Inc.

% LTI/NCFSYN
% M. G. Safonov 01/07/2003

nag1=nargin;
nag2=nargout;
nout = max(nag2,1)-1;
varargout=cell(1,nout);
if nag1==0,
	if nag2==0,
        disp('Usage [K,CL,GAM,INFO]=NCFSYN(G,Win,Wout)')
	else
        error('Too few inputs')
	end
end

% Initialize variables
G=[];
W1=[];
W2=[];
k=[];
cl=[];
gam=[];
info=struct;
info.emax=[];
info.Ks=[];
info.Gs=[];
G=varargin{1};
if ~isa(G,'lti')
    error('Plant G must be an LTI system')
end
G=ss(G);
[ny,nu]=size(G);

% Get the weights W1 and W2, and check for errors
switch nag1
    case 1
        W1=ss(eye(nu));  % Default W1,W2
        W2=ss(eye(ny));
    case 2
        W1=ss(varargin{2});
        W2=ss(eye(ny));  % Default W2
    otherwise
        W1=ss(varargin{2});
        W2=ss(varargin{3});
end 
Wcell={W1,W2};
nuny=[nu,ny];
for i=1:2,  % make sure W1, W2 have dims compatible with G
   if issiso(Wcell{i}), % Make SISO weights MIMO, if needed
       temp=Wcell{i};
       for j=1:nuny(i)-1,
          Wcell{i}=append(Wcell{i},temp);
       end
    elseif ~isequal(size(Wcell{i},i),nuny(i)),
       etmp = ['MIMO Weight W', num2str(i), ' must have'];
       etmp = [etmp num2str(nuny(i)), 'inputs and outputs.'];
       error(etmp) 
    end
end
Gs=W2*G*W1;  % Shaped plant Gs

% if ~isequal(ny,nu),
%     error('The plant G must have equal number of inputs and outputs')
% end

Ts=get(G,'Ts');
paaz=2^-4*sqrt(eps);
if Ts,  % Discrete-time
    T=abs(Ts);
    a=ssdata(G);
    nx=size(a,1);
    if (max(abs(eig(a)+1))>paaz), % if no poles near z=-1
        paaz=0;
    end   
    
    % convert Gs to continous time Gsc
     Gsc=bilin(Gs,-1,'S_Tust',[T,1-paaz]);   %  G=bilinz2s(G)
    
    % compute controller Ksc, k then transfor back to discrete time
    if nag1==4 && strcmpi(varargin{4},'ref')
      % calculate optimal NCF controller Ks w/reference command
      [Ksc,emax,Ksobs] = xncfsyn(Gsc,1,'ref','Off');
      nuW2 = size(W2,1);
      Ks=bilin(Ksc,1,'S_Tust',[T,1-paaz]);    % Ks=bilinz2s(Ksc)
      k=W1*Ks*blkdiag(W2,eye(nuW2));
    else
      [Ksc,emax,Ksobs] = xncfsyn(Gsc,1); % optimal NCF controller 
      Ks=bilin(Ksc,1,'S_Tust',[T,1-paaz]);    % Ks=bilinz2s(Ksc)
      k=W1*Ks*W2;
    end
    gam=1/emax;
else % continuous time
    %Gsmat=lti2mat(Gs);  % convert Gs to MAT
    if nag1==4 && strcmpi(varargin{4},'ref')
      % calculate optimal NCF controller w/reference command
      [Ks,emax,Ksobs] = xncfsyn(Gs,1,'ref','Off');
      nuW2 = size(W2,1);
      k=W1*Ks*blkdiag(W2,eye(nuW2));
    else
      [Ks,emax,Ksobs] = xncfsyn(Gs,1); % optimal NCF controller 
      k=W1*Ks*W2;
    end
    %Ks=mat2lti(Ksmat);  % convert result to LTI
    gam=1/emax;
end % else % continuous time

% compute CL, if required
% if nag2>1  
%     L=G*k;           % loop
%     S=inv(eye(ny)-L) % sensitivity (positive feedback)
%     cl=S-eye(ny);    % -L/(I-L) %  complementary sensitivity (positive feedback)
% end
ey=eye(ny);
eu=eye(nu);
w1=1:ny; w2=ny+1:ny+nu; w3=ny+nu+1:ny+nu+nu;
z1=w1  ; z2=w2;         z3=ny+nu+1:ny+nu+ny;
T11=zeros(ny+nu+ny,ny+nu+nu);
T11(z1,w1)=ey;
T11(z2,w3)=eu;
T11(z3,w1)=ey;
T12=T11(:,w1);
T21=[zeros(nu,ny) eu eu];
%GJB: Changed from Gs and Ks to G and K
% augp=T11+T12*Gs*T21;
% cl=lft(augp,Ks);
augp=T11+T12*G*T21;
if nag1==4 && strcmpi(varargin{4},'ref')
   cl=lft(augp,k,nu,ny);
else
   cl=lft(augp,k,nu,ny);
end

% Compute INFO, if required
if nag2>3,
	% INFO   -  struct array with various information fields, such as
	%           INFO.emax   -  the nugap robustness, EMAX=1/GAM 
	%           INFO.Gs     -  the 'shaped plant' Gs=W2*G*W1
	%           INFO.Ks     -  Ks=NCFSYN(Gs) (i.e., W1=W2=eye
    info.emax=emax;
    info.Gs=Gs;
    info.Ks=Ks;
end

if nout,
  temp={cl,gam,info};
  varargout=temp(1:nout);
end
    
%%%% End of LTI/NCFSYN --- M. G. Safonov 01/07/2003 %%%%
