function [Gm,aug,hhsv] = bstschmr(G,mrtype,no,info)
%BSTSCHMR Balanced stochastic truncation (BST) model reduction.
%
% [SS_H,AUG,HHSV] = BSTSCHMR(SS_,MRTYPE,NO,INFO) performs
%    relative error Schur model reduction on a SQUARE, STABLE G(s):=
%    (A,B,C,D). The infinity-norm of the relative error is bounded as
%    (Zhou, 1995):
%               -1                       n
%            |Gm (s)(Gm(s)-G(s))|    <= prod (1+2*si*(sqrt(1+si^2)+si))-1
%                                inf    k+1
%    where si denotes the i-th Hankel singular value of the all-pass
%    "phase matrix" of G(s).
%    The algorithm is based on the Balanced Stochastic Truncation (BST)
%    theory with Relative Error Method (REM).
%    Based on the "MRTYPE" selected, you have the following options:
%     1). MRTYPE = 1  --- no: size "k" of the reduced order model.
%     2). MRTYPE = 2  --- find k-th order reduced model that
%                         tolerance <= "no".
%     3). MRTYPE = 3  --- display all the Hankel SV of phase matrix and
%                   prompt for "k" (in this case, no need to specify "no").
%    Input variable: "info" = 'left '; or 'right' (default value)
%    Output variable "aug": aug(1,1) = no. of state removed
%                           aug(1,2) = relative error bound
%    Note that if D is not full rank, an error will result.

% R. Y. Chiang & M. G. Safonov 2/30/88
% Copyright 1988-2004 The MathWorks, Inc. 
%       $Revision: 1.1.6.2 $ $Date: 2004/08/25 19:12:28 $
% All Rights Reserved.
% ---------------------------------------------------------------------------

nag1 = nargin;
%[emsg,nag1,xsflag,Ts,A,B,C,D,mrtype,no,info]=mkargs5x('ss',varargin); error(emsg);

[A,B,C,D,Ts] = ssdata(G);
G0 = G;
D0 = D;

if Ts, error('LTI inputs must be continuous time (Ts=0)'), end
nag1 = nargin;     % NARGIN may have changed

[rd,cd] = size(D);
if nag1 == 3
   info = 'right';
end
%
if (info == 'left ') | (rd > cd)
   A = A'; temp = B; B = C'; C = temp'; D = D';
end
%
if rank(D*D') < min([rd, cd])
  %disp('   WARNING: D MATRIX IS NOT FULL RANK - - -');
  %disp('            THE PROBLEM IS ILL-CONDITIONED !');
  %D = 1e-8*eye(size(D));
  D = max(abs(eig(A)))/1000*eye(size(D));
end
%
%disp(' ')
%disp('        - - Working on Schur BST-REM model reduction - -');
[ma,na] = size(A);
P = gram(A,B);
G = P*C'+B*D';
phi = D*D';
%
qrnQ = [zeros(ma) -C';-C -phi];
[K,Q] = lqrc(A,G,qrnQ);

[up,sp,vp] = svd(P);
lr = up*diag(sqrt(diag(sp)));
[uq,sq,vq] = svd(Q);
lo = uq*diag(sqrt(diag(sq)));
hhsv = svd(lo'*lr);
%
% ------ Model reduction based on your choice of mrtype:
%
if mrtype == 1
   kk = no+1;
end
%
if mrtype == 2
   resig_1 = 1;
   kk = 1;
   for i = ma:-1:1
      resig = (1+2*hhsv(i)*(sqrt(1+hhsv(i)^2)+hhsv(i)));
      resig_1 = resig_1*resig;
      if resig_1-1 > no
         kk = i+1;
         break
      end
   end
end
%
if mrtype == 3
   no = input('Please enter the desired order: ');
   kk = no + 1;
end
%
% Check the desired order vs. minimal
%
ord = kk-1;
ordmin = min([ord,length(hhsv(find(hhsv>eps)))]);
if  ord > ordmin
   warning('A minimal order model has replaced your desired model.')
end
kk = ordmin + 1;
%
% ------ Save all the states:
%
if kk > na & sum(hhsv) > 1e-16
   aug = 0;
   Gm = G0;
   return
end
%
% ------ Disgard all the states:
%
if kk <= 1 | sum(hhsv) < 1e-16
   ahed = [];   bhed = [];
   ched = [];   dhed = D0;
   resig_1 = 1;
   h = hhsv;
   rem = prod(1+2*h.*(sqrt(1+h.^2)+h))-1;
   aug = rem;
   Gm = ss(ahed,bhed,ched,dhed);
   return
end
%
% ------ Computing the relative error bound by Zhou 1995:
%
resig_1 = 1;
h = hhsv(kk:end);
rem = prod(1+2*h.*(sqrt(1+h.^2)+h))-1;
strm = na-kk+1;
aug = rem;
%
% ---------------------- Start Model Reduction:
%
% ------ Find the left-eigenspace basis:
%
ro = (hhsv(kk-1)^2+hhsv(kk)^2)/2.;
gammaa = P*Q-ro*eye(na);
[va,ta,msa] = blkrsch(gammaa,1,na-kk+1);
vlbig = va(:,(na-kk+2):na);
%
% ------ Find the right-eigenspace basis:
%
gammad = -gammaa;
[vd,td,msd] = blkrsch(gammad,1,kk-1);
vrbig = vd(:,1:kk-1);
%
% ------ Find the similarity transformation:
%
ee = vlbig'*vrbig;
[ue,se,ve] = svd(ee);
%
seih = diag(ones(kk-1,1)./sqrt(diag(se)));
slbig = vlbig*ue*seih;
srbig = vrbig*ve*seih;
%
ahed = slbig'*A*srbig;
bhed = slbig'*B;
ched = C*srbig;
dhed = D0;
%
if (info == 'left ') | (rd > cd)
  ahed = ahed'; temp = bhed;
  bhed = ched'; ched = temp'; dhed = dhed;
end
%
Gm = ss(ahed,bhed,ched,dhed);
%
%disp(' ')
%disp(['               ' int2str(aug(1,1)), '    states removed !!'])
%
% ------ End of BSTSCHMR.M --- RYC/MGS %
