function [X,L,G,RR,Rpt] = gcare_kluge(H,factor)
%%  This GCARE is a KLUGED RCAST version for temporary use until GCARE is
%%  available in Control Toolbox - adapted from CARE
%   [X1,X2,D,L] = GCARE(H,...,'factor') returns two matrices X1, X2 
%   and a diagonal scaling matrix D such that X = D*(X2/X1)*D.
%   The vector L contains the closed-loop eigenvalues.  All outputs 
%   are empty when the associated Hamiltonian matrix has eigenvalues 
%   on the imaginary axis.

% function [X,L,G,RR] = care(A,B,Q,varargin)
%CARE  Solve continuous-time algebraic Riccati equations.
%
%   [X,L,G,RR] = CARE(A,B,Q,R,S,E)  computes the unique symmetric 
%   stabilizing solution X of the continuous-time algebraic Riccati 
%   equation
%                               -1
%      A'XE + E'XA - (E'XB + S)R  (B'XE + S') + Q = 0
%
%   or, equivalently,
%                         -1             -1                     -1
%      F'XE + E'XF - E'XBR  B'XE + Q - SR  S' = 0  with  F:=A-BR  S'.
%
%   When omitted, R,S and E are set to the default values R=I, S=0, 
%   and E=I.  Additional optional outputs include the gain matrix
%               -1
%          G = R  (B'XE + S') ,
%
%   the vector L of closed-loop eigenvalues (i.e., EIG(A-B*G,E)), 
%   and the Frobenius norm RR of the relative residual matrix.  
%
%   [X,L,G,REPORT] = CARE(A,B,Q,...,'report')  turns off error 
%   messages and returns a success/failure diagnosis REPORT instead.
%   The value of REPORT is 
%     * -1 if Hamiltonian matrix has eigenvalues too close to jw axis
%     * -2 if X=X2/X1 with X1 singular
%     * the relative residual RR when CARE succeeds.
%
%   [X1,X2,L,REPORT] = CARE(A,B,Q,...,'implicit')  also turns off 
%   error messages, but now returns matrices X1,X2 such that X=X2/X1. 
%   REPORT=0 indicates success.
%
%   See also  DARE.

%   Author(s): Alan J. Laub (1993)  (laub@ece.ucsb.edu)
%              with key contributions by Pascal Gahinet, Cleve Moler,
%              and Andy Potvin
%   Revised: 94-10-29, 95-07-20, 96-01-09, 8-21-96
%   Copyright 1986-2004 The MathWorks, Inc.


%   Assumptions: E is nonsingular, Q=Q', R=R' with R nonsingular, and
%                the associated Hamiltonian pencil has no eigenvalues
%                on the imaginary axis.
%   Sufficient conditions to guarantee the above are stabilizability,
%   detectability, and [Q S;S' R] >= 0, with R > 0.
%
%   Reference: W.F. Arnold, III and A.J. Laub, ``Generalized Eigenproblem
%	       Algorithms and Software for Algebraic Riccati Equations,''
%	       Proc. IEEE, 72(1984), 1746--1754.

ni = nargin;
no = nargout;
NoError=logical(1);  % %
RR = 0;              % %
RR = 0;              % %
n2=size(H,1);
n=fix(n2)/2;    % %
   % Compute complex Schur form
   [z,t] = schur(H);
   [z,t] = rsf2csf(z,t);
   L = diag(t);

   % Prepare vector index such that index(i) = sign(real(t(i,i)))
   index = sign(real(L));
   istab = find(index<0);

   % Exit if spectrum cannot be split equally
   if length(istab)~=n,
      if NoError,
         X = []; L = []; G = []; RR = []; Rpt = -1;  return
      else
         error('Cannot order eigenvalues; spectrum too near imaginary axis.')
      end
   else
      L = L(istab);
   end

   % Order spectrum
   [z,t] = schord(z,t,index);

%%%%%%%%%%%%%%%%%%%%
% % Fix complex e-vectors
   vv = [real(z(:,1:n)) imag(z(:,1:n))];
   [uuu,sig,vvv] = svd(vv);
   z=uuu(:,1:n);

% % 
% Check for symmetry of solution
X1 = z(1:n,1:n);
X2 = z(n+1:n2,1:n);
X12 = X1'*X2;
Asym = X12-X12';
Asym = max(abs(Asym(:))) - sqrt(eps);
if Asym > 0.1*max(abs(X12(:))),
   % Spurious separation of Hamiltonian spectrum
   if NoError,
      % % X = []; L = []; G = []; RR = -1;  return
      RR = -1; %%
   else 
      error('Cannot order eigenvalues: spectrum too near unit circle.')
   end
elseif Asym > sqrt(eps),
   warning(...
    'Solution may be inaccurate due to poor scaling or eigenvalues too near imag. axis.')
      RR = -1; %%
end

% Force exact complex conjugate pairs in L
%% L = qzeig(L);


% Set output arguments  [X,L,G,RR,Rpt] <-- [X1,X2,D,L]
   % X given in implicit form
   Rpt=RR;
   RR = L;
   %% L = ScaleFact*X2;
   L = X2;
   X = X1;
   G=eye(size(X));
