function [r,u,v]=gcd(g,opt)
% [R,U,V]=GCD(G) computes a Greatest Common Divisor R of a  
%    continuous-time LTI system G=ss(A,B,C,D) using the 
%    algorithm of Le & Safonov (IEEE Trans, AC-37(3), 1991).
%    By default, the right GCD is computed, in which case
%    G must be 'tall' (i.e., have at least as many outputs as
%    inputs) and the LTI systems R,U,V have the following 
%    properties:
%        G   = U*[R; 0]
%        V*G = [R; 0]
%        sigma(R)== sigma(G)
%        pole(R) == pole(G)
%        zero(R) == zero(G)
%        U is PxP and R is MxM, both square
%           where [P,M]=size(G) with P>=M.
%        V is the inverse of the system U
%        U and V are stable and minimum phase
%        R.A=G.A;   R.B=G.B
%
%    The sub-system V(1:M,:) is a 'squaring-down' 
%    compensator
%           R = V(1:M,:)*G 
%    that preserves the RHP poles, RHP zeros and 
%    singular values of G.
%
% [L,U,V]=GCRD(G,'left') computes the greatest common left divisor
%    of a 'fat' LTI system (at least as many inputs as outputs)
%        G=[L,0]*U; V*G=[L,0]
%
% Limitation: The D-matrix of G must be full rank

% Copyright 1988-2005 The MathWorks, Inc.
% $Revision: 1.1.6.3 $ $Date: 2005/12/22 18:23:29 $
% M. G. Safonov 11/24/01
if nargin>1 & ~isstr(opt),
   error('Optional second input argument, if present, must be the string ''Left''')
end

[p,m]=size(g);

if nargin>1 & lower(opt(1))=='l',
 	if p>m,
        error('Left Greatest Common Divisor gcd(G):  G cannot have fewer inputs than outputs')
    end
    gt=transpose(g);
    [rt,ut,vt]=gcrd(gt);
    r=transpose(rt);
    u=transpose(ut);
    v=transpose(vt);
else
	if p<m,
        error('Right Greatest Common Divisor gcd(G):  G cannot have fewer outputs than inputs')
    end
    [r,u,v]=gcrd(g);
end

%%%%% subfunction GCRD.M %%%%%
function [r,u,v]=gcrd(g)
g=ss(g); % convert to ss
[p,m]=size(g);
a=get(g,'a');
n=size(a,1);

% error checking
if p<m,
    error('G cannot have fewer outputs than inputs')
elseif p==m;
    r=g;
    u=ss(eye(m));
    v=u;
    return
end

d=get(g,'d');
if rank(d)<m,
    error('G(s) must have full column rank')
end

%  Initialize lists of indices for 
i1=1:m;     % y1 & u1 indices
i2=m+1:p;   % y2 & u2 indices

% Step 0: Scale G and square up with padding so G.D=I;
% 
% 
%              | A  |  B     0 |  
%      G(s) =  |----|----------| 
%              | C1 |  I     0 | 
%              | C2 |  0     I | 

% scalings qd,rd computed via QR algorithm
% check if scaling needed
dd=zeros(p,m);
dd(i1,i1)=eye(m);  % dd=[I;0]
d=get(g,'d');
if ~isequal(dd,d),
	[qd,rd]=qr(d);
	rd=rd(i1,:);
	g=qd'*g/rd;
    scale=1; 
else
    scale=0;
end

g=[g,ss(zeros(p,p-m))];
[a,b,c,d]=ssdata(g);
set(g,'d',eye(p));  %g.d=eye(p);
b=b(:,i1);
c1=c(i1,:);
c2=c(i2,:);
d=d(:,i1);


% Step 1:  Apply state-feedback -D'*C, which zeros c1  and 
% cancels all zeros with unobservable modes 
% (see Le & Safonov, IEEE Trans, AC-37(3), Mar.1991.
% 
%              | A-B*C1 |  B   0 |  
%     Gt(s) =  |--------|--------| 
%              | 0      |  I   0 | 
%              | C2     |  0   I | 
gt=g;
set(gt,'a',a-b*c1);            % gt.a = a-b*(d'*c)
c=get(gt,'c');
c(1:m,:)=zeros(m,n); 
set(gt,'c',c);                 % gt.c = c-d*(d'*c) = [0;c2]


% Step 2:  Factor G=UG*[R;0], with U1 anti-stable all-pass
%   Notes: (1) minreal(U1(:,1:p) has all LHP zeros of G, but no RHP zeros
%          (2) sigma(G1)=sigma(R1)
%          (3) U1 is all-pass
% 
%              | A-B*C1-B*Fg |  B  Hg |  
%     Ug(s) =  |-------------|--------| 
%              | -Fg         |  I  0  | 
%              | C2          |  0  I  | 
%
%              | A-B*C1  |  B  |  
%     Rg(s) =  |-------- |-----| 
%              | Fg      |  I  | 
gtsub = subsref(gt,substruct('()',{':' i1}));  % gt(:,i1)
[ug,rg,fg,hg]=ur(transpose(gtsub'));  

% Step 3: Minimal realization of all-pass U1 (discards RHP zeros)
%    Note: Minimal realization URED can be reliably computed since U1 is 
%    all-pass, so all Hankel singlular values are either 0 or 1
%              | At-Bt*Ft |  Bt  Ht |  
%      Ured =  |----------|---------| 
%              |-Ft       |  I   0  | 
%              | Ct       |  0   I  | 
[ured,no,svh,slb,srb]=schbal(ug,2,.5);
at = slb'*(a-b*c1)*srb;
bt = slb'*b;
ct = c2*srb;

 
% Step 4: Solve Riccati equations
[x,lx,f] = care(at,bt,ct'*ct,eye(m));  % state-feedback gain F
if isempty(at), f=bt'; end
[y,ly,h]=care(at',ct',bt*bt',eye(p-m)); 
if isempty(at), h=ct; end
h = h';                                % observer gain H


% Step 5: Build u,v,r
%              | A      | B  |
%     R(s) :=  |--------|----|
%              | F+D'*C | I  |
r=ss(a,b,f*slb'+c1,eye(m));

%  
%              |  At-Bt*F  | Bt   H  |
%     U(s) :=  |-----------|---------|
%              |  F        | I    0  |
%              | -Ct       | 0    I  |
au=at-bt*f;   bu=[bt, h];
cu=[-f; ct]; du=eye(p);
u=ss(au,bu,cu,du);

%              |  At-H'*Ct | Bt   H  |
%     V(s) :=  |-----------|---------|
%              | -F        | I    0  |
%              |  Ct       | 0    I  |
av=at-h*ct;  bv=[bt, h];
cv=[f; -ct];   dv=eye(p);
v=ss(av,bv,cv,dv);

% Step 6: Unscale g=u*r=(qd*u2)*(r3*rd)
if scale,
	u=qd*u;
	r=r*rd;
	v=v/qd;
end
% ------ End of subfunction(GCRD.M) MGS/RYC 11/24/01

%%%%%%%%% begin subfunction UR.M MGS/RYC 11/23/01
% -------------------------------------------------------------------- %
function [u,r,f,h]=ur(g)
% Factorization G=U*[R;0] for G of form G=ss(A,B,C,D) with D=[I 0]'
%     where U(s) is all-pass and R(s) square and minimum phase 
% 
%     Outputs:  
%              | A-BF |  B     H |   | A-BF |  B  H    |
%      U(s) =  |------|----------| = |------|----------|
%              | C1-F |  I     0 |   | C-DF |  D  Dperp|
%              | C2   |  0     I | 
%      where 
%   
%              | A    |  B   |
%      R(s) =  |------|------|
%              | F    |  I   |
%  where F=B'X+C is LQ optimal state-feedback for G(s)
%  for cost 
%      
%      | Q | N |     | C' |  
%      |---|---| =   |----|* [ C  D] 
%      | N'| R |     | D' |
% Limitations:  G(s) must have at least as many outputs as inputs.

% M.G. Safonov and R.Y. Chiang 11/23/01

[a,b,c,d]=ssdata(g);

% error checking
[p,m]=size(g);
if ~isequal(d,eye(size(d))),
    error('g.d~=[I; 0]')
end
if p<m,
    error('fewer outputs than inputs')
end

% solve riccati
Q = c'*c; 
N = c(1:m,:)'; % N=c'*d
R = eye(m);
QRN=[Q,N;N',R];
[x,l,f] = care(a,b,Q,R,N);
if isempty(a), f=b'; end
% [f,x]=lqrc(a,b,QRN);
h = -pinv(x)*c(m+1:end,:)';
% Form U and R
u=ss(a - b * f, [b, h], c-d*f, eye(p)); % U(s)=[Theta(s), Thetap(s)] 
r=ss(a,b,f,eye(m));       % R(s)
%
% ------ End of subfunction UR.M ---- MGS/RYC 11/23/01 % 

%
% ------ End of LTI/GCD.M ---- MGS/RYC 01/03/03 % 


