% function  [sysk,emax,sysobs]=xncfsyn(sysgw,factor,opt,display)
%
%   Synthesizes a controller to robustly stabilize
%   a family of systems given by a ball of uncertainty
%   in the normalized coprime factors of the system description.
%
%   Inputs:
%   -------
%    SYSGW  - the weighted system to be controlled.
%    FACTOR - =1 implies that an optimal controller is required.
%             >1 implies that a suboptimal controller is required
%                achieving a performance FACTOR less than optimal.
%    OPT    - 'ref'  includes a reference input (optional).
%    DISPLAY- display synthesis information to screen, default = 'On'  
%
%
%   Outputs:
%   -------
%    SYSK   -  H-infinity loopshaping controller
%    EMAX   -  stability margin as an indication robustness to
%              unstructured perturbations. EMAX is always less
%              than 1 and values of EMAX below 0.3 generally indicate
%              good robustness margins.
%    SYSOBS -  H-infinity loopshaping observer controller. This variable
%              is created only if FACTOR>1 and OPT = 'ref'
%
%   See also: HINFNORM, HINFSYN, and NUGAP

% Copyright 1988-2006 The MathWorks, Inc.
% $Revision: 1.1.6.3 $ $Date: 2006/11/17 13:59:39 $

function    [sysk,emax,sysobs]=xncfsyn(sysgw,factor,opt,display)


if nargin<2,
    disp('usage: [k,cl]=ncfsyn(sysgw,factor,opt)');
    return;
end;
sysk = [];
emax = 0;
sysobs = [];
sysgw = ss(sysgw);
szgw = size(sysgw);
p = szgw(1);
m = szgw(2);
n = length(get(sysgw,'StateName'));

if ~isa(sysgw,'ss')
    disp('SYSGW needs to be SS, TF or ZPK object');
    return;
 end;

if nargin<3, 
   opt='noref';
   display = 'On';
elseif nargin<4
   display = 'On';
end

if strcmp(opt,'noref') & (factor==1),
     % [sysnlcf,infosig] = ncfmr(sysgw,n);
     % sig = infosig.hsv{1};
     [junk,info] = ncfmr(sysgw,n);
     sig = info.hsv;
     sysnlcf = info.GL;
     if length(sig)>0,
         emax = sqrt(1-sig(1)^2);
         sysUV = xhankmr(sysnlcf,sig,0,'a');
     else % constant system case
         emax=1;
         sysUV=sysnlcf;        
     end
     sysUV = sysUV';
     sysk = -1*xcf2sys(sysUV);

end; %if strcmp(opt,'noref') & (factor==1),

if strcmp(opt,'noref') & (factor>1),
     [a,b,c,d] = ssdata(sysgw);
     R = eye(p)+d*d';
     S = eye(m)+d'*d;
     Hamx = [a-b*(S\d')*c, -(b/S)*b'; -c'*(R\c), -(a-b*(S\d')*c)'];
     Hamz = [(a-b*(d'/R)*c)', -c'*(R\c) ; -b*(S\b')  -(a-b*(d'/R)*c)];
     [x1,x2,fail] = ric_schr(Hamx);
     if fail == 0
	     X=x2/x1;
     else
	     sysk = [];
        if strcmpi(display(1:2),'On')
           disp('Error in solving the X Hamiltonian');
        end
	return
     end
     [z1,z2,fail]=ric_schr(Hamz);
     if fail == 0
	     Z=z2/z1;
     else
	     sysk = [];
        if strcmpi(display(1:2),'On')
           disp('Error in solving the Z Hamiltonian');
        end
	return
     end
     F = -S\(d'*c+b'*X);
     H = -(b*d'+Z*c')/R;
     sig = eig(Z*X);
     emax = 1/sqrt(1+max(real(sig)));
     gamma = factor/emax;
     W2 = eye(n)-gamma^(-2)*(eye(n)+Z*X);
     sysk = ss(a+b*F-W2\(Z*c'*(c+d*F)),W2\(Z*c'),-b'*X,-d');
 end; %if strcmp(opt,'noref') & (factor>1)

if (nargin>2) & strcmp(opt,'ref') & factor>1,
     [a,b,c,d] = ssdata(sysgw);
     R = eye(p)+d*d';
     S = eye(m)+d'*d;
     Hamx = [a-b*(S\d')*c, -(b/S)*b'; -c'*(R\c), -(a-b*(S\d')*c)'];
     Hamz = [(a-b*(d'/R)*c)', -c'*(R\c) ; -b*(S\b')  -(a-b*(d'/R)*c)];
     [x1,x2,fail] = ric_schr(Hamx);
     if fail == 0
	     X=x2/x1;
     else
	     sysk = [];
        if strcmpi(display(1:2),'On')
           disp('Error in solving the X Hamiltonian');
        end
	     return
     end
     [z1,z2,fail]=ric_schr(Hamz);
     if fail == 0
	     Z=z2/z1;
     else
	     sysk = [];
        if strcmpi(display(1:2),'On')
           disp('Error in solving the Z Hamiltonian');
        end
	     return
     end
     F = -S\(d'*c+b'*X);
     H = -(b*d'+Z*c')/R;
     sig = eig(Z*X);
     emax = 1/sqrt(1+max(real(sig)));
     gamma = factor/emax;
     W2 = eye(n)-gamma^(-2)*(eye(n)+Z*X);
     Hi = -(b*d'+W2\(Z*c'))/R;
     sqrtS = sqrtm(S); 
     bm = (b-W2\(Z*c'*d))/S;
     sysobs = ss(a+Hi*c,[-Hi, bm, zeros(n,m)],...
              -b'*X,[-d', zeros(m,m), -sqrtS]);
     sysk = ss(a+Hi*c-bm*b'*X,[-Hi-bm*d',-bm*sqrtS],-b'*X,[-d', -sqrtS]);
end; %  if (nargin>2) & (opt=='ref') & factor>1

if  strcmp(opt,'ref') & (factor==1),
     % [sysnrcf,sig] = ncfmr(sysgw.');
     % sysnrcf = sysnrcf.';
     [junk,info]=ncfmr(sysgw,n);
     sig=info.hsv;
     sysnrcf=info.GR;
     if length(sig)>0,
         emax = sqrt(1-sig(1)^2);
         sysUV = xhankmr(sysnrcf,sig,0,'a');
     else  % constant system case
         emax=1;
         sysUV=sysnrcf;        
     end
     if strcmpi(display(1:2),'On')
        disp(['emax = ',num2str(emax)]);
     end
     sysUV = sysUV';
     sysUV = sysUV*[-eye(p) zeros(p,m);zeros(m,p) eye(m)];
     sysL  = sysUV*[-eye(p) zeros(p,m); zeros(m,p) eye(m)]*sysnrcf;
     sysUVn = sysL\sysUV;
     sysUVn = sbmudl(sysUVn);
     nUV = length(get(sysUV,'StateName'));
     nUVn = length(get(sysUVn,'StateName'));
%     sysUVn = strunc(sysUVn,min(nUV,nUVn)); % remove non-minimal states
     sysUVn = modreal(sysUVn,min(nUV,nUVn)); % remove non-minimal states
     sysUVn = sysUVn*[eye(p) zeros(p,2*m); zeros(m,m+p) eye(m)] + ...
                  [zeros(m,p) eye(m) zeros(m,m)];
     sysk = xcf2sys(sysUVn);
  end; %if (nargin>2) & (opt=='ref') & (factor==1)
%
%