% function [sysnlcf,sig,sysnrcf] = xsncfbal(sys,yype,aug)
%
%   Finds a (truncated)  balanced realization of the normalized 
%   left and right coprime factors of the system state-space 
%   model SYS. The state-space models in SYSNLCF and SYSNRCF
%   are both balanced realizations with Hankel singular values 
%   given by SIG.  
% 
%    SYSNLCF = [Nl Ml] and Nl Nl~ + Ml Ml~ = I, SYS = Ml^(-1) Nl.
%    SYSNRCF = [Nr;Mr] and Nr~ Nr + Mr~ Mr = I, SYS = Nr Mr^(-1).
%
%   If TYPE==1, then AUG denotes the order of the balanced
%   realization of SYS. If TYPE==2, AUG denotes the TOL.
%   The results are truncated to retain all Hankel singular values 
%   greater than TOL. If TYPE and AUG are omitted then TOL is set to 
%   MAX(SIG(1)*1.0E-12,1.0E-16). A warning message is printed out 
%   if SYS is close to being undetectable, in which case the output 
%   may be unreliable.
%
%   See also: NCFMR, NCFSYN.

%   Copyright 1991-2004 MUSYN Inc. and The MathWorks, Inc.
% $Revision: 1.1.6.1 $
function [sysnlcf,sig,sysnrcf,Ham]  = sncfbal(sys,Type,arg)

if nargin==3
   if Type == 1,
       ord = arg;
   end
   if Type == 2,
       tol = arg;
   end
end

if nargin<1
  disp('usage: [sysnlcf,sig,sysnrcf] = xsncfbal(sys,tol)')
  return
end
[A,B,C,D] = ssdata(sys);
if isa(sys,'ss') || isa(sys,'zpk') || isa(sys,'tf')
   sys = ss(sys);
   szs = size(sys);
   p = szs(1);
   m = szs(2);
   n = length(get(sys,'StateName'));
else
   error('Invalid class of SYS. SYS must be SS/TF/ZPK')
end

%  Solve the GFARE soln X=S'*S

Ham = [A' zeros(n); -B*B' -A] - [C'; -B*D']*inv(eye(p)+D*D')*[D*B' C];
[U,Hamm] = schur(Ham);
if any(imag([A B;C D])) 
   [U,Hamm] = ocsf(U,Hamm);
else 
   [U,Hamm] = orsf(U,Hamm,'s'); 
end
[S,SS]=schur((U(1:n,1:n)'*U(n+1:2*n,1:n)+U(n+1:2*n,1:n)'*U(1:n,1:n))/2);
S = S*sqrt(abs(SS))*S';
[qdr,Dr]=qr([D;eye(m)]);Dr=inv(Dr(1:m,1:m));
[qdl,Dl]=qr([D';eye(p)]);Dl=inv(Dl(1:p,1:p)');

% change coordinates using U(1:n,1:n)

nmuinv= norm(inv(U(1:n,1:n)));
if nmuinv>1e12, 
  disp('Warning from XSNCFBAL: SYS is close to being undetectable.');
  disp('                      Results maybe unreliable.')
  % return
end %if nmuinv>1e12,
B=U(1:n,1:n)'*B; %temp variable
Cl=Dl*(C/U(1:n,1:n)');
C=C*U(n+1:2*n,1:n); %temp variable
Al=Hamm(1:n,1:n)';
Bl=(B-C'*D)*Dr*Dr';        
Hl=-(C'+B*D') *Dl'*Dl;       

% realization of nlcf [N M] is now pck(Al,[Bl Hl],Cl,[Dl*D;Dl])
% Calculate observability Gramian for nrcf [Nl Ml], R*R'

nm11=n:-1:1;
R=sjh6(Al(nm11,nm11),Cl(:,nm11));
R=R(nm11,nm11)';

% calculate the Hankel-singular values of [Nl Ml]

[W,T,V] = svd(S*R);
sig = diag(T);

% balancing coordinates

T = W'*S;
Cl = Cl*T'; Al = Al*T'; 
T = R*V; 
Bl = T'*Bl; Al = T'*Al; Hl = T'*Hl;

% calculate the truncated dimension nn

if nargin<3, 
   Type = 2;
	tol=max([sig(1)*1.0E-12,1.0E-16]);
end  
  
nn = n;
if Type == 2
   for i=n:-1:1, 
      if sig(i)<=tol 
         nn=i-1; 
      end
   end
elseif Type == 1
    nn = ord;
    arg = nn;
end

if Type == 3
   format short e
   format compact
   [mhsv,nhsv] = size(sig);
   disp('    Hankel Singular Values:')
   hsv'
   for i = 1:mhsv
     if sig(i) == 0
        hsvp(i) = eps;
     else
        hsvp(i) = sig(i);
     end
   end
   disp('                              (strike a key to see the plot ...)')
   pause
   subplot
   plot(20*log10(hsvp),'--');hold
   plot(20*log10(hsvp),'*');grid on;hold
   ylabel('DB')
   title('Hankel Singular Values')
   pause
   nn = input('Please assign the no. of reduced order model:');
end

if nn==0, 
   sysnrcf = [Dl*D Dl];
   sig=[];
   sysnlcf = [];
else
   sig = sig(1:nn);
   % diagonal scaling  by sig(i)^(-0.5)
   irtsig = sig.^(-0.5);
   onn=1:nn;
   Al=Al(onn,onn).*(irtsig*irtsig');
   Bl=(irtsig*ones(1,m)).*Bl(onn,:);
   Cl=Cl(:,onn).*(ones(p,1)*irtsig');
   Hl=Hl(onn,:).*(irtsig*ones(1,p));
   sysnlcf = ss(Al,[Bl Hl],Cl,[Dl*D Dl]);
end %if nn==0,

%now calculate the right coprime factorization if nargout>2.
 if nargout>2,
    if nn==0, 
		  sysnrcf = [D*Dr;Dr];
		  sig = [];
  	     sysnlcf = [];
    elseif (sig(1)<0.99)&&(nmuinv<=1e12),
		 roms2 = sqrt(1-sig.^2);
		 %sysnrcf = [zeros(nn+p+m,nn+m) nn*eye(nn+p+m,1); zeros(1,nn+m), -inf];
	 	 %sysnrcf(1:nn,1:nn)=(Al).*(roms2*(roms2.^(-1))');
  	    AA = (Al).*(roms2*(roms2.^(-1))');
		 B=(Bl-Hl*D)*Dr;
		 %sysnrcf(nn+p+1:nn+m+p,1:nn)=...
    	 Cbot = -(Dr*B').*(ones(m,1)*(sig./roms2)')...
			    -(D'*Dl'*Cl).*(ones(m,1)*roms2');
	 	 %sysnrcf(nn+1:nn+p,1:nn)=...
    	 Ctop = (Dl\Cl).*(ones(p,1)*roms2')+D*Cbot;
	 	 %sysnrcf(1:nn+m+p,nn+1:nn+m)=[B./(roms2*ones(1,m));D*Dr;Dr];
	 	 BD = [B./(roms2*ones(1,m));D*Dr;Dr];
    	 sysnrcf = ss(AA,BD(1:nn,1:m),[Ctop;Cbot],BD(nn+1:nn+m+p,1:m));
    else
	 	 sysnrcf = xsncfbal(sys.');
	 	 sysnrcf = sysnrcf.';
    end %if nn==0, sysnrcf
 end, % if nargout>2
