function sys = uss(varargin)
%USS  Specify uncertain state space.  
%   USS is the constructor for uncertain state space models 
%   (USS objects).  Uncertain state space models are
%   useful for worst-case gain and robustness analysis.
%  
%   There are three ways to construct a USS object.
%   
%   1.  Use SS to create uncertain state-space objects from 
%       uncertain state-space matrix data (See UMAT).
%
%       p = ureal('p',1);
%       A = [0 3*p; -p p^2];
%       B = [0; p];
%       C = ones(2,2);  D=zeros(2,1);
%       usys = ss(A,B,C,D);
%   
%   2.  Combine an SS, TF, or ZPK object with uncertain elements.
%
%       sys = tf(1, [1 1]);
%       k = ureal('k',1);
%       usys= k*sys;
%
%   3.  Recast a double, uncertain element (ATOM object), LTI 
%       or UMAT object as a USS object.
%
%       M = tf(1, [1 1 1]);
%       M = uss(M);
%       Here, M is a 3-state SISO system (USS object) with no uncertainty.
%
%   If M is a DOUBLE, then USYS is a static gain with no uncertain elements.
%   If M is an uncertain atom, then USYS is a static gain.
%   If M is an LTI object, then USYS has no uncertainty.
%   If M is a UMAT, then USYS is a static gain.
%
%   See also: SS, METHODS(USS)

%   Copyright 2003-2005 The MathWorks, Inc.


nin = nargin;
nvp = 0;
i = 1;
while nin>i && isa(varargin{nin-i},'char')
   nvp = nvp + 1;
   i = i + 2;
end
ain = nin - 2*nvp;

if nvp==0
   if ain==1  % USS(SS/TF/ZPK/FRD)
      x = varargin{1};
      cx = class(x);
      switch cx
      case {'zpk' 'tf' 'ss'}
         tmp = ss(x);
         if hasdelay(tmp)
             warning('Omitting DELAYs in conversion to USS');
         end
         try
            [a,b,c,d] = ssdata(tmp);
         catch
             error(['USS objects cannot currently accept varying # of states in SS array'])
         end
         tmpp = struct(tmp);
         Xs.UmatData = umat(cat(1,cat(2,a,b),cat(2,c,d)));
         Xs.StateName = get(tmp,'StateName');
         Xs.Ts = getTs(tmp);
         a4 = lti2ds(tmpp.lti);
         sys = uss(Xs,a4); % direct to USS/USS, 3 field struct & DynamicSystem
      case {'frd'}   % Despite what TMW says, we need this for BINOP(FRD,USS)
         if nvp>0
            error('Use SET to change properties.');
         end
         sys = ufrd(x);
      otherwise
         error('Not applicable to LTI models of class %s',cx);
      end
   elseif ain==5 % USS(DOUBLE,DOUBLE,DOUBLE,DOUBLE,SS/TF/ZPK/FRD)
      cain = zeros(1,5);
      for i=1:4
         cain(i) = isa(varargin{i},'double');
      end
      cain(5) = isa(varargin{5},'ss') || isa(varargin{5},'tf') || ...
         isa(varargin{5},'zpk') || isa(varargin{5},'frd')
      if all(cain==1)
         sys = uss(umat(varargin{1}),umat(varargin{2}),...
            umat(varargin{3}),umat(varargin{4}),varargin{5});
      else
         badarg = find(cain==0);
         error(['Argument #' int2str(badarg(1)) ' is incorrect class']);
      end
   end
else % USS(ARG1, PVP), something in ARG1 and/or PVP is an LTI object;nothing superior 
   sys = uss(varargin{1:ain});  % y can be only USS or UFRD
   sys = set(sys,varargin{ain+1:end});
end
