%LFT  Generalized feedback interconnection of systems.
%
%   SYS = LFT(SYS1,SYS2,NU,NY) takes two systems SYS1 and SYS2 and
%   computes a model SYS of the following feedback interconnection:
%
%                        +-------+
%            w1 -------->|       |-------> z1
%                        |  SYS1 |
%                  +---->|       |-----+
%                  |     +-------+     |
%                u |                   | y
%                  |     +-------+     |
%                  +-----|       |<----+
%                        |  SYS2 |
%           z2 <---------|       |-------- w2
%                        +-------+
%
%   The feedback loop connects the first NU outputs of SYS2 to the
%   last NU inputs of SYS1 (signals u), and the last NY outputs of
%   SYS1 to the first NY inputs of SYS2 (signals y).  The resulting
%   model SYS maps the input vector [w1;w2] to the output vector
%   [z1;z2].  The resulting model SYS is also referred to as the
%   linear fractional transformation (LFT) of SYS1 and SYS2.
%
%   SYS = LFT(SYS1,SYS2) returns
%     * the lower LFT of SYS1 and SYS2 if SYS2 has fewer inputs and
%       outputs than SYS1.  This amounts to deleting w2,z2 in the
%       above diagram.
%     * the upper LFT of SYS1 and SYS2 if SYS1 has fewer inputs and
%       outputs than SYS2.  This amounts to deleting w1,z1 above.
%
%   If SYS1 and SYS2 are arrays of LTI models, LFT returns an LTI
%   array SYS of the same dimensions where
%      SYS(:,:,k) = LFT(SYS1(:,:,k),SYS2(:,:,k),NU,NY)
%
%   Note: The LFT command is applicable to a variety of objects,
%   including LTI models (SS, ZPK, TF, FRD), uncertain models
%   and matrices (USS, UFRD, UMAT), and elementary uncertainty (UREAL,
%   UCOMPLEX, ULTIDYN,...).
%
%   See also FEEDBACK, CONNECT, LTIMODELS.



function out = lft(A,B,dim2,dim1)
% Copyright 2003-2004 The MathWorks, Inc.

% STATUS BINOP
if isa(A,'ndlft') & isa(B,'ndlft')
   sza = size(A);
   szb = size(B);
   if ~isexdok(sza(3:end),szb(3:end))
      error('Invalid extra dimensions')
   end
   if nargin==2
      [dim1,dim2] = stardims(sza,szb);
   elseif nargin~=4
      error('Incorrect number of arguments');
   end
    if dim1 > min([sza(1) szb(2)])
        error('incompatible dimensions')
        return
    elseif dim2 > min([sza(2) szb(1)])
        error('incompatible dimensions')
        return
    end
   if sza(1)==0 & sza(2)==0
      out = B;
   elseif szb(1)==0 & szb(2)==0
      out = A;
   else
      out = binop(A,B,'starp',dim1,dim2);
   end
   %
   %out = simplify(out,'nodepromote','noremove');
   out = simplify(out,'nodepromote');
else
   if nargin==2
      out = lft(ndlft(A),ndlft(B));
   elseif nargin==4
      out = lft(ndlft(A),ndlft(B),dim2,dim1);
   else
      error('Incorrect number of arguments');
   end
end
 
