function m = simplify(m,varargin)
%SIMPLIFY Simplify representation of uncertain objects
%   B = SIMPLIFY(A) performs model-reduction-like techniques to
%   detect and eliminate redundant copies of uncertain elements.
%   Depending on the result, the CLASS of B may be lower than 
%   CLASS of A. The AutoSimplify property of each uncertain element
%   in A determines what reduction methods are used.  After 
%   reduction, any uncertain element which does not actually affect
%   the result is deleted from the representation.
%
%   B = SIMPLIFY(A,'full') overrides all uncertain element's
%   AutoSimplify property, and uses 'full' reduction techniques.
%
%   B = SIMPLIFY(A,'basic') overrides all uncertain element's
%   AutoSimplify property, and uses 'basic' reduction techniques.
%
%   B = SIMPLIFY(A,'class') does no reduction.  However, any uncertain
%   elements in A with 0 occurances are eliminated, and the class of
%   B may be lower than class of A.

%function m = simplify(m)
% Copyright 2003-2004 The MathWorks, Inc.

%  binary operation OP completes calculation, calls SIMPLIFY(m,'nodep','norem')
%     SIMPLIFY loops through blocks repeatedly, calling NDLFTRED
%        NDLFTRED checks LEVEL, and calls NDLFTREDV
%           NDLFTREDV calls appropriate reduction tools

% Can return DOUBLE, ATOM, NDLFT

% STATUS UNCLEAR

levelchoices = {'off';'obvious';'basic';'full';'class'};
depromotechoices = {'nodepromote'};
elimzerochoices = {'noremove'};

vin = nargin-1;

levelflg = 0;
nodepflg = 0;
noremflg = 0;
leveloverride = [];
for i=1:vin
   idx = find(strncmpi(levelchoices,varargin{i},length(varargin{i}))) - 1;
   if length(idx)==1
      if levelflg==1
         error('Only specify one (at most) simplification level');
      end
      levelflg = 1;
      if idx==5 % move 'class' -> 'off'
         idx = 1;
      end
      leveloverride = idx;
   end
   idx = find(strncmpi(depromotechoices,varargin{i},length(varargin{i})));
   if length(idx)==1
      nodepflg = 1;
   end
   idx = find(strncmpi(elimzerochoices,varargin{i},length(varargin{i})));
   if length(idx)==1
      noremflg = 1;
   end
end

previous = inf;
redtry = 0;
while isa(m,'ndlft') & length(m.Copies)>0 & sum(m.Copies)<previous
   redtry = redtry + 1;
   previous = sum(m.Copies);
   % XXX MAJOR: if m is a double and nodepflg=noremflg==1, 
   %    next line removes extra repeated copies.
   %%% m.MatrixData = removend(m.MatrixData);
   m = ndlftred(m,2,-1);
   if ~noremflg
      m = rmzerocp(m);
   end
   %%% m.MatrixData = removend(m.MatrixData);
%    if ~noremflg
%       m.MatrixData = removend(m.MatrixData);
%       m = ndlftred(m,2,-1);
%       m = rmzerocp(m);
%       m.MatrixData = removend(m.MatrixData);
%    end
   if isa(m,'ndlft') & length(m.Copies)>0 ...
      & (sum(m.Copies)<=previous | redtry==1)
      previous = sum(m.Copies);
      %%% m.MatrixData = removend(m.MatrixData);
      m = ndlftred(m,0,1,leveloverride);
      if ~noremflg
         m = rmzerocp(m);
      end
      %%% m.MatrixData = removend(m.MatrixData);
%       if ~noremflg
%          m.MatrixData = removend(m.MatrixData);
%          m = ndlftred(m,0,1,leveloverride);
%          m = rmzerocp(m);
%          m.MatrixData = removend(m.MatrixData);
%       end
   end
end
m = ndlftred(m,2,-1);

if ~nodepflg
   m = depromote(m);
end
