function out = ucomplex(a1,a2,varargin)
%UCOMPLEX   Create uncertain complex parameters.
%   A = UCOMPLEX('NAME',NOMINALVALUE) creates an uncertain complex
%   parameter, with name 'NAME', and nominal value NOMINALVALUE (complex,
%   scalar, DOUBLE).
% 
%   The uncertainty (potential deviation from NOMINALVALUE) is described
%   in two different manners:
%      RADIUS (radius of disc centered at NOMINALVALUE)
%      PERCENTAGE (disc size is percentage of magnitude of NOMINALVALUE)
%   The MODE property determines which description remains invariant
%   if the NOMINALVALUE is changed (the other is derived).
% 
%   The default MODE is RADIUS, and the default radius is 1.
% 
%   The property AUTOSIMPLIFY controls how expressions involving the real
%   parameter are simplified. Its default value is 'basic', which means
%   elementary methods of simplification are applied as operations are
%   completed.  Other values for AUTOSIMPLIFY are 'off' (no simplification
%   performed) and 'full' (model-reduction-like techniques are applied).
% 
%   Trailing Property/Value pairs are allowed, some examples are given below.
%   A = UCOMPLEX('NAME',NOMINALVALUE,'P1',V1,'P2',V2,...)
% 
%   % EXAMPLE: (CUT/PASTE)
%   % Create a UCOMPLEX with name 'a', nominal value 4+j, Radius 2.5
%   a = ucomplex('a',4+j,'Radius',2.5);
%   % Check that Percentage Uncertainty is consistent
%   [a.Radius a.Percentage/100*abs(a.NominalValue)]
% 
%   See also UREAL, GET(UCOMPLEX)

% Copyright 2003-2004 The MathWorks, Inc.

superiorto('ss','tf','zpk','frd');

if nargin>1
   IMode = 1; % 'Radius';
   IData = 1;
   if ~(isa(a1,'char') & ndims(a1)==2 & size(a1,1)==1)
      error('Invalid NAME.  NAME should be a string');
   end
   if ~(isa(a2,'double') & ndims(a2)==2 & all(size(a2)==[1 1]))
      error('Invalid NominalValue.  NominalValue should be scalar, double');
   end

   A = atom(a1,a2); % use default for AutoSimplify - no 3rd arg
   out.Data = [IData IMode];
   out = class(out,'ucomplex',A);
   nin = nargin-2;
   endqueue = [];
   for i=1:nin/2
      if strncmpi('Mode',varargin{2*(i-1)+1},length(varargin{2*(i-1)+1}))
         endqueue = [endqueue;i];
      else
         out = set(out,varargin{2*(i-1)+1},varargin{2*i});
         pv = {'Radius','Percentage'};
         idx = find(strncmpi(pv,varargin{2*(i-1)+1},length(varargin{2*(i-1)+1})));
         if length(idx)==1
            out = set(out,'Mode',pv{idx});
         end
      end
   end
   for i=1:length(endqueue)
      ii = endqueue(i);
      out = set(out,varargin{2*(ii-1)+1},varargin{2*ii});
   end
elseif nargin==0
   A = atom(LOCALgeneratename,0);
   out.Data = [1 1];
   out = class(out,'ucomplex',A);
elseif nargin==1
   error('Invalid construction syntax for UCOMPLEX objects.  At least two arguments are required.');
end

function A = LOCALgeneratename
   str = int2str(ceil(9999*rand));
   lstr = length(str);
   pdd = '0'*ones(1,6-lstr);
   A = ['UNNAMED_tag_' pdd str];
   A = 'UNNAMED';

