function out = ucomplexm(name,nominal,varargin)
%UCOMPLEXM   Create uncertain complex matrix.
%   A = UCOMPLEXM('NAME',NOMINALVALUE) creates an uncertain complex
%   matrix, with name 'NAME', and nominal value NOMINALVALUE (complex,
%   DOUBLE).
%
%   The properties WL and WR are invertible "weighting" matrices whose
%   default values are identity matrices.  The values represented
%   by the uncertain object A are all matrices B which satisfy
%   norm(inv(WL)*(B - A.NominalValue)*inv(WR)) <= 1.
% 
%   The property AUTOSIMPLIFY controls how expressions involving the
%   uncertain matrix are simplified.  Its default value is 'basic', which means
%   elementary methods of simplification are applied as operations are
%   completed.  Other values for AUTOSIMPLIFY are 'off' (no simplification
%   performed) and 'full' (model-reduction-like techniques are applied).
% 
%   Trailing Property/Value pairs are allowed, some examples are given below.
%   A = UCOMPLEXM('NAME',NOMINALVALUE,'P1',V1,'P2',V2,...)
% 
%   % EXAMPLE: (CUT/PASTE)
%   % Create a UCOMPLEXM with name 'F', nominal value [1 2 3;4 5 6], and
%   % weighting matrices WL = diag([.1 .3]), WR = diag([.4 .8 1.2])
%   F = ucomplexm('F',[1 2 3;4 5 6],'WL',diag([.1 .3]),'WR',diag([.4 .8 1.2]));
%   % Get the property list of F
%   get(F)
%   % Sample the uncertain object F 
%   sample(F)
% 
%   See also UCOMPLEX, ULTIDYN, GET(UCOMPLEXM)

% Copyright 2003-2004 The MathWorks, Inc.

superiorto('ss','tf','zpk','frd');

if nargin>1
	szn = size(nominal);
	wl = eye(szn(1));
	wr = eye(szn(2));
	if isa(nominal,'double') & ndims(nominal)==2
       A = atom(name,nominal);
       out.WL = wl;
       out.WR = wr;
       out = class(out,'ucomplexm',A);
       if length(varargin)>0
          out = set(out,varargin{:});
       end
	else
       error('Invalid UCOMPLEXM construction');
	end
elseif nargin==0
   A = atom(LOCALgeneratename,0);
   out.WL = 1;
   out.WR = 1;
   out = class(out,'ucomplexm',A);
elseif nargin==1
   error('Invalid construction syntax for UCOMPLEXM objects.  At least two arguments are required.');
end

function A = LOCALgeneratename
   str = int2str(ceil(9999*rand));
   lstr = length(str);
   pdd = '0'*ones(1,6-lstr);
   A = ['UNNAMED_tag_' pdd str];
   A = 'UNNAMED';


