function out = feedback(s1,s2,s3,s4,s5)
%  FEEDBACK  Feedback connection of two uncertain system models. 
%  
%     USYS = FEEDBACK(USYS1,USYS2) computes an uncertain system USYS for
%     the closed-loop feedback system
%  
%            u --->O---->[ USYS1 ]----+---> y
%                  |                  |           y = USYS * u
%                  +-----[ USYS2 ]<---+
%  
%     Negative feedback is assumed and the resulting system USYS 
%     maps u to y.  To apply positive feedback, use the syntax
%     USYS = FEEDBACK(USYS1,USYS2,+1).
%  
%     USYS = FEEDBACK(USYS1,USYS2,FEEDIN,FEEDOUT,SIGN) builds the more
%     general feedback interconnection:
%                        +--------+
%            v --------->|        |--------> z
%                        |  USYS1  |
%            u --->O---->|        |----+---> y
%                  |     +--------+    |
%                  |                   |
%                  +-----[  USYS2  ]<--+
%  
%     The vector FEEDIN contains indices into the input vector of USYS1
%     and specifies which inputs u are involved in the feedback loop.
%     Similarly, FEEDOUT specifies which outputs y of USYS1 are used for
%     feedback.  If SIGN=1 then positive feedback is used.  If SIGN=-1 
%     or SIGN is omitted, then negative feedback is used.  In all cases,
%     the resulting UFRD model USYS has the same inputs and outputs as USYS1 
%     (with their order preserved).
%  
%     If USYS1 and USYS2 are arrays of USS models, FEEDBACK returns an UFRD
%     array USYS of the same dimensions where 
%        USYS(:,:,k) = FEEDBACK(USYS1(:,:,k),USYS2(:,:,k)) .
%  
%     See also IC, LFT, PARALLEL, SERIES, SYSIC.

% Copyright 2003-2005 The MathWorks, Inc.

% XXXMINOR needs to be extended with checking, XXXOK, XXXHELP.
ni = nargin;
if ni==2
   s3 = -1;
elseif ni==4
   s5 = -1;
end

if ni==1
   error('Invalid number of input arguments.');
elseif ni<=3
   sz1 = size(s1);
   tmp = [eye(sz1(1));eye(sz1(1))]*s1*[eye(sz1(2)) s3*eye(sz1(2))];
   out = lft(tmp,s2);
elseif ni == 5 || ni == 4
   a = s1;
   b = s2;
   feedin = s3;
   feedout = s4;
   sza = size(a);
   % delete the names before doing the I/O duplication.  after duplication, 
   % add back the correct names to the original inputs and outputs.  
   NewAOutputName = [pvget(a,'OutputName');repmat({''},[length(feedout(:)) 1])];
   NewAInputName = [pvget(a,'InputName');repmat({''},[length(feedin(:)) 1])];
   a = pvset(a,'OutputName',repmat({''},[sza(1) 1]));
   a = pvset(a,'InputName',repmat({''},[sza(2) 1]));
   L.type = '()';
   L.subs = {[1:sza(1) feedout(:).'] [1:sza(2) feedin(:).']};
   asel = subsref(a,L);
   asel = pvset(asel,'OutputName',NewAOutputName);
   asel = pvset(asel,'InputName',NewAInputName);
   out = lft(asel,s5*b);
end