function out = lft(A,B,dim2,dim1)
%LFT  Generalized feedback interconnection of systems.
%
%   SYS = LFT(SYS1,SYS2,NU,NY) takes two systems SYS1 and SYS2 and
%   computes a model SYS of the following feedback interconnection:
%
%                        +-------+
%            w1 -------->|       |-------> z1
%                        |  SYS1 |
%                  +---->|       |-----+
%                  |     +-------+     |
%                u |                   | y
%                  |     +-------+     |
%                  +-----|       |<----+
%                        |  SYS2 |
%           z2 <---------|       |-------- w2
%                        +-------+
%
%   The feedback loop connects the first NU outputs of SYS2 to the
%   last NU inputs of SYS1 (signals u), and the last NY outputs of
%   SYS1 to the first NY inputs of SYS2 (signals y).  The resulting
%   model SYS maps the input vector [w1;w2] to the output vector
%   [z1;z2].  The resulting model SYS is also referred to as the
%   linear fractional transformation (LFT) of SYS1 and SYS2.
%
%   SYS = LFT(SYS1,SYS2) returns
%     * the lower LFT of SYS1 and SYS2 if SYS2 has fewer inputs and
%       outputs than SYS1.  This amounts to deleting w2,z2 in the
%       above diagram.
%     * the upper LFT of SYS1 and SYS2 if SYS1 has fewer inputs and
%       outputs than SYS2.  This amounts to deleting w1,z1 above.
%
%   If SYS1 and SYS2 are arrays of LTI models, LFT returns an LTI
%   array SYS of the same dimensions where
%      SYS(:,:,k) = LFT(SYS1(:,:,k),SYS2(:,:,k),NU,NY)
%
%   Note: The LFT command is applicable to a variety of objects,
%   including LTI models (SS, ZPK, TF, FRD), uncertain models
%   and matrices (USS, UFRD, UMAT), and elementary uncertainty (UREAL,
%   UCOMPLEX, ULTIDYN,...).
%
%   See also FEEDBACK, CONNECT, LTIMODELS.
% Copyright 2003-2004 The MathWorks, Inc.

if nargin==0 || nargin==1 || nargin==3
   error('Invalid syntax - need 2 (or 4) input arguments');
end

if ~isa(A,'ufrd') 
   A = lift2ufrd(A,B.Frequency,B.Units);
elseif ~isa(B,'ufrd')
   B = lift2ufrd(B,A.Frequency,A.Units);
else   
   % Check frequency vectors
   try
      % give priority to rad/s if both FRD's
      [freq,units] = freqcheck(A.Frequency,A.Units,B.Frequency,B.Units);
   catch einfo
      throw(einfo);
   end
end

sza = size(A);
szb = size(B);
if nargin==2
   [dim1,dim2] = stardims(sza,szb);
end

% XXXMINOR in CTB, there can be systems with no inputs and/or no outputs.
%  is that allowed in our setting?
if dim1 > min([sza(1) szb(2)])
   error('NY exceeds number of inputs of SYS2 or number of outputs of SYS1.')
elseif dim2 > min([sza(2) szb(1)])
   error('NU exceeds number of outputs of SYS2 or number of inputs of SYS1.')
end
if sza(1)==0 & sza(2)==0
   out = B;
elseif szb(1)==0 & szb(2)==0
   out = A;
else
   try
      out = binop(A,B,'lft',dim2,dim1);
   catch einfo
      throw(einfo);
   end
end