%PARENSET Low level set command for UFRD objects.

% Copyright 2003-2004 The MathWorks, Inc.
function B = parenset(M,L,RHS)

if isempty(RHS)
   error('Empty assignment not supported for UFRD objects.');
end
% Initialize M if it is empty, using L and RHS
if isempty(M)
   % RHS must be UFRD
   M = LOCALEmptyInit(L,RHS);
end
   
if ~isa(M,'ufrd') 
   M = lift2ufrd(M,RHS.Frequency,RHS.Units);
elseif ~isa(RHS,'ufrd')
   RHS = lift2ufrd(RHS,M.Frequency,M.Units);
else   
   % Check frequency vectors
   try
      % give priority to rad/s if both FRD's
      [freq,units] = freqcheck(M.Frequency,M.Units,RHS.Frequency,RHS.Units);
   catch einfo
      throw(einfo)
   end
end

% Convert Logicals to Doubles
for i=1:length(L)
   if isa(L{i},'logical');
      L{i} = find(L{i});
   end
end

szM = size(M);
lenL = length(L);
szRHS = size(RHS);

% Some logical variables
scalarR = prod(szRHS)==1;
IORow = szM(1)==1;
IOColumn = szM(2)==1;
ADRow = length(szM)>=3 && szM(3)==1;
ADColumn = length(szM)>=4 && szM(4)==1;
AssignOnAllAD = lenL<=2;
IOSidx = lenL==1;
% Below can be 0, 1, etc.  Note that 1 is unconventional
NonTrivialAD = length(szM) - 2 - (length(szM)==4 && szM(4)==1);

% Allow for single indexing in vectors (regardless of array dimensions)
if IOSidx   % Single Index
   if IORow
      L = [{1} L];
   elseif IOColumn
      L = [L {1}];
   else
      error('A(I)=B is not allowed since A is not a column or row'); 
   end
end
lenL = length(L);

%
if AssignOnAllAD
   L = [L repmat({':'},[1 NonTrivialAD])];
end
lenL = length(L);

% Convert Group References to numbers
saveL = cell(2,1);
for i=1:2
   if isa(L{i},'double') && max(L{i})>szM(i)
      saveL{i} = L{i};
      L{i} = 1:szM(i);
   end
end
L12 = utRefCheck(M.dynamicsys,L(1:2),szM(1:2));
L(1:2) = L12;
for i=1:2
   if ~isempty(saveL{i})
      L{i} = saveL{i};
   end
end

% Get L filled with numbers
if lenL>length(szM)
   szMforL = [szM ones(1,lenL-length(szM))];
elseif lenL<length(szM)
   szMforL = [szM(1:lenL-1) prod(szM(lenL:end))];  % could be length3
else
   szMforL = szM;
end
if lenL>length(szRHS)
   szRforL = [szRHS ones(1,lenL-length(szRHS))];
elseif lenL<length(szM)
   szRforL = [szRHS(1:lenL-1) prod(szRHS(lenL:end))];  % could be length3
else
   szRforL = szRHS;
end
for i=1:lenL
   if isequal(L{i},':')
      if szMforL(i)==1 || szRforL(i)==1 || szMforL(i)==szRforL(i)
         L{i} = 1:max([szRforL(i) szMforL(i)]);
      else
         error('Subscripted assignment dimension mismatch.');
      end
   end
end

% Expand Scalar RHS, using L
if scalarR
   eachidxnum = zeros(1,lenL);
   for i=1:lenL
      eachidxnum(i) = length(unique(L{i}));
   end
   RHS = repsys(RHS,eachidxnum);
   szRHS = size(RHS);  % should be eachidxnum
end

% if length(L)<length(szM), then M needs to be temporarily reshaped.
lenL = length(L);
reshapeflag = 0;
if lenL<length(szM)
   % it must be that lenL>=3, and length(szM)>=4
   newszM = [szM(1:lenL-1) prod(szM(lenL:end))]; % at least length=3
   if length(newszM)==2 % never happens
      newszM = [newszM 1 1]; % pad to 4, if necessary
   elseif length(newszM)==3
      newszM = [newszM 1]; % pad to 4, if necessary
   end
   % newszM is at least 1x4
   M = reshape(M,newszM(3:end)); % **NOTE**: RESHAPE only acts on ArrayDims
   reshapeflag = 1;
   origszM = szM;
   szM = newszM;
else
   szM = [szM ones(1,lenL-length(szM))]; %% why??
end

% Grow M if necessary.
% If M had to be reshaped, then it is illegal to have
% the last index in L be bigger than the last reshaped
% index in M.
for i=1:lenL
   if max(L{i})>szM(i)
      if reshapeflag==1 && i==lenL
         msg = 'Attempt to grow array along ambiguous dimension.';
         error(msg);
      else
         M = cat(i,M,umat(zeros([szM(1:i-1) max(L{i})-szM(i) szM(i+1:end)])));
         szM(i) = max(L{i});
         origszM(i) = szM(i);
      end
   end
end

% Fix overspecified entries of L, create W.
W = cell(1,lenL);
for i=1:length(L)
   [L{i},W{i}] = LOCALcorrectidx(L{i});
end
% Reselect relevant entries from RHS
RL.type = '()';
RL.subs = W;
RHS = subsref(RHS,RL);

replaceidx = L;
noreplaceidx = cell(1,lenL);
resortidx = cell(1,lenL);
for i=1:lenL
   if max(replaceidx{i})<=szM(i) && all(replaceidx{i}>0) && ...
         all(ceil(replaceidx{i})==floor(replaceidx{i}))
      noreplaceidx{i} = 1:szM(i);
      noreplaceidx{i}(replaceidx{i}) = [];
      [dum,resortidx{i}] = sort([replaceidx{i} noreplaceidx{i}]);
   else
      error('Subscripted assignment dimension mismatch.');
   end
end

% Do all concatenating, cheat in a try/catch
try
   useL = replaceidx;
   B = RHS;
   for i=1:lenL
      for j=1:i-1
         useL{j} = [replaceidx{j} noreplaceidx{j}];
      end
      useL{i} = noreplaceidx{i};
      B = cat(i,B,parenget(M,useL));
   end

   % Re-Sort every dimension's "rows"
   B = parenget(B,resortidx);

   if reshapeflag==1
      B = reshape(B,origszM(3:end));
   end
catch
   error('Subscripted assignment dimension mismatch.');
end



function [vals,where] = LOCALcorrectidx(I)
J = I(:).';
if isempty(I)
   vals = zeros(1,0);
   where = zeros(1,0);
else
   where = 1;
   vals = J(1);
   for k=2:length(J)
      idx = find(J(k)==vals);
      if ~isempty(idx)
         where(idx) = k;
      else
         vals = [vals J(k)];
         where = [where k];
      end
   end
end


function MINIT = LOCALEmptyInit(L,RHS)
if length(L)>=ndims(RHS)
   for i=1:length(L)
      if isequal(L{i},':')
         szM(i) = size(RHS,i);
      elseif isa(L{i},'double') && ndims(L{i})==2 && min(size(L{i}))==1 ...
            && all(ceil(L{i})==floor(L{i})) && all(L{i}>0)
         maxLi = max(L{i});
         if maxLi>=size(RHS,i)
            szM(i) = maxLi;
         else
            error('Inconsistent index and rhs argument.');
         end
      else
         error('Inconsistent index and rhs argument.');
      end
   end
else
   error('Inconsistent index and rhs argument.');
end
RD = umat(zeros([szM(1:2) length(RHS.Frequency) szM(3:end)]));
MINIT = ufrd(RD,RHS.Frequency,'Units',RHS.Units,RHS.Ts);



% if length(L)==2
%    szm = size(m);
%    replacerows = L{1};
%    replacecols = L{2};
%    keeprows = 1:szm(1);
%    keeprows(replacerows) = [];
%    [tmp,rowrearrange] = sort([replacerows keeprows]);
%    keepcols = 1:szm(2);
%    keepcols(replacecols) = [];
%    [tmp,colrearrange] = sort([replacecols keepcols]);
%    m2dot = parenget(m,{keeprows [replacecols keepcols]});
%    m12 = parenget(m,{replacerows keepcols});
%    % Get New Dynamic System
%    tmps.type = '()';
%    tmps.subs = L;
%    newds = subsasgn(m.dynamicsys,tmps,RHS.dynamicsys);
%    sztmp = size(RHS);
%    RHS = pvset(RHS,'dynamicsys',dynamicsys(sztmp(1),sztmp(2)));
%    sztmp = size(m12);
%    m12 = pvset(m12,'dynamicsys',dynamicsys(sztmp(1),sztmp(2)));
%    sztmp = size(m2dot);
%    m2dot = pvset(m2dot,'dynamicsys',dynamicsys(sztmp(1),sztmp(2)));
%    newm = [RHS m12;m2dot];
%    tmps.type = '()';
%    tmps.subs = L;
%    newds = subsasgn(m.dynamicsys,tmps,RHS.dynamicsys);
%    newm = pvset(newm,'dynamicsys',newds);
%    B = parenget(newm,{rowrearrange colrearrange});
% else
%    error('Only input/output assignment is supported for UFRDs.');
% end
