function varargout = mkmimopert(lvec,rvec,freq,TS,N)
% MKMIMOPERT
%   SYS = MKMIMOPERT(L,R,OMEGA,TS,Normalizer) creates
%   a state-space system SYSN with the following properties
%
%     SYSN(OMEGA) = L*R
%     NORM(SYSN,'inf') = NORM(L)*NORM(R)
%     SYSN(0) = 0
%     SYSN(HIGHFREQUENCY) = 0
%
%   L must be a complex-valued column vector, R a complex-valued
%   row vector, and OMEGA a positive number.  The state dimension
%   of SYSN is length(L) + length(R) + 1;
%   SYS is the unnormalized version (using N.S).
%
%   [A,B,C,D,TS] = MKMIMOPERT(L,R,OMEGA,TS,Normalizer) also works.
%

% Copyright 2004 The MathWorks, Inc.

lvec = lvec(:);  % column
rvec = rvec(:).';% row
nl = length(lvec);
nr = length(rvec);
if max(abs(lvec))==0
   lvec = randn(nl,1) + sqrt(-1)*randn(nl,1);
   LFac = 0;
else
   LFac = norm(lvec);
   lvec = lvec/LFac;
end
if max(abs(rvec))==0
   rvec = randn(1,nr) + sqrt(-1)*randn(1,nr);
   RFac = 0;
else
   RFac = norm(rvec);
   rvec = rvec/RFac;
end
      
anglevec = [angle(lvec);-angle(rvec.')];
ak = LOCALmaxasep(anglevec);
while isempty(ak)
   % Make small (relative) perturbations to L and R
   lvec = (1 + 1e-6*(randn(nl,1) + sqrt(-1)*randn(nl,1))).*lvec;
   rvec = (1 + 1e-6*(randn(1,nr) + sqrt(-1)*randn(1,nr))).*rvec;
   anglevec = [angle(lvec);-angle(rvec.')];
   ak = LOCALmaxasep(anglevec);
end

% Code above already ensures that AK will not be empty.  Code
% below is "old", and handles the general case.  If ISEMPTY(AK),
% then there will be problems with the state-dimensions of perturbations. 
if ~isempty(ak) && ak<=nl
   lvec = exp(-sqrt(-1)*anglevec(ak))*lvec;
   rvec = exp(sqrt(-1)*anglevec(ak))*rvec;
   luse = [lvec(ak+1:end,1);lvec(1:ak-1,1)];  % want to think of this as 0x1
%    %L = [real(lvec(ak));vecorat(freq,luse,TS)];
%    L = vecorat(freq,luse,TS,real(lvec(ak)),1); % force COLUMN
%    Lsys = L([nl-ak+2:nl 1:nl-ak+1]);
   [LA,LB,LC,LD] = vecorat(freq,luse,TS,real(lvec(ak)),1); % force COLUMN
   rowidx = [nl-ak+2:nl 1:nl-ak+1];
   LC = LC(rowidx,:);
   LD = LD(rowidx,:);
   [RA,RB,RC,RD] = vecorat(freq,rvec,TS);
elseif ~isempty(ak) && ak>nl
   lvec = exp(-sqrt(-1)*anglevec(ak))*lvec;
   rvec = exp(sqrt(-1)*anglevec(ak))*rvec;
   ak = ak - nl;
   ruse = [rvec(1,ak+1:end) rvec(1,1:ak-1)];
   %R = [real(rvec(ak)) vecorat(freq,ruse,TS)];
   [RA,RB,RC,RD] = vecorat(freq,ruse,TS,real(rvec(ak)),2); % force ROW
   colidx = [nr-ak+2:nr 1:nr-ak+1];
   RB = RB(:,colidx);
   RD = RD(:,colidx);
   [LA,LB,LC,LD] = vecorat(freq,lvec,TS);
else
   [LA,LB,LC,LD] = vecorat(freq,lvec,TS);
   [RA,RB,RC,RD] = vecorat(freq,rvec,TS);
end
LB = LB*sqrt(LFac);
LC = LC*sqrt(LFac);
LD = LD*LFac;
RB = RB*sqrt(RFac);
RC = RC*sqrt(RFac);
RD = RD*RFac;

[SA,SB,SC,SD] = utfit1omega(freq,TS);
[yflag,betascalar,idflag] = LOCALislftscalar(N.S,nl,nr);
if idflag
   [A,B,C,D] = LOCALtriplemult(LA,LB,LC,LD,...
      SA,SB,SC,SD,...
      RA,RB,RC,RD);
elseif yflag
   if betascalar>=0
      f1 = sqrt(betascalar);
      f2 = f1;
   else
      f1 = sqrt(-betascalar);
      f2 = -f1;
   end
   [A,B,C,D] = LOCALtriplemult(LA,LB,LC,LD,...
      SA,f1*SB,f2*SC,betascalar*SD,...
      RA,RB,RC,RD);
else
   [A,B,C,D] = LOCALtriplemult(LA,LB,LC,LD,...
      SA,SB,SC,SD,...
      RA,RB,RC,RD);
   [A,B,C,D] = LOCALlftsysdouble(A,B,C,D,N.S);
end
if nargout==1
   varargout = cell(1,1);
   varargout{1} = ss(A,B,C,D,TS);
else
   varargout = cell(1,5);
   varargout{1} = A;
   varargout{2} = B;
   varargout{3} = C;
   varargout{4} = D;
   varargout{5} = TS;
end

function [AA,BB,CC,DD] = LOCALlftsysdouble(A,B,C,D,S)
nx = size(A,1);
nu = size(B,2);
ny = size(C,1);
T11 = A;
T12 = B;
T21 = C;
T22 = D;
B11 = S(1:nu,1:ny);
B12 = S(1:nu,ny+(1:nu));
B21 = S(nu+(1:ny),1:ny);
B22 = S(nu+(1:ny),ny+(1:nu));
imtb = eye(ny) - T22*B11;
imbt = eye(nu) - B11*T22;
XX = imtb\T21;
YY = imbt\B12;
AA = T11 + T12*B11*XX;
BB = T12*YY;
CC = B21*XX;
DD = B22+B21*T22*YY;


function [A,B,C,D] = LOCALtriplemult(A1,B1,C1,D1,...
   A2,B2,C2,D2,...
   A3,B3,C3,D3)
n1 = size(A1,1);
n2 = size(A2,1);
n3 = size(A3,1);
B2C3 = B2*C3;
B2D3 = B2*D3;
D2C3 = D2*C3;
D2D3 = D2*D3;
B1C2 = B1*C2;
D1C2 = D1*C2;
B1D2C3 = B1*D2C3;
B1D2D3 = B1*D2D3;
D1D2C3 = D1*D2C3;
D1D2D3 = D1*D2D3;
nx = n1+n2+n3;
A = zeros(nx,nx);
x1s = 1;
x1e = n1;
x2s = x1e+1;
x2e = x1e + n2;
x3s = x2e + 1;
x3e = x2e + n3;
x1 = x1s:x1e;
x2 = x2s:x2e;
x3 = x3s:x3e;
A(x1,x1) = A1;
A(x2,x2) = A2;
A(x3,x3) = A3;
A(x1,x2) = B1C2;
A(x1,x3) = B1D2C3;
A(x2,x3) = B2C3;
B = [B1D2D3;B2D3;B3];
C = [C1 D1C2 D1D2C3];
D = D1D2D3;

function [A,B,C] = LOCALislftscalar(mat,n,m)
dim = n+m;
dimn = dim*n;
inc = dim + 1;
cand = mat(m+1,1);

checkvec = [(m+1):inc:dimn (dimn+1):inc:(dim*(dim-1)+m)];
if all(mat(checkvec)==cand)
   mat(checkvec) = [];
   if all(mat==0)
      A = 1;
      B = cand*cand;
      C = cand==1;
   else
      A = 0;
      B = 0;
      C = 0;
   end
else
   A = 0;
   B = 0;
   C = 0;
end
      
function  [loc] = LOCALmaxasep(v)
v = v(:);
lv = length(v);
dmat = abs(mod(repmat(v,[1 lv]) - repmat(v.',[lv 1]),2*pi));
dmat = dmat + 8*eye(lv);
md = min(dmat,[],2);
[mx,idx] = max(md);
if mx>0
   loc = idx(1);
else
   loc = [];
end