function [perfmarg,BadAtoms,Report,info] = robustperf(sys,opt,SETUP)
%ROBUSTPERF   Performance margin of uncertain systems.
%
%[PERFMARG,PERFMARGUNC,REPORT,INFO] = ROBUSTPERF(SYS) calculates the
%robust performance margin of the uncertain system SYS.
%
%The performance (as measured by input/output gain) of a nominally-stable
%uncertain system will generally degrade (increased gain) for specific
%values of its uncertain elements.  ROBUSTPERF computes the Robust 
%Performance Margin, which is one measure of the level of degradation
%brought on by the modeled uncertainty.  A performance robustness margin
%greater than 1 means that the input/output gain (H-infinity norm) of the
%uncertain system remains less than 1 for all values of the the uncertain
%system's modeled uncertainty.  A performance robustness margin less than 1
%implies that certain values of the uncertain elements, within their
%specified ranges, lead to an input/output gain greater than 1.
%
%Numerically, a margin of 0.4 (for example) implies two things: for all
%values of uncertain elements which are less than 0.4 normalized units
%away from their nominal values, the input/output gain remains less than
%2.5 (i.e., 1/0.4);  and, there is a collection of uncertain elements
%which are less than or equal to 0.4 normalized units away from their
%nominal values that results in an input/output gain of 2.5.  See
%ACTUAL2NORMALIZED for converting between actual and normalized deviations
%from the nominal value of an uncertain element.
%
%The output arguments are:
%  PERFMARG  Structure with the following fields:
%           UpperBound: Upper bound on performance margin.
%           LowerBound: Lower bound on performance margin. 
%    CriticalFrequency: Frequency at which the minimum robust performance
%                       margin occurs, associated with PERFMARG.UpperBound.
%  PERFMARGUNC  Structure of values of uncertain elements associated with
%           PERFMARG.UpperBound. The fieldnames are the names of uncertain 
%           elements of SYS.  
%  REPORT  Char array with text description of robustness analysis results.
%  INFO  Structure with the following fields:
%           Sensitivity: Structure with local sensitivities of
%                        PERFMARG.UpperBound to each uncertain elements' range.
%             Frequency: Frequency vector associated with analysis.
%    BadUncertainValues: Cell array (same length as Frequency vector).
%                        Each entry is a structure, similar to PERFMARGUNC,
%                        but associated with the robust performance analysis
%                        at that specific frequency.
%             MussvBnds: Upper and lower bounds from MUSSV.
%             MussvInfo: Structure of compressed data from MUSSV. 
% 
%[PERFMARG,PERFMARGUNC,REPORT,INFO] = ROBUSTPERF(SYS,OPTS) specifies 
%options described in OPTS.  (See ROBOPT for more details on the options.) 
%
%See also: LOOPMARGIN, MUSSV, ROBOPT, ROBUSTSTAB, WCGAIN, WCSENS, WCMARGIN.

% Copyright 2004-2005 The MathWorks, Inc.

if nargin==0
   disp('usage: [PERFMARG,PERFMARGUNC,REPORT,INFO] = ROBUSTPERF(SYS,OPTS)');
   return
end
if nargin==1
    opt = robopt;
    SETUP = [];
 elseif nargin==2
    SETUP = [];
end
if ~isa(opt,'robopt')
   error('The OPTS input argument must a ROBOPT object.');
end
if nargout==0
   outputcell = cell(1,1);
else
   outputcell = cell(nargout,1);
end
szsys = size(sys);
exd = szsys(3:end);

% Add performance block and call robuststab
rp = ucomplexm('rcastreserved_rp',zeros(szsys(2),szsys(1))); % XXX MINOR: Convention for naming?
sys2 = feedback(sys,rp,+1);
%[stabmarg,BadAtoms,Report,infostab] = robuststab(sys2,opt,SETUP);
[outputcell{:}] = robuststab(sys2,opt,SETUP);
if nargout<=1
   stabmarg = outputcell{1};
elseif nargout==2
   stabmarg = outputcell{1};
   BadAtoms = outputcell{2};
elseif nargout==3
   stabmarg = outputcell{1};
   BadAtoms = outputcell{2};
   ReportRS   = outputcell{3};
elseif nargout==4
   stabmarg = outputcell{1};
   BadAtoms = outputcell{2};
   ReportRS   = outputcell{3};
   infostab = outputcell{4};
end

perfmarg.UpperBound = stabmarg.UpperBound;
perfmarg.LowerBound = stabmarg.LowerBound;
perfmarg.CriticalFrequency = stabmarg.DestabilizingFrequency;

if nargout>=2
   BadAtoms = rmfield(BadAtoms,'rcastreserved_rp');
end

if nargout==4 % User wanted INFORP
   info.Sensitivity = rmfield(infostab.Sensitivity,'rcastreserved_rp');
   info.Frequency = infostab.Frequency;
   % Deleted as per conversation with Pascal    
   %info.wcperfmarg = infostab.wcstabmarg;
   %info.wcunc = rmfield(infostab.wcunc,'rcastreserved_rp');
   % MEANING WAS
   %      wcperfmarg: Worst (largest) value in PERFMARG.UpperBound.  Relevant when
   %                  SYS has array dimensions, otherwise equals PERFMARG.UpperBound.
   %           wcunc: Chosen from PERFMARGUNC, associated with the largest
   %                  robust performance.  Relevant when SYS has array
   %                  dimensions, otherwise equals PERFMARGUNC.
   info.BadUncertainValues = infostab.BadUncertainValues;
   info.MussvBnds = infostab.MussvBnds;
   info.MussvInfo = infostab.MussvInfo;

   nf = length(info.Frequency);
   for i1 = 1:nf
       info.BadUncertainValues{i1} = rmfield(info.BadUncertainValues{i1},'rcastreserved_rp');
   end
end

if nargout >=3 %User wants the REPORT
   Report = cell([exd 1]);
   if strcmpi(opt.Sensitivity,'On')
      for i1 = 1:prod(exd)
         sensreport = ReportRS(:,:,i1);
          Report{i1}=LOCALmkmessage(perfmarg.UpperBound(i1),perfmarg.LowerBound(i1),...
              perfmarg.CriticalFrequency(i1),get(sys,'Units'),sensreport(5:end,:));
      end
   else
      for i1 = 1:prod(exd)
          Report{i1}=LOCALmkmessage(perfmarg.UpperBound(i1),perfmarg.LowerBound(i1),...
              perfmarg.CriticalFrequency(i1),get(sys,'Units'),[]);
      end
   end
   Report = reportconvert(Report);
end

%--------- Local subroutine -------------------------------------%
function text = LOCALmkmessage(pku,pkl,freq,units,sensreport);

%% Redesigned Message Output
if isequal(pku,-inf)
   text = ['Performance Margin is -inf, since NominalValue is unstable.'];
else
   text = ['Uncertain System achieves a robust performance margin of ' ...
              num2str(pku,'%0.4g') '.']; 
   if isinf(pkl)
      text2 = [' -- No modeled uncertainty exists to cause the '...
                    'performance to degrade.'];
      text3 = '';
   else
      text2 = [' -- A model uncertainty exists of size '...
              num2str(100*pkl,'%0.3g') '% resulting in a performance margin'];
      text3 = ['    of ' num2str(1/pkl,'%0.3g') ' at ' ...
              num2str(freq,'%0.3g') ' ' units '.'];
   end
   if isempty(sensreport)
      text = strvcat(text,text2,text3);
   else
      text = strvcat(text,text2,text3,sensreport);
   end
end