function [stabmarg,BadAtoms,Report,info] = robuststab(sys,opt,SETUP)
%ROBUSTSTAB   Stability margins of nominally stable uncertain systems.
%
%[STABMARG,DESTABUNC,REPORT,INFO] = ROBUSTSTAB(SYS) calculates robust 
%stability margins of the uncertain multivariable system SYS.  A stability
%robustness margin greater than 1 means that the uncertain system is stable
%for all values of its modeled uncertainty.  A stability robustness margin
%less than 1 implies that certain allowable values of the uncertain
%elements, within their specified ranges, lead to instability.
%
%Numerically, a margin of 0.5 (for example) implies two things: the
%uncertain system remains stable for all values of uncertain elements
%which are less than 0.5 normalized units away from their nominal values;
%and, there is a collection of uncertain elements which are less than or
%equal to 0.5 normalized units away from their nominal values that results
%in instability.  See ACTUAL2NORMALIZED for converting between actual and
%normalized deviations from the nominal value of an uncertain element.
%
%  STABMARG  Structure with the following fields:
%      UpperBound: Upper bound on stability margin.
%      LowerBound: Lower bound on stability margin. 
%      DestabilizingFrequency: Frequency at which instability occurs, 
%             associated with STABMARG.UpperBound.
%  DESTABUNC  Structure of values of uncertain elements which cause 
%             instability. The fieldnames are the names of uncertain 
%             elements of SYS.
%  REPORT  Char array with text description of robustness analysis results.
%  INFO  Structure with the following fields:
%      Sensitivity: Structure with local sensitivities of STABMARG.UpperBound
%                   to each uncertain elements' range.
%      Frequency: Frequency vector associated with analysis.
%      BadUncertainValues: Cell array (same length as Frequency vector) with
%                   uncertain elements which cause pole migration across
%                   the stability boundary at the corresponding frequency.
%      MussvBnds: Upper and lower bounds from MUSSV.
%      MussvInfo: Structure of compressed data from MUSSV. 
%
%[STABMARG,DESTABUNC,REPORT,INFO] = ROBUSTSTAB(SYS,OPTS) specifies 
%options described in OPTS.  (See ROBOPT for more details on the options.) 
%
%If SYS is an UFRD, then the analysis is performed on the frequency grid 
%within SYS. Note that the stability of the nominal system (SYS.NominalValue)
%is not verified by the computation as it is an FRD object.  If SYS is 
%an USS, then the stability of SYS.NominalValue is first checked, an 
%appropriate frequency grid is generated (automatically), and the analysis 
%performed on that frequency grid.  
%
%See also: ACTUAL2NORMALIZED, LOOPMARGIN, MUSSV, ROBOPT, ROBUSTPERF, WCGAIN, 
% WCSENS, WCMARGIN.

% Copyright 2004-2006 The MathWorks, Inc.

if nargin==0
   disp('usage: [STABMARG,DESTABUNC,REPORT,INFO] = ROBUSTSTAB(SYS,OPT)');
   return
end

if nargin==1
    opt = robopt;
    SETUP = [];
 elseif nargin==2
    SETUP = [];
end
if strcmpi(opt.Display,'Off')
   opt.Mussv = [opt.Mussv 's']; % Suppress progress information
end
if ~isa(opt,'robopt')
   error('The OPTS input argument must a ROBOPT object.');
end
if nargout<3
    set(opt,'Sensitivity','Off');
end

% Determine if USS/ROBUSTSTAB was calling function
ussflag = 0;
if isequal(SETUP,'uss')
    SETUP = [];
   ussflag = 1;
end


newsys = rmzerocp(sys);
nblkorig = length(pvget(sys,'AtomCell'));
info = struct('Sensitivity',[],...
   'Frequency',[],...
   'BadUncertainValues',[],...
   'MussvBnds',[],...
   'MussvInfo',[]);
M = lftdata(newsys); % M is an FRD
szsys = size(newsys);
exd = szsys(3:end);
npts = length(newsys.Frequency);
names = pvget(newsys,'UNames');

M = M(1:end-szsys(1),1:end-szsys(2));
try
   % it will fail if there are any repeated, non-scalar blocks.
   [oldmublk] = getmutoolsblkstruct(newsys);
catch
   rethrow(lasterror);
end
tmp23 = pvget(newsys,'N23BlkStructure');
mublk = tmp23.N3;
bidx = blk2idx(mublk);
[rp,cp,nblk,iosize,blkiosize,szmd] = ndlftinfo(pvget(newsys.ResponseData,'NDlftData'));

AtomCell = pvget(newsys,'AtomCell');

if nblk==0
   % need to be careful here, should probably do something about 0-copy
   % blocks
   warning('No uncertainty to analyze');
   return
end

% if M has EXD, bnds will have EXD
[bnds,tinfo] = mussv(M,oldmublk,opt.Mussv,[],[],SETUP); % bnds and info data are FRDs
info.MussvInfo = tinfo;
info.MussvBnds = bnds;

stabmarg.Max = zeros([1 1 exd]);
stabmarg.Min = zeros([1 1 exd]);
stabmarg.Max = repmat(-inf,[1 1 exd]);
stabmarg.Min = repmat(-inf,[1 1 exd]);
stabfreq.Max  = zeros([1 1 exd]);
stabfreq.Min  = zeros([1 1 exd]);
DestabNamesValues = cell(nblkorig,2);
DestabNamesValues(1:nblk,1) = names;
analysistext = cell([1 1 exd]);
if strcmpi(opt.Sensitivity,'On')
   sensitivity = zeros([nblkorig 1 exd]);
else
   sensitivity = zeros([nblkorig 0 exd]);
end
destabpert = cell([nblkorig 1]);
Sdestabpert = cell([nblkorig 1]);
gsrangedata = zeros([nblkorig 2 exd]);

for k = 1:nblk
   switch class(AtomCell{k})
   case {'ureal','ucomplex'}
      nomval = pvget(AtomCell{k},'NominalValue');
      destabpert{k} = repmat(nomval,[1 1 npts exd]);
      DestabNamesValues{k,2} = repmat(nomval,[1 1 exd]); % realp, complexp
   case 'ucomplexm'
      szdel = size(AtomCell{k});
      nomval = pvget(AtomCell{k},'NominalValue');
      destabpert{k} = repmat(nomval,[1 1 npts exd]);
      DestabNamesValues{k,2} = repmat(nomval,[1 1 exd]); % complexm      
   case 'ultidyn'
      szdel = size(AtomCell{k});
      nx = szdel(1) + szdel(2) + 1;  % see MKMIMOPERT to understand state dimension
      ny = szdel(1);
      nu = szdel(2);
      Sdestabpert{k} = struct('A',{zeros([nx nx npts exd])},...
        'B',{zeros([nx szdel(2) npts exd])},...
        'C',{zeros([szdel(1) nx npts exd])},...
        'D',{zeros([szdel(1) szdel(2) npts exd])});
       initialValue = pvget(AtomCell{k},'SSNominalValue',sys.Ts);
       destabpert{k} = depromote(repmat(uss(initialValue),[1 1 npts exd]));
       DestabNamesValues{k,2} = depromote(repmat(uss(initialValue),[1 1 exd])); % ltidynamics
   otherwise
      error('ROBUST STABLIITY analysis allows only for uncertain atoms of the type: ureal, ultidyn, ucomplex or ucomplexm.');
   end
end

%-- loop on extra dimensions
if isempty(SETUP)
   LBINDXval = zeros(prod(exd),1);
   for i=1:prod(exd);
      [pkl,pklomi] = max(bnds(1,2,i).ResponseData(:));
      [pku,pkuomi] = max(bnds(1,1,i).ResponseData(:));
      if pkl==0
         stabmarg.Max(1,1,i) = inf;
      else
         stabmarg.Max(1,1,i) = 1/pkl;
      end
      if pku==0
         stabmarg.Min(1,1,i) = inf;
      else
         stabmarg.Min(1,1,i) = 1/pku;
      end
      lbindx = pklomi;
      LBINDXval(i) = pklomi;
      stabfreq.Max(1,1,i) = bnds.Frequency(pklomi);
      stabfreq.Min(1,1,i) = bnds.Frequency(pkuomi);
      % Analysis Text
   %    analysistext{1,1,i} = LOCALmkmessage(pku,pkl,bnds.Frequency(lbindx(1)),...
   %       get(M,'Units'),opt,sensitivity,AtomCell);
      prt = mussvunwrap(info.MussvInfo.pvec(1,:,i),info.MussvInfo.blk); % prt is an FRD
      %  the following is a matrix of data, not a cell. therefore this must be
      %  reshaped into a cell array of perts. Also these perts are in the 
      %  literal = 0 mode and therefore need to be translated back into the
      %  original coordinates

      for j = 1:npts
         prtm = prt.ResponseData(:,:,j); % 2D double
         for k = 1:nblk
            % if(abs(bnds(1,2,i).ResponseData(j)-1/.42562831207)<.1)
            %    [1/bnds(1,2,i).ResponseData(j) norm(prtm) j i]
            % end
            switch class(AtomCell{k})
            case {'ureal','ucomplex'}
               nprt = prtm(cp(k),rp(k));
               [N] = pvget(AtomCell{k},'Normalizer');
               destabpert{k}(:,:,j,i) = N.S(2,2) + N.S(2,1)*nprt/(1-N.S(1,1)*nprt)*N.S(1,2);
            case 'ucomplexm'
               szatm = size(AtomCell{k});
               nprt = prtm(cp(k):cp(k)+szatm(1)-1,rp(k):rp(k)+szatm(2)-1);
               [N] = pvget(AtomCell{k},'Normalizer');
               destabpert{k}(:,:,j,i) = lft(nprt,N.S);
            case 'ultidyn'
               szatm = size(AtomCell{k});
               nprt = prtm(cp(k):cp(k)+szatm(1)-1,rp(k):rp(k)+szatm(2)-1);
               [N] = pvget(AtomCell{k},'Normalizer');
               [u,s,v] = svd(nprt);
               wlvec = u(:,1)*sqrt(s(1,1));
               wrvec = sqrt(s(1,1))*v(:,1)';
               [Ap,Bp,Cp,Dp] = mkmimopert(wlvec,wrvec,newsys.Frequency(j),newsys.Ts,N);
               Sdestabpert{k}.A(:,:,j,i) = Ap;
               Sdestabpert{k}.B(:,:,j,i) = Bp;
               Sdestabpert{k}.C(:,:,j,i) = Cp;
               Sdestabpert{k}.D(:,:,j,i) = Dp;
            end
         end
      end
   %    for k = 1:nblk
   %       DestabNamesValues{k,2}(:,:,i) = destabpert{k}(:,:,lbindx,i);
   %    end
      if strcmpi(opt.Sensitivity,'On')
         gamma = opt.VaryUncertainty/100;
         if gamma<=0
            warning('Resetting VaryUncertainty to default value');
            gamma = opt.Default.VaryUncertainty/100;
         end
         idx = find(bnds(1,1,i).ResponseData >= pku*0.85); %1 x <=npts
         if length(idx) == 1
             if idx == 1 
                 idx = [idx idx+1];
             else
                 idx = [idx-1 idx];
             end
         end
         matg = frd(M.ResponseData(:,:,idx,i),newsys.Frequency(idx));
         sfactor = zeros(nblk,1);

         for k = 1:nblk
            tmpmat = matg;
            tmpmat.ResponseData(rp(k):rp(k+1)-1,:) = ...
                    (1+gamma)*tmpmat.ResponseData(rp(k):rp(k+1)-1,:);
            if strcmpi(opt.Display,'On') 
               name = pvget(AtomCell{k},'Name');
               if ~(length(name)>=13 && strcmpi('rcastreserved',name(1:13)))
                  if k==1
                     fprintf(['Sensitivity Calc: Uncertainty ' name ', ']);
                  else
                  fprintf(['                  Uncertainty ' name ', ']);
   %                   xtxt = repmat('\b',1,length(lname)+2);
   %                   fprintf([xtxt]);
   %                   fprintf([name ', ']);
                  end
               end
               sidx = find(opt.Mussv=='s');
               opt.Mussv(sidx) = [];
               lname = name;
            end
   %           if k<nblk
   %              sbnds = mussv(tmpmat,oldmublk,[opt.Mussv 'b']);
   %           else
            if strcmpi(opt.Display,'On') && ...
               ~(length(name)>=13 && strcmpi('rcastreserved',name(1:13)))
              sbnds = mussv(tmpmat,oldmublk,opt.Mussv);
            else
              sbnds = mussv(tmpmat,oldmublk,[opt.Mussv 's']);
            end
            pkus = sbnds.ResponseData(1,1,:);
            oldpeak = bnds(1,1,i).ResponseData(:,:,idx);
            oldpeak = oldpeak(:);
            pkus = pkus(:);
            if any(oldpeak==0 & pkus>0)
               sfactor(k) = inf;
            elseif all(oldpeak==0 & pkus==0)
               sfactor(k) = 1;
            else
               keepgz = find(oldpeak>0);
               sfactor(k) = max(abs( (pkus(keepgz)-oldpeak(keepgz)) ./ (gamma*oldpeak(keepgz)) ));
            end
         end
         sensitivity(1:nblk,1,i) = round(100*sfactor);
         analysistext{1,1,i} = LOCALmkmessage(pku,pkl,bnds.Frequency(lbindx(1)),...
            get(M,'Units'),ussflag,opt,sensitivity,AtomCell);
      else
         analysistext{1,1,i} = LOCALmkmessage(pku,pkl,bnds.Frequency(lbindx(1)),...
            get(M,'Units'),ussflag);
      end
      for k=1:nblk
         N = pvget(AtomCell{k},'Normalizer');
         switch class(AtomCell{k})
         case 'ureal'
            %gsrangedata(k,1,i) = starp (-1/pku,N.S);
            nprt = -1/pku;
            gsrangedata(k,1,i) = N.S(2,2) + N.S(2,1)*nprt/(1-N.S(1,1)*nprt)*N.S(1,2);
            %gsrangedata(k,2,i) = starp (1/pku,N.S);
            nprt = 1/pku;
            gsrangedata(k,2,i) = N.S(2,2) + N.S(2,1)*nprt/(1-N.S(1,1)*nprt)*N.S(1,2);
         case 'ucomplex' 
            gsrangedata(k,1,i) = pvget(AtomCell{k},'NominalValue');
            gsrangedata(k,2,i) = 1/pku*pvget(AtomCell{k},'Radius');
         case 'ultidyn' 
            gsrangedata(k,1,i) = 0;
            % XXX need to consider positive reals
            if newsys.Ts == 0
               gsrangedata(k,2,i) = 1/pku*pvget(AtomCell{k},'Bound');
            else
               gsrangedata(k,2,i) = 1/pku;
            end
         case 'ucomplexm' 
            gsrangedata(k,1,i) = norm(pvget(AtomCell{k},'NominalValue'));
            WR = pvget(AtomCell{k},'WR');
            WL = pvget(AtomCell{k},'WL');
            WRn = norm(WR);
            WLn = norm(WL);
            gsrangedata(k,2,i) = 1/sqrt(pku)*WRn*WLn;
         end
      end
   end
   for k = 1:nblk
      switch class(AtomCell{k})
         case {'ureal' 'ucomplex' 'ucomplexm'}
            for i=1:prod(exd)
               DestabNamesValues{k,2}(:,:,i) = destabpert{k}(:,:,LBINDXval(i),i);
            end
         case 'ultidyn'
            destabpert{k} = ss(Sdestabpert{k}.A,...
               Sdestabpert{k}.B,...
               Sdestabpert{k}.C,...
               Sdestabpert{k}.D,newsys.Ts);
            Apert = repmat(Sdestabpert{k}.A(:,:,LBINDXval(1),1),[1 1 exd]);
            Bpert = repmat(Sdestabpert{k}.B(:,:,LBINDXval(1),1),[1 1 exd]);
            Cpert = repmat(Sdestabpert{k}.C(:,:,LBINDXval(1),1),[1 1 exd]);
            Dpert = repmat(Sdestabpert{k}.D(:,:,LBINDXval(1),1),[1 1 exd]);
            for i=2:prod(exd)
               Apert(:,:,i) = Sdestabpert{k}.A(:,:,LBINDXval(i),i);
               Bpert(:,:,i) = Sdestabpert{k}.B(:,:,LBINDXval(i),i);
               Cpert(:,:,i) = Sdestabpert{k}.C(:,:,LBINDXval(i),i);
               Dpert(:,:,i) = Sdestabpert{k}.D(:,:,LBINDXval(i),i);
            end
            DestabNamesValues{k,2} = ss(Apert,Bpert,Cpert,Dpert,newsys.Ts);
      end
   end
end

% Handles BLOCKS with zero copies, which
% were removed at the beginning.
bno = pvget(sys,'UNames');
deporig = pvget(sys.ResponseData,'DepData');
[C,Idx] = setdiff(bno,names);
for k=1:length(C)
   names = [names;C(k)];
   missingatom = pvget(deporig,C{k});
   nomval = pvget(missingatom,'NominalValue');
   switch class(missingatom)
   case 'ureal'
      gsrange = [-inf inf];
   case 'ucomplex' 
      gsrange = [pvget(missingatom,'NominalValue') inf];
   case 'ultidyn' 
      gsrange = [0 inf];
   case 'ucomplexm' 
      gsrange = [norm(pvget(missingatom,'NominalValue')) inf];
   end
   DestabNamesValues{nblk+k,1} = C{k};
   for i=1:prod(exd)
      DestabNamesValues{nblk+k,2}(:,:,i) = nomval;
      sensitivity(nblk+k,i) = 0;
      for j=1:npts
         destabpert{nblk+k,1}(:,:,j,i) = nomval;
      end
      gsrangedata(nblk+k,:,i) = gsrange;
   end
end
idx = zeros([length(bno),1]);
for i=1:length(bno)
   idx(i) = find(strcmp(bno{i},names));
end
DestabNamesValues = DestabNamesValues(idx,:);
gsrangedata = gsrangedata(idx,:,:);
gsrangedata = reshape(gsrangedata(idx,:,:),[length(idx) 2 exd]);
destabpert = destabpert(idx);

pidx = [1 2 4:4+length(exd)-1 3];
sspidx = [4:4+length(exd)-1 3]-2; % 1, 2,... refer to array dimensions
odestabpert = destabpert; % copy, for rearrangement into 
% 06/10/04: Do not need to do this, as DESTABPERT will be returned differently
% for k=1:nblkorig
%    if isa(destabpert{k}(:,:,1),'ss')
%       %XXXMINOR need to replace with SS PERMUTE
%       %XXXMAJOR why did we want to switch it back?
%       destabpert{k} = depromote(permute(uss(destabpert{k}),sspidx));
%    else
%       destabpert{k} = permute(destabpert{k},pidx);
%    end
% end

DestabilizingFrequency = stabfreq.Max;
%info.DestabUnc = destabpert;
%info.StabilityRange = gsrangedata;
Report = analysistext;

if strcmpi(opt.Sensitivity,'On')
   sensitivity(:,:,:) = sensitivity(idx,:,:);
else
   % 06/10/04 add, to match doc
   % tmpsize = size(sensitivity);
   % sensitivity = repmat(nan,[tmpsize(1) 1 tmpsize(3:end) 1 1]);
   sensitivity = repmat(nan,[nblkorig 1 exd]);
end


% Make all variables conform to documentation
if nargout == 4
    DSP = cell(npts,1);
    placeh = cell(length(odestabpert),1);
    for i=1:npts
       for j=1:length(placeh)
          placeh{j} = cell(exd);
          for k=1:prod(exd)
             % if odestabpert{j} is a double, then this is right, as dimensions
             % 1 and 2 are the row/column, dimension 3 corresponds to frequency,
             % and dimensions 4 onward associate to the array dimensions.
             placeh{j}{k} = odestabpert{j}(:,:,i,k);
             % if odestabpert{j} is a SS, then this is right too, as dimensions
             % 1 and 2 are the output/input, dimension 3 corresponds to frequency,
             % and dimensions 4 onward associate to the array dimensions.  The
             % reason for the difference in the PERMUTE several lines above is
             % that PERMUTE treats row/col and input/output different for
             % DOUBLES and SS.
          end
       end
       tmp = [DestabNamesValues(:,1) placeh]';
       DSP{i} = struct(tmp{:});
    end
    info.Frequency = sys.Frequency;
    info.BadUncertainValues = DSP;
end

stabmarg = struct('UpperBound',rm12(stabmarg.Max),'LowerBound',rm12(stabmarg.Min),...
         'DestabilizingFrequency',rm12(DestabilizingFrequency));
BadAtoms = nv2struct(DestabNamesValues(:,1),DestabNamesValues(:,2));
Report = rm12(Report);

tmp = [bno nd22dcellwexdcell(sensitivity)].';
info.Sensitivity = struct(tmp{:});

if isempty(SETUP)
   Report = reportconvert(Report);
end

%---------- Local subroutine ------------------------------%
function text = LOCALmkmessage(pku,pkl,freq,units,ussflag,opt,sensitivity,AtomCell);

if nargin==5
   opt = [];
end

if pkl >= 1
   text = ['Uncertain System is NOT robustly stable to modeled uncertainty.'];
elseif pku > 1
   if ussflag == 0
      text = strvcat('Assuming nominal UFRD system is stable ...',...
      'Uncertain System is possibly NOT robustly stable to modeled uncertainty.');
   else
      text = 'Uncertain System is possibly NOT robustly stable to modeled uncertainty.';
   end
else
   if ussflag == 0
   text = strvcat('Assuming nominal UFRD system is stable ...',...
      'Uncertain System is robustly stable to modeled uncertainty.');
   else
      text = 'Uncertain System is robustly stable to modeled uncertainty.';
   end
end

if pku==0
   text1 = [' -- It can tolerate an infinite amount of modeled'];
else
   text1 = [' -- It can tolerate up to ' num2str(100/pku,'%0.3g') '% of the modeled'];
end
if pkl==0
   %XXXMAJOR this message seems weird to me.
   text1 = [ text1 ' uncertainty.'];
   text2 = [' --   No modeled uncertainty exists to cause an instability at ' num2str(freq,'%0.3g') ' ' units '.'];
   text3 = '';
else
   text1 = [ text1 ' uncertainty.'];
   text2 = [' -- A destabilizing combination of the modeled uncertainty of size ' num2str(100/pkl,'%0.3g') '% exists'];
   text3 = ['    causing an instability at ' num2str(freq,'%0.3g') ' ' units '.'];
   text2 = [' -- A destabilizing combination of ' num2str(100/pkl,'%0.3g') '% of the modeled uncertainty exists,'];
   text3 = ['    causing an instability at ' num2str(freq,'%0.3g') ' ' units '.'];
end
if ~isempty(opt)
   SensReport = LOCALsensmessage(sensitivity,AtomCell);
   text = strvcat(text,text1,text2,text3,SensReport{1});
else
   text = strvcat(text,text1,text2,text3);
end



%---------- Local subroutine ------------------------------%
function SensReport = LOCALsensmessage(SMsens,atomnames);
% SMub = STABMARG.UpperBound
% SMsens = info.Sensitivity

szm = size(SMsens);
if length(szm)==2
   exd = 1;
else
   exd = szm(3:end);
end
SensReport = cell(exd);
na = length(atomnames);

examp = 25;
text = cell(1,1);
for i1=1:prod(exd)
   text{i1} = [' -- Sensitivity with respect to uncertain element ...'];
    for i2 = 1:na
       name = get(atomnames{i2},'name');
       if ~(length(name)>=13 && strcmpi('rcastreserved',name(1:13)))
          sens = SMsens(i2,i1); 
          drop = round( (sens/100)*examp );
          text1 = ['   ''' name ''' is ' num2str(sens) '%.  '];
          text2 = ['Increasing ''' name ''' by ' num2str(examp) '%' ...
             ' leads to a ' num2str(drop) '% decrease in the margin.'];
      %        text2 = ['If the uncertainty range of ' name ' is'];
      %        text3 = ['   enlarged by ' num2str(examp) '%'];
      %        text4 = [' then the stability margin should drop by about ' ...
      %            num2str(drop) '%.' ];
          text{i1} = strvcat(text{i1}, [text1 text2]);
       end
    end
    
    SensReport{i1} = strvcat(text);
end



