function out = stack(dim,varargin)
% STACK  Stack UFRDs into UFRD array.
% 
%  USYSG = STACK(ARRAYDIM,USYSG1,USYSG2,...) produces an array of UFRD
%  USYSG by stacking the UFRD USYSG1, USYSG2,... along
%  the array dimension ARRAYDIM.  All UFRDs must have the same 
%  number of inputs and outputs.
% 
%  For example, if USYSG1 and USYSG2 are two UFRDs with the 
%  same I/O dimensions,
%    * STACK(1,USYSG1,USYSG2) produces a 2-by-1 UFRD array
%    * STACK(2,USYSG1,USYSG2) produces a 1-by-2 UFRD array
%    * STACK(3,USYSG1,USYSG2) produces a 1-by-1-by-2 UFRD array.
% 
%  You can also use STACK to concatenate UFRD arrays USYSG1,USYSG2,...
%  along some array dimension ARRAYDIM.
% 
%  See also HORZCAT, VERTCAT, APPEND.

% Copyright 2003-2005 The MathWorks, Inc.
if nargin<2
   error('Need at least 2 arguments.');
end
if ~(isa(dim,'double') && ndims(dim)==2 && all(size(dim)==[1 1]) && ...
      dim>0 && floor(dim)==ceil(dim))
   error('DIM should be a positive integer.');
end
if nargin==2
   out = varargin{1};
elseif nargin==3
   A = varargin{1};
   B = varargin{2};
   if ~isa(A,'ufrd') 
      A = lift2ufrd(A,B.Frequency,B.Units);
   elseif ~isa(B,'ufrd')
      B = lift2ufrd(B,A.Frequency,A.Units);
   else   
      % Check frequency vectors
         try
            % give priority to rad/s if both FRD's
            [freq,units] = freqcheck(A.Frequency,A.Units,B.Frequency,B.Units);
         catch einfo
            throw(einfo)
         end
   end
   sza = size(A);
   szb = size(B);

   % XXXOK
   Ts = binaryts(pvget(A,'Ts'),pvget(B,'Ts'),isstatic(A),isstatic(B));
   ResponseData = stack(dim+1,A.ResponseData,B.ResponseData);
   DS = A.dynamicsys+B.dynamicsys;

   szRD = size(ResponseData);
   if length(szRD)==2  || ( length(szRD)>= 3 && szRD(3)==1 )
      ResponseData = repmat( ResponseData, [1 1 length(A.Frequency) ones(1,length(szRD)-3)] );   
   end
   out = ufrd(ResponseData,A.Frequency,'UNITS',A.Units,Ts,DS);
else
   out = stack(dim,stack(dim,varargin{1},varargin{2}),varargin{3:end});
end
