function out = ufrd(varargin)
%UFRD   Conversion to Uncertain Frequency Response Data model.
%   Uncertain Frequency Response Data (UFRD) models usually arise
%   in the conversion of an uncertain state-space (USS) system to
%   its uncertain frequency response.   They also automatically
%   arise when Frequency Response Data models (FRD) are combined
%   with uncertain matrices (UMAT).
%
%Conversion from USS:
%   SYSG = UFRD(SYS,FREQS,'UNITS',UNITS) converts a USS model SYS to
%   a UFRD model SYSG by frequency response.  UNITS specifies the
%   units of the frequencies in FREQS, which may be 'rad/s' or 'Hz',
%   and will default to 'rad/s' if the last two arguments are omitted.
%   The result is a UFRD model.
%
%Conversion from FRD:
%   SYSG = UFRD(SYS) converts an FRD model SYS to a UFRD model SYSG
%   with no uncertain elements.
%
%Specifying Property Values:
%   In all syntax above, the input list can be followed by pairs
%        'PropertyName1', PropertyValue1, ...
%   to set the various properties of the UFRD model.
%
%Creation (this use is more rare, and unlikely to be done by user):
%   SYS = UFRD(RESPONSEDATA,FREQS) creates a UFRD from the RESPONSEDATA
%   and FREQS.  RESPONSEDATA should be a UMAT array, whose 1st array
%   dimension (ie., size(RESPONSEDATA,3)) aligns with the frequency

% Copyright 2003-2004 The MathWorks, Inc.
%   vector FREQS.
%
%   SYS = UFRD(RESPONSEDATA,FREQS,TS) also specifies sampling time.
%
%   SYS = UFRD(RESPONSEDATA,FREQS,REFSYS) causes SYS to inherit all
%   LTI properties from REFSYS.
%
%   SYS = UFRD(RESPONSEDATA,FREQS,'UNITS',UNITS) additional specifies the
%   units of FREQS.
%
%   SYS = UFRD(RESPONSEDATA,FREQS,'UNITS',UNITS,TS)
%
%   SYS = UFRD(RESPONSEDATA,FREQS,'UNITS',UNITS,REFSYS)

%   UFRDs are usually created by
%   manipulation of USS, FRD, UMAT and ATOM, rather than by direct
%   use of the constructor.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% March 11, 2003.  The code should handle (all can be followed by PName/Value pairs)
% RESPONSEDATA(UMAT), FREQUENCY
% RESPONSEDATA(UMAT), FREQUENCY, TS
% RESPONSEDATA(UMAT), FREQUENCY, refLTIModel
% RESPONSEDATA(UMAT), FREQUENCY,'Units',CHAR
% RESPONSEDATA(UMAT), FREQUENCY,'Units',CHAR, TS
% RESPONSEDATA(UMAT), FREQUENCY,'Units',CHAR, refLTI/refDS
% RESPONSEDATA(UMAT), FREQUENCY,'Units',CHAR, TS, DS (our call)
%
% USS, FREQUENCY
% USS, FREQUENCY,'Units',CHAR
%
% FRD
% UFRD

superiorto('double',...
   'ureal','ucomplex','ucomplexm','udyn','ultidyn',...
   'atom',...
   'ss','tf','zpk','ndlft','frd','umat','uss');

validate = 1;
mkatend = 1;

% Determine how many Property NameValue pairs (NVP)
% AIN denotes number of leading arguments (not name/value pairs)
nin = nargin;
nvp = 0;
i = 1;
while nin>i && ...
      ~(~isa(varargin{nin-i},'char') || ...
      strncmpi('Units',varargin{nin-i},length(varargin{nin-i})) || ...
      (nin>i+1 && strncmpi('Units',varargin{nin-i-1},length(varargin{nin-i-1}))))
   nvp = nvp + 1;
   i = i + 2;
end
ain = nin - 2*nvp;

if ain == 0
   a1 = umat;
   a2 = zeros(0,1);
   a3 = 'rad/s';
   a4 = utarbts;
   a5 = dynamicsys(0,0);
   validate = 0;
elseif ain==1
   arg1 = varargin{1};
   % no validation on true casts - why don't we do this in USS?
   validate = 0;
   switch(class(arg1))
   case {'ufrd'}
      if nvp==0
         mkatend = 0;
         out = arg1;
      else
         error('Use SET to change properties');
      end
   case {'frd'}
      if hasdelay(arg1)
         warning('Omitting DELAYs in conversion to UFRD');
      end
      if nvp==0
         [a1tmp,a2] = frdata(arg1);
         a1 = umat(a1tmp);
         a3 = get(arg1,'Units');
         [a5,a4] = lti2ds(arg1);
      else
         error('Convert to UFRD, then use SET to change properties');
      end
   otherwise
      estr = ['Unable to convert ' upper(class(arg1)) ' to UFRD format.'];
      error(estr);
   end
elseif ain>=2
   arg1 = varargin{1};
   arg2 = varargin{2};
   if isa(arg2,'double') && ndims(arg2)==2 && min(size(arg2))==1
      % Check FREQUENCY HERE - still to do....
      switch class(arg1)
      case {'umat'}
         if ain==2   % ResponseData(umat), freq
            a1 = arg1;
            a2 = arg2;
            a3 = 'rad/s';
            a4 = utarbts;
            sza1 = size(a1);
            a5 = dynamicsys(sza1(1),sza1(2));
         elseif ain==3
            arg3 = varargin{3};
            if isa(arg3,'double') & ndims(arg3)==2 & max(size(arg3))==1
               a1 = arg1;
               a2 = arg2;
               a3 = 'rad/s';
               a4 = arg3;
               sza1 = size(a1);
               a5 = dynamicsys(sza1(1),sza1(2));
            elseif isa(arg3,'tf') || isa(arg3,'zpk') || isa(arg3,'ss')
               a1 = arg1;
               a2 = arg2;
               a3 = 'rad/s';
               dum = struct(arg3);
               [a5,a4] = lti2ds(dum.lti);
            elseif isa(arg3,'frd') 
               a1 = arg1;
               a2 = arg2;
               % XXX: This is due to the way CTB implements this feature in FRD
               a3 = 'rad/s';
               dum = struct(arg3);
               [a5,a4] = lti2ds(dum.lti);
            elseif isa(arg3,'ufrd') | isa(arg3,'uss')
               a1 = arg1;
               a2 = arg2;
               % XXX: See above for precedence
               a3 = 'rad/s';
               a4 = arg3.Ts;
               a5 = arg3.dynamicsys;
            else
               error('3rd argument should be SAMPLE TIME, or reference system.');
            end
         elseif ain>=4
            arg3 = varargin{3};
            arg4 = varargin{4};
            len3 = length(arg3);
            len4 = length(arg4);
            % UMAT, FREQUENCY,'Units',CHAR, TS
            % UMAT, FREQUENCY,'Units',CHAR, refLTI/refDS
            % UMAT, FREQUENCY,'Units',CHAR, TS, DS (our call)
            if isa(arg3,'char') && strncmpi(arg3,'Units',len3) && ...
                  isa(arg4,'char') && (strncmpi(arg4,'Hz',len4) || strncmpi(arg4,'rad/s',len4))
               a1 = arg1;
               a2 = arg2;
               a3 = arg4;
               a4 = utarbts;
               sza1 = size(arg1);
               if ain==4
                  a5 = dynamicsys(sza1(1),sza1(2));
               elseif ain==5
                  % if ARG5 is REFSYS, we are using TS from REFSYS, but not
                  % the UNITS -- those are coming from ARG3/ARG4.
                  arg5 = varargin{5};
                  if isa(arg5,'double') & ndims(arg5)==2 & max(size(arg5))==1
                     a4 = arg5;
                     a5 = dynamicsys(sza1(1),sza1(2));
                  elseif isa(arg5,'tf') || isa(arg5,'zpk') || isa(arg5,'ss')
                     dum = struct(arg5);
                     [a5,a4] = lti2ds(dum.lti);
                  elseif isa(arg5,'frd') 
                     % XXX: check CTB implementation regarding Ts/Units
                     % (ie., who wins?)
                     dum = struct(arg5);
                     [a5,a4] = lti2ds(dum.lti);
                  elseif isa(arg5,'ufrd') | isa(arg5,'uss')
                     % XXX: See above for precedence
                     a4 = arg5.Ts;
                     a5 = arg5.dynamicsys;
                  else
                     estr = ['5th argument should be SAMPLE TIME or reference system.'];
                     error(estr);
                  end
               elseif ain==6
                  a4 = varargin{5};
                  a5 = varargin{6};
               else
                  estr = 'Invalid number of arguments.';
                  error(estr);
               end
            else
               estr = ['Invalid UNITS specification: 3rd argument should be '];
               estr = [estr '''UNITS'' and 4th argument should be ''Hz'' or ''rad/s'''];
               error(estr);
            end
         end
      case {'uss'}
         if ain==2   % uss, freq
            a2 = arg2;
            a3 = 'rad/s';
            a4 = arg1.Ts;
            a5 = pvget(arg1,'dynamicsys');
         elseif ain==4
            arg3 = varargin{3};
            arg4 = varargin{4};
            len3 = length(arg3);
            len4 = length(arg4);
            if isa(arg3,'char') && strncmpi(arg3,'Units',len3) && ...
                  isa(arg4,'char') && (strncmpi(arg4,'Hz',len4) || strncmpi(arg4,'rad/s',len4))
               a2 = arg2;
               a3 = arg4;
               a4 = arg1.Ts;
               a5 = pvget(arg1,'dynamicsys');
            else
               estr = ['Invalid UNITS specification: 3rd argument should be '];
               estr = [estr '''UNITS'' and 4th argument should be ''Hz'' or ''rad/s'''];
               error(estr);
            end
         else
            error('Invalid number of arguments');
         end
         if allatomslti(arg1)
            if arg1.Ts==utarbts && ~isstatic(arg1)
               estr = ['XXXMINOR, need to make sure this does not happen.'];
               error(estr);
            end
            %lsys = pvget(arg1,'MofMDelta'); % this is SS, with LTI-filled
            lsys = lftdata(arg1); % this is SS, with LTI-filled
            a4 = lsys.Ts; %if -2, then now 0, otherwise as it was. XXXMINOR
            thisOmega = a2;
            if strncmpi(a3,'Hz',length(a3))
               thisOmega = (2*pi)*a2;
            end
            MatrixData = freqresp(lsys,thisOmega); % NDDouble, 3rd dimension is w
            tmp2 = pvget(arg1,'theDELTAasNDLFT');
            DepData = pvget(tmp2,'DepData');
            Copies = pvget(tmp2,'Copies');
            Literal = pvget(tmp2,'Literal');
            a1 = umat(ndlft(MatrixData,DepData,Copies,Literal));
         else
            error('Some atoms are not linear, time-invariant, conversion to UFRD is not defined');
         end
      otherwise
         error(['Invalid first argument class ' class(arg1)])
      end
   else
      error('2nd argument should be Frequency vector.');
   end
end

% Create OUT, if necessary
if mkatend
   out.ResponseData = a1;
   out.Frequency = a2(:);
   % Fix up the actual UNITS field, since abreviations may have been
   % entered.
   ta3 = 'rad/s';
   if strcmpi(a3(1),'h')
      ta3 = 'Hz';
   end
   out.Units = ta3;
   out.Ts = a4;
   out = class(out,'ufrd',gsref(0),a5);
end

% Handle all Property Name/Value pairs with one call to SET.
% Do not call SET with only 1 argument, as that has a different meaning.
if nin>ain
   out = set(out,varargin{ain+1:end});
   validate = 0; % SET automatically validates
end

% Check validity of OUT
if validate==1
   [pflag,out] = isvalid(out);
   if pflag==0
      error('Invalid UFRD construction');
   end
end
