%WCGAIN   Calculates worst-case gain of an uncertain system.
% "Worst-case" refers to maximum over the uncertainty, and "norm" is
% measured by the maximum over frequency of the maximum singular value
% of the frequency response matrix (ie., the H-infinity norm).
% 
% [MAXGAIN,MAXGAINUNC,INFO] = WCGAIN(SYS) calculates upper and lower
% bounds for the worst-case gain of an uncertain frequency response
% data object (UFRD).  
% 
% MAXGAIN  Structure with the following fields:
%         LowerBound: Upper bound on worst-case gain. 
%         UpperBound: Lower bound on worst-case gain. 
%  CriticalFrequency: Frequency at which the maximum gain occurs.
% MAXGAINUNC  Structure of worst-case uncertainty values
% INFO  Structure with the following fields:
%        Sensitivity: Structure of percentages corresponding to MAXGAIN 
%                     sensitivity to variations in each uncertainty level.
%          Frequency: Frequency vector used in analysis.
%         ArrayIndex: Array index (for arrays of uncertain systems)
%                     where the maximum gain occurs.  Value is scalar,
%                     for single index use.
% 
% [MAXGAIN,MAXGAINUNC,INFO] = WCGAIN(SYS,OPTS) specifies options
% described in OPTS.  See WCGOPT for more details on the options for
% WCGAIN. 
% 
%See also LOOPMARGIN, MUSSV, LTI/NORM, ROBUSTSTAB, WCGOPT, WCSENS, WCMARGIN.

% Copyright 2003-2006 The MathWorks, Inc.

function [MaxGain,WCUncertainty,info,Data] = ...
   wcgain(M,opts,PrevData,SETUP,StableFlag)
nin = nargin;
% Never actually uses StableFlag.  Argument list is identical to USS/WCGAIN.
if nin==1
   opts = wcgopt;
   PrevData = [];
   SETUP = [];
   StableFlag = [];
elseif nin==2
   PrevData = [];
   SETUP = [];
   StableFlag = [];
elseif nin==3
   SETUP = [];
   StableFlag = [];
elseif nin==4
   StableFlag = [];
end

info = struct('Sensitivity',[],'MaximumGain',[],'BadAtomValues',[],'ArrayIndex',[]);

% convert to two column form
szM = size(M);
TheFrequency = M.Frequency; 
nfpts = length(M.Frequency);
nAD = length(szM)-2; % will be 0 or >=2
Y = unique(sort(opts.ArrayDimPtWise));
idx = find(Y>nAD);
Y(idx)=[];
opts.ArrayDimPtWise=Y; % Note: this is never actually referenced
                       % again.  Calls to PVWCG implicitly are only
                       % PKOVER, so ArrayDimPtWise is not referenced.
AD = szM(3:end); % empty or >=2
X = 1:nAD;
X(Y)=[];
E=AD;
E(X)=1;
% At this point, both AD and E should be 1-by-nAD, and X and Y should
% be 1-by-??, the lengths of which should add to nAD.  [E] (or [E 1 1],
% enforcing that everything be thought of as at least 2-dimensional)
% is the same as the e_i in the documentation.  Note: AD, E, X and Y have
% nothing(!) to do with FreqPtWise. 
if opts.FreqPtWise==1
   YY = [0 Y];
   XX = X;
   EE = [nfpts E];
else
   EE = E;
   YY = Y;
   XX = [0 X];
end
numX = length(XX);
numY = length(YY);
slots = [szM(1) szM(2) nfpts AD];
YY=YY+1;
XX=XX+1;
nYpts = prod(slots(YY+2)); % frequency is in here (if appropriate)
nXpts = prod(slots(XX+2)); % frequency is in here (if appropriate)

% build mu block structure
m = pvget(M.ResponseData,'NDlftData'); % M is UFRD, M.RD is UMAT, m is ndlft
morig = m;
m = rmzerocp(m);

[rporig,cporig,nblkorig,iosizeorig,blkiosizeorig,szmdorig] = ndlftinfo(morig);
[rp,cp,nblk,iosize,blkiosize,szmd] = ndlftinfo(m);
BS = cell(nblk,1);
crows = [];
ccols = [];
AtomCell = pvget(m.DepData,'AtomCell');
mublk = zeros(0,3);
tmublk = zeros(0,2);
Copies = pvget(m,'Copies');
for i=1:nblk
   rep = Copies(i);
   switch class(AtomCell{i})
   case 'ureal'
      mublk = [mublk;1 1 -rep];
      tmublk = [tmublk;ones(rep,2)];
   case 'ucomplex'
      mublk = [mublk;1 1 rep];
      tmublk = [tmublk;ones(rep,2)];
      crows = [crows rp(i):rp(i+1)-1];
      ccols = [ccols cp(i):cp(i+1)-1];
   case 'ucomplexm'
      szdel = size(AtomCell{i});
      crows = [crows rp(i):rp(i+1)-1];
      ccols = [ccols cp(i):cp(i+1)-1];
      if (rep==1) | isequal(szdel,[1 1])
         mublk = [mublk;szdel  rep];
         tmublk = [tmublk;szdel];
      else
         msg1 = ['Uncertain element ' pvget(AtomCell{i},'Name') ' is a '];
         msg2 = ['repeated, non-scalar UCOMPLEXM.\n'];
         msg3 = ['WCGAIN cannot solve this problem.'];
         msg = sprintf([msg1 msg2 msg3]);
         error(msg);
      end
   case 'ultidyn'
      szdel = size(AtomCell{i});
      crows = [crows rp(i):rp(i+1)-1];
      ccols = [ccols cp(i):cp(i+1)-1];      
      if (rep==1) 
         mublk = [mublk;szdel  rep];
         tmublk = [tmublk;szdel];          
      elseif isequal(szdel,[1 1])
         mublk = [mublk; szdel  rep];
         tmublk = [tmublk; ones(rep,2)];
      else
         msg1 = ['Uncertain element ' pvget(AtomCell{i},'Name') ' is a '];
         msg2 = ['repeated, non-scalar ULTIDYN.\n'];
         msg3 = ['WCGAIN cannot solve this problem.'];
         msg = sprintf([msg1 msg2 msg3]);
         error(msg);
      end
   otherwise
      msg1 = ['Uncertain element ' pvget(AtomCell{i},'Name') ' is a '];
      msg2 = ['UDYN.\n'];
      msg3 = ['WCGAIN cannot solve this problem.'];
      msg = sprintf([msg1 msg2 msg3]);
      error(msg);
   end
end
crows = [crows rp(end):rp(end)+iosize(1)-1];
ccols = [ccols cp(end):cp(end)+iosize(2)-1];
cblkidx = find(mublk(:,3)>0);
cmublk = mublk(cblkidx,:);
tmublk = [tmublk;fliplr(iosize)];
ctmublk = [tmublk(cblkidx,:);fliplr(iosize)];
bidx = blk2idx(mublk);

% Switch Literal
Liter = pvget(m,'Literal');
mz = setlitr(m,find(Liter==1),0);
if isempty(SETUP)
   mdata = pvget(mz,'MatrixData');
	if nblk>0
      matND = munochk(mdata,tmublk,'x');
   else
      matND = mdata;
	end
else
   matND = zeros([szmd(1) szmd(2) slots(3:end)]);
end
% Convert to 2 column form
matND = permute(matND,[1 2 XX+2 YY+2]);
matND = reshape(matND,[szmd(1) szmd(2) nXpts nYpts]);

% (code from wcg.m)
% If there are lots of YY's,
%    then Data should be a cell array of size slots(YY).
% If there is only one YY,
%    then Data should be a cell array of size slots(YY)-by-1.
% If there are no YY's,
%    then Data should be a 1x1 cell array.
% OR, (single index) Data should be a nYpts-by-1 cell array
Data = cell(nYpts,1);  % reshape(Data,[slots(YY) 1 1]);
if isempty(PrevData)
   PrevData = cell(nYpts,1); 
end

pkloc = zeros([nYpts 1]); % ??, not sure
LowerBnd = repmat(inf,[nYpts 1]); % OK
UpperBnd = repmat(inf,[nYpts 1]); % OK
wcidx = 1; % for use in Setup
topts = wcgopt('MBadThreshold',opts.MBadThreshold,...
   'ABadThreshold',opts.ABadThreshold);
%topts.Display = 'Off';
idxXcell = cell(1,numX);
idxYcell = cell(1,numY);
freq = zeros([nYpts 1]); % doesn;t make sense to me...
sidx = zeros([nYpts 1]);
FXY = cell(1,nAD+1);
if isempty(SETUP)
	if nblk == 0
       for i=1:nYpts
          LowerBnd(i) = -1;
          for j=1:nXpts
             tmpnorm = norm(matND(:,:,j,i));
             if tmpnorm>LowerBnd(i)
                idx = j;
                LowerBnd(i) = tmpnorm;
             end
          end
          pkloc(i) = idx; 
          UpperBnd(i) = LowerBnd(i);
       end
	else
       if strcmpi(opts.LowerBoundOnly,'On')
          for i=1:nYpts
             Data{i} = pvwcg(matND(:,:,:,i),mublk,opts,PrevData{i});
             [LowerBnd(i),idx] = max(Data{i}.SfreqLmax);
             UpperBnd(i) = max(Data{i}.SfreqUmax);
             pkloc(i) = idx(1); % idx=1 if vecXdims==0
          end
       else
          for i=1:nYpts
             if ~isempty(cmublk) && length(cblkidx)~=nblk        
                [tmpData] = pvwcg(matND(crows,ccols,:,i),cmublk,topts,[]);
                cbnd.ratio = tmpData.SbestU/tmpData.SbestL;
                cbnd.diff = tmpData.SbestU-tmpData.SbestL;
                % matND has only 2 array dimensions
                Data{i} = pvwcg(matND(:,:,:,i),mublk,opts,PrevData{i},cbnd);
             else  
                Data{i} = pvwcg(matND(:,:,:,i),mublk,opts,PrevData{i});
             end   
             [LowerBnd(i),idx] = max(Data{i}.SfreqLmax);
             UpperBnd(i) = max(Data{i}.SfreqUmax);
             pkloc(i) = idx(1); % idx=1 if vecXdims==0
          end
       end
   end
   
   for i=1:nYpts
       if numX>0
          [idxXcell{:}] = ind2sub(slots(XX+2),pkloc(i));
          FXY(XX) = idxXcell;
       end
       if numY>0
          [idxYcell{:}] = ind2sub(slots(YY+2),i);
          FXY(YY) = idxYcell;
       end
       freq(i) = FXY{1};
       if isempty(AD) % edge case
          sidx(i) = 1;
       else
          sidx(i) = sub2ind(AD,FXY{2:end});
       end
   end
   ScalarIndex = reshape(sidx,[EE 1 1]);
   LowerBnd = reshape(LowerBnd,[EE 1 1]);
   UpperBnd = reshape(UpperBnd,[EE 1 1]);
   if opts.FreqPtWise==1
      lbound = cell(nfpts,1);
      ubound = cell(nfpts,1);
      arrayindex = cell(nfpts,1);
      for i=1:nfpts
         lbound{i} = reshape(LowerBnd(i,:),[E 1 1]);
         ubound{i} = reshape(UpperBnd(i,:),[E 1 1]);
         arrayindex{i} = reshape(ScalarIndex(i,:),[E 1 1]);
      end
      [dum,fidx] = max(LowerBnd(:,:),[],1);
      critfreq = reshape(TheFrequency(fidx),[E 1 1]);
   else
      lbound = LowerBnd;
      ubound = UpperBnd;
      critfreq = reshape(TheFrequency(freq),[E 1 1]);
      arrayindex = ScalarIndex;
   end
   [MaximumGain,wcidx] = max(LowerBnd(:));
else % SETUP
   GENERALPKLOC = 1;
   for i=1:nYpts
       if numX>0
          [idxXcell{:}] = ind2sub(slots(XX+2),GENERALPKLOC);
          FXY(XX) = idxXcell;
       end
       if numY>0
          [idxYcell{:}] = ind2sub(slots(YY+2),i);
          FXY(YY) = idxYcell;
       end
       if isempty(AD) % edge case
          sidx(i) = 1;
       else
          sidx(i) = sub2ind(AD,FXY{2:end});
       end
   end
   ScalarIndex = reshape(sidx,[EE 1 1]);
   critfreq = reshape(repmat(TheFrequency(1),[prod(E) 1]),[E 1 1]);
   MaximumGain = inf;
   if opts.FreqPtWise==1
      lbound = cell(nfpts,1);
      ubound = cell(nfpts,1);
      arrayindex = cell(nfpts,1);
      for i=1:nfpts
         lbound{i} = reshape(repmat(inf,[prod(E) 1]),[E 1 1]);
         ubound{i} = reshape(repmat(inf,[prod(E) 1]),[E 1 1]);
         arrayindex{i} = reshape(ScalarIndex(i,:),[E 1 1]);;
      end
   else
      lbound = reshape(repmat(inf,[prod(E) 1]),[E 1 1]);
      ubound = reshape(repmat(inf,[prod(E) 1]),[E 1 1]);
      arrayindex = ScalarIndex;
   end
end

for i=1:bidx.full.num
   ii = bidx.full.origloc(i);
   %block will be {'ucomplex' 'complexm' 'ultidyn'}
   sza = size(AtomCell{ii});
   if isempty(SETUP)
      [N] = pvget(AtomCell{ii},'Normalizer');
      S = N.S;
      BS{ii} = zeros([sza EE]);
      for k=1:nYpts
         BS{ii}(:,:,k) = lft(Data{k}.AllWlvec{pkloc(k)}{i}*Data{k}.AllWrvec{pkloc(k)}{i},S);
      end
   else
      NV = pvget(AtomCell{ii},'NominalValue');
      for k=1:nYpts
         BS{ii}(:,:,k) = NV;
      end
   end
end
for i=1:bidx.repcomp.num
   ii = bidx.repcomp.origloc(i);
   %block will be {'ucomplex' 'complexm' 'ultidyn'}
   sza = size(AtomCell{ii});
   BS{ii} = zeros([sza EE]);            
   [N] = pvget(AtomCell{ii},'Normalizer');
   if isempty(SETUP)
      for k=1:nYpts
         nprt = Data{k}.AllWrepcomp{pkloc(k)}(i);  
         BS{ii}(:,:,k) =  N.S(2,2) + N.S(2,1)*nprt/(1-N.S(1,1)*nprt)*N.S(1,2);
         % SAME AS, also below a few times
         %BS{ii}(:,:,k) = starp(Data{k}.AllWrepcomp{pkloc(k)}(i),S);
      end
   else
      BS{ii}(:) =  N.S(2,2);
   end
end
for i=1:bidx.sreal.num
   ii = bidx.sreal.origloc(i);
   sza = size(AtomCell{ii});
   BS{ii} = zeros([sza EE]);
   [N] = pvget(AtomCell{ii},'Normalizer');
   if isempty(SETUP)
      for k=1:nYpts
         nprt = Data{k}.AllWsreal{pkloc(k)}(i);
         BS{ii}(:,:,k) = N.S(2,2) + N.S(2,1)*nprt/(1-N.S(1,1)*nprt)*N.S(1,2);
      end
   else
      BS{ii}(:) = N.S(2,2);
   end
end
for i=1:bidx.repreal.num
   ii = bidx.repreal.origloc(i);
   sza = size(AtomCell{ii});
   BS{ii} = zeros([sza EE]);
   [N] = pvget(AtomCell{ii},'Normalizer');
   if isempty(SETUP)
      for k=1:nYpts
         nprt = Data{k}.AllWrepreal{pkloc(k)}(i);
         BS{ii}(:,:,k) = N.S(2,2) + N.S(2,1)*nprt/(1-N.S(1,1)*nprt)*N.S(1,2);
      end
   else
      BS{ii}(:) = N.S(2,2);
   end
end

names = pvget(m,'UNames');
% Handles BLOCKS with zero copies, which were removed at
% the beginning.  We grwo BS dynamically, which is not great,
% but probably not too bad.
bno = pvget(morig,'UNames');
deporig = pvget(morig,'DepData');
[C,Idx] = setdiff(bno,names);
for i=1:length(C)
   names = [names;C(i)];
   thisatom = pvget(deporig,C{i});
   sza = size(thisatom);
   if isa(thisatom,'ultidyn')
      nomval = pvget(thisatom,'SSNominalValue',M.Ts);
      BS{nblk+i} = ss(zeros([sza EE]));
   else
      nomval = pvget(thisatom,'NominalValue');
      BS{nblk+i} = (zeros([sza EE]));
   end
   for k=1:nYpts
      BS{nblk+i,1}(:,:,k) = nomval;
   end
end
origidx = zeros(length(bno),1);
for i=1:length(bno)
   origidx(i) = find(strcmp(bno{i},names));
   % If origidx(i)>nblk, then the i'th block in original order has
   % zero copies.  TRUE.  The 1st nontrivial block's original
   % location comes from doing find(origidx==1);
end
BS = BS(origidx);
names = names(origidx);

WCNamesValues = [names BS];
tmpM = morig;
%tmpM = m;

atomcell = pvget(tmpM,'AtomCell');
for i=1:length(atomcell);
   if isa(atomcell{i},'ultidyn') && origidx(i)<=nblk
      % Only work on ULTIDYN with >0 copies - if setup,
      % they need to be initialized, if notsetup, they need to
      % be converted from DOUBLEs
      bpasdouble = WCNamesValues{i,2};
      WCNamesValues{i,2} = ss(zeros([size(atomcell{i}) EE]));
      if isempty(SETUP)
         N = pvget(atomcell{i},'Normalizer');
         for k=1:nYpts
            omega = TheFrequency(freq(k));
            nbs = lft(bpasdouble(:,:,k),N.T);
            [u,s,v] = svd(nbs);
            wlvec = u(:,1)*sqrt(s(1,1));
            wrvec = sqrt(s(1,1))*v(:,1)';
            if norm(wrvec)==0 || norm(wlvec)==0
               value = pvget(atomcell{i},'SSNominalValue',M.Ts);
            else
               value = mkmimopert(wlvec,wrvec,omega,M.Ts,N);
            end
            WCNamesValues{i,2}(:,:,k) = value;
         end
      else % in setup, put the correct StateSpace stuff into the active ULTIDYN's
         nomval = pvget(atomcell{i},'SSNominalValue',M.Ts);
         for k=1:nYpts
            WCNamesValues{i,2}(:,:,k) = nomval;
         end
      end
   end
end
WCUncertainty = nv2struct(WCNamesValues(:,1),WCNamesValues(:,2));
if nblkorig>0
   BadAtomValues = WCUncertainty(wcidx(1));
else
   % for no uncertainty, XXX
   BadAtomValues = struct;
   WCUncertainty = repmat(BadAtomValues,[EE 1 1]);
end
if opts.FreqPtWise==1
   tmpcell = cell(nfpts,1);
   for i=1:nfpts
      tmpcell{i} = reshape(WCUncertainty(i,:),[E 1 1]);
   end
   WCUncertainty = tmpcell;
end

if strncmpi(opts.Sensitivity,'on',2) && nblkorig>0 && isempty(SETUP)
   gamma = opts.VaryUncertainty/100;
   topts = opts;
   topts.Sensitivity = 'Off'; %don't need when calling WCGAIN again
   topts.FreqPtWise = 0; %only called with 1 freq for sensitivity calc
   topts.LowerBoundOnly = 'On'; %use LowerBoundOnly for sensitive
   if opts.FreqPtWise == 0
      sens2unc = cell(nblkorig,1);
      [sens2unc{:}] = deal(zeros([1 1 E 1 1]));
      for i=1:prod(E)
         fidx = find(TheFrequency==critfreq(i));
         L.type = '()';
         L.subs = {':',':',arrayindex(i)};
         mtmp = subsref(M,L);
         mtmp = ufrd(mtmp.ResponseData(:,:,fidx),critfreq(i),M.Ts); %UFRD, 1 freq, no EXD
         mxlg = LowerBnd(i); % scalar
         for k=1:nblk
            kk = find(origidx==k);
            mm = LOCALuscl(mtmp,atomcell{kk},gamma);
            lbnd = wcgain(mm,topts);
            tmp  = (lbnd.LowerBound-mxlg)/((gamma)*mxlg);
            sens2unc{kk}(i) = 100*max([tmp 0],[],2);
         end
      end
      maxgainsens = nv2struct(names,sens2unc);
   else %opts.FreqPtWise==1
      maxgainsens = cell(nfpts,1);
      sens2unc = cell(nfpts,1);
      tmp = cell(nblkorig,1);
      [tmp{:}] = deal(zeros([1 1 E 1 1]));
      [sens2unc{:}] = deal(tmp);
      for j=1:nfpts
         mr = M.ResponseData(:,:,j,arrayindex{j}); % UMAT with EXD at single FREQxYgrid
         mtmp = ufrd(mr,TheFrequency(j),M.Ts); %UFRD, 1 freq, 1 EXD
         mxlg = lbound{j}; %E
         for k=1:nblk
            kk = find(origidx==k);
            mm = LOCALuscl(mtmp,atomcell{kk},gamma);
            lbnd = wcgain(mm,topts);
            tmp  = (lbnd.LowerBound(:)-mxlg(:))./((gamma)*mxlg(:));
            sens2unc{j}{kk}(:) = 100*max([tmp 0*tmp],[],2);
         end
         maxgainsens{j} = nv2struct(names,sens2unc{j});
      end
   end
else
   if opts.FreqPtWise == 0
      if nblkorig>0
         % for no uncertainty, XXX
         sens2unc = cell(nblkorig,1);
         [sens2unc{:}] = deal(repmat(nan,[1 1 E 1 1]));
         maxgainsens = nv2struct(names,sens2unc);
      else
         maxgainsens = repmat(struct,[EE 1 1]);
      end
   else %opts.FreqPtWise==1
      maxgainsens = cell(nfpts,1);
      if nblkorig>0
         sens2unc = cell(nfpts,1);
         tmp = cell(nblkorig,1);
         [tmp{:}] = deal(repmat(nan,[1 1 E 1 1]));
         [sens2unc{:}] = deal(tmp);
         for j=1:nfpts
            maxgainsens{j} = nv2struct(names,sens2unc{j});
         end
      else
         for j=1:nfpts
            maxgainsens{j} = repmat(struct,[E 1 1]);
         end
      end
   end
end

MaxGain.LowerBound = lbound;
MaxGain.UpperBound = ubound;
MaxGain.CriticalFrequency = critfreq;
info.Sensitivity = maxgainsens;
info.Frequency = TheFrequency;
info.MaximumGain = MaximumGain;
info.BadAtomValues = BadAtomValues;
info.ArrayIndex = arrayindex;

%Return bounds as FRD
if opts.FreqPtWise==1
   szf = size(lbound{1});
   F = length(szf);
   temp = cat(F+1,lbound{:});
   temp = permute(temp,[F+2 F+3 F+1 1:F]);
   MaxGain.LowerBound = frd(temp,TheFrequency,M.Ts,'Units',M.Units);
   temp = cat(F+1,ubound{:});
   temp = permute(temp,[F+2 F+3 F+1 1:F]);
   MaxGain.UpperBound = frd(temp,TheFrequency,M.Ts,'Units',M.Units);
end

% Decided to not return this information in INFO structure
info = rmfield(info,'MaximumGain');
info = rmfield(info,'BadAtomValues');

%-------------------------------------------------
function mm = LOCALuscl(mtmp,aname,gamma)

mm = mtmp;
mmu = pvget(mm,'Uncertainty');
uaname = aname;
switch class(aname)
  case 'ureal'
     uaname.PlusMinus = (1+gamma)*aname.PlusMinus;
  case 'ucomplex'
     uaname.Radius = (1+gamma)*aname.Radius;
  case 'ucomplexm'
     uaname.WL = sqrt(1+gamma)*aname.WL;
     uaname.WR = sqrt(1+gamma)*aname.WR;
  case 'ultidyn'
     if isequal(aname.Type,'GainBounded')
        uaname.Bound = (1+gamma)*aname.Bound;
     else
        uaname.Bound = aname.Bound - gamma*abs(aname.Bound);
     end
end
mmu = pvset(mmu,aname.Name,uaname);
mm = pvset(mm,'Uncertainty',mmu);



