function [wci,wco] = wcmargin(P,C,opts)
%WCMARGIN  Worst-case stability margins of uncertain feedback loops.
%
%   WCMARG = WCMARGIN(L) calculates the worst-case loop-at-a-time 
%   disk margins for the N-channel uncertain feedback loop:
%
%            u --->O---->[ L ]----+---> y
%                  |              |         
%                  +<-------------+
%
%   For each single-input, single-output loop L(i,i), the worst-case 
%   disk margins provide mimimum guaranteed gain and phase margins 
%   (in the classical sense) and help determine how much combined 
%   gain/phase variations the feedback loop can robustly tolerate.  
%   See LOOPMARGIN and DMPLOT for more details on the disk margins. 
%
%   The N-by-N loop transfer matrix L is specified as a USS or UFRD
%   model.  Negative feedback is assumed in the feedback path 
%   (specify -L otherwise).  
%
%   WCMARG is a N-by-1 structure where WCMARG(i) gives the worst-case 
%   disk margins for the i-th feedback channel.  The fields 
%   of WCMARG(i) are as follows:
%        GainMargin: Worst-case disk gain margin (lower bound on the
%                    worst-case gain margin for the i-th loop L(i,i))
%       PhaseMargin: Worst-case disk phase margin in degrees (lower 
%                    bound on the worst-case phase margin for L(i,i))
%         Frequency: Frequency where the margin disk touches the
%                    Nyquist plot envelope for the uncertain loop
%                    transfer L(i,i) (rad/sec)
%       Sensitivity: Structure with one field per uncertain element 
%                    in L. The field values are the sensitivity of
%                    the worst-case margins to each uncertain element.
%
%   [WCMARGI,WCMARGO] = WCMARGIN(P,C) calculates the worst-case disk
%   margins of the multivariable feedback loop consisting of a 
%   controller C, of size NU-by-NY, in negative feedback with a plant P,
%   of size NY-by-NU.  Either P or C (or both) are uncertain models.
%   Only specify the portion of C in the feedback path for "2-dof" 
%   architectures.  This returns:
%     * NU-by-1 structure WCMARGI containing the worst-case,
%       loop-at-a-time disk margins at the plant inputs (L = C*P)
%     * NY-by-1 structure WCMARGO containing the worst-case,
%       loop-at-a-time disk margins at the plant outputs (L = P*C).
%
%   WCMARG = WCMARGIN(L,OPT) and [WCMARGI,WCMARGO] = WCMARGIN(P,C,OPT)  
%   further specify options OPT (see WCGOPT for more details).
%   By default, WCMARGIN does not compute margin sensitivities to 
%   uncertainties.  To compute sensitivities, do
%
%      opt = wcgopt('Sensitivity','on');
%      [wcmargi,wcmargo] = wcmargin(P,C,opt)
%
%   Example: Consider a nominal 2-input, 2-output plant with 8% modeling 
%   errors at the plant outputs and a variation in the gain of the first 
%   input channel between 0.97 and 1.06. You can calculate the worst-case 
%   gain and phase margins as follows:
%       ingain1 = ureal('ingain1',1,'Range',[0.97 1.06]);
%       a = [-0.2 10;-10 -0.2];
%       b = [ingain1 0;0 1];
%       c = [1 8;-10 1];
%       d = zeros(2,2);
%       sat = ss(a,b,c,d);
%       unmod = ultidyn('unmod',[2 2],'Bound',0.08);
%       sat = (eye(2)+unmod)*sat;
%       k = [1 -2;0 1];
%       [wcmi,wcmo] = wcmargin(sat,k)
%
%   See also LOOPMARGIN, DMPLOT, LOOPSENS, ROBUSTSTAB, WCGAIN, WCGOPT, 
%            WCSENS.           

%   Copyright 2003-2004 The MathWorks, Inc.
%   $Revision: 1.1.6.2 $ $Date: 2004/12/26 21:48:20 $

wco=[];
nin = nargin;
if nin == 1 % must be Loop Transfer Function, square, C=I, 
   szp = size(P);
   if szp(1)==szp(2) 
      C = eye(szp(1),szp(1));
      opts = wcgopt;
   else
      error('Loop Transfer Function must be square');
   end
elseif nin == 2 % either Loop Transfer Function with OPT or P,C
   szp = size(P);
   if isa(C,'wcgopt') || isempty(C) % must be  Loop Transfer Function with OPT
      if szp(1)==szp(2)
         opts = C;
         C = eye(szp(1),szp(1));
      else
         error('Loop Transfer Function must be square');
      end
   else
      opts = [];
   end
end

if  ~isempty(opts)
   if  ~isa(opts,'wcgopt')
      error('The OPT input argument must a WCGOPT object.');
   end
end

szp = size(P);
szc = size(C);
if szc(1)==szp(2) & szp(1)==szc(2)
   nu = szp(2);
   ny = szp(1);
   [info,sntisnto] = loopsens(P,C); 
%   if ~(isa(sntisnto,'ufrd') || isa(sntisnto,'uss'))
    if ~(isa(sntisnto,'frd') || isa(sntisnto,'ss') || isa(sntisnto,'tf') || ...
          isa(sntisnto,'zpk') || isa(sntisnto,'ufrd') || isa(sntisnto,'uss'))
      error('P or C must be either be a USS or UFRD object.');
   end
   if ~all(isnan(info.Stable(:))) && all(info.Stable(:)==0)
      [WCDiskI,WCDiskO] = LOCALunstable;
      return
   end
   % Input Margins
   L.type = '()';
   L.subs = {1:2*nu,1:nu};
   mtmp = subsref(sntisnto,L);
   SpTi = [eye(nu) eye(nu)]*mtmp;
   if isa(SpTi,'ss') || isa(SpTi,'zpk') || isa(SpTi,'tf')
      [mag,ph,omega,FocusInfo] = genfresp(SpTi(:,:,1),4,[]);
      SpTig = frd(SpTi,omega);
   elseif isa(SpTi,'uss')
      [mag,ph,omega,FocusInfo] = genfresp(get(SpTi(:,:,1),'NominalValue'),4,[]);
      if length(omega)>15
         omega = omega(4:end-4);
      end
      SpTig = frd(SpTi,omega);
   elseif isa(SpTi,'frd') || isa(SpTi,'ufrd')
      SpTig = SpTi;
   end
   sz = size(sntisnto);
   exd = sz(3:end);
   pexd = prod(exd);
   WCDiskI = cell(nu,1);
   GMCell = {};
   PMCell = {};
   WCell = {};
   SStruc = [];
   if isempty(opts)
      opts = wcgopt('ArrayDimPtWise',1:length(sz));
   end
   opts.FreqPtWise = 0; % finding just the PEAK over frequency
   for i = 1:nu
      L.type = '()';
      L.subs = {i,i};
      tmp = subsref(SpTig,L);
      [wcmaxgi,WCNamesValuesi,Infoi] = wcgain(tmp,opts,[],[],info.Stable);
      wcWi = wcmaxgi.CriticalFrequency;
%       if isa(SpTi(i,i),'ufrd') || isa(SpTi(i,i),'uss')
         %Perhaps we should use the upper bound to be more conservative?
         maxg = wcmaxgi.LowerBound;
         ming = wcmaxgi.UpperBound;
%       else
%          maxg = wcmaxgi;
%          ming = wcmaxgi;
%       end
      %Limit max GM to inf and PM to 90 deg for Uncertain  cases
      [WCDiskGMi,WCDiskPMi] = LOCALminmg(maxg,1.001);
      wcsisomarg.kmi{i} = maxg;
      if strcmpi(opts.Sensitivity,'On')
         % Why not just return the Sensitivity info from WCGAIN?
%          wcSensi = LOCALsens(maxg,Infoi.Sensitivity,opts.VaryUncertainty/100);
%          wcSensi = reshape(wcSensi,[1 prod(exd)]).';
         wcSensi = Infoi.Sensitivity;
      else
         wcSensi = [];
      end
      WCDiskGMi = cat(3,1./WCDiskGMi,WCDiskGMi);
      WCDiskGMi = reshape(WCDiskGMi,[pexd 2]);
      WCDiskPMi = cat(3,-WCDiskPMi,WCDiskPMi);
      WCDiskPMi = reshape(WCDiskPMi,[pexd 2]);
      wcWi = reshape(wcWi,[1 pexd]).';
      GMCell = [GMCell mat2cell(WCDiskGMi,ones(prod(exd),1),2)];
      PMCell = [PMCell mat2cell(WCDiskPMi,ones(prod(exd),1),2)];
      WCell  = [WCell mat2cell(wcWi,ones(prod(exd),1),1)];
      if isempty(wcSensi)
         SStruc = [];
      else
         if i==1
            SStruc = wcSensi;
         else
            SStruc = cat(length(exd)+1,SStruc,wcSensi);
         end
      end
   end
   if ~isempty(exd)
      GMCell = reshape(GMCell',[nu exd]);
      PMCell = reshape(PMCell',[nu exd]);
      WCell  = reshape(WCell,[nu exd]);
   else
      GMCell = reshape(GMCell',[nu 1]);
      PMCell = reshape(PMCell',[nu 1]);
      WCell  = reshape(WCell,[nu 1]);
   end
   wci = struct('GainMargin',GMCell,'PhaseMargin',PMCell,...
                           'Frequency',WCell,'Sensitivity',[]);
   if ~isempty(SStruc)
      if ~isempty(exd)
         SStruc = permute(SStruc,[length(exd)+1 1:length(exd)]);
      end
      for i=1:prod(size(SStruc))
         wci(i).Sensitivity = SStruc(i);
      end
   end
%    if nu==1 && ~isempty(exd)
%       wci = reshape(wci,[exd]);
%    end

   if nargout == 2
      % Output Margins
      L.type = '()';
      L.subs = {2*nu+1:2*nu+2*ny,nu+1:nu+ny};
      mtmp = subsref(sntisnto,L);
      SpTo = [eye(ny) eye(ny)]*mtmp;
      if isa(SpTi,'frd') || isa(SpTi,'ufrd')
         SpTog = SpTo;
      else
         SpTog = frd(SpTo,omega);
      end
      GMCell = {};
      PMCell = {};
      WCell = {};
      SStruc = [];
      for i = 1:ny
         L.type = '()';
         L.subs = {i,i};
         tmp = subsref(SpTog,L);
         [wcmaxgo,WCNamesValueso,Infoo] = wcgain(tmp,opts,[],[],info.Stable);
         wcWo = wcmaxgo.CriticalFrequency;
         maxg = wcmaxgo.LowerBound;
         if strcmpi(opts.Sensitivity,'On')
            % Why not just return the Sensitivity info from WCGAIN?
            wcSenso = Infoo.Sensitivity;
         else
            wcSenso = [];
         end         
         %Limit max GM to inf and PM to 90 deg for Uncertain cases
         [WCDiskGMo,WCDiskPMo] = LOCALminmg(maxg,1.001);
         WCDiskGMo = cat(3,1./WCDiskGMo,WCDiskGMo);
         WCDiskGMo = reshape(WCDiskGMo,[pexd 2]);
         WCDiskPMo = cat(3,-WCDiskPMo,WCDiskPMo);
         WCDiskPMo = reshape(WCDiskPMo,[pexd 2]);
         wcWo = reshape(wcWo,[1 pexd]).';
         GMCell = [GMCell mat2cell(WCDiskGMo,ones(prod(exd),1),2)];
         PMCell = [PMCell mat2cell(WCDiskPMo,ones(prod(exd),1),2)];
         WCell  = [WCell mat2cell(wcWo,ones(prod(exd),1),1)];
         if isempty(wcSenso)
            SStruc = [];
         else
            if i==1
               SStruc = wcSenso;
            else
               SStruc = cat(length(exd)+1,SStruc,wcSenso);
            end
         end
      end
      if ~isempty(exd)
         GMCell = reshape(GMCell',[ny exd]);
         PMCell = reshape(PMCell',[ny exd]);
         WCell  = reshape(WCell,[ny exd]);
      else
         GMCell = reshape(GMCell',[ny 1]);
         PMCell = reshape(PMCell',[ny 1]);
         WCell  = reshape(WCell,[ny 1]);
      end
      wco = struct('GainMargin',GMCell,'PhaseMargin',PMCell,...
                              'Frequency',WCell,'Sensitivity',[]);
      if ~isempty(SStruc)
         if ~isempty(exd)
            SStruc = permute(SStruc,[length(exd)+1 1:length(exd)]);
         end
         for i=1:prod(size(SStruc))
            wco(i).Sensitivity = SStruc(i);
         end
      end
%       if nu==1 && ~isempty(exd)
%          wco = reshape(wco,[exd]);
%       end
   end
else
   error('I/O dimensions of P don''t match feedback channel widths in C.')
end

function [GM,PM] = LOCALminmg(mg,Bnd);

idx = find(mg<=Bnd);
if ~isempty(idx)
   GM = zeros(size(mg));
   GM(idx) = inf;
   PM = zeros(size(mg));
   PM(idx) = 90;
   gidx = 1:prod(size(mg));
   gidx(idx) = [];
   if ~isempty(gidx)
      GM(gidx) = (1+1./mg(gidx))./(1-1./mg(gidx));
      PM(gidx) = (180/pi)*2*atan(1./mg(gidx));
   end
else
   GM = (1+1./mg)./(1-1./mg);
   PM = atan(1./mg);
   PM = (180/pi)*2.*PM;
end

% The following Local subroutine is currently not called. GJB 15Jun04 
function [osens] = LOCALsens(mg,sens,gamma);

osens.PM = sens;
osens.GM = sens;
% FX maybe inf in locations. This leads to a "Divide by zero" Warning
infloc = find(mg==inf);
fx = mg;
fx(infloc) = 100000;
fxi = 1./fx;
GM = (1+fxi)./(1-fxi);
PM = (180/pi)*2*atan(fxi);
tmpGM = (-2./(fx-1).^2).*fx./GM;
tmpPM = (-360/pi)./(fx.^2+1).*fx./PM;
szt = size(tmpGM);
tmpGM(infloc) = nan;
tmpPM(infloc) = nan;
% Modified based on new SENS output from WCGAIN, 15Jun04 GJB
% not sure if this is correctly correct for arrays
fnames = fieldnames(sens);
for i=1:length(fnames)
   eval(['xx = sens.' fnames{1} ';']);
   tmp = abs(tmpGM.*reshape(xx,szt));
   osens.GM.Values(i,:) = round(tmp(:));
   tmp = abs(tmpPM.*reshape(xx,szt));
   osens.PM.Values(i,:) = round(tmp(:));
end

function [wci,wco] = LOCALunstable;

wci = struct('GainMargin',[],'PhaseMargin',[],'Frequency',[],...
             'Sensitivity',[]);
wco = wci;
