function nvalue = actual2normalized(atom,value)
%ACTUAL2NORMALIZED   Computes the normalized-distance between
%  an uncertain atom's nominal value and a given value.
%
%  NDIST = ACTUAL2NORMALIZED(A,V) is the normalized-distance
%  between the nominal value of the uncertain atom A and the
%  given value V.  If A is a UREAL, then NDIST may be positive
%  or negative, reflecting that V is greater than, or less than
%  the nominal value.  If A is any other class of uncertain atom,
%  then NDIST is nonnegative.
%
%  If V is an array of values, then NDIST will be an array of
%  normalized distances.
%
%  Normalized-Distance from Nominal is used in interpreting the
%  robustness margins computed by the ROBUSTSTAB and ROBUSTPERF
%  commands.
%
%  See also: NORMALIZED2ACTUAL, ROBUSTSTAB, ROBUSTPERF

% Copyright 2003-2004 The MathWorks, Inc.

if nargin<2
    error('ndist = actual2normalized(atom,v)');
end

if ~( isa(value,'ss') || isa(value,'tf')  || isa(value,'frd') || ...
        isa(value,'zpk') )
       error('For ULTIDYN, the actual value must be a SS, TF, ZPK, or FRD.');
end

HinfTol = 5e-4;
GB = strcmp(atom.Type,'GainBounded');
Bound = atom.Bound;
N = pvget(atom,'Normalizer');
szv = size(value);
nvalue = zeros([1 1 szv(3:end)]);
if isequal(szv(1:2),size(atom))
   if value.Ts==0
      for i=1:prod(szv(3:end))
         pp = pole(value(:,:,i));
         if max(real(pp))<0
            if GB
               nvalue(1,1,i) = norm(value(:,:,i),inf,HinfTol)/Bound;
            else
               natom = lft(value(:,:,i),N.T);    
               pp = pole(natom);
               if max(real(pp))<0
                  nvalue(1,1,i) = norm(natom,inf,HinfTol);
               else
                  nvalue(1,1,i) = inf;
               end
            end
         else
            nvalue(1,1,i) = inf;
         end
      end
   else
      for i=1:prod(szv(3:end))
         pp = pole(value(:,:,i));
         if max(abs(pp))<1
            if GB
               nvalue(1,1,i) = norm(value(:,:,i),inf,HinfTol)/Bound;
            else
               natom = lft(value(:,:,i),N.T);    
               pp = pole(natom);
               if max(abs(pp))<1
                  nvalue(1,1,i) = norm(natom,inf,HinfTol);
               else
                  nvalue(1,1,i) = inf;
               end
            end
         else
            nvalue(1,1,i) = inf;
         end
      end
   end
else
   error('Value is not the correct dimension for ATOM');
end
