function varargout = bode(varargin)
%BODE  Bode frequency response of Uncertain USS models.
%
%  BODE(SYS) draws sample Bode plots of the uncertain model SYS (created 
%  with either USS or UFRD). The uncertain system SYS is sampled at 10 
%  values, as well as its nominal value, and the bode response of these 
%  11 systems is computed and plotted. The frequency range and number 
%  of points are chosen automatically.
%       
%  BODE(SYS,N) samples SYS at N values, instead of the default 10.
%  The Bode plot of the nominal value is also plotted.
%
%  BODE(SYS,N,{WMIN,WMAX}) draws the Bode plots for frequencies
%  between WMIN and WMAX (in radians/second).
%
%  BODE(SYS,[],{WMIN,WMAX}) allows specification of WMIN and WMAX 
%  using the default number of 10 samples.
%
%  BODE(SYS,N,W) uses the user-supplied vector W of frequencies, in
%  radian/second, at which the Bode response is to be evaluated.  
%  See LOGSPACE to generate logarithmically spaced frequency vectors.
%
%  BODE(SYS,[],W) allows specification of the user-supplied frequency
%  vector W using the default number of 10 samples.
%
%  BODE(SYS1,SYS2,...,N,W) graphs the Bode response of multiple USS, UFRD, 
%  TF, SS, ZPK models SYS1,SYS2,... on a single plot. The number of  
%  samples N and the frequency vector W are optional.  You can specify a 
%  color, line style, and marker for each model, as in  
%       bode(sys1,'r',sys2,'y--',sys3,'gx').
%  With this conventional specification (or if no linetype is
%  specified), both the samples and the nominal will use the same
%  linetype. 
%   
%  In order to specify different line types for the samples and
%  nominal, use a slash '/' within the linetype specifier.  For
%  instance, 'r/b+:' will plot the samples in solid red, with no
%  markers, and will plot the nominal in dotted blue,with '+' sign
%  data markers.
%
%  For discrete-time models with sample time Ts, BODE uses the
%  transformation Z = exp(j*W*Ts) to map the unit circle to the 
%  real frequency axis.  The frequency response is only plotted 
%  for frequencies smaller than the Nyquist frequency pi/Ts, and 
%  the default value 1 (second) is assumed when Ts is unspecified.
% 

%  See also bodemag, nichols, nyquist, sigma, freqresp, ltiview, ltimodels.
% Copyright 2003-2004 The MathWorks, Inc.

nout = nargout;
if nout>0
   % XXXX 
   error('BODE with a USS or UFRD cannot be called with output arguments.')
end

nin = nargin;
specificW = 0;
specificN = 0;
if nin>0
   if isa(varargin{nin},'cell') % last arg specifies WMIN and WMAX
        W = varargin{nin};
        specificW = 1;
        N = varargin{nin-1};
        specificN = 1; 
        nin = nin-2;
        if nin==0
            error('Invalid W, {WMIN,WMAX} or N specification.');
        end
   elseif isa(varargin{nin},'double')
      L = varargin{nin};
      if ndims(L)==2 && min(size(L))==1
         specificL = 1;
         nin = nin - 1;
         if nin>0 && isa(varargin{nin},'double') || isempty(varargin{nin})
            L1 = varargin{nin};
            if (ndims(L1)==2 && min(size(L1))==1) || isempty(L1)
               specificN = 1;
               specificW = 1;
               N = L1;
               W = L;
               nin = nin - 1;
            else
               error('Invalid W, {WMIN,WMAX} or N specification.');
            end
         else
            specificN = 1;
            N = L;
         end
      else
         error('Invalid W, {WMIN,WMAX} or N specification.');
      end
   end
end

if specificN==0 || isempty(N)
   N = 10;
elseif ~(isa(N,'double') && ndims(N)==2 && all(size(N)==[1 1]) ...
   && floor(N)==ceil(N) && imag(N)==0 && N>0)
   error('N must be a positive integer');
end

samcell = cell(1,0);
nomcell = cell(1,0);
flt = 0;
i = 1;
while i<=nin
   switch class(varargin{i})
      case{'uss' 'ufrd'}
         flt = 0;
         prev = 1;
         tmpu = usample(varargin{i},N);
         tmpnom = pvget(varargin{i},'NominalValue');
         samcell = [samcell {tmpu}];
         nomcell = [nomcell {tmpnom}];
      case {'ss' 'tf' 'zpk' 'frd'}
         flt = 0;
         prev = 2;
         samcell = [samcell varargin(i)];
         % do nothing - varargin{i} is ok, newcell does not grow
      case {'char'}
         if flt==1
            error('Invalid argument list - successive line-types not allowed.');
         else
            flt = 1;
         end
         if prev==1
            tmpidx = find(varargin{i}=='/');
            if isempty(tmpidx)
               samcell = [samcell varargin(i)];
               nomcell = [nomcell varargin(i)];
            else  
               samcell = [samcell {varargin{i}(1:tmpidx-1)}];
               nomcell = [nomcell {varargin{i}(tmpidx+1:end)}];
            end
         elseif prev==2
            samcell = [samcell varargin(i)];
         end
       otherwise
          %keyboard
         error('BODE only supports SS, TF, ZPK, UFRD and USS objects');
   end
   i = i + 1;
end

if specificW==1
   nomcell = [nomcell {W}];
end

bode(samcell{:},nomcell{:});
   