% DIAG Diagonal uncertain matrices and diagonals of an uncertain matrix.
%    If V is an uncertain vector, DIAG(V) puts V on the main diagonal.
%    If X is an uncertain matrix, DIAG(X) is the main diagonal of X.
%    Hence DIAG(DIAG(X)) is a diagonal uncertain matrix.
% 
%  Example
%     a = ureal('a',10)
%     V = [1+a 2 3 4]
%     MV = diag(V)
%  produces a diagonal matrix MV of size 4-by-4. Given an 
%  uncertain matrix M, an uncertain vector of the diagonal 
%  elements of M, VM are found using DIAG.
%     M = [1+a 2; 3 4]
%     VM = diag(M)
%  An uncertain matrix of the diagonal entries of VM results from 
%     MVM = diag(diag(M))
% 
%  See also APPEND, BLKDIAG.
function out = diag(m,k)
% Copyright 2003-2004 The MathWorks, Inc.
% XXXCOMMENT XXXOK XXXMINOR are we allowed to have this?
% XXXDS what about DIAG in the other classes?

if nargin==1
   k=0;
elseif isempty(k)
   out=[];
   warning('K-th diagonal input must be an integer scalar.');
   return;
elseif isa(k,'double') && ndims(k)==2 && all(size(k)==[1 1])
   k = fix(real(k));   
else
   error('k must be a scalar');
end

szm = size(m);
if szm(1)==1 || szm(2)==1   % single row or single column
   dim = max(szm(1:2));
   
   out = [];
   if szm(1)==1
      for i=1:dim
         S(1).type='()';
         S(1).subs={1,i};
         out = blkdiag(out,subsref(m,S));
      end
   else
      for i=1:dim
         S(1).type='()';
         S(1).subs={i,1};
         out = blkdiag(out,subsref(m,S));
      end
   end

   if k>0
      out = [zeros(szm,k) out; zeros(k,k) zeros(k,szm)];
   elseif k<0
      k=abs(k);
      out = [zeros(k,szm) zeros(k,k); out zeros(szm,k)];      
   end
   
else % grab diagonal entries of a matrix
   
   fr = pvget(m,'Frequency');
   nfr = length(fr);
   Ts = pvget(m,'Ts');

   out = [];
   if k>=0
      if k+1>szm(2)
         out = ufrd(umat(zeros(0,1,nfr)),fr,Ts); % XXXX
      else
         for i=(k+1):min([szm(2) k+szm(1)])
            S(1).type='()';
            S(1).subs={i-k,i};
            out = vertcat(out,subsref(m,S));
         end  
      end
   else
      k = abs(k);
      if k+1>szm(1)
         out = ufrd(umat(zeros(0,1,nfr)),fr,Ts); % XXXX
      else  
         for i=(k+1):min([szm(1) k+szm(2)])
            S(1).type='()';
            S(1).subs={i,i-k};
            out = vertcat(out,subsref(m,S));
         end            
      end
   end
   
end