function out = ultidyn(name,sza,varargin)
%ULTIDYN   Creates an uncertain linear time-invariant object.
%
%  H = ULTIDYN('NAME',IOSIZE) creates an uncertain, linear,
%  time-invariant dynamic object, with name 'NAME', and of
%  size IOSIZE, [number-of-outputs number-of-inputs], which
%  is real, scalar, 1-by-2 DOUBLE.
%
%  The property TYPE is 'PositiveReal' or 'GainBounded' (default).
%  The property BOUND is a real, scalar, which describes the
%  possible behaviors modeled by the uncertain object.
%    If TYPE is 'PositiveReal', then H+H' >= 2*BOUND;
%    If TYPE is 'GainBounded', then, || H || <= BOUND.
%  
%  Trailing Property/Value pairs are allowed in the construction.
%  H = ULTIDYN(NAME,IOSIZE,'P1',V1,'P2',V2,...) 
%
%  H = ULTIDYN('H',[2 3],'Type','GainBounded','Bound',7) creates
%  a ULTIDYN object with internal name 'H', dimension 2-by-3, 
%  norm bounded by 7.
%
%  The property SAMPLESTATEDIM is a positive integer, defining
%  the state dimension of random samples of the uncertain object
%  when sampled with USAMPLE.  The default value is 1.
%
%  The property AUTOSIMPLIFY controls how expressions involving the real 
%  parameter are simplified. Its default value is 'basic', which means 
%  elementary methods of simplification are applied as operations are 
%  completed.  Other values for AUTOSIMPLIFY are 'off' (no simplification 
%  performed) and 'full' (model-reduction-like techniques are applied).  
%
%  % EXAMPLE (CUT/PASTE)
%  % create MIMO (3-by-4) process model
%  P = rss(5,3,4);   % 5 states, 3 outputs, 4 inputs
%  % add 20% multiplicative uncertainty to the input of P
%  Punc = P*(eye(4) + ultidyn('DeltaM',[4 4],'Bound',0.2));
%
%  See also: UREAL, GET(ULTIDYN), USS.

%  The property SAMPLESTATENAMING (default is 'on') controls whether
%  or not random samples of the object have a nonempty StateName
%  property.

% Copyright 2003-2004 The MathWorks, Inc.

superiorto('ss','tf','zpk','frd');
if nargin==0
   str = int2str(ceil(9999*rand));
   lstr = length(str);
   pdd = '0'*ones(1,6-lstr);
   name = ['UNNAMED_tag_' pdd str];
   name = ['UNNAMED'];
   A = atom(name,0);
   out.Type = 'GainBounded';
   out.Bound = 1;
   out.SampleStateDim = 1;
   out.SampleStateNaming = 'on';
   out = class(out,'ultidyn',A);
elseif nargin>=1
   if nargin==1
      sza = [1 1];
   end
   A = atom(name,zeros(sza));
   out.Type = 'GainBounded';
   out.Bound = 1;
   out.SampleStateDim = 1;
   out.SampleStateNaming = 'on';
   out = class(out,'ultidyn',A);
   
   nin = nargin-2;
   endqueue = [];
   for i=1:nin/2
      if strncmpi('Type',varargin{2*(i-1)+1},length(varargin{2*(i-1)+1}))
         out = set(out,varargin{2*(i-1)+1},varargin{2*i});
      else
         endqueue = [endqueue;i];
      end
   end
   for i=1:length(endqueue)
      ii = endqueue(i);
      out = set(out,varargin{2*(ii-1)+1},varargin{2*ii});
   end
end
