function sysout = feedback(s1,s2,varargin)
%  FEEDBACK  Feedback connection of two uncertain system models. 
%  
%     USYS = FEEDBACK(USYS1,USYS2) computes an uncertain system USYS for
%     the closed-loop feedback system
%  
%            u --->O---->[ USYS1 ]----+---> y
%                  |                  |           y = USYS * u
%                  +-----[ USYS2 ]<---+
%  
%     Negative feedback is assumed and the resulting system USYS 
%     maps u to y.  To apply positive feedback, use the syntax
%     USYS = FEEDBACK(USYS1,USYS2,+1).
%  
%     USYS = FEEDBACK(USYS1,USYS2,FEEDIN,FEEDOUT,SIGN) builds the more
%     general feedback interconnection:
%                        +--------+
%            v --------->|        |--------> z
%                        |  USYS1  |
%            u --->O---->|        |----+---> y
%                  |     +--------+    |
%                  |                   |
%                  +-----[  USYS2  ]<--+
%  
%     The vector FEEDIN contains indices into the input vector of USYS1
%     and specifies which inputs u are involved in the feedback loop.
%     Similarly, FEEDOUT specifies which outputs y of USYS1 are used for
%     feedback.  If SIGN=1 then positive feedback is used.  If SIGN=-1 
%     or SIGN is omitted, then negative feedback is used.  In all cases,
%     the resulting USS model USYS has the same inputs and outputs as USYS1 
%     (with their order preserved).
%  
%     If USYS1 and USYS2 are arrays of USS models, FEEDBACK returns an USS
%     array USYS of the same dimensions where 
%        USYS(:,:,k) = FEEDBACK(USYS1(:,:,k),USYS2(:,:,k)) .
%  
%     See also ICONNECT, LFT, PARALLEL, SERIES, SYSIC.

% Copyright 2004 The MathWorks, Inc.

if isa(s1,'umat') && isa(s2,'umat')
   sysout = depromote(feedback(uss(s1),uss(s2),varargin{:}));
else
   sysout = feedback(umat(s1),umat(s2),varargin{:});
end