function [out,NVout] = gridureal(sys,varargin)
% GRIDUREAL   Grid UREAL atoms over their range
%
%    B = GRIDUREAL(A,N) substitutes N samples of the uncertain
%    real parameters in A returning an array of size [SIZE(A) N].  
%    The N samples are generated by uniformly gridding each UREAL 
%    in A with N points (N=1 uses the nominal of each UREAL) and then
%    evaluating A at the N "diagonal" points in the parameter space.
%    For example, if (a,b,c,d) are four ureals in A then A is
%    evaluated at:
%       (a(1),b(1),c(1),d(1)) ; ...; (a(N),b(N),c(N),d(N))
%    If A includes uncertain objects other than uncertain reals, then 
%    B will be an uncertain object. GRIDUREAL(A) is the same as 
%    GRIDUREAL(A,1). 
%
%    [B,SampleValues] = GRIDUREAL(A,N) additionally returns the specific
%    sampled values (as a STRUCT whose fieldnames are the names of A's
%    uncertain elements) of the uncertain reals.  Hence, B is the
%    same as USUBS(A,SampleValues).
%
%    [B,SampleValues] = GRIDUREAL(A,NAMES,N) samples only the uncertain
%    reals listed in the NAMES variable (CELL, or CHAR array). Any
%    entries of NAMES that are not elements of A are simply ignored. 
%    Note that GRIDUREAL(A,fieldnames(A.Uncertainty),N) is the same 
%    as GRIDUREAL(A,N).
%
%    [B,SampleValues] = GRIDUREAL(A,NAMES1,N1,NAMES2,N2,...) takes N1 
%    samples of the uncertain elements listed in NAMES1, and N2 samples of
%    the uncertain elements listed in NAMES2 and so on.  SIZE(B) will
%    equal [SIZE(A) N1 N2 ...].
% 
%    % EXAMPLE 1: (CUT/PASTE)
%    % Create two UREALS
%    gamma = ureal('gamma',4);
%    tau = ureal('tau',.5,'Percentage',30);
%    % Create Uncertain Plant
%    P = tf(gamma,[tau 1]);
%    % Create Integral Controller based on nominal plant parameters
%    KI = 1/(2*tau.Nominal*gamma.Nominal);
%    C = tf(KI,[1 0]);
%    % Create Uncertain Closed-Loop System
%    CLP = feedback(P*C,1);
%    % Plot (many) Uncertain Plant step responses 
%    subplot(2,1,1); step(gridureal(P,20))
%    % Plot (many) Uncertain Closed-Loop step responses
%    subplot(2,1,2); step(gridureal(CLP,20))
% 
%    % EXAMPLE 2: (CUT/PASTE)
%    % Make 4 UREALs, and a simple uncertain matrix
%    a=ureal('a',1); b=ureal('b',2); c=ureal('c',3); d=ureal('d',4);
%    m = [a b;c d];
%    % Sample the (a,b) space in 5 places, and the (c,d) space in 3
%    % places, and evaluate m at the 15 "grid"-points 
%    m1 = gridureal(m,{'a';'b'},5,{'c';'d'},3);
%    % Sample the (a,b,c,d) space at 15 places, and evaluate m at
%    % these 15 points 
%    m2 = gridureal(m,{'a';'b';'c';'d'},15);
%    % The 2,1 entry of m1 (which is c) only takes on 3 distinct values,...
%    subplot(2,1,1)
%    hist(m1(2,1,:))
%    % while the 2,1 entry of m2 (which is also c) takes
%    % on 15 distinct values uniformly through its range
%    subplot(2,1,2)
%    hist(m2(2,1,:))
%
%    See also USAMPLE, USUBS.

% Copyright 2003-2004 The MathWorks, Inc.
nin = nargin;
okpas = 1;
if nargin>=2 && isequal(varargin{end},-1)
   okpas = 0;
   nin = nin - 1;
end
if nin==1
   names = pvget(sys,'UNames');
   Ln = max([1 length(names)]);
   N = num2cell(repmat(1,[Ln 1]));
   grouping = ones(Ln,1);
elseif nin==2
   if isa(varargin{1},'double') && ndims(varargin{1})==2 && ...
         imag(varargin{1})==0 && floor(varargin{1})==ceil(varargin{1}) && ...
         varargin{1}>0
      names = pvget(sys,'UNames');
      Ln = max([1 length(names)]);
      N = num2cell(repmat(varargin{1},[Ln 1]));
      grouping = ones(Ln,1);
   else
      error('In 2-argument call, the 2nd argument should be a positive integer.');
   end
elseif ceil((nin+1)/2)==floor((nin+1)/2)
   [names,N,grouping] = nvlist(varargin{1:nin-1});
else
   error('Invalid Inputs');
end
[dum,uslots] = unique(grouping);
Nslots = [N{uslots}];
Nslots = Nslots(:)';
Fslots = ones(size(Nslots));

% Suppose A is 6x7, then B=gridureal(A,'a',9) gives B=6x7x9(x1), 
% then C = gridureal(B,'fred',4) might give C=6x7x9x1x4, or should
% it give 6x7x9x4.  Well, it should certainly give the same as
% Ctilde = usmaple(A,'a',9,'fred',4), which would have to be 6x7x9x4.
% This implies that ALL trailing one array dimensions are ignored!
% This creates one problem though.  gridureal(A,'p1',N,'p2',3) will
% be different dimension than gridureal(gridureal(A,'p1',N),'p2',3)
% if N==1, but will be the same dimension if N>1.
sz = size(sys);
nad = length(sz) - 2;
ad = sz(3:end);
if nad==2 && ad(2)==1
   nad = 1;
   ad = ad(1);
end
adpad = ones(1,nad);

% This is the only difference between the UMAT and USS/UFRD code.
% Its probably against TMW policy, but I'm making them all the same
% for simplicity.
TS = [];
if isa(sys,'uss') || isa(sys,'ufrd')
   TS = utarbts(pvget(sys,'Ts'));
end

L = length(names);
values = cell(L,1);
cvalues = cell(L,1);
sysU = pvget(sys,'Uncertainty'); 
for i=1:L
   [ac,pflag,estr] = pvget(sysU,names{i});
   if pflag==1
      Fslots(grouping(i)) = Nslots(grouping(i));
   end
end
Nslots = Fslots;

keepnames = [];
NGroups = grouping(end);
for i=1:L
   [ac,pflag,estr] = pvget(sysU,names{i});
   if pflag==1
      Nslotstmp = Nslots;
      Nslotstmp(grouping(i)) = 1;
      keepnames = [keepnames;i];
      tt = pvget(ac,'Traits');
      if tt.linear==0
         error('Can only sample uncertain objects linear uncertainty')
      end
      szd = size(ac);
      nexd = length(sz) - 2 + (grouping(i)-1);
      pad = ones(1,nexd);
      if strcmp(class(ac),'ureal')
         tmp=pvget(ac,'Range');
         if N{i}==1
             % Return the nominal for gridding of one point
             vv = reshape( sum(tmp)/2  ,[1 1 pad N{i}]);
         else   
             vv = reshape( linspace(tmp(1),tmp(2),N{i}) ,[1 1 pad N{i}]);
         end    
         if okpas==1
            vvg = repmat(vv,[1 1 adpad Nslotstmp]);
            cvv = reshape(num2cell(vvg,[1 2]),[adpad Nslots 1]);
         end
      else
          error('Can only grid uncertain real parameters');
      end   
      values{i} = vv;
      if okpas==1
         cvalues{i} = cvv;
      end
   end
end
%out = usubs(sys,names(keepnames),values(keepnames));
if okpas==1
   PNVcell = [names(keepnames) cvalues(keepnames)]';
else
   PNVcell = [names(keepnames) values(keepnames)]';
end
NVout = struct(PNVcell{:});
out = usubs(sys,NVout);


% OLD GRIDUREAL CODE: Updated to mimic usample
%
% b = gridureal(a,atomname1,n1,atomname2,n2,...)
% 
% Replaces each named UREAL object in A with a uniform grid over its 
% range, consisting of N points.  Each instance leads to an extra array
% dimension in B.  If A contains only UREAL atoms, and all are
% gridded, then the result B will be of class DOUBLE, FRD, or SS,
% depending on A.  The extra array dimensions of B are in the
% order specified in the argument list.
% 
% if nargin>1
%    [blkid,values] = nvlist(varargin{:});
% else
%    get(A)
%    blkid = pvget(A,'UNames');
%    % XXXMINOR what are we doing here?  Why 10?  What if the
%    % atom is not a UREAL?
%    values = num2cell(repmat(10,[length(blkid) 1]));
% end
% if isa(A,'ureal')
%    A = umat(A);
% elseif ~isa(A,'uss') & ~isa(A,'umat') & ~isa(A,'ufrd')
%    error('A must be an uncertain object');
% end
% 
% B = A;
% for i=1:length(blkid)
%    szb = size(B);
%    exd = szb(3:end);
%    nexd = length(exd);
%    range = get(get(B.Unc,blkid{i}),'Range'); %XXXMAJOR, what if named atom is not UREAL?
%    val = linspace(range(1),range(2),values{i}); % DOUBLE
%    aval = reshape(val,[1 1 ones(1,nexd) values{i}]);
%    B = usubs(B,blkid{i},aval);
% end

%XXX Why are we doing this below?  It really makes no sense.
%  We are simply enforcing???  If we want singletons whereever
% there was actually no dependence, then we don't need to do this.
% i believe we have that in the above USUBS call.  Suppose convention
% is that in Names1,N1,Names2,N2, etc that if NamesK contains
% no active names, then a singleton slot is created.  I beleive then
% we are done, and there is no need for the conditional activity below.
% we need to change NSLOTS based on what is found.
% if isempty(keepnames) || max(grouping(keepnames))<NGroups
%    disp('in last conditional...');
%    lastdim = length(sz) + NGroups;
%    repidx = [ones(1,lastdim-1) N{end}];
%    if isa(out,'ss') %XXXCTB should fix this by adding REPMAT
%       out = pvget(repmat(uss(out),repidx),'NominalValue');
%    else
%       out = repmat(out,repidx);
%    end
% end
