function sysout = series(s1,s2,varargin)
%SERIES  Series interconnection of the two uncertain models.
%
%                                  +------+
%                           v2 --->|      |
%                  +------+         USYS2 |-----> y2
%                  |      |------->|      |
%         u1 ----->|      |y1   u2 +------+
%                  |USYS1 |
%                  |      |---> z1
%                  +------+
%
%   USYS = SERIES(USYS1,USYS2,OUTPUTS1,INPUTS2) connects two USS models 
%   USYS1 and USYS2 in series such that the outputs of USYS1 specified by
%   OUTPUTS1 are connected to the inputs of USYS2 specified by INPUTS2.  
%   The vectors OUTPUTS1 and INPUTS2 contain indices into the outputs 
%   and inputs of USYS1 and USYS2, respectively.  The resulting USS model 
%   USYS maps u1 to y2.
%
%   If OUTPUTS1 and INPUTS2 are omitted, SERIES connects USYS1 and USYS2
%   in cascade and returns
%                     USYS = USYS2 * USYS1 .
%
%   If USYS1 and USYS2 are arrays of USS models, SERIES returns an LTI
%   array USYS of the same size where 
%      USYS(:,:,k) = SERIES(USYS1(:,:,k),USYS2(:,:,k),OUTPUTS1,INPUTS2) .
%
%   See also APPEND, PARALLEL, FEEDBACK, ICONNECT, SYSIC.

% Copyright 2004 The MathWorks, Inc.

if isa(s1,'umat') && isa(s2,'umat')
   sysout = depromote(series(uss(s1),uss(s2),varargin{:}));
else
   sysout = series(umat(s1),umat(s2),varargin{:});
end