function out = ss(varargin)
%SS  Creates state-space model or converts model to state space.
%       
%   SYS = SS(A,B,C,D) creates a SS object SYS representing the 
%   continuous-time state-space model
%         dx/dt = Ax(t) + Bu(t)
%          y(t) = Cx(t) + Du(t)
%   You can set D=0 to mean the zero matrix of appropriate dimensions.
%   If one or more of the matrices A,B,C,D have uncertainty, SS returns 
%   an uncertain state-space (USS) model (Robust Control Toolbox only).
%
%   SYS = SS(A,B,C,D,Ts) creates a discrete-time state-space model with  
%   sample time Ts (set Ts=-1 if the sample time is undetermined).
%
%   SYS = SS creates an empty SS object.
%   SYS = SS(D) specifies a static gain matrix D.
%
%   In all cases above, the input list can be followed by pairs
%      'PropertyName1', PropertyValue1, ...
%   that set various model properties (see LTIPROPS for details).
%
%   You can create arrays of state-space models by using ND arrays for
%   A,B,C,D.  The first two dimensions of A,B,C,D define the 
%   number of states, inputs, and outputs, while the remaining 
%   dimensions specify the array sizes.  For example, 
%      sys = ss(rand(2,2,3,4),[2;1],[1 1],0)
%   creates a 3x4 array of SISO state-space models.  You can also
%   use indexed assignment and STACK to build SS arrays:
%      sys = ss(zeros(1,1,2))     % create 2x1 array of SISO models
%      sys(:,:,1) = rss(2)        % assign 1st model
%      sys(:,:,2) = ss(-1)        % assign 2nd model
%      sys = stack(1,sys,rss(5))  % add 3rd model to array
%
%   SYS = SS(SYS) converts a system SYS of arbitrary class to state 
%   space, that is, computes a state-space realization of SYS.
%
%   SYS = SS(SYS,'min') computes a minimal realization of SYS.
%
%   See also LTIMODELS, LTIPROPS, DSS, RSS, DRSS, SSDATA, TF, ZPK, FRD.

% Copyright 2003-2004 The MathWorks, Inc.
%   $Revision: 1.1.6.2 $  $Date: 2004/08/25 19:13:02 $
try
   out = uss(varargin{:});
catch
   rethrow(lasterror);
end

