function C = stack(dim,varargin)
% STACK  Stack UMAT objects into UMAT array.
% 
%  MATU = STACK(ARRAYDIM,MATU1,MATU2,...) produces an array of UMAT
%  MATU by stacking the UMAT MATU1,MATU2,... along
%  the array dimension ARRAYDIM.  All UMATs must have the same 
%  number of inputs and outputs.
% 
%  For example, if MATU1 and MATU2 are two UMATs with the 
%  same I/O dimensions,
%    * STACK(1,MATU1,MATU2) produces a 2-by-1 UMAT array
%    * STACK(2,MATU1,MATU2) produces a 1-by-2 UMAT array
%    * STACK(3,MATU1,MATU2) produces a 1-by-1-by-2 UMAT array.
% 
%  You can also use STACK to concatenate UMAT arrays MATU1,MATU2,...
%  along some array dimension ARRAYDIM.
% 
%  See also HORZCAT, VERTCAT, APPEND.

% Copyright 2003-2004 The MathWorks, Inc.

if nargin<2
   error('Need at least 2 arguments.');
end
if ~(isa(dim,'double') && ndims(dim)==2 && all(size(dim)==[1 1]) && ...
      dim>0 && floor(dim)==ceil(dim))
   error('DIM should be a positive integer.');
end
if nargin==2
   C = varargin{1};
elseif nargin==3
   if isa(varargin{1},'umat') & isa(varargin{2},'umat')
      A = varargin{1};
      B = varargin{2};
      try
         C = umat(stack(dim,A.NDlftData,B.NDlftData));
      catch
         rethrow(lasterror);
      end   
   else
      C = stack(dim,umat(varargin{1}),umat(varargin{2}));
   end
elseif nargin>3
   C = stack(dim,stack(dim,umat(varargin{1}),umat(varargin{2})),varargin{3:end});
end


