function out = umat(a1,a2,a3)
%UMAT   Create uncertain matrices.
%   UMAT is the constructor for UMAT (uncertain matrix) objects. 
%   It converts a variety of data types into UMAT objects. Uncertain 
%   matrices are building blocks for uncertain models and are 
%   useful for worst-case gain and robustness analysis.
%
%   There are two ways to create UMAT objects. 
%   1.  Create uncertain elements (ATOM objects) and combine them:
%
%       p = ureal('p',1); 
%       M = [0 p; 1 p^2];
%       M is a 2-by-2 uncertain matrix (UMAT object).
%
%   2.  Create a UMAT object directly by recasting doubles and LTI objects:
%   
%       M = umat(1);
%       Here M is a 1-by-1 matrix (UMAT object) with no uncertainty.
%
%   See also UREAL, UCOMPLEX, METHODS(UMAT), USAMPLE, USUBS
 
%   Copyright 2003-2004 The MathWorks, Inc.

%XXXOK, XXXCOMMENT, XXXPRECEDENCE (frd), XXXMINOR 
% Standard promotions to UMAT
%
superiorto('double',...
   'ureal','ucomplex','ucomplexm','udyn','ultidyn',...
   'atom',...
   'ss','tf','zpk','frd','ndlft');

validate = 1;
nin = nargin;
if nargin==2
   validate = a2;
   nin = nargin - 1;
end

mkatend = 1;
if nin==1
   switch class(a1)
   case {'umat'}
      out = a1;
      mkatend = 0;
      validate = 0;
   case {'ureal' 'ucomplex' 'ucomplexm' 'double'}
      a1 = ndlft(a1);
      validate = 1;
   case {'udelay'}
      a1 = ndlft(a1);
      validate = 1;
   case {'ndlft'}
      validate = 0;
   case {'ultidyn' 'udyn'}
      %XXXX perhaps these should immediately go to USS. CANT
      % right now, they are just promoted to UMAT, as in the
      % 5 atom cases above.
      a1 = ndlft(a1);
      validate = 1;
   case {'uss'}
      out = a1;
      mkatend = 0;
      validate = 0;
   case {'ss' 'tf' 'zpk'}
      out = uss(a1);
      mkatend = 0;
      validate = 0;
   case {'frd'}
      out = ufrd(a1);
      mkatend = 0;
      validate = 0;
   otherwise
      error('Invalid UMAT conversion');
   end
else
   % no validation on creating EMPTY
   validate = 0;
   mkatend = 0;
   out.NDlftData = ndlft;
   out = class(out,'umat',gsref(0));
end
% Create OUT, if necessary
if mkatend
   % XXXX check if NDLFTDATA has any dynamic blocks. don't bother
   out.NDlftData = a1;
   out = class(out,'umat',gsref(0));
end
% Check validity of OUT
if validate==1
   [pflag,out] = isvalid(out);
   if pflag==0
      error('Invalid UMAT construction');
   end
end
