function nvalue = actual2normalized(atom,avalue)
%ACTUAL2NORMALIZED   Computes the normalized-distance between
%  an uncertain atom's nominal value and a given value.
%
%  NDIST = ACTUAL2NORMALIZED(A,V) is the normalized-distance
%  between the nominal value of the uncertain atom A and the
%  given value V.  If A is a UREAL, then NDIST may be positive
%  or negative, reflecting that V is greater than, or less than
%  the nominal value.  If A is any other class of uncertain atom,
%  then NDIST is nonnegative.
%
%  If V is an array of values, then NDIST will be an array of
%  normalized distances.
%
%  Normalized-Distance from Nominal is used in interpreting the
%  robustness margins computed by the ROBUSTSTAB and ROBUSTPERF
%  commands.
%
%  See also: NORMALIZED2ACTUAL, ROBUSTSTAB, ROBUSTPERF

% Copyright 2003-2004 The MathWorks, Inc.

if nargin<2
    error('nvalue = actual2normalized(atom,avalue)');
end
if ~isa(avalue,'double') && isreal(avalue)
   error('For UREAL, the actual value must be a real double.');
end

szv = size(avalue);
nvalue = zeros(szv);
N=pvget(atom,'Normalizer');
TotalRange = N.TotalRange;
T11 = N.T(1,1);
T12 = N.T(1,2);
T21 = N.T(2,1);
T22 = N.T(2,2);

if isempty(N.Tarb)
  %nvalue = lft(avalue,N.T);    
  nvalue=T22+T12*T21*avalue./(1-T11*avalue);
else
  Tarb11 = N.Tarb(1,1);
  Tarb12 = N.Tarb(1,2);
  Tarb21 = N.Tarb(2,1);
  Tarb22 = N.Tarb(2,2);
  %nvalue = lft(lft(avalue,N.Tarb),N.T);
  tmp = Tarb22+Tarb12*Tarb21*avalue./(1-Tarb11*avalue);
  nvalue=T22+T12*T21*tmp./(1-T11*tmp);
end
% Fix
ltrange = find(avalue<TotalRange(1));
nvalue(ltrange) = -inf;
gtrange = find(avalue>TotalRange(2));
nvalue(gtrange) = inf;

