function out = ureal(a1,a2,varargin)
%UREAL   Create uncertain real parameters.
%   A = UREAL('NAME',NOMINALVALUE) creates an uncertain real parameter,
%   with name 'NAME', and nominal value NOMINALVALUE (real, scalar, DOUBLE).
%
%   The uncertainty (potential deviation from NOMINALVALUE) is described
%   in three different (equivalent) manners: 
%     RANGE (an interval containing NOMINALVALUE)
%     PERCENTAGE (deviation from NOMINALVALUE) 
%     PLUSMINUS (additive deviation from NOMINALVALUE)
%
%   The MODE property determines which description remains invariant
%   if the NOMINALVALUE is changed (the other descriptions are derived).   
%
%   The default MODE is PLUSMINUS, and [-1 1] is the default additive
%   uncertainty.  Hence, A = UREAL('A',5) creates a UREAL with name 'A',
%   nominal value 5, and +-1 uncertainty (ie., possible range of [4 6]).
%     
%   Trailing Property/Value pairs are allowed.
%   A = UREAL('NAME',NOMINALVALUE,'P1',V1,'P2',V2,...) sets the 
%   properties P1, P2, ... to the values V1, V2, ...
%
%   A = UREAL('NAME',NOMINALVALUE,'Range',DATA) sets the Range values
%   to that in DATA, and automatically sets MODE to RANGE.
%   Here, DATA should be a real, 1-by-2 DOUBLE, representing the
%   left and right endpoints of the interval represented by the
%   UREAL object.  It should be that DATA(1) < NOMINALVALUE < DATA(2).
%
%   A = UREAL('NAME',NOMINALVALUE,'Percentage',DATA) sets the Percentage 
%   values to that in DATA, and automatically sets MODE to PERCENTAGE.
%
%   A = UREAL('NAME',NOMINALVALUE,'PlusMinus',DATA) sets the PlusMinus 
%   values to that in DATA, and automatically sets MODE to PLUSMINUS.
%
%   A = UREAL('NAME',NOMINALVALUE,'Mode','Range','Percentage',DATA) sets 
%   the Percentage values to that in DATA, but changes the MODE to RANGE.
%
%   The property AUTOSIMPLIFY controls how expressions involving the real 
%   parameter are simplified. Its default value is 'basic', which means 
%   elementary methods of simplification are applied as operations are 
%   completed.  Other values for AUTOSIMPLIFY are 'off' (no simplification 
%   performed) and 'full' (model-reduction-like techniques are applied).  
%
%   % EXAMPLES: (CUT/PASTE)
%   % Create a UREAL with name 'a', nominal value 4.
%   a = ureal('a',4);
%   % Check that Percentage Uncertainty in a is indeed +-25%
%   a.Percentage
% 
%   % Create a UREAL with name 'b', nominal value 5, and range [2 6].
%   % Note that MODE is automatically set to 'RANGE'
%   b = ureal('b',5,'Range',[2 6]);
% 
%   % Create a UREAL by specifying a percentage of uncertainty, but force
%   % MODE to be 'RANGE'
%   c = UREAL('c',4,'Mode','Range','Percentage',25);
% 
%   See also UCOMPLEX, GET(UREAL), ULTIDYN, UMAT.

% Copyright 2003-2004 The MathWorks, Inc.


superiorto('ss','tf','zpk','frd');

if nargin>1
   IMode = 2; % 'PlusMinus';
   IPM = [-1 1];
   if isa(a1,'char') & ndims(a1)==2 & (isempty(a1) | size(a1,1)==1)
      if isempty(a1)
         a1 = LOCALgeneratename;
         error('Invalid NAME.  NAME can not be empty'); 
      end
   else
      error('Invalid NAME.  Should be a CHAR');
   end
   if ~(isa(a2,'double') & ndims(a2)==2 & all(size(a2)==[1 1]) & isreal(a2))
      %if isa(a2,'char') & ndims(a2)==2 & size(a1,1)==1
      %   if ceil(nargin/2)==floor(nargin/2)
      error('Invalid NOMINALVALUE.  NOMINALVALUE should be real, scalar');
   end
   IData = [a2+IPM IMode];
   A = atom(a1,a2); % use default for AutoSimplify
   out.Data = IData;
   out = class(out,'ureal',A);
   nin = nargin-2;
   endqueue = [];
   for i=1:nin/2
      if strncmpi('Mode',varargin{2*(i-1)+1},length(varargin{2*(i-1)+1}))
         endqueue = [endqueue;i];
      else
         out = set(out,varargin{2*(i-1)+1},varargin{2*i});
         pv = {'Range';'PlusMinus';'Percentage'};
         idx = find(strncmpi(pv,varargin{2*(i-1)+1},length(varargin{2*(i-1)+1})));
         if length(idx)==1
            out = set(out,'Mode',pv{idx});
         end
      end
   end
   for i=1:length(endqueue)
      ii = endqueue(i);
      out = set(out,varargin{2*(ii-1)+1},varargin{2*ii});
   end
elseif nargin==0
   A = atom(LOCALgeneratename,0);
   out.Data = [-1 1 2];
   out = class(out,'ureal',A);
elseif nargin==1
   error('Invalid construction syntax for UREAL objects.  At least two arguments are required.');
end

function A = LOCALgeneratename
% str = int2str(ceil(9999*rand));
% lstr = length(str);
% pdd = '0'*ones(1,6-lstr);
% A = ['UNNAMED_tag_' pdd str];
   A = 'UNNAMED';
