function varargout = bodemag(varargin)
%BODEMAG  Bode magnitude plot for Uncertain USS/UFRD models.
%
%  BODEMAG(SYS) draws sample Bode magnitude plots of the uncertain model 
%  SYS (created with either USS or UFRD). The uncertain system SYS is 
%  sampled at 20 values, as well as its nominal value, and the bode 
%  response of these 21 systems is computed and plotted. The frequency 
%  range and number of points are chosen automatically.
%
%  BODEMAG(SYS,{WMIN,WMAX}) draws the magnitude plots for frequencies
%  between WMIN and WMAX (in radians/second).
%
%  BODEMAG(SYS,W) uses the user-supplied vector W of frequencies, in
%  radian/second, at which the frequency response is to be evaluated.  
%  See LOGSPACE to generate logarithmically spaced frequency vectors.
%
%  BODEMAG(SYS1,SYS2,...,W) graphs the magnitude plot of multiple USS, 
%  UFRD, TF, SS, ZPK models SYS1,SYS2,... on a single plot. The 
%  frequency vector W are optional.  You can specify a color, line style,
%  and marker for each model, as in  
%       bode(sys1,'r',sys2,'y--',sys3,'gx').
%  With this conventional specification (or if no linetype is
%  specified), both the samples and the nominal will use the same
%  linetype. 
%
%  For discrete-time models with sample time Ts, BODEMAG uses the
%  transformation Z = exp(j*W*Ts) to map the unit circle to the 
%  real frequency axis.  The frequency response is only plotted 
%  for frequencies smaller than the Nyquist frequency pi/Ts, and 
%  the default value 1 (second) is assumed when Ts is unspecified.
% 
%  See also BODE, NYQUIST

%  See also bode, nichols, nyquist, sigma, freqresp, ltiview, ltimodels.
% Copyright 2003-2004 The MathWorks, Inc.
nin = nargin;
nout = nargout;
mname = mfilename;
if nout>0
   strerr = ' with a USS or UFRD cannot be called with output arguments.';
   error([upper(mname) strerr])
end

N = 20;  % Number of times to sample uncertain system
   
samcell = cell(1,0);
for i=1:nin
    c=class(varargin{i});
    % RESHAPE to be PROD(extra dim)x1
    if isa(varargin{i},'dynamicsys')
       sys = varargin{i};
       sz = size(sys);
       nad = length(sz) - 2;
       ad = sz(3:end);
       if nad==2 && ad(2)==1
          nad = 1;
          ad = ad(1);
       end
       if strcmp(c,'uss') || strcmp(c,'ufrd')
           tmpu = reshape(usample(sys,N),[prod(ad)*N 1]);
           tmpnom = reshape(pvget(sys,'NominalValue'),[prod(ad) 1]);
           tmpu = stack(1,tmpu,tmpnom);         
           samcell = [samcell {tmpu}];
       else
           % Do nothing and let lti version do the error checking
           samcell = [samcell {sys}];
       end 
    else
    % Do nothing and let lti version do the error checking
       samcell = [samcell varargin(i)];
    end
 end
eval([mname '(samcell{:});']);
  