function sys = connect(varargin)
%CONNECT  Arbitrary interconnection of USS models.
%
%   CONNECT constructs the aggregate model for a given block 
%   diagram interconnection of USS models.  You can specify 
%   the block diagram connectivity in two ways:
%
%   Name-based interconnection
%     In this approach, you name the input and output signals of 
%     all USS blocks SYS1, SYS2,... in the block diagram, including 
%     the summation blocks. The aggregate model SYS is then built
%     by SYS = CONNECT(SYS1,SYS2,...,INPUTS,OUTPUTS) where INPUTS
%     and OUTPUTS are the names of the block diagram external I/Os
%     (specified as strings or cell arrays of strings).
%
%     Example 1: Given SISO USS models C and G, you can construct 
%     the closed-loop transfer T from r to y using:
%
%                   e       u
%           r --->O-->[ C ]---[ G ]-+---> y
%               - |                 |       
%                 +<----------------+
%         
%        C.InputName = 'e';  C.OutputName = 'u';
%        G.InputName = 'u';  G.OutputName = 'y';
%        Sum = sumblk('e','r','y','+-');
%        T = connect(G,C,Sum,'r','y')
%
%     Example 2: If C and G above are two-input, two-output models
%     instead, you can form the MIMO transfer T from r to y using
%         
%        C.InputName = {'e1','e2'};   C.OutputName = {'u1','u2'};
%        G.InputName = C.OutputName;  G.OutputName = {'y1','y2'};
%        r = {'r1','r2'};
%        Sum = sumblk(C.InputName,r,G.OutputName,'+-');
%        T = connect(G,C,Sum,r,G.OutputName)   
%
%   Index-based interconnection
%     In this approach, first combine all USS blocks into an 
%     aggregate, unconnected model BLKSYS using APPEND. Then 
%     construct a matrix Q where each row specifies one of the 
%     connections or summing junctions in terms of the input 
%     vector U and output vector Y of BLKSYS.  For example, the 
%     row [3 2 0 0] indicates that Y(2) feeds into U(3), while 
%     the row [7 2 -15 6] indicates that Y(2) - Y(15) + Y(6) 
%     feeds into U(7).  The aggregate model SYS is then obtained by 
%     SYS = CONNECT(BLKSYS,Q,INPUTS,OUTPUTS) where INPUTS and  
%     OUTPUTS are index vectors into U and Y selecting the block  
%     diagram external I/Os.
%
%     Example: You can construct the closed-loop model T for the 
%     block diagram above as follows:
%        BLKSYS = append(C,G);   
%        % U = inputs to C,G.  Y = outputs of C,G
%        % Here Y(1) feeds into U(2) and -Y(2) feeds into U(1)
%        Q = [2 1; 1 -2]; 
%        % External I/Os: r drives U(1) and y is Y(2)
%        T = connect(BLKSYS,Q,1,2)
%
%   See also SYSIC, ICONNECT, SUMBLK, APPEND, SERIES, PARALLEL, FEEDBACK, LFT.

%   Author(s): G. Balas, P. Gahinet 06-Dec-2005
%   Copyright 2005 The MathWorks, Inc.
%   $Revision: 1.1.8.1 $  $Date: 2006/01/26 01:49:32 $

ni = nargin;
if ni<2
   error('Not enough input arguments.')
else
   IndexBased = isa(varargin{2},'double');
end

if IndexBased
   % SYS = CONNECT(BLKSYS,Q,INPUTS,OUTPUTS)
   sys = varargin{1};
   q = varargin{2};
   Size = size(sys);
   ny = Size(1);  nu = Size(2);
   % Build feedback matrix K such that U = K*Y + R forms
   % the desired connected system.
   k = zeros(nu,ny);
   % Go through rows of Q
   for i=1:size(q,1)
      % Remove zero elements from each row of Q
      qi = q(i,find(q(i,:)));
      n = length(qi);
      % Put the zeros and +-ones in K
      if n>1
         iu = qi(1);
         iy = abs(qi(2:n));
         if iu<1 || iu>nu || any(iy<1 | iy>ny)
            error('Some indices in row %d of Q are out of range.',i)
         end
         k(iu,iy) = sign(qi(2:n));
      end
   end
   % Close loop
   try
      sys = feedback(sys,k,+1);
   catch
      rethrow(lasterror)
   end
   % Select just the outputs and inputs wanted:
   if ni==4
      try
         %sys = utSubParen(sys,varargin([4 3]));
         sys = parenget(sys,varargin([4 3]));
      catch
         error('Specified INPUTS/OUTPUTS indices are invalid or out of range.')
      end
   end
else
   % Name-based IC
   % SYS = CONNECT(SYS1,SYS2,...,INPUTS,OUTPUTS)
   if ni<3
      error('Not enough input arguments.')
   end
   % Append all blocks
   sys = append(varargin{1:ni-2});
   % Get I/O names
   InputNames = get(sys,'InputName');
   OutputNames = get(sys,'OutputName');
   if length(unique(OutputNames))<length(OutputNames)
      error('Output signal names cannot be repeated in the block diagram.')
   end
   % Signal branching gives rise to repeated input names. Reduce repeated 
   % inputs to single occurrence driving a gain block ONES(nrepeat,1)
   [inames,junk,J] = unique(InputNames);
   nur = length(inames);
   if nur<length(InputNames)
      InputNames = inames;
      % Compute selection matrix M such that A = M*B
      % B = EYE * B -> A = B(J) = M(J,:) * B where M = EYE
      M = eye(nur);
      try
         sys = sys * M(J,:);
         set(sys,'InputName',inames);
      catch
         rethrow(lasterror)
      end
   end
   % Check if the external I/Os are defined and locate them in full I/O list
   iu = localFindIn(InputNames,varargin{ni-1});
   if isempty(iu)
      error('Some external inputs do not appear in any block.')
   end
   iy = localFindIn(OutputNames,varargin{ni});
   if isempty(iy)
      error('Some external outputs do not appear in any block.')
   end
   % Determine feedback channels
   [junk,feedin,feedout] = intersect(InputNames,OutputNames);
   % Close loop
   nfeed = length(feedin);
   if nfeed>0
      try
         sys = feedback(sys,eye(nfeed),feedin,feedout,1);
      catch
         rethrow(lasterror)
      end
%       % Eliminate structurally nonminimal states and delays (SS only)
%       if isStateSpace(sys)
%          sys = sminreal(sys);
%       end
   end
   % Select desired final I/Os
   %sys = utSubParen(sys,{iy,iu});
   sys = parenget(sys,{iy,iu});
end


%---------- Local Function ------------------

function idx = localFindIn(A,B)
% Computes index s.t. B = A(IDX)
if ischar(B)
   B = {B};
end
[Bu,junk,iu] = unique(B);  % B = Bu(iu)
[junk,ia,ib] = intersect(A,Bu);
if length(ib)<length(Bu)
   % Not all strings in B are in A
   idx = []; return
end
[junk,is] = sort(ib);  % Bu = A(ia(is))
idx = ia(is(iu));
