% DKSYN  Synthesis of a robust controller via mu-synthesis D-K iteration.
%
% [K,CLP,BND] = DKSYN(P,NMEAS,NCONT) synthesizes a controller,
% K, for the open-loop plant model, P via the D-K iteration approach
% to mu-synthesis.  P is a USS and has NMEAS measurement outputs and 
% NCONT control inputs.  It is assumed that the measurement outputs
% and control inputs correspond to the last NMEAS outputs and NCONT
% inputs.  K is the controller, CLP is the closed-loop interconnection
% with P and BND is the robust performance bound the closed-loop 
% system CLP achieves.  K, CLP and BND are related as follows:
%
%   CLP = LFT(P,K);
%   BNDI = ROBUSTPERF(CLP);
%   BND = 1/BNDI.LowerBound
%
% [K,CLP,BND] = DKSYN(P,NMEAS,NCONT,OPT) specifies options 
% described in DKITOPT. OPT is created via the command OPT=DKITOPT;
%
% OPT is an object with the following properties:
%   FrequencyVector:  Frequency vector used for analysis. Default 
%      is an empty matrix which results in the frequency range and number 
%      of points  used to be chosen automatically.
%   InitialController:  Controller used to initiate first iteration.
%      Default is an empty SS object.
%   AutoIter:  Automated mu-synthesis mode. Default is 'on'. 
%   DisplayWhileAutoIter: Displays iteration progress in AutoIter
%      mode.  Default is 'Off'.
%   StartingIterationNumber:  Starting iteration number. Default is 1.
%   NumberOfAutoIterations:  Number of iterations to perform, Default
%      is 10.
%   AutoScalingOrder:  State order for fitting D-scaling data. 
%      Default is 5.
%   AutoIterSmartTerminate:  Automatic termination of iteration 
%      procedure based on progress of design iteration. Default
%      is 'on'.
%   AutoIterSmartTerminateTol: Tolerance used AutoIterSmartTerminate.
%      Default is 0.005.
%   
% [K,CLP,BND,DKINFO] = DKSYN(P,NMEAS,NCONT,PREVDKINFO,OPT) 
% PREVDKINFO is a structure from a previous attempt at designing a
% robust controller using DKSYN.  DPREVDKINFO is used when the DKSYN
% starting iteration is not 1 (OPT.StartingIterationNumber~=1) to
% determine the correct D-scalings to initiate the iteration 
% procedure. 
%
% DKINFO is a N-by-1 cell array where N is the total number of 
% iterations performed. The i'th cell contains a structure with
% the following fields:
%         K: Controller at i'th iteration, an SS object.
%      Bnds: Robust performance bound the closed-loop system.
%        DL: Left D-scale, an SS object.
%        DR: Right D-scale, an SS object.
% MussvBnds: Upper and lower mu bounds, an FRD object.
% MussvInfo: Structure returned from MUSSV at each iteration.
%
% See Also: DKITOPT, H2SYN, HINFSYN, LOOPMARGIN, MUSSV, ROUSTSTAB, 
%           ROBUSTPERF, WCGAIN, WCSENS, WCMARGIN.

% Copyright 2003-2004 The MathWorks, Inc.

%-------------------------------------------------------------------%
% mutools commands to convert: GENPHASE, FITSYS, MSF, HINFSYN (mike),
%                               SBMUDL
% mutools commands converted but??  :  D2CBIL/C2DBIL
%-------------------------------------------------------------------%

function [K,clpic,maxbnd,dkinfo] = dksyn(P,NMEAS,NCONT,varargin); 

if nargin==3
   prevdkinfo = [];
   opt = dkitopt;
elseif nargin==4 %4th argument is either PREVDKINFO or OPT
   fnames = fieldnames(varargin{1});
   optfnames = fieldnames(dkitopt);
   if length(fnames)==1 && all(strcmp(fnames,{'K','Bnd','DL','DR','MussvInfo'}))
      prevdkinfo = varargin{1};
      opt = dkitopt;
      muinfo = prevdkinfo.MussvInfo;
   elseif (length(fnames)==length(optfnames) && all(strcmp(fnames,optfnames))) || ...
           isempty(varargin{1})
      opt = varargin{1};
      prevdkinfo = [];
   else %error in 4th input
      error('Invalid Structure for OPT or PREVDKINFO')
   end
elseif nargin==5
   if ~isempty(varargin{1})
      fnames = fieldnames(varargin{1}{1});
      infofnames = {'K';'Bnd';'DL';'DR';'MussvBnds';'MussvInfo'};
      if length(fnames)==length(infofnames) && ...
            all(strcmp(fnames,infofnames))
         prevdkinfo = varargin{1};
      else
         error('Invalid PREVDKINFO structure')
      end
   else
       prevdkinfo = [];
   end
   if ~isempty(varargin{2})
      optfnames = fieldnames(dkitopt);
      fnames = fieldnames(varargin{2});
      if length(fnames)==length(optfnames) && all(strcmp(fnames,optfnames))
         opt = varargin{2};
      else
         error('The OPT input argument must a DKITOPT object.')
      end
   else
      opt = dkitopt;
   end
elseif nargin<3
   error('Must have at least three input arguments')
else
   error('Invalid number of input arguments')
end   
   
%check dimensions
szp = size(P);
if length(szp)>2
   error('DKSYN does not handle extra array dimensions')
end
if (szp(1)-NMEAS)<1
   error('Insufficient number of P outputs and measurements')
end
if (szp(2)-NCONT)<1
   error('Insufficient number of P inputs and controls')
end
if strcmpi(opt.AutoIter,'On')
   auto = 1;
else
   auto = 0;
end
if strcmpi(opt.DisplayWhileAutoIter,'On') || ...
      strcmpi(opt.AutoIter,'Off')
   auto_visual = 1;
else
   auto_visual = 0;
end
Pnom = [];

if isempty(opt.FrequencyVector)
   if isa(P,'uss')
      unclist = pvget(P,'Uncertainty');
      unames = pvget(unclist,'UNames');
      for i=1:length(unclist)
         nam = pvget(unclist,unames{i});
         uclass = class(nam);
         if isequal(uclass,'udyn')
            error(['DKSYN is not implemented for ''' uclass ''' atom types']);
         end
      end
      Pnom = pvget(P,'NominalValue');
   elseif isa(P,'ss') || isa(P,'tf') || isa(P,'szp')
      Pnom = P;
   end
   if ~isempty(Pnom)
      OMEGA = dkpickfreq(Pnom);
   else
      OMEGA = [];
   end
else
   OMEGA = opt.FrequencyVector;
end
if P.Ts==0
   DISCRETE = 0;
else
   DISCRETE = 1;
end

% hinfsyn parameters which will disappear
RICMTHD = 2;
EPP = 1e-6;
EPR = 1e-8;
GMAX = 100;
GMIN = 0;
GTOL = 0.01*(GMAX-GMIN);
GMAX_PLAN = 2;

axisvar = 'liv,m';
if abs(max(diff(OMEGA)) - min(diff(OMEGA))) < 1e-5
    axisvar = 'iv,m';
end

if isa(P,'ss') || isa(P,'tf') || isa(P,'szp') % no uncertainty
   iter_num = opt.StartingIterationNumber+1;
   NOMINAL = P;
   [K,clpic,clpic_g,gf,OMEGA,errormsg] = ...
      LOCALhinfsyn(NOMINAL,NMEAS,NCONT,opt,iter_num,NOMINAL,...
                     OMEGA,GMIN,GMAX,GTOL,axisvar);
   if ~isempty(errormsg)
      dkinfo{iter_num}.errormsg = errormsg;
      maxbnd = [];
   else
      maxbnd = norm(clpic,inf);
   end
   return
elseif isa(P,'umat') || isa(P,'ufrd')
   %call cmmusyn
   error('DKSYN is not set up to handle UMATs or UFRDs')
   return
elseif isa(P,'uss')
   %remove zero copies from P and get BLK info
   newsys = rmzerocp(P);
   %BLK = pvget(newsys,'muToolsBlkStructure'); % old mutools blk structure
   BLK = getmutoolsblkstruct(newsys); % old mutools blk structure
   BLK = [BLK; szp(2)-NCONT szp(1)-NMEAS]; % Add performance channel
   NOMINAL = lftdata(newsys); % M is an SS with Delta on top
   sznom = size(NOMINAL);
   names = pvget(newsys,'UNames');
else
   error('DKSYN requires a USS object')
end

if isempty(opt.InitialController)
   K = [];
else
   K = opt.InitialController;
   szk = size(K);
   if szk(1)~=NCONT || szk(2)~=NMEAS
      error('Invalid dimension of opt.InitialController')
   else
      clp = lft(NOMINAL,K);
      evls = pole(clp);
      if P.Ts==0 && any(real(evls)>0)
         msg = str2mat('Initial Controller does not stability closed-loop',...
            ' and will NOT be used.');
         warning(msg)
      elseif P.Ts~=0 && any(abs(evls)>1)
         warning('Initial Controller does not stability closed-loop')
         warning(' Initial Controller will NOT be used')
      end
   end
end

iter_num = opt.StartingIterationNumber;
iter_stop = opt.StartingIterationNumber + opt.NumberOfAutoIterations - 1;
autoord = opt.AutoScalingOrder;
stophist = zeros(0,3);

if iter_num == 1
	dl = eye(sznom(1),sznom(1));
	dr = eye(sznom(2),sznom(2));
	prevd = 0;
	fiti = 0;
	dispdata = [];
else
   if ~isempty(prevdkinfo)
		Dscale = prevdkinfo{iter_num}.MussvInfo.dvec;
		dl = prevdkinfo{iter_num}.DL;
		dr = prevdkinfo{iter_num}.DR;
		sens = prevdkinfo{iter_num}.MussvInfo.sens;
		bnds = prevdkinfo{iter_num}.MussvBnds;
        muinfo = prevdkinfo{iter_num}.MussvInfo;
		K = prevdkinfo{iter_num}.K;
        dkinfo = prevdkinfo(1:iter_num);
		dispdata = [];
   else
	   error('Starting iteration is invalid -- PREVDKINFO does not exist')
   end
   if iter_num == 2     % haven't fit D's yet
		dl = [];
		dr = [];
		%use Dscale Freq, so it matched MUSSV Freq
		g_g = frd(lft(NOMINAL,K),Dscale.Frequency); 
		clpic = starp(NOMINAL,K);
		clpic_g = frd(clpic,Dscale.Frequency);
   else    %(>2)
		IC = dl*NOMINAL/dr;   % D's absorbed
		clpic = lft(NOMINAL,K);
		clpic_g = frd(clpic,Dscale.Frequency);
   end
   if isa(dl,'double')
      ndl = 0;
   else
      ndl = length(dl.StateName);
   end
   if isa(dr,'double')
      ndr = 0;
   else
      ndr = length(dr.StateName);
   end
   prevd = ndl + ndr;  % order of D's
end
again = 1;


if iter_num > 1
	if (length(OMEGA)~=length(Dscale.Frequency) || all(OMEGA(:)==Dscale.Frequency)) ...
	      && (auto == 0 || auto_visual == 1)
		disp(' ')
		disp('**** New OMEGA frequency data will be incorporated ****')
		disp('****       after next H-infinity control design       ****')
	end
end
%if ~isempty(get(0,'children'))
%   [mw,sw,notours] = findmuw;
%   if ~isempty(notours) && auto_visual==1
%      figure(notours(1))
%      clf
%   end
%end
while again == 1 & iter_num<=iter_stop
   dkinfo{iter_num} = struct('K',[],'Bnd',[],'DL',[],'DR',[],'MussvBnds',[],...
      'MussvInfo',[],'errormsg',[]);
   if iter_num > 1
      maxsens = norm(sens,inf);
      if maxsens==0
         szs = size(sens);
         sens = sens+ones(1,szs(2));
      end
      szclp = size(clpic_g);
      if auto == 0
             [dlnewj,drnewj] =  ...
                xmsf(clpic_g,bnds,muinfo,sens,abs(BLK),[],DISCRETE);
      else
         [dlnewj,drnewj] =  ...
                xmsfbatch(clpic_g,bnds,muinfo,sens,abs(BLK),...
                    autoord,auto_visual,DISCRETE);
      end
      tmpdlg = frd(dlnewj,clpic_g.Frequency);
      tmpdrg = frd(drnewj,clpic_g.Frequency);
      x_g = tmpdlg*clpic_g/tmpdrg;
      mnum = fnorm(x_g);
      if auto==0 | auto_visual == 1
         clf
         uplot(axisvar,bnds,'-',mnum,'--');
         title(['MU bnds (solid) and ||D*M*D^-1|| (dashed): ITERATION  ',int2str(iter_num)])
         drawnow;
         pause(2)
      end
      dl = blkdiag(dlnewj,eye(NMEAS));
      dr = blkdiag(drnewj,eye(NCONT));
      IC = dl*NOMINAL/dr;
      oldgm = GMAX;
      if GMAX_PLAN == 1
         maxmnum = norm(mnum,inf);
         GMAX = 1.2*maxmnum;
      elseif GMAX_PLAN == 2
         x = lft(NOMINAL,K);
         maxmnum = norm(dlnewj*x/drnewj,inf);
         GMAX = 1.02*maxmnum;
      end
      GTOL = 0.02*(GMAX - GMIN);
   else
      IC = NOMINAL;
   end
   if auto==0 | auto_visual==1
      disp(' ');disp(' ');disp(' ')
	 	disp(['Iteration Number:  ', int2str(iter_num)])
		disp(setstr(ones(1,19+length(int2str(iter_num)))*'-'));disp(' ')
   end

%   Design an H-infinity control law for the interconnection structure in IC
   if auto==0
      disp('Information about the Interconnection Structure IC:');
      size(IC)
   end
   [K,clpic,clpic_g,gf,OMEGA,errormsg] = ...
     LOCALhinfsyn(IC,NMEAS,NCONT,opt,iter_num,NOMINAL,OMEGA,GMIN,GMAX,GTOL,axisvar);
   if ~isempty(errormsg)
      dkinfo{iter_num}.errormsg = errormsg;
   else
      dkinfo{iter_num}.errormsg = [];
   end

   if auto==0
      disp(' ')
   end

%   now calculate an upper and lower bounds for mu for the control
%   design

    if isempty(errormsg)
       if auto_visual==1
           disp('Calculating MU of closed-loop system:')
           [bnds,muinfo] = mussv(clpic_g,abs(BLK),'c');
           if min(min(BLK)) < 0
               mbnds = mussv(clpic_g,BLK);
           end
       else
           [bnds,muinfo] = mussv(clpic_g,abs(BLK),'s');
           if min(min(BLK)) < 0
               mbnds = mussv(clpic_g,BLK,'s');
           end
       end
       Dscale = muinfo.dvec;
       Dscale_data = Dscale.ResponseData;
       szd = size(Dscale_data);
       Dscale_data = reshape(Dscale_data,[szd(2) szd(3)])';
       Dscale_dataNorm = Dscale_data(:,szd(2)*ones(1,szd(2))).\Dscale_data;
       Dscale_dataNorm = reshape(Dscale_dataNorm',[1 szd(2) szd(3)]);
       Dscale.ResponseData = Dscale_dataNorm; % normalized D

       if min(min(BLK)) < 0 & auto_visual==1
           clf; subplot(211)
           uplot(axisvar,bnds)
           title(['CLOSED-LOOP COMPLEX MU: CONTROLLER #', int2str(iter_num)])
           xlabel('FREQUENCY  (rad/s)')
           ylabel('COMPLEX MU')
           subplot(212)
           uplot(axisvar,mbnds)
           title(['CLOSED-LOOP MIXED MU: CONTROLLER #', int2str(iter_num)])
           xlabel('FREQUENCY  (rad/s)')
           ylabel('MIXED MU')
           %subplot(111)
           disp('')
           if auto==0
               disp(' MU plots for control design:      Strike any key to continue')
               pause;
           else
               drawnow
           end
       elseif auto_visual==1
           uplot(axisvar,bnds)
           title(['CLOSED-LOOP MU: CONTROLLER #', int2str(iter_num)])
           xlabel('FREQUENCY  (rad/s)')
           ylabel('MU')
           disp('')
           if auto==0
               disp(' MU plots for control design:      Strike any key to continue')
               pause;
           else
               drawnow
           end
       end

   %----------------------------------------------------%
   %                                                    %
   %    YOU CAN EASILY ADD MORE PLOTS/CALCULATIONS      %
   %          AND DISPLAYS AT THIS POINT                %
   %                                                    %
   %----------------------------------------------------%

       if isa(K,'double')
          nx_K = 0;
       else
         nx_K = length(K.StateName);
       end
       if isa(dl,'double')
          nx_dl = 0;
       else
          nx_dl = length(dl.StateName);
       end
       if isa(dr,'double')
          nx_dr = 0;
       else
          nx_dr = length(dr.StateName);
       end
       prevd = nx_dl + nx_dr;    % order of D's
       maxbnds = norm(bnds(1,1),inf);
       mtype = [iter_num; nx_K; prevd; gf; maxbnds];
    else
       mtype = [iter_num; 0; 0; inf; inf];
    end
    dispdata = [dispdata mtype];
    [mrows,mcols] = size(dispdata);
    if mcols > 5
        mtype = dkdispla(dispdata(:,mcols-4:mcols));
    else
        mtype = dkdispla(dispdata);
    end
    if isempty(errormsg)
       % update DKINFO structure
       dl_prev = dl;
       dr_prev = dr;
       maxbnd = norm(bnds(1,1),inf);
       dkinfo{iter_num}.K = K;
       dkinfo{iter_num}.Bnd = maxbnd;
       dkinfo{iter_num}.DL = dl;
       dkinfo{iter_num}.DR = dr;
       dkinfo{iter_num}.MussvBnds = bnds;
       dkinfo{iter_num}.MussvInfo = muinfo;
       sens = dkinfo{iter_num}.MussvInfo.sens;
      if strcmpi(opt.AutoIterSmartTerminate,'On') && strcmpi(opt.AutoIter,'On')
          again = 1;
          if iter_num > opt.StartingIterationNumber+1
             stophist(1) = norm(dkinfo{iter_num-2}.MussvBnds(1,1),inf);
             stophist(2) = norm(dkinfo{iter_num-1}.MussvBnds(1,1),inf);
             stophist(3) = norm(dkinfo{iter_num}.MussvBnds(1,1),inf);
             stopdiff = diff(stophist);
             rtol = opt.AutoIterSmartTerminateTol*stophist(end);
             if all(abs(stopdiff)<rtol) || stopdiff(end)>20*rtol
                again = 0; % terminate DK iteration due to no progress or increase
             else
                again = 1;
             end
          end
      end
    else
       dkinfo{iter_num}.K = [];
       dkinfo{iter_num}.Bnd = inf;
       dkinfo{iter_num}.DL = [];
       dkinfo{iter_num}.DR = [];
       dkinfo{iter_num}.MussvBnds = [];
       dkinfo{iter_num}.MussvInfo = [];
       again = 0;
    end
    
    first = 0;     % no longer first iteration
    iter_num = iter_num + 1;
    if auto_visual==1
        clc;home
        disp(mtype)
        disp(' ')
    end
    if auto==0
        ag = notherdk(1);
        if ag > 0
            again = 1;
        else
            again = 0;
        end
     end
end  %while again
if auto_visual==1
    disp([' Next MU iteration number:  ', int2str(iter_num)]);
end

%Construct output variables K, clpic and maxbnd from closed-loop system
% that achieved the smallest maxbnd
maxbnd = inf;
for i = 1:length(dkinfo)
   tmp = dkinfo{i}.Bnd;
   if tmp<maxbnd
      maxbnd = tmp;
      mini = i;
   end
end
K = dkinfo{mini}.K;
clpic = lft(P,K);

%----------------------- End of DKSYN ------------------------------------------%
function [K,clpic,clpic_g,gf,OMEGA,errormsg] = ...
     LOCALhinfsyn(IC,NMEAS,NCONT,opt,iter_num,NOMINAL,OMEGA,GMIN,GMAX,GTOL,axisvar);

K = []; clpic = []; clpic_g = []; gf = []; errormsg = [];
% hinfsyn parameters which will disappear
RICMTHD = 2;
EPP = 1e-6;
EPR = 1e-8;
hinf_go = 1;
cnt = 0;
if strcmpi(opt.DisplayWhileAutoIter,'On') || ...
      strcmpi(opt.AutoIter,'Off')
   auto_visual = 1;
   hinfdisp = 'On';
else
   auto_visual = 0;
   hinfdisp = 'Off';
end
while hinf_go
	%--convert for now to mutools------%
	if ~isempty(opt.InitialController) && opt.StartingIterationNumber==iter_num
      K = opt.InitialController;
      g = lft(IC,K);
      gf = norm(g,inf);
	else
      % calls RCT HINFSYN
      [K,g,gf] = hinfsyn(IC,NMEAS,NCONT,'GMAX',GMAX,'GMIN',GMIN,'TOLGAM',GTOL,...
            'DISPLAY',hinfdisp);
	end
	SKIP = 1;
	if isempty(gf)
      SKIP= 0;
      if strcmpi(opt.AutoIter,'Off')
          disp('----  Upper bound for GAMMA is too small!!  ----');
          disp('-----  Need to increase GAMMA Upper bound  -----');
          [GMAX,GMIN,GTOL,EPP,EPR] = chhsparm(GMAX,GMIN,GTOL,EPP,EPR);
      else
          % GMIN = GMAX;
          GMAX = 2*GMAX;
          GTOL = 2*GTOL;
          cnt = cnt + 1;
          if cnt == 15
              %error('Cannot find a stabilizing controller, check GMAX');
              %return
              errormsg = 'Cannot find a stabilizing controller, check GMAX';
              SKIP = 0;
              hinf_go = 0;
          end
      end
	elseif gf==-1           % A, B_2 unstabilizable
      %error('Open-Loop Interconnection is not stabilizable through U (B_2)');
      %return
      SKIP = 0;
      errormsg = 'Open-Loop Interconnection is not stabilizable through U (B_2)';
      hinf_go = 0;
	elseif gf==-2           % A, C_2 detectable
      %error('Open-Loop Interconnection is not detectable through Y (C_2)');
      %return
      SKIP = 0;
      errormsg = 'Open-Loop Interconnection is not detectable through Y (C_2)';
      hinf_go = 0;
	end
	if SKIP==1
      chflg = 1;
      while chflg == 1
         clpic = lft(NOMINAL,K);
         clpic_g = frd(clpic,OMEGA);
         g_g = frd(g,OMEGA);  % has rational scalings
         if auto_visual==1
            g_gs = svd(g_g);
            uplot(axisvar,g_gs)
            title('SINGULAR VALUE PLOT: CLOSED-LOOP RESPONSE')
            xlabel('FREQUENCY (rad/s)')
            ylabel(' MAGNITUDE')
            drawnow
         end
         if strcmpi(opt.AutoIter,'Off')
            disp(' '); disp(' ')
            disp('Singular Value plot of closed-loop system in GRAPHICS window')
            disp('Make sure that chosen Frequency range is appropriate')
            RE = reomega(1);
            if RE == 1
               [OMEGA,chflg] = chomega(OMEGA(:));
               axisvar = 'liv,m';
               if abs(max(diff(OMEGA)) - min(diff(OMEGA))) < 1e-5
                   axisvar = 'iv,m';
               end
            else
               chflg = 0;
            end
            %RE = rerunhi(1);
            %if RE == 0
            hinf_go = 0;
            %else
            %    [GMAX,GMIN,GTOL,EPP,EPR] = ...
            %        chhsparm(GMAX,GMIN,GTOL,EPP,EPR);
            %end
          else
             chflg = 0;
             hinf_go = 0;
          end
      end
   end
end     % while hinf_go
