function B = imp2exp(A,yidx,uidx)
%IMP2EXP Converts an implicit linear relationship to explicit
%   input-output relation.
%
%   B = IMP2EXP(A,YIDX,UIDX) transforms a linear constraint on
%   variables Y and U of the form A(:,[YIDX;UIDX])*[Y;U] = 0 into
%   an explicit input/output relationship Y = B*U.  The index
%   vectors YIDX and UIDX refer to the columns (inputs) of A as
%   referenced by the explicit relationship for B.
%
%   See also INV.

% Copyright 2004 The MathWorks, Inc.

if isa(yidx,'double') && isa(uidx,'double') && ...
      ndims(yidx)==2 && ndims(uidx)==2 && ...
      min(size(yidx))==1 && min(size(uidx))==1 && ...
      all(ceil(yidx)==floor(yidx)) && all(ceil(uidx)==floor(uidx)) && ...
      min(yidx)>0 && min(uidx)>0 && ...
      length(yidx)>0 && length(uidx)>0
   yidx = yidx(:);
   uidx = uidx(:);
   szA = size(A);
   if max(yidx)>szA(2)
      error('Indices in YIDX exceed the column/input dimension of A.');
   end
   if max(uidx)>szA(2)
      error('Indices in UIDX exceed the column/input dimension of A.');
   end
   if ~isempty(intersect(yidx,uidx))
      error('Indices UIDX and YIDX must not have any elements in common.');
   end
%    if ~length(unique(yidx))==length(yidx)
%       error('Indices specified in YIDX should be unique.');
%    end
   if ~length(unique(uidx))==length(uidx)
      error('Indices specified in UIDX should be unique.');
   end
   nvartosolve = szA(2) - length(uidx);
   neq = szA(1);
   if nvartosolve<neq
      msg1 = 'Relationship is likely over constrained, so that\n';
      msg2 = 'given U, there may not be Y to satisfy constraints.';
      error(sprintf([msg1 msg2]));
   end
   if nvartosolve>neq
      msg1 = 'Relationship is likely under constrained, so that\n';
      msg2 = 'given U, there will be many Y satisfying constraints.';
      error(sprintf([msg1 msg2]));
   end
   if length(uidx)==1 || min(diff(sort(uidx)))>0
      unyidx = unique(yidx);
      missingyidx = (1:szA(2))';  % Given U, assume you can solve for notU
      missingyidx([unyidx;uidx]) = [];
      yidxext = [yidx;missingyidx];
      sn = '_rcastreserved';
      % Operate on ExtendedY
      try
         UmatData = blk2ioch(imp2exp(uss2umat(A,sn),yidxext,uidx),sn,'t','unchanged');
      catch
         rethrow(lasterror);
      end
      DS = utImp2Exp(A.dynamicsys,yidxext,uidx);
      NX = length(A.StateName);
      % Select appropriate output channels
      NY = length(yidx);
      B = uss(UmatData(1:NX+NY,:),A.StateName,A.Ts,DS(1:NY,:));
   else
      error('Elements of UIDX must be unique.');
   end
else
   error('UIDX and YIDX must be vectors of positive integers.');
end
