function varargout = impulse(varargin)
%IMPULSE  Impulse response of Uncertain USS models.
%
%   IMPULSE(SYS) plots sample impulse responses of the uncertain model
%   SYS. The uncertain system SYS is sampled at 20 values, as well as its
%   nominal value, and the impulse response of these 21 systems is
%   computed and plotted.
%
%   For multi-input models, independent impulse commands are applied to 
%   each input channel.  The time range and number of points are chosen 
%   automatically.  For continuous systems with direct feedthrough, the 
%   infinite pulse at t=0 is disregarded.
%
%   IMPULSE(SYS,TFINAL) simulates the impulse responses from t=0 to the 
%   final time t=TFINAL.  For discrete-time models with unspecified 
%   sampling time, TFINAL is interpreted as the number of samples.
%
%   IMPULSE(SYS,T) uses the user-supplied time vector T for simulation. 
%   For discrete-time models, T should be of the form  Ti:Ts:Tf 
%   where Ts is the sample time.  For continuous-time models,
%   T should be of the form  Ti:dt:Tf  where dt will become the sample 
%   time for the discrete approximation to the continuous system. The
%   impulse is always assumed to arise at t=0 (regardless of Ti).
%
%   IMPULSE(SYS1,SYS2,...,T) plots the impulse response of multiple USS, 
%   TF, SS, ZPK models SYS1,SYS2,... on a single plot.  You can also
%   specify a color, line style, and marker for each system, as in 
%      impulse(sys1,'r',sys2,'y--',sys3,'gx').
%   With this conventional specification (or if no linetype is
%   specified), both the samples and the nominal will use the same
%   linetype. 
%
%  See also STEP

%   See also IMPULSE, INITIAL, LSIM, LTIVIEW, LTIMODELS.
% Copyright 2003-2004 The MathWorks, Inc.

nin = nargin;
nout = nargout;
mname = mfilename;
if nout>0
   strerr = ' with a USS or UFRD cannot be called with output arguments.';
   error([upper(mname) strerr])
end

N = 20;  % Number of times to sample uncertain system
   
samcell = cell(1,0);
for i=1:nin
    c=class(varargin{i});
    % RESHAPE to be PROD(extra dim)x1
    if isa(varargin{i},'dynamicsys')
       sys = varargin{i};
       sz = size(sys);
       nad = length(sz) - 2;
       ad = sz(3:end);
       if nad==2 && ad(2)==1
          nad = 1;
          ad = ad(1);
       end
       if strcmp(c,'uss') || strcmp(c,'ufrd')
           tmpu = reshape(usample(sys,N),[prod(ad)*N 1]);
           tmpnom = reshape(pvget(sys,'NominalValue'),[prod(ad) 1]);
           tmpu = stack(1,tmpu,tmpnom);         
           samcell = [samcell {tmpu}];
       else
           % Do nothing and let lti version do the error checking
           samcell = [samcell {sys}];
       end 
    else
    % Do nothing and let lti version do the error checking
       samcell = [samcell varargin(i)];
    end
 end
eval([mname '(samcell{:});']);
   