function varargout = nichols(varargin)
%NICHOLS  Nichols frequency response of Uncertain USS/UFRD models.
%       
%   NICHOLS(SYS) draws sample Nichols plots of the uncertain model SYS 
%   (created with either USS or UFRD). The uncertain system SYS is sampled
%   at 20 values, as well as its nominal value, and the bode response of 
%   these 21 systems is computed and plotted.  If SYS is a USS, the 
%   frequency range and number of points are chosen automatically.  See 
%   BODE for details on the notion of frequency in discrete-time.
%
%   NICHOLS(SYS,{WMIN,WMAX}) draws the Nichols plot for frequencies
%   between WMIN and WMAX (in radian/second).
%
%   NICHOLS(SYS,W) uses the user-supplied vector W of frequencies, in
%   radians/second, at which the Nichols response is to be evaluated.  
%   See LOGSPACE to generate logarithmically spaced frequency vectors.
%
%   NICHOLS(SYS1,SYS2,...,W) plots the Nichols plot of multiple LTI
%   models SYS1,SYS2,... on a single plot.  The frequency vector W
%   is optional.  You can also specify a color, line style, and marker 
%   for each system, as in  
%      nichols(sys1,'r',sys2,'y--',sys3,'gx').
%
%   [MAG,PHASE] = NICHOLS(SYS,W) and [MAG,PHASE,W] = NICHOLS(SYS) return
%   the response magnitudes and phases in degrees (along with the 
%   frequency vector W if unspecified).  No plot is drawn on the screen.  
%   If SYS has NY outputs and NU inputs, MAG and PHASE are arrays of 
%   size [NY NU LENGTH(W)] where MAG(:,:,k) and PHASE(:,:,k) determine 
%   the response at the frequency W(k).
%
%   See also BODE, NYQUIST, SIGMA, FREQRESP, LTIVIEW, LTIMODELS.

nin = nargin;
nout = nargout;
mname = mfilename;
if nout>0
   strerr = ' with a USS or UFRD cannot be called with output arguments.';
   error([upper(mname) strerr])
end

N = 20;  % Number of times to sample uncertain system
   
samcell = cell(1,0);
for i=1:nin
    c=class(varargin{i});
    % RESHAPE to be PROD(extra dim)x1
    if isa(varargin{i},'dynamicsys')
       sys = varargin{i};
       sz = size(sys);
       nad = length(sz) - 2;
       ad = sz(3:end);
       if nad==2 && ad(2)==1
          nad = 1;
          ad = ad(1);
       end
       if strcmp(c,'uss') || strcmp(c,'ufrd')
           tmpu = reshape(usample(sys,N),[prod(ad)*N 1]);
           tmpnom = reshape(pvget(sys,'NominalValue'),[prod(ad) 1]);
           tmpu = stack(1,tmpu,tmpnom);         
           samcell = [samcell {tmpu}];
       else
           % Do nothing and let lti/bode do the error checking
           samcell = [samcell {sys}];
       end 
    else
    % Do nothing and let lti/bode do the error checking
       samcell = [samcell varargin(i)];
    end
 end
eval([mname '(samcell{:});']);
   
