function B = repsys(A,varargin)
% REPSYS Replicate and tile UFRD and USS objects.
%
%  B = REPSYS(A,K) is the same as B = blkdiag(A,...,A) with
%  A repeated K times.
%
%  B = REPSYS(A,M,N) creates an UFRD array B consisting of an M-by-N
%  tiling of copies of A.  Hence size(B,1) equals M*size(A,1), and
%  size(B,2) equals N*size(A,2), and size(B,k) equals size(A,k) for
%  all k >= 3.
% 
%  B = REPSYS(A,[M N]) accomplishes the same result as REPSYS(A,M,N).
% 
%  B = REPSYS(A,[M N P ...]) tiles the array A to produce a
%  M-by-N-by-P-by-... block UFRD array.  A can be N-D.
% 
%  See also BLKDIAG.

% Copyright 2003-2004 The MathWorks, Inc.
if length(varargin)==1
   rmat = varargin{1};
elseif length(varargin)>1
   rmat = [varargin{:}];
else
   error('REPSYS requires at least 2 inputs')
end

if isa(rmat,'double') && ndims(rmat)==2 && min(size(rmat))==1 && ...
      all(ceil(rmat)==floor(rmat))
   if length(rmat)==1
      if rmat>0
         B = A;
         for i=2:rmat
            B = blkdiag(B,A);
         end
      else
         error('Tiling specification should be positive.');
      end
   else
      rmat = rmat(:)';
      fbname = '_rcastreserved';
      uresult = repmat(uss2umat(A,fbname),varargin{:});
      UmatData = blk2ioch(uresult,fbname,'t','unchanged');
      B = uss(UmatData,A.StateName,A.Ts,utRepSys(A.dynamicsys,rmat));
   end
else
   error('Invalid tiling specifications.');
end
