function [stabmarg,BadAtoms,Report,info] = robuststab(m,opts);
%ROBUSTSTAB   Stability margins of nominally stable uncertain systems.
%
%[STABMARG,DESTABUNC,REPORT,INFO] = ROBUSTSTAB(SYS) calculates robust 
%stability margins of the uncertain multivariable system SYS.  A stability
%robustness margin greater than 1 means that the uncertain system is stable
%for all values of its modeled uncertainty.  A stability robustness margin
%less than 1 implies that certain allowable values of the uncertain
%elements, within their specified ranges, lead to instability.
%
%Numerically, a margin of 0.5 (for example) implies two things: the
%uncertain system remains stable for all values of uncertain elements
%which are less than 0.5 normalized units away from their nominal values;
%and, there is a collection of uncertain elements which are less than or
%equal to 0.5 normalized units away from their nominal values that results
%in instability.  See ACTUAL2NORMALIZED for converting between actual and
%normalized deviations from the nominal value of an uncertain element.
%
%  STABMARG  Structure with the following fields:
%      UpperBound: Upper bound on stability margin.
%      LowerBound: Lower bound on stability margin. 
%      DestabilizingFrequency: Frequency at which instability occurs, 
%             associated with STABMARG.UpperBound.
%  DESTABUNC  Structure of values of uncertain elements which cause 
%             instability. The fieldnames are the names of uncertain 
%             elements of SYS.
%  REPORT  Char array with text description of robustness analysis results.
%  INFO  Structure with the following fields:
%      Sensitivity: Structure with local sensitivities of STABMARG.UpperBound
%                   to each uncertain elements' range.
%      Frequency: Frequency vector associated with analysis.
%      BadUncertainValues: Cell array (same length as Frequency vector) with
%                   uncertain elements which cause pole migration across
%                   the stability boundary at the corresponding frequency.
%      MussvBnds: Upper and lower bounds from MUSSV.
%      MussvInfo: Structure of compressed data from MUSSV. 
%
%[STABMARG,DESTABUNC,REPORT,INFO] = ROBUSTSTAB(SYS,OPTS) specifies 
%options described in OPTS.  (See ROBOPT for more details on the options.) 
%
%If SYS is an UFRD, then the analysis is performed on the frequency grid 
%within SYS. Note that the stability of the nominal system (SYS.NominalValue)
%is not verified by the computation as it is an FRD object.  If SYS is 
%an USS, then the stability of SYS.NominalValue is first checked, an 
%appropriate frequency grid is generated (automatically), and the analysis 
%performed on that frequency grid.  
%
%See also: ACTUAL2NORMALIZED, LOOPMARGIN, MUSSV, ROBOPT, ROBUSTPERF, WCGAIN, 
% WCSENS, WCMARGIN.

% Copyright 2003-2006 The MathWorks, Inc.

nin = nargin;
if nin==0
   disp('usage: [STABMARG,DESTABUNC,REPORT,INFO] = ROBUSTSTAB(SYS,OPT)');
   return
end

% Never actually uses SETUP.  Argument list is exactly consistent with
% UFRD/WCGAIN.
if nin==1
   opts = robopt;
   PrevData = [];
   SETUP = [];
   StableFlag = [];
elseif nin==2
   PrevData = [];
   SETUP = [];
   StableFlag = [];
elseif nin==3
   SETUP = [];
   StableFlag = [];
elseif nin==4
   StableFlag = [];
end
if ~isa(opts,'robopt')
   error('The OPTS input argument must a ROBOPT object.');
end
if nargout<3
    set(opts,'Sensitivity','Off');
end


%XXXMINOR genfresp doesn't work with LTI arrays
L.type = '()';
L.subs = {':',':',1};
m1 = subsref(m,L);
m1nom = pvget(m1,'NominalValue');
[mag,ph,omega] = genfresp(m1nom,4,[]);
% if length(omega)>15
%    omega = omega(4:end-4);
% end
omega = elimfreqpoints(omega,get(m1nom,'Ts'));
nfpts = length(omega);

% check the poles, eliminate unstable entries, frequency response
szm = size(m);
exd = szm(3:end);
nexd = length(exd);
ngpts = prod(exd);

mnom = pvget(m,'NominalValue');
if isempty(StableFlag)
   StableFlag = ones([exd 1 1]);
   switch m.Ts
      case 0
         for i=1:ngpts
            if any(real(pole(mnom(:,:,i)))>=0) % continuous, unstable
               StableFlag(i) = 0;
            end
         end
      otherwise
         for i=1:ngpts
            if any(abs(pole(mnom(:,:,i)))>=1)
               StableFlag(i) = 0;
            end
         end
   end
end
ustabindx = find(StableFlag==0);
if length(ustabindx)==ngpts  %all plants unstable
   % Rely on SETUP in UFRD to get these right
   [stabmarg,BadAtoms,Report,info] = robuststab(frd(m,0),opts,1);
   for i=1:prod(size(Report))
      Report{i} = 'Stability Margin is -inf, since NominalValue is unstable';
   end
   Report = reportconvert(Report);
   return
end
mg = ufrd(m,omega);
% A SETUP call to UFRD/ROBUSTSTAB
[stabmarg,BadAtoms,Report,info] = robuststab(mg,opts,1);
for i=1:prod(size(Report))
   Report{i} = 'Stability Margin is -inf, since NominalValue is unstable';
end

mgr = reshape(mg,[ngpts 1]);
GoodIndex = (1:ngpts)';
GoodIndex(ustabindx) = [];
mgrs = mgr(:,:,GoodIndex);
[stabmargS,BadAtomsS,ReportS,infoS] = robuststab(mgrs,opts,'uss');
stabmarg.LowerBound(GoodIndex) = stabmargS.LowerBound;
stabmarg.LowerBound(ustabindx) = -inf;
stabmarg.UpperBound(GoodIndex) = stabmargS.UpperBound;
stabmarg.UpperBound(ustabindx) = -inf;
stabmarg.DestabilizingFrequency(GoodIndex) = stabmargS.DestabilizingFrequency;
stabmarg.DestabilizingFrequency(ustabindx) = NaN;
BadAtoms(GoodIndex) = BadAtomsS; 
for k=1:nfpts
   info.BadUncertainValues{k}(GoodIndex) = infoS.BadUncertainValues{k};   
end   
info.MussvInfo.dvec.ResponseData(:,:,:,GoodIndex) = infoS.MussvInfo.dvec.ResponseData;
info.MussvInfo.pvec.ResponseData(:,:,:,GoodIndex) = infoS.MussvInfo.pvec.ResponseData;
if ~isempty(infoS.MussvInfo.gvec)
   info.MussvInfo.gvec.ResponseData(:,:,:,GoodIndex) = infoS.MussvInfo.gvec.ResponseData;
end
info.MussvInfo.sens.ResponseData(:,:,:,GoodIndex) = infoS.MussvInfo.sens.ResponseData;
info.MussvBnds.ResponseData(:,:,:,GoodIndex) = infoS.MussvBnds.ResponseData;
%info.StabilityRange(:,:,GoodIndex) = infoS.StabilityRange;

szRS = size(ReportS);
if length(szRS)==2
   Report(GoodIndex) = {ReportS};
else
   Report(GoodIndex) = mat2cell(ReportS,szRS(1),szRS(2),ones(1,szRS(3)));
end
info.Sensitivity(GoodIndex) = infoS.Sensitivity;

Report = reportconvert(Report);
