function Sys = series(Sys1,Sys2,outputs1,inputs2)
%SERIES  Series interconnection of the two uncertain models.
%
%                                  +------+
%                           v2 --->|      |
%                  +------+         USYS2 |-----> y2
%                  |      |------->|      |
%         u1 ----->|      |y1   u2 +------+
%                  |USYS1 |
%                  |      |---> z1
%                  +------+
%
%   USYS = SERIES(USYS1,USYS2,OUTPUTS1,INPUTS2) connects two USS models 
%   USYS1 and USYS2 in series such that the outputs of USYS1 specified by
%   OUTPUTS1 are connected to the inputs of USYS2 specified by INPUTS2.  
%   The vectors OUTPUTS1 and INPUTS2 contain indices into the outputs 
%   and inputs of USYS1 and USYS2, respectively.  The resulting USS model 
%   USYS maps u1 to y2.
%
%   If OUTPUTS1 and INPUTS2 are omitted, SERIES connects USYS1 and USYS2
%   in cascade and returns
%                     USYS = USYS2 * USYS1 .
%
%   If USYS1 and USYS2 are arrays of USS models, SERIES returns an LTI
%   array USYS of the same size where 
%      USYS(:,:,k) = SERIES(USYS1(:,:,k),USYS2(:,:,k),OUTPUTS1,INPUTS2) .
%
%   See also APPEND, PARALLEL, FEEDBACK, ICONNECT, SYSIC.

% Copyright 2003-2004 The MathWorks, Inc.

ni = nargin;
if ni==2,
   Sys = Sys2 * Sys1;
   return
elseif ni~=4,
   error('Wrong number of arguments (must be 2 or 4).');
end

% Error checking
sizes1 = size(Sys1);
sizes2 = size(Sys2);
lo = length(outputs1);
li = length(inputs2);
if ~isequal(sizes1(3:end),sizes2(3:end)) & length(sizes1)>2 & length(sizes2)>2,
   error('LTI arrays USYS1 and USYS2 must have compatible dimensions.')
elseif li~=lo,
   error('Vectors OUTPUTS1 and INPUTS2 must have matching lengths.')
elseif li>sizes2(2),
   error('Length of INPUTS2 exceeds number of inputs in USYS2.');
elseif lo>sizes1(1),
   error('Length of OUTPUTS1 exceeds number of outputs in USYS1.');
elseif min(inputs2)<=0 | max(inputs2)>sizes2(2),
   error('Index out of range in INPUTS2.')
elseif min(outputs1)<=0 | max(outputs1)>sizes1(1),
   error('Index out of range in OUTPUTS1.')
end

% Build series interconnection
Sys = Sys2(:,inputs2) * Sys1(outputs1,:);

