function varargout = sigma(varargin)
% SIGMA  Singular value plot of uncertain UFRD/USS models.
% 
%  SIGMA(SYS) produces a singular value (SV) plot of the frequency 
%  response of the nominal model SYS (SYS.NominalValue) and 20 random
%  models from the set of uncertain models. The frequency range and 
%  number of points are chosen automatically. See BODE for details on 
%  the notion of frequency in discrete time.
% 
%  SIGMA(SYS,{WMIN,WMAX})  draws the SV plot for frequencies ranging
%  between WMIN and WMAX (in radian/second).
% 
%  SIGMA(SYS,W) uses the user-supplied vector W of frequencies, in
%  radians/second, at which the frequency response is to be evaluated.  
%  See LOGSPACE to generate logarithmically spaced frequency vectors.
% 
%  SIGMA(SYS,W,TYPE) or SIGMA(SYS,[],TYPE) draws the following
%  modified SV plots depending on the value of TYPE:
%         TYPE = 1     -->     SV of  inv(SYS)
%         TYPE = 2     -->     SV of  I + SYS
%         TYPE = 3     -->     SV of  I + inv(SYS) 
%  SYS should be a square system when using this syntax.
% 
%  SIGMA(SYS1,SYS2,...,W,TYPE) draws the SV response of several LTI
%  models SYS1,SYS2,... on a single plot.  The arguments W and TYPE
%  are optional.  You can also specify a color, line style, and marker 
%  for each system, as in  sigma(sys1,'r',sys2,'y--',sys3,'gx').
% 
%  SV = SIGMA(SYS,W) and [SV,W] = SIGMA(SYS) return the singular 
%  values SV of the frequency response (along with the frequency 
%  vector W if unspecified).  No plot is drawn on the screen. 
%  The matrix SV has length(W) columns and SV(:,k) gives the
%  singular values (in descending order) at the frequency W(k).
% 
%  See also BODE, NICHOLS, NYQUIST.
 

% Copyright 2003-2004 The MathWorks, Inc.

nin = nargin;
nout = nargout;
mname = mfilename;
if nout>0
   strerr = ' with a USS or UFRD cannot be called with output arguments.';
   error([upper(mname) strerr])
end

N = 20;  % Number of times to sample uncertain system
   
samcell = cell(1,0);
for i=1:nin
    c=class(varargin{i});
    % RESHAPE to be PROD(extra dim)x1
    if isa(varargin{i},'dynamicsys')
       sys = varargin{i};
       sz = size(sys);
       nad = length(sz) - 2;
       ad = sz(3:end);
       if nad==2 && ad(2)==1
          nad = 1;
          ad = ad(1);
       end
       if strcmp(c,'uss') || strcmp(c,'ufrd')
           tmpu = reshape(usample(sys,N),[prod(ad)*N 1]);
           tmpnom = reshape(pvget(sys,'NominalValue'),[prod(ad) 1]);
           tmpu = stack(1,tmpu,tmpnom);         
           samcell = [samcell {tmpu}];
       else
           % Do nothing and let lti version do the error checking
           samcell = [samcell {sys}];
       end 
    else
    % Do nothing and let lti version do the error checking
       samcell = [samcell varargin(i)];
    end
 end
eval([mname '(samcell{:});']);
  