function out = simplify(a,varargin)
%SIMPLIFY Simplify representation of USS
%   B = SIMPLIFY(A) performs model-reduction-like techniques to
%   detect and eliminate redundant copies of uncertain elements.
%   Depending on the result, the CLASS of B may be lower than A
%   (which is USS).  Hence B may be a DOUBLE, ATOM, UMAT, SS or USS.
%   The AutoSimplify property of each uncertain element in
%   A determines what reduction methods are used.  After reduction,
%   any uncertain element which does not actually affect the result
%   is deleted from the representation.
%
%   B = SIMPLIFY(A,'full') overrides all uncertain element's
%   AutoSimplify property, and uses 'full' reduction techniques.
%
%   B = SIMPLIFY(A,'basic') overrides all uncertain element's
%   AutoSimplify property, and uses 'basic' reduction techniques.
%
%   B = SIMPLIFY(A,'class') does no reduction.  However, any uncertain
%   elements in A with 0 occurances are eliminated, and the class of
%   B may be lower than class of A.

% Copyright 2003-2004 The MathWorks, Inc.

% XXXLATERMAJOR - issues below make this NOTDONE
% What does SIMPLIFY do with regard to STATES?
%   if there are no states, does it return an SS/USS?  or is it DOUBLE/UMAT
%   if there are no uncertainties, does it return SS?  or still USS
%   Those are basically depromotion questions.  In an SS, there is no difference
%   between having no states, having 0 copies of integrators.  in UMAT's, the
%   situation is a bit different.  you may 0 copies of a parameter, which is
%   subtley different than not depending on the parameter at all.
%
%   BUT, subtle issue: in a UMAT, if there are zero copies of the only uncertainty,
%   then DEPROMOTE both eliminates, and depromotes to DOUBLE.  But, remove-zero-copies
%   keeps it as a UMAT.
%   M = [a a+1;3+a a] - a;
%   simplify(M,'off','noremove') -- result seems wrong here.  if you don't remove, then...
%   simplify(M,'off','nodep')
%
%   the problem seems to stem from 1 line in NDLFT/DEPROMOTE, which starts by calling
%   rmzerocp.  this has been commented out, and now the examples above exhibit proper
%   behavior.
%
%   The NODEPROMOTE and NOREMOVE and LEVEL apply only to the ATOMS
%   STATES, are reduced with FULL (or not at all - depends on NOMINREAL flag)
%   If there are, or ultimately become, no states, then the result will
%   be of lower precedence than USS.  Any .dynamicsys info is lost.


nx = length(a.StateName);
minrealchoices = {'nominreal'};
vin = nargin-1;

i = 1;
go = 1;
minrealflag = 1;
while i<=vin & go==1
   idx = find(strncmpi(minrealchoices,varargin{i},length(varargin{i})));
   if length(idx)==1
      go = 0;
      minrealflag = 0;
   end
   i = i + 1;
end
% A has a Ts, but TMPA does not
if a.Ts==0
   tmpname = 'RCASTRESERVEDCONTINUOUS';
else
   tmpname = 'RCASTRESERVEDDISCRETE';
end
tmpa = uss2umat(a,tmpname,minrealflag);
newa = simplify(tmpa,varargin{:}); % ATOM, UMAT or DOUBLE
if isa(newa,'atom') | isa(newa,'double')
   newa = umat(newa);
end
[bidx,copies,rp,cp] = gblkinfo(newa,tmpname);

a3 = a.Ts;
a4 = a.dynamicsys;
if copies==nx
   %IF NO REDUCTION ACTUALLY TAKES PLACE, THEN THE DATA IS NOT MODIFIED, keep STATENAME
   %if minrealflag==1, rebuild with STATENAME = repmat({''},[nx 1])
   a1 = blk2ioch(newa,tmpname,'t','unchanged');
   a2 = a.StateName;
   out = uss(a1,a2,a3,a4);
elseif copies>0
   a1 = blk2ioch(newa,tmpname,'t','unchanged');
   a2 = repmat({''},[copies 1]);
   out = uss(a1,a2,a3,a4);
elseif copies==0
   a1 = delblk(newa,tmpname);
   a2 = repmat({''},[0 1]);
   out = uss(a1,a2,a3,a4);
elseif isinf(copies)
   %warning('USS/SIMPLIFY, Why did this happen?');
   %XXMINOR what is this protecting against?, if the block isn't even there
   %XX Answer: 20+ lines above, we run USS2UMAT, then simplify, then lift
   %back to UMAT if necessary.  For a USS without states, the combined
   %effect of all of this eliminates the block with TMPNAME, so that COPIES
   %from the GBLKINFO call is -inf.  Seems ok ~ that's why we're in here.
   a1 = newa;
   a2 = repmat({''},[0 1]);
   out = uss(a1,a2,a3,a4);
end
% XXXCONSOLIDATE - all code below is common to all 3 versions
depromotechoices = {'nodepromote'};
vin = nargin-1;

nodepflg = 0;
for i=1:vin
   idx = find(strncmpi(depromotechoices,varargin{i},length(varargin{i})));
   if length(idx)==1
      nodepflg = 1;
   end
end

if nodepflg==0
   out = depromote(out);
end
