function sysout = ssbal(sys,midomega,FSflag,BLTflag)
%SSBAL  Diagonal state/uncertainty scaling which preserves the 
%   uncertain input/output map of an uncertain system.
%
%   USYSOUT = SSBAL(USYS)  yields a system whose input/output
%   and uncertain properties are the same as USYS, an USS object.
%   The numerical conditioning of USYSOUT is usually better than
%   that of USYS, improving the accuracy of additional computations
%   performed with USYSOUT. USYSOUT is an USS object. The balancing 
%   algorithm uses MUSSV to balance the constant uncertain state-space
%   matrices in discrete-time. If USYS  is a continuous-time uncertain
%   system, the uncertain state-space is mapped via a bilinear 
%   transformation into discrete-time for balancing.
%
%   USYSOUT = SSBAL(USYS,Wc) defines the critical frequency, Wc, for
%   the bilinear prewarp transformation from continuous-time to
%   discrete-time. The default value of Wc is 1 when the nominal 
%   uncertain system is stable and 1.25*MXEIG when it is unstable. 
%   MXEIG corresponds to the value of real of the most positive 
%   pole of USYS.
%
%   USYSOUT = SSBAL(USYS,Wc,FSflag) sets the scaling flag, FSflag, to
%   handle repeated uncertain parameters. Setting FSflag=1 uses full 
%   matrix scalings to balance the repeated uncertain parameter
%   blocks. FSflag=0, the default, uses a single, positive scalar to 
%   balance the repeated uncertain parameter blocks. 
%
%   USYSOUT = SSBAL(USYS,Wc,FSflag,BLTflag) sets the bilinear 
%   transformation flag, BLTflag. Setting BLTflag=1, default, 
%   transforms the continuous-time system USYS to discrete-time for 
%   balancing. BSTflag=0 does not transform do this transformation, 
%   rather the continuous-time state-space data from USYS is balanced. 
%   Note that if USYS is a discrete-time, no bilinear transformation 
%   is performed.
%
%   SSBAL does not work on an array of uncertain systems. An error 
%   message is generated to alert the user of this. 
%
%  See also: C2D, D2C, MUSSV, MUSSVEXTRACT

% Copyright 2004 The MathWorks, Inc.

if nargin<3
   FSflag = 0;
   BLTflag = 1;
elseif nargin<4
   BLTflag = 1;
end
tublk = zeros(0,2);
szsys = size(sys);

switch class(sys)
case 'uss'
   if length(szsys)>2
      error('SSBAL only applies to a single uncertain system.');
   end
   [M,Delta] = lftdata(sys);
   SN = pvget(sys,'StateName');
   Nx = length(SN);
   Ts = pvget(sys,'Ts');
   [Af,Bf,Cf,Df] = ssdata(M); % all 2-d doubles
   if Ts==0 & BLTflag==1
      if nargin==1 || isempty(midomega)
         eigA= eig(Af);
         idx = find(real(eigA)>0 & abs(imag(eigA))<1e-6*abs(real(eigA)));
         if isempty(idx)
            midomega = 1;
         else
            midomega = 1.2*max(real(eigA(idx)));
         end
      end
      [As,Bs,Cs,Ds] = ssdata(c2d(M,2/midomega,'tustin'));
   else
      [As,Bs,Cs,Ds] = ssdata(M);
   end
   ublk = getmutoolsblkstruct(sys,1);
   nblk = size(ublk,1);
   for i=1:nblk
      if ublk(i,2)==0
         if FSflag==1
            tublk = [tublk;abs(ublk(i,:))];
         else
            tublk = [tublk;ones(abs(ublk(i,1)),2)];
         end
      else
         tublk = [tublk;abs(ublk(i,:))];
      end
   end
case 'ss'
   if length(szsys)>2
      error('SSBAL only applies to a single uncertain system.');
   end
   SN = pvget(sys,'StateName');
   [Af,Bf,Cf,Df] = ssdata(sys);
   Ts = sys.Ts;
   Nx = size(Af,1);
   if Ts==0 & BLTflag==1
      if nargin==1 || isempty(midomega)
         midomega = 1;
      end
      [As,Bs,Cs,Ds] = ssdata(c2d(sys,2/midomega,'tustin'));
   else
      [As,Bs,Cs,Ds] = ssdata(sys);
   end
otherwise
   error([class(sys) ' is invalid for SSBAL.']);
end
            
if Nx>0
   if FSflag==1
      stateblk = [Nx 0];
   else
      stateblk = ones(Nx,2);
   end
else
   stateblk = zeros(0,2);
end
blk = [stateblk;tublk;szsys([2 1])];
W = [As Bs;Cs Ds]; % states/perts/IO
[bounds,info] = mussv(W,blk,'Us');
[dl,dr] = mussvunwrap(info);

W = dl*([Af Bf;Cf Df]/dr);
A = W(1:Nx,1:Nx);
B = W(1:Nx,Nx+1:end);
C = W(Nx+1:end,1:Nx);
D = W(Nx+1:end,Nx+1:end);
nC = norm(C);
nB = norm(B);
if nC>0 & nB>0
   rat = sqrt(nB/nC);
   B = B/rat;
   C = C*rat;
end

if FSflag==1
   SN = repmat({''},[Nx 1]);
else
   for i=1:Nx
      SN{i} = [SN{i} '(units modified)'];
   end
end

switch class(sys)
case 'uss'
   UmatData = lft(Delta,[D C;B A]);
   UmatData = UmatData([szsys(1)+1:end 1:szsys(1)],[szsys(2)+1:end 1:szsys(2)]);
   sysout = uss(UmatData,SN,Ts,pvget(sys,'dynamicsys'));
case 'ss'
   sysout = ss(A,B,C,D,sys);
   sysout.StateName = SN;
otherwise
   error('Invalid')
end
