function out = stack(dim,varargin)
% STACK  Stack USSs into USS array.
% 
%  USYS = STACK(ARRAYDIM,USYS1,USYS2,...) produces an array of USS
%  USYS by stacking the USS USYS1,USYS2,... along
%  the array dimension ARRAYDIM.  All USSs must have the same 
%  number of inputs and outputs.
% 
%  For example, if USYS1 and USYS2 are two USSs with the 
%  same I/O dimensions,
%    * STACK(1,USYS1,USYS2) produces a 2-by-1 USS array
%    * STACK(2,USYS1,USYS2) produces a 1-by-2 USS array
%    * STACK(3,USYS1,USYS2) produces a 1-by-1-by-2 USS array.
% 
%  You can also use STACK to concatenate USS arrays USYS1,USYS2,...
%  along some array dimension ARRAYDIM.
% 
%  See also HORZCAT, VERTCAT, APPEND.

% Copyright 2003-2005 The MathWorks, Inc.
if nargin<2
   error('Need at least 2 arguments.');
end
if ~(isa(dim,'double') && ndims(dim)==2 && all(size(dim)==[1 1]) && ...
      dim>0 && floor(dim)==ceil(dim))
   error('DIM should be a positive integer.');
end
if nargin==2
   out = varargin{1};
elseif nargin==3
   if isa(varargin{1},'uss') & isa(varargin{2},'uss')
      A = varargin{1};
      B = varargin{2};
      Ts = binaryts(A.Ts,B.Ts,isstatic(A),isstatic(B));
      sza = size(A);
      szb = size(B);
      nxA = length(A.StateName);
      nxB = length(B.StateName);
      if nxA>nxB
         if Ts==0
            newbu = blkdiag(-eye(nxA-nxB),B.UmatData);
         else
            newbu = blkdiag(zeros(nxA-nxB),B.UmatData);
         end
         if isequal(A.StateName(1:nxB),B.StateName) || prod(szb==0)
            sn = A.StateName;
         else
            sn = repmat({''},[nxA 1]);
         end
         B = uss(newbu,sn,Ts,B.dynamicsys);
      elseif nxA<nxB
         if Ts==0
            newau = blkdiag(-eye(nxB-nxA),A.UmatData);
         else
            newau = blkdiag(zeros(nxB-nxA),A.UmatData);
         end
         if isequal(B.StateName(1:nxA),A.StateName) || prod(sza==0)
            sn = B.StateName;
         else
            sn = repmat({''},[nxB 1]);
         end
         A = uss(newau,sn,Ts,A.dynamicsys);
      else
         if isequal(A.StateName,B.StateName)
            sn = A.StateName;
         else
            sn = repmat({''},[nxA 1]);
         end
      end
      fbname = 'RCASTRESERVED';
      uresult = stack(dim,uss2umat(A,fbname),uss2umat(B,fbname));
      UmatData = blk2ioch(uresult,fbname,'t','unchanged');
      out = uss(UmatData,sn,Ts,plus(A.dynamicsys,B.dynamicsys));
   else
      out = stack(dim,uss(varargin{1}),uss(varargin{2}));
   end
else
   out = stack(dim,stack(dim,uss(varargin{1}),uss(varargin{2})),varargin{3:end});
end
