function [out,NVout] = usample(sys,varargin)
%USAMPLE   Generates random samples of uncertain matrix or system.
%   B = USAMPLE(A,N) picks N random samples of the uncertainty in A and 
%   returns these samples in an array B of size [SIZE(A) N].  A can be 
%   an uncertain element, matrix, or system (see UMAT, USS, UFRD, UREAL,...) 
%   USAMPLE(A) is the same as USAMPLE(A,1).
%
%   [B,SAMPLES] = USAMPLE(A,N) also returns the uncertainty samples in 
%   the N-by-1 structure array SAMPLES whose field names and values are 
%   the names and sample values of the uncertain elements in A.  Note 
%   that B is the same as USUBS(A,SAMPLES).
%
%   [B,SAMPLES] = USAMPLE(A,NAMES,N) samples only the uncertain
%   elements listed in the string or string vector NAMES. The output B 
%   is uncertain unless NAMES includes all uncertain elements.  Any 
%   entry of NAMES that does not appear in A is ignored.  
%
%   [B,SAMPLES] = USAMPLE(A,NAMES1,N1,NAMES2,N2,...) takes N1 samples of
%   the uncertain elements listed in NAMES1, N2 samples of the uncertain
%   elements listed in NAMES2, and so on.  The resulting B has size 
%   [SIZE(A) N1 N2 ...].
%
%   The scalar parameter BW in
%      [B,SAMPLES] = USAMPLE(A,N,BW),
%      [B,SAMPLES] = USAMPLE(A,NAMES,N,BW) and 
%      [B,SAMPLES] = USAMPLE(A,NAMES1,N1,NAMES2,N2,BW)
%   affects how ULTIDYN elements within A are sampled, restricting the
%   poles of the samples.  If A is a continuous-time USS or UFRD, then
%   the poles of sampled GainBounded ULTIDYN elements in SAMPLES will each
%   have magnitude <= BW.  If A is a discrete-time, then sampled GainBounded
%   ULTIDYN elements are obtained by Tustin transformation of a continuous
%   system, again using BW as the (continuous) pole magnitude bound.  
%   If the ULTIDYN type is PositiveReal, then the samples are obtained by
%   bilinearly transforming (see "Normalizing Functions for Uncertain Atoms"
%   in the documentation) across the input-output channels of the GainBounded
%   elements described above.
%
%   USAMPLE is useful for Monte-Carlo analysis of uncertain systems.
%
%   Example 1: 
%      % Create an uncertain model of P(s) = g/(tau*s+1) where
%      % g varies in [3,5] and tau = 0.5 +/- 30%
%      g = ureal('g',4);
%      tau = ureal('tau',.5,'Percentage',30);
%      P = tf(g,[tau 1]);
%      % Create an integral controller based on nominal (g,tau)
%      C = tf(1,[1 0]) / (2*tau.Nominal*g.Nominal)
%      % Plot the plant and closed-loop responses for 20 samples
%      % of (g,tau)
%      subplot(2,1,1); step(usample(P,20))
%      subplot(2,1,2); step(usample(feedback(P*C,1),20))
%
%   Example 2:
%      % Create 4 uncertain parameters a,b,c,d and form M=[a b;c d]
%      a=ureal('a',1); b=ureal('b',2); c=ureal('c',3); d=ureal('d',4);
%      M = [a b;c d];
%      % Pick 5 samples for (a,b), 3 samples for (c,d), and evaluate
%      % M at the corresponding 5x3=15 combinations
%      M1 = usample(M,{'a';'b'},5,{'c';'d'},3)
%      % Evaluate M at 15 samples of (a,b,c,d)
%      M2 = usample(M,{'a';'b';'c';'d'},15)
%
%   See also USUBS, USS, UFRD, UMAT, UREAL, UCOMPLEX, ULTIDYN.

%   Copyright 2003-2006 The MathWorks, Inc.
%   $Revision: 1.1.6.4 $  $Date: 2006/12/27 21:20:30 $

nin = nargin;
okpas = 1;
bandwidth = [];
if nin>=2 && isequal(varargin{end},-1)
   okpas = 0;
   nin = nin - 1;
end
if nin>=3 && ...
      isscalar(varargin{nin-1}) && isa(varargin{nin-1},'double') && ...
      isscalar(varargin{nin-2}) && isa(varargin{nin-2},'double')
   bandwidth =  varargin{nin-1};
   nin = nin-1;
end

if nin==1
   names = pvget(sys,'UNames');
   Ln = max([1 length(names)]);
   N = num2cell(repmat(1,[Ln 1]));
   grouping = ones(Ln,1);
elseif nin==2
   if isa(varargin{1},'double') && ndims(varargin{1})==2 && ...
         imag(varargin{1})==0 && floor(varargin{1})==ceil(varargin{1}) && ...
         varargin{1}>0
      names = pvget(sys,'UNames');
      Ln = max([1 length(names)]);
      N = num2cell(repmat(varargin{1},[Ln 1]));
      grouping = ones(Ln,1);
   else
      error('In 2-argument call, the 2nd argument should be a positive integer.');
   end
elseif ceil((nin+1)/2)==floor((nin+1)/2)
   [names,N,grouping] = nvlist(varargin{1:nin-1});
else
   error('Invalid Inputs');
end
[dum,uslots] = unique(grouping);
Nslots = [N{uslots}];
Nslots = Nslots(:)';
Fslots = ones(size(Nslots));

% Suppose A is 6x7, then B=usample(A,'a',9) gives B=6x7x9(x1), 
% then C = usample(B,'fred',4) might give C=6x7x9x1x4, or should
% it give 6x7x9x4.  Well, it should certainly give the same as
% Ctilde = usmaple(A,'a',9,'fred',4), which would have to be 6x7x9x4.
% This implies that ALL trailing one array dimensions are ignored!
% This creates one problem though.  usample(A,'p1',N,'p2',3) will
% be different dimension than usample(usample(A,'p1',N),'p2',3)
% if N==1, but will be the same dimension if N>1.
sz = size(sys);
nad = length(sz) - 2;
ad = sz(3:end);
if nad==2 && ad(2)==1
   nad = 1;
   ad = ad(1);
end
adpad = ones(1,nad);

% This is the only difference between the UMAT and USS/UFRD code.
% Its probably against TMW policy, but I'm making them all the same
% for simplicity.
TS = [];
if isa(sys,'uss') || isa(sys,'ufrd')
   TS = utarbts(pvget(sys,'Ts'));
end

L = length(names);
values = cell(L,1);
cvalues = cell(L,1);
sysU = pvget(sys,'Uncertainty');
for i=1:L
   [ac,pflag,estr] = pvget(sysU,names{i});
   if pflag==1
      Fslots(grouping(i)) = Nslots(grouping(i));
   end
end
Nslots = Fslots;

keepnames = [];
NGroups = grouping(end);
for i=1:L
   [ac,pflag,estr] = pvget(sysU,names{i});
   if pflag==1
      Nslotstmp = Nslots;
      Nslotstmp(grouping(i)) = 1;
      keepnames = [keepnames;i];
      tt = pvget(ac,'Traits');
      if tt.linear==0
         error('Can only sample uncertain objects linear uncertainty')
      end
      szd = size(ac);
      nexd = length(sz) - 2 + (grouping(i)-1);
      pad = ones(1,nexd);
      switch class(ac)
      case {'ureal' 'ucomplex'}
         vv = reshape(pvget(ac,'Instance',N{i}),[1 1 pad N{i}]);
         if okpas==1
            vvg = repmat(vv,[1 1 adpad Nslotstmp]);
            cvv = reshape(num2cell(vvg,[1 2]),[adpad Nslots 1]);
         end
      case {'ucomplexm'}
         vv = reshape(pvget(ac,'Instance',N{i}),[szd pad N{i}]);
         if okpas==1
            vvg = repmat(vv,[1 1 adpad Nslotstmp]);
            cvv = reshape(num2cell(vvg,[1 2]),[adpad Nslots 1]);
         end
      case {'ultidyn'}
         dimlist = [pad N{i} 1]; % keep the trailing 1 in case pad is EMPTY
         % TS versus [] in UMAT/SAMPLE
         vv = reshape(pvget(ac,'Instance',[],TS,N{i},bandwidth),dimlist); 
         if okpas==1
            vvg = pvget(repmat(uss(vv),[1 1 adpad Nslotstmp]),'NominalValue');
            svv = size(vvg);
            cvv = cell([svv(3:end) 1 1]);
            for k=1:prod(svv(3:end))
               cvv{k} = vvg(:,:,k);
            end
         end
      end
      %values{i} = umat(vv); % really old? -- ? make it a UMAT to handle N-d BINOP
      values{i} = vv;
      if okpas==1
         cvalues{i} = cvv;
      end
   end
end
%out = usubs(sys,names(keepnames),values(keepnames));
%NVNVNV = [names(keepnames) values(keepnames)]';
%out = usubs(sys,NVNVNV{:}); % only use n,v,n,v,n,v,n,v

%XXX Why are we doing this below?  It really makes no sense.
%  We are simply enforcing???  If we want singletons whereever
% there was actually no dependence, then we don't need to do this.
% i believe we have that in the above USUBS call.  Suppose convention
% is that in Names1,N1,Names2,N2, etc that if NamesK contains
% no active names, then a singleton slot is created.  I beleive then
% we are done, and there is no need for the conditional activity below.
% we need to change NSLOTS based on what is found.
% if isempty(keepnames) || max(grouping(keepnames))<NGroups
%    disp('in last conditional...');
%    lastdim = length(sz) + NGroups;
%    repidx = [ones(1,lastdim-1) N{end}];
%    if isa(out,'ss') %XXXCTB should fix this by adding REPMAT
%       out = pvget(repmat(uss(out),repidx),'NominalValue');
%    else
%       out = repmat(out,repidx);
%    end
% end
if okpas==1
   PNVcell = [names(keepnames) cvalues(keepnames)]';
else
   PNVcell = [names(keepnames) values(keepnames)]';
end
NVout = struct(PNVcell{:});
out = usubs(sys,NVout);