function out = uss(varargin)
%USS  Specify uncertain state space.  
%   USS is the constructor for uncertain state space models 
%   (USS objects).  Uncertain state space models are
%   useful for worst-case gain and robustness analysis.
%  
%   There are three ways to construct a USS object.
%   
%   1.  Use SS to create uncertain state-space objects from 
%       uncertain state-space matrix data (See UMAT).
%
%       p = ureal('p',1);
%       A = [0 3*p; -p p^2];
%       B = [0; p];
%       C = ones(2,2);  D=zeros(2,1);
%       usys = ss(A,B,C,D);
%   
%   2.  Combine an SS, TF, or ZPK object with uncertain elements.
%
%       sys = tf(1, [1 1]);
%       k = ureal('k',1);
%       usys= k*sys;
%
%   3.  Recast a double, uncertain element (ATOM object), LTI 
%       or UMAT object as a USS object.
%
%       M = tf(1, [1 1 1]);
%       M = uss(M);
%       Here, M is a 3-state SISO system (USS object) with no uncertainty.
%
%   If M is a DOUBLE, then USYS is a static gain with no uncertain elements.
%   If M is an uncertain atom, then USYS is a static gain.
%   If M is an LTI object, then USYS has no uncertainty.
%   If M is a UMAT, then USYS is a static gain.
%
%   See also: SS, METHODS(USS)

% Copyright 2003-2004 The MathWorks, Inc.
% $Revision: 1.1.8.4 $  $Date: 2004/12/09 02:56:17 

superiorto('umat','double',...
   'ureal','ucomplex','ucomplexm','udyn','ultidyn',...
   'atom',...
   'ss','tf','zpk','frd');

validate = 1;
mkatend = 1;
ourcall = 0;

% Determine how many Property NameValue pairs (NVP)
% AIN denotes number of leading arguments (not name/value pairs)
nin = nargin;
nvp = 0;
i = 1;
while nin>i && isa(varargin{nin-i},'char')
   nvp = nvp + 1;
   i = i + 2;
end
ain = nin - 2*nvp;

if nvp==0
   if ain == 0
      a1 = umat;
      a2 = cell(0,1);
      a3 = utarbts;
      a4 = dynamicsys(0,0);
   elseif ain==1 %  USS(USS), USS(DOUBLE), USS(PSS)
      arg1 = varargin{1};
      switch(class(arg1))
      case {'uss'}
         if nvp==0
            mkatend = 0;
            out = arg1;
         else
            error('Use SET to change properties');
         end
      case {'double'}
         if ispsys(arg1)
            % Special handling of PSYS -> USS
            try
               [sys,delta] = LocalAFF2USS(arg1);
            catch
               rethrow(lasterror)
            end
            out = lft(blkdiag(delta{:}),sys);
            return
         end
         a1 = arg1;
         szm = size(a1);
         a1 = umat(a1);
         a2 = cell(0,1);
         a3 = utarbts;
         a4 = dynamicsys(szm(1),szm(2));
      otherwise
         error('Invalid first argument');
      end
   elseif ain==2 % USS(STRUCT,DS)
      a1 = varargin{1};
      arg2 = varargin{2};
      if isa(a1,'struct') && isa(arg2,'dynamicsys')
         if isequal(fieldnames(a1),{'UmatData';'StateName';'Ts'})
            mkatend = 0;
            out = class(a1,'uss',gsref(0),arg2);
         else
            error('Fields of Struct must be: UmatData, StateName, Ts.');
         end
      else
         error('Direct construction of USS requires struct and dynamicsys objects.');
      end
   elseif ain>=4  % USS(D,D,D,D) or USS(D,D,D,D,D)
      cain = zeros(1,4);
      for i=1:4
         cain(i) = isa(varargin{i},'double');
      end
      if ~all(cain==1)
         badarg = find(cain==0);
         error(['Argument #' int2str(badarg(1)) ' is incorrect class']);
      end
      [a1,a2,a4,ecode,estr] = LOCALfunctionSS(umat(varargin{1}),...
         umat(varargin{2}),umat(varargin{3}),umat(varargin{4}));
      if ecode
         error(estr)
      end
      a3 = 0; % possibly overwritten below, based on ARG5
      if ain==5
         arg5 = varargin{5};
         if isa(arg5,'double') & ndims(arg5)==2 & all(size(arg5)==[1 1]) & ...
               isreal(arg5)
            a3 = arg5;
         else
            error('5th argument should be SampleTime');
         end
      end
   else
      error('Invalid number of arguments');
   end
else 
   out = uss(varargin{1:ain});  % OUT can be only USS or UFRD
   out = set(out,varargin{ain+1:end});
   validate = 0;
   mkatend = 0;
end
if mkatend
   out.UmatData = a1;
   out.StateName = a2;
   out.Ts = a3;
   out = class(out,'uss',gsref(0),a4);
end

% Check validity of OUT
if validate==1
   [pflag,out,estr] = isvalid(out);
   if pflag==0
      error(estr);
   end
end


function [sys,delta] = LocalAFF2USS(affsys)
% Convert affine p-sys into USS
if affsys(2,1)~=2,
   error('First argument is not an affine parameter-dependent system');
end
[typ,nv,ns,ni,no]=psinfo(affsys);
pv=psinfo(affsys,'par');
[typ,np,npv]=pvinfo(pv);
if ~strcmp(typ,'box'),
   error('PV must be of type ''box'' (see PVEC)');
end
[range,rate]=pvinfo(pv,'par');
tol=eps^(2/3);

% center parameter range by redefining S0
pmed=(range(:,1)+range(:,2))/2;
s0=psinfo(affsys,'eval',pmed);

% get delta bounds
bnds=(range(:,2)-range(:,1))/2;

% construct the nominal plant SYS and the uncertainty DELTA
[a0,b0,c0,d0,e0]=ltiss(s0);
if rcond(e0) < eps,
   error('E(p) is singular for the median value of p');
end
delta = cell(1,0);

L=[]; M=[]; R=[]; S=[]; T=[];
Warn = false;

for i=2:nv,
   [ai,bi,ci,di,ei]=ltiss(psinfo(affsys,'sys',i));
   [u,s,v,rk]=svdparts([ei ai bi;zeros(no,ns) ci di],0,tol);

   if rk > 0
      W=u*diag(sqrt(s));   Z=diag(sqrt(s))*v';
      L=[L  W(1:ns,:)];  M=[M  W(ns+1:ns+no,:)];
      T=[T ; Z(:,1:ns)]; R=[R ; Z(:,ns+1:2*ns)];
      S=[S ; Z(:,2*ns+1:2*ns+ni)];
      % Uncertainty
      if max(abs(rate(i-1,:))) > 0,
         Warn = true;
      end
      p = ureal(sprintf('p%d',i-1),0,'PlusMinus',bnds(i-1));
      delta = [delta,repmat({p},[1 rk])];
   end
end

if Warn
   warning('Ignoring rate bounds on parameter variations when converting to USS.')
end

A = e0\a0;
B = e0\[L b0];
C = [R-T*A ; c0];
D = [-T*B + [zeros(size(S,1),size(M,2)) S] ; M d0];
sys = ss(A,B,C,D);


function [umatdata,sn,dsfield,ecode,estr] = LOCALfunctionSS(a,b,c,d)
ecode = 0; estr = '';
sza = size(a);
szb = size(b);
szc = size(c);
szd = size(d);
if sza(1)~=sza(2)
   ecode = 1;
   estr = 'A-matrix needs to be square';
end
if sza(1)~=szb(1)
   ecode = 1;
   estr = 'A and B matrices need the same number of rows';
end
if sza(2)~=szc(2)
   ecode = 1;
   estr = 'A and C matrices need the same number of columns';
end
if isempty(d) || (isa(d,'double') && ndims(d)==2 && all(size(d)==[1 1]) && d==0)
   d = zeros(szc(1),szb(2));
end
szd = size(d);
if szd(1)~=szc(1)
   ecode = 1;
   estr = 'C and D matrices need the same number of rows';
end
if szd(2)~=szb(2)
   ecode = 1;
   estr = 'B and D matrices need the same number of columns';
end
if ~ecode
   umatdata = [a b;c d];
   dsfield = dynamicsys(szc(1),szb(2));
   sn = cell(sza(1),1);
   for i=1:sza(1)
      sn{i} = '';
   end
elseif isempty(a) && isempty(b) && isempty(c)
   umatdata = d;
   dsfield = dynamicsys(szd(1),szd(2));
   sn = cell(0,1);
   ecode = 0;
   estr = '';
else
   umatdata = [];
   dsfield = [];
   sn = [];
end

