function [MaxGain,WCNamesValues,Info,Data,StableFlag] = ...
   wcgain(M,opts,PrevData,SETUP,StableFlag)
%WCGAIN   Calculates worst-case gain of an uncertain system.
% "Worst-case" refers to maximum over the uncertainty, and "norm" is
% measured by the maximum over frequency of the maximum singular value
% of the frequency response matrix (ie., the H-infinity norm).
% 
% [MAXGAIN,MAXGAINUNC,INFO] = WCGAIN(SYS) calculates upper and lower
% bounds for the worst-case gain of an uncertain state-space object (USS).  
% 
% MAXGAIN  Structure with the following fields:
%         LowerBound: Upper bound on worst-case gain. 
%         UpperBound: Lower bound on worst-case gain. 
%  CriticalFrequency: Frequency at which the maximum gain occurs.
% MAXGAINUNC  Structure of worst-case uncertainty values
% INFO  Structure with the following fields:
%        Sensitivity: Structure of percentages corresponding to MAXGAIN 
%                     sensitivity to variations in each uncertainty level.
%          Frequency: Frequency vector used in analysis.
%         ArrayIndex: Array index (for arrays of uncertain systems)
%                     where the maximum gain occurs.  Value is scalar,
%                     for single index use.
% 
% [MAXGAIN,MAXGAINUNC,INFO] = WCGAIN(SYS,OPTS) specifies options
% described in OPTS.  See WCGOPT for more details on the options for
% WCGAIN. 
% 
%See also LOOPMARGIN, MUSSV, LTI/NORM, ROBUSTSTAB, WCGOPT, WCSENS, WCMARGIN.

% Copyright 2003-2004 The MathWorks, Inc.

nin = nargin;
% Never uses SETUP, but argument list is identical to UFRD/WCGAIN.
if nin==1
   opts = wcgopt;
   PrevData = [];
   SETUP = [];
   StableFlag = [];
elseif nin==2
   PrevData = [];
   SETUP = [];
   StableFlag = [];
elseif nin==3
   SETUP = [];
   StableFlag = [];
elseif nin==4
   StableFlag = [];
end

if ~allatomslti(M)
   %error('Some atoms are not linear, time-invariant, WCGAIN is not defined');
   error('All uncertain elements must be linear, time-invariant.');
end


%XXXMINOR genfresp doesn't work with LTI arrays
L.type = '()';
L.subs = {':',':',1};
M1 = subsref(M,L);
M1nom = pvget(M1,'NominalValue');
if isstatic(M1nom)
   omega = 1;
else  
   [mag,ph,omega] = genfresp(M1nom,4,[]);
   if length(omega)>15
      omega = omega(4:end-4);
   end
   %omega = logspace(-1,2,3); % XXX FOR TESTING
end
nfpts = length(omega);

szM = size(M);  % note that size(USS) will never have length==3
AD = szM(3:end); % empty or >=2; empty means 1-by-1, nonempty means NOT 1-by-1
nAD = length(AD);
Y = unique(sort(opts.ArrayDimPtWise));
idx = find(Y>nAD);
Y(idx)=[];
X = 1:nAD;
X(Y)=[];
E=AD;
E(X)=1;
nYgpts = prod(AD(Y)); % 1 or greater, never 0, even if "not present"
nXgpts = prod(AD(X)); % 1 or greater, never 0, even if "not present"

Mnom = pvget(M,'NominalValue');
if isempty(StableFlag)
   StableFlag = ones([AD 1 1]);
   switch M.Ts
      case 0
         for i=1:prod(AD)
            if any(real(pole(Mnom(:,:,i)))>=0) % continuous, unstable
               StableFlag(i) = 0;
            end
         end
      otherwise
         for i=1:prod(AD)
            if any(abs(pole(Mnom(:,:,i)))>=1)
               StableFlag(i) = 0;
            end
         end
   end
end
ustabindx = find(StableFlag==0);
Mg = ufrd(M,omega);

if length(AD)>0 % either Y and/or X is NonEmpty
	theIdx = cell(nAD,1);
	[theIdx{:}] = ind2sub(AD,ustabindx);
   %XXX GJB 15jan04, new matlab changed behaviour of SUB2IND and caused
   % the old code to bomb.
   if isempty(Y)
      NUBadYsing = [];
   elseif length(Y)==1
      NUBadYsing = (sub2ind([AD(Y) 1],theIdx{Y})); %bad Y's as a single index
   else
      NUBadYsing = (sub2ind(AD(Y),theIdx{Y})); %bad Y's as a single index within Ygrid
   end      
   % BadYsing contains the bad Y's as a single index within Ygrid
   [BadYsing,Iidx,Jidx] = unique(NUBadYsing);
   
   if length(BadYsing)==nYgpts || (isempty(Y) && ~isempty(ustabindx))
      % HOW: Y is empty and there is at least one within the Xgrid, or ...
      %      at every Ypt, there is an unstable among the Xpts
      MaxGain.LowerBound = repmat(inf,[E 1 1]);
      MaxGain.UpperBound = repmat(inf,[E 1 1]);
      MaxGain.CriticalFrequency = repmat(0,[E 1 1]);
      WCNamesValues = [];
      Info.Sensitivity = [];
      Info.Frequency = [];
      %Info.MaximumGain = [];
      %Info.BadAtomValues = [];    
      Info.ArrayIndex = [];
      return
   else
      % Call SETUP to get the dimensions REALLY correct
      [MaxGain,WCNamesValues,Info] = wcgain(Mg,opts,[],1,[]);
      % Now permute...
		Mg = permute(Mg,[X Y]);  % even if both X and Y are empty, this is fine
		Mg = reshape(Mg,[prod(AD(X)) prod(AD(Y))]); % if Xand Y are empty, then [1 1], ok
		GoodYsing = (1:prod(AD(Y)))'; % if Y is empty, this equals 1
		GoodYsing(BadYsing) = []; % if Y is empty, BadSing is [], so this equals 1.  Good.
      % Anything of dimension E, will be filled with Out(GoodYSing) = OutS
		Mg = Mg(:,:,:,GoodYsing); % only array locations with Stable CLP
   end
elseif StableFlag==0
  % Single system, nominally unstable
  MaxGain.LowerBound = inf;
  MaxGain.UpperBound = inf;
  MaxGain.CriticalFrequency = 0;
  WCNamesValues = [];
  Info.Sensitivity = [];
  Info.Frequency = [];
  % Info.MaximumGain = [];
  % Info.BadAtomValues = [];    
  Info.ArrayIndex = [];
  return    
else
   [MaxGain,WCNamesValues,Info] = wcgain(Mg,opts,[],1,[]);
   GoodYsing = 1;
end

% The actual computational call to UFRD/WCGAIN
tempopts = opts;
tempopts.ArrayDimPtWise = [2];  % Y
[MaxGainS,WCNamesValuesS,InfoS,Data] = wcgain(Mg,tempopts,PrevData,[],[]);
MaxGain.CriticalFrequency(GoodYsing) = MaxGainS.CriticalFrequency;
if opts.FreqPtWise==0
   MaxGain.LowerBound(GoodYsing) = MaxGainS.LowerBound;
   MaxGain.UpperBound(GoodYsing) = MaxGainS.UpperBound;
   WCNamesValues(GoodYsing) = WCNamesValuesS;
   Info.Sensitivity(GoodYsing) = InfoS.Sensitivity;
   Info.Frequency = InfoS.Frequency;
   [Info.MaximumGain,wcidx] = max(MaxGain.LowerBound(:));
   Info.BadAtomValues = WCNamesValues(wcidx);
   Info.ArrayIndex(GoodYsing) = InfoS.ArrayIndex;
else
   mxgn = -1;
   for i=1:nfpts
      MaxGain.LowerBound.ResponseData(1,1,i,GoodYsing) = ...
         MaxGainS.LowerBound.ResponseData(1,1,i,:);
      MaxGain.UpperBound.ResponseData(1,1,i,GoodYsing) = ...
         MaxGainS.UpperBound.ResponseData(1,1,i,:);
      WCNamesValues{i}(GoodYsing) = WCNamesValuesS{i};
      Info.Sensitivity{i}(GoodYsing) = InfoS.Sensitivity{i};
      Info.ArrayIndex{i}(GoodYsing) = InfoS.ArrayIndex{i};
      [tmpgn,tmpidx] = max(MaxGain.LowerBound.ResponseData(1,1,i,:));
      if tmpgn>mxgn
         mxgn = tmpgn;
         wcidx = tmpidx;
         wci = i;
      end
   end
   Info.Frequency = InfoS.Frequency;
   Info.MaximumGain = mxgn;
   Info.BadAtomValues = WCNamesValues{wci}(wcidx);
end
% Decided to not return this information in INFO structure
Info = rmfield(Info,'MaximumGain');
Info = rmfield(Info,'BadAtomValues');
