function wcst = wcsens(P,C,varargin)
% WCSENS  Worst-case sensitivity of plant-controller feedback loop.
%
%WCST = WCSENS(L) calculates the worst-case sensitivity and 
%complementary sensitivity functions for the feedback loop consisting 
%of the loop transfer function L, of size N-by-N, in negative feedback 
%with an N-by-N identity matrix. If L is a USS, the frequency range 
%and number of points used to calculate WCMARG are chosen automatically.
% 
% WCST is a structure with the following substructures:
%    Si: Worst-case input-to-plant Sensitivity function 
%    Ti: Worst-case input-to-plant Complementary Sensitivity function 
%    So: Worst-case output-of-plant Sensitivity function 
%    To: Worst-case output-of-plant Complementary Sensitivity function 
%   PSi: Worst-case plant times input-to-plant Sensitivity function 
%   CSo: Worst-case compensator times output-of-plant Sensitivity function 
%Stable: 1 if nominal closed loop is stable, 0 otherwise. NaN for FRDs.
%
%WCST = WCSENS(P,C) calculates the worst-case sensitivity and 
%complementary sensitivity functions for the feedback loop C in 
%negative feedback with P.  C should only be the compensator in the 
%feedback path, not any reference channels, if  it is a "2-dof" 
%architecture. If P and C are SS/TF/ZPK or USS objects, the 
%frequency range and number of points are chosen automatically.
% 
%Each sensitivity substructures, WCST.Si, WCST.Ti, etc, is also a 
%structure containing five fields: 
%  MaximumGain: Structure containing a lower bound, lbound, and upper bound, 
%           ubound, on the sensitivity function and the critical value
%           of frequency, critfreq, at which the maximum gain occurs.
%  BadUncertainValues: Structure containing values of uncertain elements 
%           which maximize the System gain.  The fieldnames of 
%           BadUncertainValues, are the names of uncertain elements of the 
%           System field. 
%  System: Sensitivity transfer matix associated with structure, i.e. for
%           WCST.Si is the input sensitivity function.
%  BadSystem: Worst-case sensitivity transfer matix associated with the
%           the System field (FRD object).
%  Sensitivity: Structure whose fieldnames are names of uncertain elements
%           of the 'System' field. Values of fields are positive numbers, 
%           each entry indicating the local sensitivity of the worst-case 
%           gain in 'maxgain'. Default is [].
%
%WCST = WCSENS(L,TYPE,SCALING) AND WCST = WCSENS(P,C,TYPE,SCALING) allow
%selection of the desired Sensitivity and Complementary Sensitivity 
%functions, TYPE, as 
%   'Si', 'Ti', 'So', 'To', 'PSi', 'CSo'  correspond to the Sensitivity
%       and Complementary Sensitivity functions.
%   'S' or 'T' selects all Sensitivity functions ('Si', So', 'CSo','PSi') or
%       all Complementary Sensitivity functions ('Ti', To').
%   'Input' or 'Output' selects all input Sensitivity functions 
%       ('Si', Ti', 'PSi') or all output Sensitivity functions ('So','To', CSo').
%   'All' selects all Sensitivity functions (default).
%
%SCALING defines the scaling of the sensitivity function used in the 
%worst-case analysis. SCALING is either the character strings 'Absolute'
%(default), 'Relative' or a SS/TF/ZPK/FRD object. If SCALING is 'Relative'
%or a SS/TF/ZPK/FRD object, the worst-case analysis peaks over frequency
%relative to the magnitude of the nominal value of the sensitivity
%function. If SCALING is a SS/TF/ZPK/FRD object, its input/output 
%dimensions should be either 1-by-1 or dimensions compatible with P and C
%respectively. TYPE and SCALING can also be combined in a cell array, e.g.
%
% WCST = WCSENS(P,C,{'Ti','So'},'Abs','Si','Rel','PSi',WT)
%
%This calling sequence indicates that the 'Ti' and 'So' worst-case 
%sensitivites are calculated with respect to an absolute magnitude weight.
%The 'Si' worst-case sensitivity is calculated with respect to an
%relative magnitude weight and the 'PSi' worst-case sensitivity is
%calculated with respect to WT scaling.
%
%WCST = WCSENS(L,OPT), WCST = WCSENS(L,TYPE,SCALING,OPT),
%WCST = WCSENS(P,C,OPT) or WCST = WCSENS(P,C,TYPE,SCALING,OPT) specifies 
%options described by OPT.  (See WCGOPT for more details on the options.) 
%The default behavior does not compute sensitivity of the worst-case 
%margins to the uncertainties, MarginSens.  To compute sensitivities, 
%create a WCGOPT options object, and set the Sensitivity property to 'on'.
%
% OPT = WCGOPT('Sensitivity','on');
% WCST = WCMARGIN(P,C,OPT)
%
%If L, P or C are arrays, the substructures of WCST will have the appropriate
%array dimensions.
%
% Example
%
% Consider a feedback loop with a first order plant and a PI controller. 
% The time constant is uncertain and the feedback loop accounts for 
% unmodeled dynamic uncertainty.
%
%   delta = ultidyn('delta',[1 1]);
%   tau = ureal('tau',5,'range',[4 6]);
%   P = tf(1,[tau 1])*(1+0.25*delta);
%   C = tf([4 4],[1 0]);
%   looptransfer = loopsens(P,C);
%   Snom = looptransfer.Si.NominalValue;
%   om = logspace(-0.5,1,30);
%   wcst = wcsens(frd(P,om),C,'Si');    
%   Swc = wcst.Si.BadSystem;
%   bodemag(frd(Snom,Swc.Frequency),'b-',Swc,'r-.');
%
% See also LOOPSENS, LOOPMARGIN, MUSSV, ROBUSTSTAB, USUBS,
%   WCGOPT, WCGAIN, WCMARGIN.

% Copyright 2003-2004 The MathWorks, Inc.
% $Revision: 1.1.6.1 $ $Date: 2004/11/18 23:41:43 $

nin = nargin;
exin = nin-2;

% figure out if being called with just Loop Transfer Function.
if nin == 1 % must be Loop Transfer Function, square, C=I, 
   szp = size(P);
   if szp(1)==szp(2)
      C = eye(szp(1),szp(1));
      opts = [];
   else
      error('Loop Transfer Function must be square');
   end
elseif nin == 2 % either Loop Transfer Function with OPT or P,C
   szp = size(P);
   % must beL and something else
   if  ~( isa(C,'double') || isuncertain(C) || isa(C,'lti') )
      if szp(1)==szp(2)
         varargin{1} = C;
         C = eye(szp(1),szp(1));
         nin = 3;
      else
         error('Loop Transfer Function must be square');
      end
   end
% must be  Loop Transfer Function with OPT
elseif ~( isa(C,'double') || isuncertain(C) || isa(C,'lti') )
   szp = size(P);
   if szp(1)==szp(2) 
      exin = exin+1;
      tmp{1} = C;
      C = eye(szp(1),szp(1));
      for ii=1:nin-2
         tmp{ii+1} = varargin{ii};
      end
      nin = nin+1;
      varargin = tmp;
   else
      error('Loop Transfer Function must be square');
   end
end


if nin<3
   opts = [];
   varargin{1} = 'All';
   varargin{2}  = 'Absolute';
   exin = 2;
end
if nin == 3 %either OPT or TYPE
   if isempty(varargin{1})
      opts = [];
      varargin{1} = 'All';
      varargin{2}  = 'Absolute';
   elseif isa(varargin{1},'wcgopt') %will change to isa(varargin{1},'struct')
      opts = varargin{1};
      varargin{1} = 'All';
      varargin{2}  = 'Absolute';
   elseif isa(varargin{1},'cell') || isa(varargin{1},'char')
      opts = [];
      varargin{2}  = 'Absolute';
   else
      error('Invalid TYPE description')
   end
   exin = 2;
else
   if mod(exin,2) %odd # of EX inputs, last must be OPT
      if ~isa(varargin{exin},'wcgopt')
         error('Invalid number of TYPE, SCALING and OPT')
      else
         opts = varargin{exin};
         exin = exin - 1;
      end
   elseif exin==2 %is either TYPE/SCALING or TYPE/OPT
      if isa(varargin{exin},'wcgopt')
         opts = varargin{exin};
         varargin{2}  = 'Absolute';
      else
         opts = [];
      end
   else % even # of EX inputs
      opts = [];
   end
end

if isempty(opts)
   opts = wcgopt;
end
[names,values,grouping] = nvlist(varargin{1:exin});
wcst = [];
            
typecell = {'Si' 'no';'Ti' 'no'; 'So' 'no'; 'To' 'no'; 'PSi' 'no'; 'CSo' 'no'};

for i=1:length(names)
   xname = names{i};
   if any(strncmpi(xname,'All',3)) % allsens functions
     typecell(:,2) = repmat(values(i),6,1);
   elseif any(strcmpi(xname,'T')) % wants all comp sens functions
      typecell([2 4],2) = repmat(values(i),2,1);
   elseif any(strcmpi(xname,'S')) % wants all comp sens functions
      typecell([1 3],2) = repmat(values(i),2,1);
   elseif any(strncmpi(xname,'I',1)) % wants all input sens functions
      typecell([1 2 5],2) = repmat(values(i),3,1);
   elseif any(strncmpi(xname,'O',1)) % wants all output sens functions
      typecell([3 4 6],2) = repmat(values(i),3,1);
   elseif strcmpi(xname,'Si') || strcmpi(xname,'Ti') || strcmpi(xname,'So') ...
         || strcmpi(xname,'To') || strcmpi(xname,'PSi') ||strcmpi(xname,'CSo')
      xloc = find(strcmpi(typecell(:,1),xname));
      typecell(xloc,2) = values(i);
   else
      error('Invalid sensitivity or complementary sensitivity function type requested')
   end
end
% All options should now be set in terms of Si, Ti, So, To, PSi, CSo
szp = size(P);
szc = size(C);
if szc(1)==szp(2) & szp(1)==szc(2)
   nu = szp(2);
   ny = szp(1);
   [info,sntisnto] = loopsens(P,C); 
   if ~(isa(sntisnto,'frd') || isa(sntisnto,'ss') || isa(sntisnto,'tf') || ...
         isa(sntisnto,'zpk') || isa(sntisnto,'ufrd') || isa(sntisnto,'uss'))
      error('P or C must be from SS/TF/ZPK/FRD/USS/UFRD classes.');
   end
	%TYPECELL contains all the info regarding how to compute the sensitivities.
	%a 'no' in the 2nd cell entry indicates that entry should not be calculated.
   wcst = struct('Si',[],'Ti',[],'So',[],'To',[],'PSi',[],'CSo',[],'Stable',[]);
   wcst.Stable = info.Stable;
   go = 1;
   L.type = '()';
   %-------------------------------  Si ----------------------------------------------%
   if ~isequal(typecell{1,2},'no') && go %case 'Si'
      L.subs = {1:nu,1:nu};
      Si = subsref(sntisnto,L);
      scale = typecell{1,2};
      [wcst,go] = LOCALwcsens(Si,scale,opts,'Si',wcst);
   end
   %-------------------------------  Ti ----------------------------------------------%
   if ~isequal(typecell{2,2},'no') && go %case 'Ti'
      L.subs = {nu+1:2*nu,1:nu};
      Ti = -subsref(sntisnto,L);
%      Ti = -sntisnto(nu+1:2*nu,1:nu);
      scale = typecell{2,2};
      [wcst,go] = LOCALwcsens(Ti,scale,opts,'Ti',wcst);
   end
   %-------------------------------  So ----------------------------------------------%
   if ~isequal(typecell{3,2},'no') && go %case 'So'
      L.subs = {2*nu+1:2*nu+ny  , nu+1:nu+ny};
      So = subsref(sntisnto,L);
%      So =  sntisnto(   2*nu+1:2*nu+ny  , nu+1:nu+ny);
      scale = typecell{3,2};
      [wcst,go] = LOCALwcsens(So,scale,opts,'So',wcst);
 end
   %-------------------------------  To ----------------------------------------------%
   if ~isequal(typecell{4,2},'no') && go %case 'To'
      L.subs = {2*nu+ny+1:2*nu+2*ny, nu+1:nu+ny};
      To = subsref(sntisnto,L);
%      To = -sntisnto(2*nu+ny+1:2*nu+2*ny, nu+1:nu+ny);
      scale = typecell{4,2};
      [wcst,go] = LOCALwcsens(To,scale,opts,'To',wcst);
   end
   %-------------------------------  PSi ----------------------------------------------%
   if ~isequal(typecell{5,2},'no') && go %case 'PSi'
      L.subs = {2*nu+1:2*nu+ny,1:nu};
      PSi = subsref(sntisnto,L);
%      PSi = sntisnto(2*nu+1:2*nu+ny,1:nu);
      scale = typecell{5,2};
      [wcst,go] = LOCALwcsens(PSi,scale,opts,'PSi',wcst);
   end
   %-------------------------------  CSo ----------------------------------------------%
   if ~isequal(typecell{6,2},'no') && go %case 'CSo'
      L.subs = {2*nu+1:2*nu+ny,1:nu};
      CSo = subsref(sntisnto,L);
%      CSo = sntisnto(2*nu+1:2*nu+ny,1:nu);
      scale = typecell{6,2};
      [wcst,go] = LOCALwcsens(CSo,scale,opts,'CSo',wcst);
   end
else
   error('Incorrect dimension')
end

function [wcst,go] = LOCALwcsens(ST,scale,opts,name,wcst)

go = 1;

System = ST;

[tmpg,scale,topts] = LOCALprewt(ST,scale,opts,name);
[maxgain,wcunc,info] = wcgain(tmpg,topts);

if isinf(maxgain.UpperBound)
   go = 0;
else
   %if Si USS -> SiBad USS, Si UFRD -> SiBad UFRD
   BadSystem = usubs(ST,wcunc); 
   if ~isa(BadSystem,'frd')
      utmp = frd(BadSystem,info.Frequency);
   else
      utmp = BadSystem;
   end
   unwtLB = wcgain(utmp,topts);
   weightedmaxgain = maxgain;
   unweightedmaxgain.LowerBound = unwtLB.LowerBound;
   scl = unwtLB.LowerBound/maxgain.LowerBound;
   unweightedmaxgain.UpperBound = maxgain.UpperBound*scl;
   unweightedmaxgain.CriticalFrequency = unwtLB.CriticalFrequency;
   % Return UNWEIGHTED maxgain instead of WEIGHTED maxgain
   wcanal = struct('MaximumGain',unweightedmaxgain,'BadUncertainValues',wcunc,...
      'System',System,'BadSystem',utmp,'Sensitivity',info.Sensitivity); 
   wcst = setfield(wcst,name,wcanal);
end

function [STout,scaleout,optout] = LOCALprewt(STin,scale,optin,STtype);

% STin is now a UFRD for a specific S and T case
STout = STin;
optout = optin;
if isa(scale,'char') && strncmpi(scale,'Abs',1)
   scaleout = scale;
elseif isa(scale,'char') && strncmpi(scale,'Rel',1)
   % "Relative" scaling, or any scaling, implies interested in the PEAK, since 
   %  scaling wouldn't matter if you are doing PTWISE.
   if isa(STin,'ufrd') || isa(STin,'frd')
      if isa(STin,'ufrd')
         STnom = get(STin,'NominalValue');
         scaleout = 1/fnorm(STnom,inf); %sigma max across freq and array dim
      else
         scaleout = 1/fnorm(STin,inf); %sigma max across freq and array dim
      end
      STout = scaleout*STin;
   elseif isa(STin,'ss') || isa(STin,'tf') || isa(STin,'zpk') || isa(STin,'uss')
      msg = ['P and/or C must be a UFRD or FRD for Relative Scaling of ' STtype ' Sensitivity'];
      error(msg)
   end
   %optout.FreqPtWise = 0;              %looking just for the peak??
elseif isa(scale,'double')
   try
      scaleout = abs(scale);
      STout = scaleout*STin; %scale STin
   catch
      msg = [STtype ' and Scaling array dimensions are inconsistent'];
      error(msg)
   end
elseif isa(scale,'ss') || isa(scale,'tf') || isa(scale,'zpk') || ...
      isa(scale,'frd') || isa(scale,'uss') || isa(scale,'ufrd')
   if STin.Ts == scale.Ts || (STin.Ts == -1 & scale.Ts ~= 0) ...
         || (STin.Ts ~= 0 & scale.Ts == -1)
      szm = size(scale);
      szst = size(STin);
      if szm(1)==szm(2) && szm(1)==1 % correct IO dim
         szml = length(szm);
         if szml==2 || isequal(szm(3:szml),szst(3:szml))
            %-------------- SCALE ia an FRD or UFRD ----------------------%
            if isa(scale,'frd') || isa(scale,'ufrd')
               if ~(isa(STin,'ufrd') || isa(STin,'frd'))
                  STin = frd(STin,scale.Frequency);
               end
               if isequal(scale.Frequency,STin.Frequency)
                  if isa(scale,'frd')
                     scaleout = abs(scale);
                  else
                     scalenom = get(scale,'NominalValue');
                     scaleout = abs(scalenom);
                  end
               else
                  msg = [STtype ' and Scaling have inconsistent frequency vectors'];
                  error(msg)
               end
            %-------------- SCALE ia an SS/TF/ZPK or USS -----------------%
            elseif (isa(scale,'ss') || isa(scale,'tf') || isa(scale,'zpk')) ...
                  || isa(scale,'uss')
               if isa(STin,'ufrd') || isa(STin,'frd')
                  if isa(scale,'uss')
                     scalenom = get(scale,'NominalValue');
                     scaleout = abs(frd(scalenom,STin.Frequency));
                  else
                     scaleout = abs(frd(scale,STin.Frequency));
                  end
               else
                  if isa(scale,'ss') || isa(scale,'tf') || isa(scale,'zpk')
                     scaleout = scale;
                  else
                     scaleout = get(scale,'NominalValue');
                  end
               end
            end
         else
            msg = [STtype ' and Scaling array dimensions are inconsistent'];
            error(msg)
         end
      else
         msg = [STtype ' Scaling is not a single-input/single-output system'];
         error(msg)
      end
      STout = scaleout*STin; %scale STin
   else
      msg = [STtype ' and Scaling have different sample times'];
      error(msg)
   end
else
   msg = ['Invalid ' STtype ' Scaling, must be a SS/TF/PZK/FRD of correct dimension'];
   error(msg)
end
