function out = wcgopt(varargin)
%WCGOPT    Create a WCGAIN OPTIONS object.
%  OPTIONS = WCGOPT('NAME1',VALUE1,'NAME2',VALUE2,...) creates a
%  options object for the command WCGAIN.
% 
%  OPTIONS = WCGOPT, by itself, creates an object, initialized
%  with all default values.
%
%  WCGOPT PROPERTIES
%
%  Sensitivity - Computes margin sensitivity to individual uncertainties. 
%  Default is 'on'.
%
%  LowerBoundOnly:  If LowerBoundOnly is 'on', then only the lower bound
%  computation will be performed.  The default value is 'off', which implies
%  that both upper and lower bounds for worst-case gain are computed.
%
%  FreqPtWise - Apply stopping criteria based on upper/lower bounds 
%  (described below)at every frequency point (as opposed to just the peak 
%  value).  FreqPtWise=1 activates the pointwise criteria.  In order to 
%  only compute the peak value to within tolerance, use 0.  Default = 0.
%
%  ArrayDimPtWise: Relevant for USS/UFRD/SS/FRD arrays.  For indices
%  specified in ArrayDimPtWise, the stopping criteria based on upper/lower
%  bounds (described below) is used at every point in array dimensions
%  specified in ArrayDimPtWise, being applied to the peak value over
%  all other array dimensions.  Default = [].
%
%  If FreqPtWise==1, then the computation terminates when at EVERY frequency
%  point at least one of the following four conditions is true:
%
%     UpperBound - LowerBound <= AbsTol
%     UpperBound - LowerBound <= Reltol*UpperBound
%     UpperBound <= AGoodThreshold + MGoodThreshold*Norm(NominalValue)
%     LowerBound >= ABadThreshold + MBadThreshold*Norm(NominalValue)
%
%  If FreqPtWise==0, then the computation terminates when ANY one of the four
%  conditions below is true:
%
%     PeakUpperBound - PeakLowerBound <= AbsTol
%     PeakUpperBound - PeakLowerBound <= Reltol*PeakUpperBound
%     at every frequency: 
%          UpperBound <= AGoodThreshold + MGoodThreshold*Norm(NominalValue))
%     at some frequency: 
%          LowerBound >= ABadThreshold + MBadThreshold*Norm(NominalValue)
%
%  In both situations above, the stopping condition is applied at every 
%  point in array dimensions specified in ArrayDimPtWise.   UpperBound and 
%  LowerBound are the peak value over all other array dimensions.
%
%  Default - Structure with default values of all WCGOPT properties
%  Meaning - Structure with concise description of all WCGOPT properties
%
%  VaryUncertainty - Percentage variation of uncertainty for sensitivity
%    calculations.  Default is 25.
%  AbsTol - Upper and Lower Absolute Stopping Tolerance.  Default = 0.02.
%  RelTol - Upper/Lower Relative Stopping Tolerance.  Default = 0.05.
%  MGoodThreshold - Multiplicative (UpperBound) Stopping Threshold, Default = 1.04.
%  AGoodThreshold - Additive (UpperBound) Stopping Threshold, Default = 0.05.
%  MBadThreshold - Multiplicative (LowerBound) Stopping Threshold, Default = 5.
%  ABadThreshold - Additive (LowerBound) Stopping Threshold, Default = 20.
%
%  NTimes - Number of Restarts in LowerBound search (positive integer), 
%           Default = 2.
%  MaxCnt - Number of cycles in Lower Bound search (positive integer), 
%           Default = 3.
%
%  MaxTime - Maximum computation time allowed (in seconds).  The computation
%  is prematurely terminated if this much real time elapses before the 
%  computation is complete.  All quantities that have been computed are 
%  returned.  Default = 720.
%
% See also DKITOPT, ROBOPT, WCGAIN, WCNORM, WCSENS, WCMARGIN.

% Copyright 2003-2006 The MathWorks, Inc.

%  Display - Displays progress of computations, Default is 'on'. 
%
% Undocumented call--opt = wcgopt(OptCell).  This version is
% called by wcgopt/loadobj


superiorto('char','double','optunit');    

validate = 1;
nin = nargin;
if ceil(nin/2)~=floor(nin/2)
   validate = varargin{end};
   nin = nargin - 1;
end
hnin = nin/2;
mkatend = 1;
% OptCell = {...
% optunit('Sensitivity','on','on',...
%    'Compute sensitivity to Individual Uncertainties [''off'' | {''on''}]',1);...
% optunit('LowerBoundOnly','off','off',...
%         'Computes only lower bound for worst-case gain [{''off''} | ''on'']',1);...
% optunit('FreqPtWise',0,0,'Compute to tolerance at every point, not just peak (0 or 1)',1);...
% optunit('ArrayDimPtWise',[],[],'Array Indices Compute to tolerance at every point, not just peak',1);...
% optunit('VaryUncertainty',25,25,...
%    'Percentage variation of uncertainty for sensitivity calculations',1);...
% optunit('AbsTol',0.02,.02,'Upper-Lower Absolute Stopping Tolerance',1);...
% optunit('RelTol',0.05,.05,'Upper/Lower Relative Stopping Tolerance',1);...
% optunit('MGoodThreshold',1.04,1.04,'Multiplicative (UpperBound) Stopping Threshold',1);...
% optunit('AGoodThreshold',.05,.05,'Additive (UpperBound) Stopping Threshold',1);...
% optunit('MBadThreshold',20,20,'Multiplicative (LowerBound) Stopping Threshold',1);...
% optunit('ABadThreshold',5,5,'Additive (LowerBound) Stopping Threshold',1);...
% optunit('NTimes',2,2,'Number of Restarts in LowerBound search (positive integer)',0);...
% optunit('MaxCnt',3,3,'Number of cycles in Lower Bound search (positive integer)',0);...
% optunit('MaxTime',720,720,'Maximum Computation Time Allowed (seconds)',1);...
% %optunit('Display','off','off','Displays progress of computations [{''off''} | ''on'']',1);...
% };

HH={'Sensitivity','on','on',...
   'Compute sensitivity to Individual Uncertainties [''off'' | {''on''}]',1;...
'LowerBoundOnly','off','off',...
 'Computes only lower bound for worst-case gain [{''off''} | ''on'']',1;...
'FreqPtWise',0,0,...
 'Compute to tolerance at every point, not just peak (0 or 1)',1;...
'ArrayDimPtWise',[],[],...
 'Array Indices Compute to tolerance at every point, not just peak',1;...
'VaryUncertainty',25,25,...
   'Percentage variation of uncertainty for sensitivity calculations',1;...
'AbsTol',0.02,.02,'Upper-Lower Absolute Stopping Tolerance',1;...
'RelTol',0.05,.05,'Upper/Lower Relative Stopping Tolerance',1;...
'MGoodThreshold',1.04,1.04,...
 'Multiplicative (UpperBound) Stopping Threshold',1;...
'AGoodThreshold',.05,.05,...
 'Additive (UpperBound) Stopping Threshold',1;...
'MBadThreshold',20,20,...
 'Multiplicative (LowerBound) Stopping Threshold',1;...
'ABadThreshold',5,5,...
 'Additive (LowerBound) Stopping Threshold',1;...
'NTimes',2,2,...
 'Number of Restarts in LowerBound search (positive integer)',0;...
'MaxCnt',3,3,...
 'Number of cycles in Lower Bound search (positive integer)',0;...
'MaxTime',720,720,...
 'Maximum Computation Time Allowed (seconds)',1;...
 };

szH = size(HH);
OptCell = cell(szH(1),1);
for i=1:szH(1)
   OptCell{i} = optunit(HH{i,:});
end
Cache.Names = HH(:,1);
Cache.Strings = HH(:,4);

if mkatend
   out.OptCell = OptCell;
   out.Cache = Cache;
   out = class(out,'wcgopt',gsref(0));
end

if nargin==1 & iscell(varargin{1})
    % Input is a cell array of optunits. This is used by wcgopt/loadobj
    OptCell = varargin{1};
    for i=1:length(OptCell)
        out = set(out,pvget(OptCell{i},'Name'),pvget(OptCell{i},'Value'));
    end    
else
    if ceil(hnin)==floor(hnin)
       for i=1:hnin
          out = set(out,varargin{2*(i-1)+1},varargin{2*i});
       end
    end

    % Check validity of OUT.  Without any output arguments it will
    % error out with appropriate error message if not valid.
    %if isequal(validate,1)
    if validate==1
       isvalid(out);
    end
end
    
    