function [M,Delta,BlkStruct,NormDeltaCell,N4Blk] = lftdata(A,L)
%LFTDATA   Decomposes an uncertain matrix or system.
%
%   Uncertain matrices/systems are modeled as a fixed matrix/system
%   M in feedback with a normalized, block-diagonal matrix DELTA of 
%   uncertain elements.  LFTDATA extracts the M, DELTA components
%   from UMAT, USS, or UFRD objects.
%
%   [M,DELTA] = LFTDATA(A) takes an uncertain matrix/system A and 
%   extracts the M, DELTA components such that A = LFT(DELTA,M).
%   If A is a UMAT (respectively, USS and UFRD), M is a DOUBLE array
%   (respectively, SS model and FRD model).  DELTA is a UMAT describing
%   the normalized uncertainty in A (WCNORM(DELTA) is 1).  
%
%   [M,DELTA] = LFTDATA(A,LIST) only pulls out the uncertainty elements
%   specified in LIST (a cell array of names).  In this case, DELTA only 
%   includes the selected uncertain elements, the other uncertain elements 
%   remain in M, and M retains the class of A.
%
%   [M,DELTA,BLKSTRUCT] = LFTDATA(A) also returns an N-by-1 structure array
%   BLKSTRUCT where BLKSTRUCT(i) describes the i-th normalized uncertain 
%   element.  This uncertainty description can be passed directly to the  
%   low-level mu-analysis function MUSSV.
%
%   [M,DELTA,BLKSTRUCT,NORMUNC] = LFTDATA(A) also returns the cell array 
%   NORMUNC of normalized uncertain elements.  Each normalized element 
%   has the string 'Normalized' appended to its original name to avoid
%   confusion.  Note that LFT(BLKDIAG(NORMUNC{:}),M) is equivalent to A.
%
%   See also LFT, MUSSV.

%   Copyright 2003-2004 The MathWorks, Inc.
%   $Revision: 1.1.6.3 $  $Date: 2004/08/25 19:13:16 $

switch class(A)
case {'ureal' 'ucomplex' 'ucomplexm' 'ultidyn' 'udyn'}
   [M,Delta,BlkStruct,NormDeltaCell,N4Blk] = lftdata(umat(A));
case {'ss' 'double' 'frd' 'tf'}
   M = A;
   Delta = umat;
   BlkStruct = pvget(Delta,'BlkStructure');
   NormDeltaCell = cell(0,1);
   N4Blk = zeros(0,4);
end