function [cm,dm,mm,info]=loopmargin(sys,varargin)
%LOOPMARGIN  Comprehensive stability margin analysis of LTI and
%            Simulink feedback loops.
%
% LTI Feedback Loops:
% 
% [CM,DM,MM] = LOOPMARGIN(L) analyzes the N-channel multivariable 
% feedback loop:
% 
%         u --->O---->[ L ]----+---> y
%             - |              |         
%               +<-------------+
% 
% The N-by-N loop transfer matrix L can be any LTI model.  Negative
% feedback is assumed in the feedback path (specify -L otherwise).
% 
% LOOPMARGIN computes three types of stability margins:
% 
% * The classical gain and phase margins CM for each individual feedback
%   channel (loop-at-a-time margins) with all other loops closed (see 
%   ALLMARGIN for details on the fields of CM).
% 
% * The disk margin DM for each individual channel with all other loops
%   closed.  See DMPLOT for details.  DM is a N-by-1 structure with fields:
%      GainMargin: Disk gain margin (absolute value)
%     PhaseMargin: Disk phase margin (degrees)
%       Frequency: Frequency (in rad/sec) at which the margin disk
%                  touches the Nyquist plot of the loopgain
% 
% * The multi-loop disk margin, MM, indicates how much simultaneous 
%   (across channels), independent gain and phase variations can be
%   tolerated in each feedback channel while still maintaining stability
%   of the closed-loop system (note that for single-loop control systems, 
%   this is same as DM).  MM is a scalar structure with fields:
%      GainMargin: Guaranteed bound on simultaneous, independent, gain 
%                  variations allowed in all feedback channels.
%     PhaseMargin: Guaranteed bound on simultaneous, independent, phase 
%                  variations allowed in all feedback channels (degrees).
%       Frequency: Associated with GainMargin/PhaseMargin fields (rad/sec).
% 
% Unless L is FRD, the frequency range and number of points used to
% calculate DM and MM margins are chosen automatically. 
% 
% Output arguments can be limited to only those requested using an
% optional character string argument.  [M1,M2] = LOOPMARGIN(L,'m,c')
% returns the multi-loop diskmargin ('m') in M1, and the classical
% margins ('c') in M2.  Use 'd' to specify the disk margin.  This
% optional 2nd argument may be any combination, in any order, of the
% 3 characters 'c', 'd' and 'm'.
% 
% L can be an LTI array, and the margin computation is simply repeated
% for all array dimensions.  If SIZE(L) equals [N N N1 N2 ...], then
% SIZE(CM) will equal [N N1 N2 ...], SIZE(DM) will equal [N N1 N2 ...],
% and SIZE(MM) will equal [1 N1 N2 ...].
% 
% [CMI,DMI,MMI,CMO,DMO,MMO,MMIO] = LOOPMARGIN(P,C) computes the stability
% margins of the multivariable feedback loop consisting of the controller
% C in negative feedback with the plant P.  
% 
%         u --->O--->[ C ]-->[ P ]---+---> y
%             - |                    |         
%               +<-------------------+
% 
% * The CMI, DMI and MMI structures give the loop-at-a-time gain and 
%   phase margins, disk margins. and independent channel margins at 
%   the plant inputs, respectively (L = C*P)
% * The CMO, DMO and MMO structures give the corresponding margins at
%   the plant outputs (L = P*C)
% * The structure MMIO gives the disk margins for simultaneous, independent
%   variations in all input and output channels of the feedback loop.
% 
% Output arguments can be limited to only those requested using a
% character string option argument.  [M1,M2,M3] = LOOPMARGIN(P,C,'mo,ci,mm')
% returns the multi-loop diskmargin at plant output ('mo') in M1,
% the classical margins at the plant input ('ci') in M2, and the disk
% margins for simultaneous, independent variations in all input and
% output channels ('mm') in M3.  This optional 3rd argument may be any
% combination, in any order, of the 7 character pairs 'ci', 'di',
% 'mi', 'co', 'do', 'mo' and 'mm'.
% 
% If either P or C are LTI arrays, then the dimensioning rules described
% above for L also apply, resulting in multidimensional struct arrays for
% the output arguments.
%  
% Example:
%   a = [-0.2 10;-10 -0.2];
%   b = eye(2);
%   c = [1 8;-10 1];
%   d = zeros(2,2);
%   sat = ss(a,b,c,d);
%   k = [1 -2;0 1];
%   [cmi,dmi,mmi,cmo,dmo,mmo,mmio] = loopmargin(sat,k)
%   [dmi,dmo,mmio] = loopmargin(sat,k,'di,do,mm')
%
% SIMULINK Models:
% 
% [CM,DM,MM] = LOOPMARGIN(MODEL,BLOCKS,PORTS) does a multi-loop 
% stabilily margin analysis using the Simulink  Control Design (SCD). 
% MODEL specifies the name of the Simulink diagram for analysis. The 
% margin analysis points are defined at the output ports (PORTS) of
% blocks (BLOCKS) within the model. BLOCKS is a cell array of full block
% path names and PORTS is a vector of the same dimension as BLOCKS.
%
% Three types of stability margins are computed: loop-at-a-time classical 
% gain and phase margins (CM), loop-at-a-time disk margins (DM) and a 
% multi-loop disk margin (MM).
% 
% [CM,DM,MM] = LOOPMARGIN(MODEL,BLOCKS,PORTS,OP) uses the operating point 
% object, OP to create linearized systems from the Simulink MODEL.  
% 
% [CM,DM,MM,INFO] = LOOPMARGIN(MODEL,BLOCKS,PORTS,OP) returns INFO in
% addition to the margins. INFO is a structure with fields OperatingPoint,
% LinearizationIO and SignalNames corresponding to the analysis. Additional
% input arguments for LOOPMARGIN are OPTIONS for linearization and
% SNAPSHOTTIMES for linearization times. See LINEARIZE for more
% information.
%
% Margin output arguments can be limited to only those requested using an
% optional character string argument.  INFO is always the last output. 
% For example,  [MM,CM,INFO] = LOOPMARGIN(MODEL,BLOCKS,PORTS,'m,c') returns 
% the multi-loop diskmargin ('m') in MM, the classical margins ('c') in CM, 
% and the INFO structure.
% 
% See also ALLMARGIN, DMPLOT, LMIO, LOOPSENS, OPERPOINT, ROBUSTSTAB,
%         WCMARGIN,  WCGAIN.

%   Copyright 2007 The MathWorks, Inc. and MUSYN Inc.

if exist(sys)~=4 %4 if A is a MDL-file on MATLAB's search path
   error('Invalid simulink model name');
end

% Call utility function
[cm,dm,mm,info]=scdloopmargin(sys,varargin{:});
