function [info,sntisnto] = loopsens(P,C)
%LOOPSENS  Sensitivity functions of plant-controller feedback loop.
%
%LOOPTRANSFER = LOOPSENS(P,C) analyzes the multivariable feedback 
%loop consisting of C in negative feedback with P.  C should only be the 
%compensator in the feedback path, without reference channels, if  it is a 
%"2-dof" architecture.
%
%LOOPTRANSFER is a structure with the following fields:
%      Poles: closed-loop poles
%     Stable: 1 if nominal closed loop is stable, 0 otherwise. NaN for FRDs.
%         Si: input-to-plant sensitivity function
%         Ti: input-to-plant complementary sensitivity function
%         Li: input-to-plant loop transfer function
%         So: output-of-plant sensitivity function
%         To: output-of-plant complementary sensitivity function
%         Lo: output-of-plant loop transfer function
%        PSi: plant times input-to-plant sensitivity function
%        CSo: compensator times output-of-plant sensitivity function
% 
%See also LOOPMARGIN, ROBUSTSTAB, ROBUSTPERF, WCSENS, WCMARGIN.

% Copyright 2003-2004 The MathWorks, Inc.

szp = size(P);
szc = size(C);
if szc(1)==szp(2) & szp(1)==szc(2)
   ny = szp(1);
   nu = szp(2);
   [sntisnto,li,lo] = LOCALallst(P,C);
   if isa(sntisnto,'ss') | isa(sntisnto,'tf') | isa(sntisnto,'zpk') 
      info.Poles = pole(sntisnto);
   elseif isa(sntisnto,'uss') 
      %warning('System is uncertain, computing poles of the nominal system.');
      info.Poles = pole(sntisnto.Nominal);
   else
      szl = size(li);
      info.Poles = nan([1 1 szl(3:end)]);
      info.Stable = nan([1 1 szl(3:end)]);
   end
   if ~all(isnan(info.Poles(:)))
      szp = size(info.Poles);
      info.Stable = ones([1 1 szp(3:end)]);
      for i=1:prod(szp(3:end))
         %XXXMINOR check sample time only once
         if sntisnto.Ts == 0 && any(real(info.Poles(:,:,i))>=0) % continuous, unstable
            info.Stable(:,:,i) = 0;
         elseif sntisnto.Ts ~= 0 && any(abs(info.Poles(:,:,i))>=1)
             info.Stable(:,:,i) = 0;
         end
      end
   else
      szl = size(li);
      info.Stable = nan([1 1 szl(3:end)]);
   end
   info.Si = sntisnto(1:nu,1:nu);
   info.Ti = -sntisnto(nu+1:2*nu,1:nu);
   info.Li = li;
   info.So =  sntisnto(   2*nu+1:2*nu+ny  , nu+1:nu+ny);
   info.To = -sntisnto(2*nu+ny+1:2*nu+2*ny, nu+1:nu+ny);
   info.Lo = lo;
   info.PSi = sntisnto(2*nu+1:2*nu+ny,1:nu);
   info.CSo = -sntisnto(1:nu,nu+1:nu+ny);
else
   error('Invalid Dimension')
end


function [SNTISNTO,Li,Lo] = LOCALallst(P,C)

szp = size(P);
szc = size(C);
if szc(1)==szp(2) & szp(1)==szc(2)
   %e1 = icsignal(szc(1));
   %e2 = icsignal(szc(1));
   %d1 = icsignal(szc(1));
   %e3 = icsignal(szc(2));
   %e4 = icsignal(szc(2));
   %d2 = icsignal(szc(2));
   %m = ic;
   %m.eq{1} = e4 == P*e1;
   %m.eq{2} = e2 == C*(-e3);
   %m.eq{3} = e1 == e2 + d1;
   %m.eq{4} = e3 == e4 + d2;
   %m.Input = [d1;d2];
   %m.Output = [e1;e2;e3;e4];
   %SNTISNTO2 = m.System;
   % H = [Si -CSo;-Ti -CSo;PSi So;PSi -To];
   ny = szp(1);
   nu = szp(2);
   tmp1 = [eye(nu) zeros(nu,ny) eye(nu);zeros(nu,nu+ny) eye(nu);...
          zeros(ny,nu) eye(ny) zeros(ny,nu);zeros(ny,nu+ny+nu);...
          zeros(ny,nu) -eye(ny) zeros(ny,nu)];
   tmp2 = [zeros(2*nu,ny);eye(ny);eye(ny);-eye(ny)];
   tmp3 = [eye(nu) zeros(nu,ny) eye(nu)];
   % This works fine whether P is a SS or uss
   SNTISNTO = lft(tmp1+tmp2*P*tmp3,C);
   Li = C*P;
   Lo = P*C;
else
  error('Invalid dimensions');
end
