function [varargout] =  mussv(matin,blk,varargin)
%MUSSV  Upper and lower bounds of the Structured Singular Value,
%   and upper bounds of the Generalized structured singular value.
%
%   [BOUNDS,MUINFO] = MUSSV(M,BLK) computes upper and lower bounds of the
%   structured singular value of M with respect to the block diagonal
%   matrix structure described by BLK.  BLK is NBLK-by-2, with the i'th
%   row describing the i'th uncertainty block.   If the i'th block is
%   of the form:
%     delta EYE(N), and delta restricted to be real, set BLK(i,:) = [-N 0];
%     delta EYE(N), and delta allowed to be complex, set BLK(i,:) = [N 0];
%     a R-by-C complex matrix, set BLK(i,:) = [R C].
%
%   The format used in the 3rd output argument from LFTDATA is also
%   acceptable for describing the block structure.
% 
%   If M is an ND array, then the computation is performed pointwise along
%   the 3rd and higher array dimensions.  In that case, SIZE(BOUNDS,1)=1,
%   SIZE(BOUNDS,2)=2, and for K > 2, SIZE(M,K) = SIZE(BOUNDS,K).
%
%   MUINFO is a structure containing information that verifies (ie.,
%   proves) the bounds.  The information must be extracted using
%   MUSSVEXTRACT.
%
%   If M is an FRD, the computations are performed pointwise in frequency
%   (as well the array dimensions) and BOUNDS is dimensioned accordingly.
%
%   [BOUNDS,MUINFO] = MUSSV(M,BLK,OPT) specifies options described by the
%   optional options argument OPT.  OPT is a character string containing
%   any combinations of the following characters:
%
%       'a'   Upper bound to greatest accuracy, using LMI solver
%       'an'  Same as 'a', but without automatic-prescaling
%       'd'   Display warnings
%       'f'   Fast upper bound (typically not as tight as the default)
%       'g6'  Use gain-based lower bound multiple times (in this case 
%             10+6*10 times), larger number typically gives better lower
%             bound.  This is an alternative to the default lower bound 
%             which uses a power iteration.
%       'i'   ReInitialize lower bound computation at each new matrix
%             (only relevant if M is ND array or FRD)
%       'm7'  Randomly reinitialize lower bound iteration multiple times (in
%             this case 7 times), a larger number typically uses
%             more computation but often yields a better lower bound.
%       'o'   Run "old" algorithms, from version 3.1.1 and before.
%             Included to allow exact replication of earlier calculations.
%       's'   Suppress progress information (silent)
%       'U'   Upper-bound "only" (lower bound uses a fast/cheap algorithm)
%       'x'   Decrease iterations in lower bound computation (faster but
%             not as tight as default).  Use 'U' for an even faster lower
%             bound.
%
%   [UBOUND,Q] = MUSSV(M,F,BLK) computes an upper bound of the
%   Generalized Structured Singular Value of the pair (M,F), with respect
%   to the block-diagonal uncertainty structure described by BLK.  The
%   upper bound guarantees that for all matrices DELTA with structure
%   defined by BLK, and NORM(DELTA)<=1/UBOUND, the matrix [I-DELTA*M;F] is
%   full column rank.  The matrix Q satisfies MUSSV(M+Q*F,BLK,'a')<=UBOUND.
%
%   [UBOUND,Q] = MUSSV(M,F,BLK,'s') suppresses the progress information.
%
%See also: LFTDATA, MUSSVEXTRACT, ROBUSTSTAB, ROBUSTPERF, WCGAIN, WCSENS,
%          WCMARGIN.

%       'c'  default
%       'C'  same as 'g'
%       'Cn' same as 'gn'
%       'r'  same as i
%       'R7' same as m7
%       't'  (still accept) increase iterations in lower bound computation
%       'w'  ignore, as it is default
% ignore w, ignore c, C->a, r->i. R->m

% Mathematically, [I-Delta*M;C] is
%   guaranteed not to lose
%   column rank for all perturbations Delta consistent with
%   BLK, with norm < 1/UBND.  This is verified by a matrix Q,
%   which satisfies  MUSSV(M+QC,BLK,'C')<=UBND.  The optimal
%   choice of Q (which minimizes the upper bound) is solved
%   by reformulating the optimization into a semidefinite
%   program.


% Authors:  Matthew P. Newlin, Peter M. Young and MUSYN INC
% Copyright 1991-2004 MUSYN Inc. and The MathWorks, Inc.
% $Revision: 1.1.8.9 $

if nargin<=1
   error('Need at least two input arguments.');
end

if nargin>=3 && isa(varargin{1},'double')
   % BLK is actually E, and VARARGIN{1} is BLK
   E = blk;
   blk = varargin{1};
   try
      blk = blkstruct2N2(blk);
   catch
      rethrow(lasterror);
   end
	if nargout>0
	   varargout = cell(1,nargout);
	   [varargout{:}] = genmussvcalc(matin,E,blk,varargin{2:end});
	else
	   [varargout{1}] = genmussvcalc(matin,E,blk,varargin{2:end});
	end
else
   % Convert Block Structure
   try
      blk = blkstruct2N2(blk);
   catch
      rethrow(lasterror);
   end
    if nargin>=3 & ~isempty(find(varargin{1}=='o'))
        % Call old mussvcalc code if option 'o' is called
        if nargout>0
            varargout = cell(1,nargout);
            [varargout{:}] =  mussvcalc(matin,blk,varargin{:});
        else
            [varargout{1}] =  mussvcalc(matin,blk,varargin{:});
        end
    else
        % Otherwise call new mussvcalc code
        if nargout>0
            varargout = cell(1,nargout);
            [varargout{:}] =  mussvcalc2(matin,blk,varargin{:});
        else
            [varargout{1}] =  mussvcalc2(matin,blk,varargin{:});
        end
    end
end

   
