function [cflag,allupaths,allunames,upaths,unames,csumchar,incompatible,...
   syscell,fnames,iiunq,jjunq,fnidx,infocell,ucell,...
   diffsysdata,dflag] = usiminfo(sname,silent)
%USIMINFO  Find USS System blocks within a specified Simulink model and
%             checks for consistency.
%
% It is possible to have uncertain atoms of the same name throughout
% a Simulink model.  The helper functions USIMSAMP and USIMFILL assume
% that these represent the same entity.  Hence uncertain atoms of the
% same name should have the same object properties and Uncertainty
% Selection value in the USS System pull down menu.  USIMINFO returns the 
% locations of all USS System blocks within a Simulink model and determines
% if these compatibility conditions are satisfied. 
%
% [CFLAG,ALLUPATHS,ALLUNAMES,UPATHS,UNAMES,CSUMCHAR] = USIMINFO(SNAME,SILENT)
%
% Inputs:
%  SNAME     - Simulink diagram name
%  SILENT    - Display inconsistencies between uncertain atoms if isn't 
%              empty. Default is empty.
%
% Outputs:
%  CFLAG     - Compatibility flag set to 1 if all uncertainties are
%              consistent, set to 0 if an uncertainty definition(s) is
%              inconsistent and set to -1 if common uncertainties in
%              different blocks have different Uncertainty Selection.
%  ALLUPATHS - Path names of USS blocks in the model (cell).
%  ALLUNAMES - Uncertainties names in Simulink model (cell).
%  UPATHS    - Path names associated with each ALLUNAMES entry (cell).
%  UNAMES    - Uncertainty names associated with each ALLUPATHS entry (cell).
%  CSUMCHAR  - Character array with description of uncertainties and their 
%               associated block path names. Empty if there is a conflict 
%               with UNAMES.
%
% See also: usubs, usimfill, usimsamp


% undocumented calling arguments
%
% [ALLSYS,ALLUNAMES,UPATHS,UNAMES,CSUMCHAR,INCOMPATIBLE,SYSCELL,...
%      FNAMES,IIUNQ,JJUNQ,FNIDX,INFOCELL,UCELL] = USIMINFO(SNAME,SILENT)
%
% Inputs:
%  SNAME  - Simulink diagram name
%  SILENT - Display clashes in uncertain atoms if isn't empty. Default
%            is empty.
%
% Outputs:
%  ALLUPATHS - Path names of USS/UMAT blocks in the model (cell).
%  ALLUNAMES - Uncertainties names in Simulink model (cell).
%  UNAMES    - Uncertainty names associated with each ALLUPATHS entry (cell).
%  UPATHS    - Path names associated with each ALLUNAMES entry (cell).
%  CSUMCHAR  - Display with description of uncertainties and their 
%               associated block path names. Character array this is
%               empty if there is a conflict with UNAMES.
%  CFLAG     - Compatibility flag set to 1 if all uncertainties are
%              consistent, set to 0 if an uncertainty definition(s) is
%              inconsistent and set to -1 if common uncertainties in
%              different blocks have different Uncertainty Selection.
%  SYSCELL   - Cell of USS/UMAT blocks in the model
%  FNAMES    - All uncertainties names in the model.
%  IIUNQ     - Index of FNUNIQUE in FNAMES
%  JJUNQ     - Index of FNAMES in FNUNIQUE
%  FNIDX     - Index of non-unique uncertainty names in FNAMES 
%  INFOCELL  - Index of non-unique uncertainty names in FNAMES 
%  UCELL     - For each uncertainty, the block path names 
%DIFFSYSDATA - Block information about consistency of USS/UMAT data and
%               uncertain variable when User Specfied Uncertainty selected.
%  DFLAG     - Compatibility vector associated with each USS/UMAT block.
%              Entry is 1 'User Specified Uncertainty' selected and the
%              uncertain system and uncertain data are consistent or 'Nominal'
%              is selected. Entry set to 0 if uncertain system and 
%              uncertain data.

% Find all the USS blocks. Note that each USS block's 'Tag' property will
% be set to 'USS'

% Authors: Gary J. Balas and Andy K. Packard
% Copyright 2005-2006 MUSYN Inc.
if nargin==1
   silent = [];
end
% everything is consistent
cflag = 1;
UNCTYPEIDX = 3;

load_system(sname);
allsys = find_system(sname,'LookUnderMasks','all','Tag','USS'); % Finds all USS blocks

mv = get_param(allsys,'MaskValues'); % same size as ALLSYS (?)
syscell = cell(size(allsys));
TSList = zeros(size(allsys));
datacell = cell(size(allsys));
Deltacell = cell(size(allsys));
infocell = cell(size(allsys));
diffsysdata = cell(size(allsys));
fnames = [];
fnidx = 1;

for i=1:length(mv)
   % system variable
   sysname = get_param(allsys{i},'sys');
   %syscell{i} = uss(evalin('base',sysname));
   syscell{i} = uss(LocalEvaluate(sysname,sname));
   TSList(i) = get(syscell{i},'Ts');
   [m,del] = lftdata(syscell{i});
   Deltacell{i} = del;
   fn = fieldnames(syscell{i}.Uncertainty);
   fnames = [fnames;fn];
   infocell{i} = fn;
   fnidx(i+1) = fnidx(i)+length(fn);
   % uncertainty variable
   dataname = get_param(allsys{i},'logname');
   if ~isempty(dataname)
      try
         % don't do EVALIN, follow CTB Blocks, using a LOCALEvaluate
         %datacell{i} = evalin('base',dataname);
         datacell{i} = LocalEvaluate(dataname,sname);
         if isa(datacell{i},'struct')
            dn = fieldnames(datacell{i});
         else
            dn = [];
         end
      catch
         datacell{i} = [];
         dn = [];
      end
   else
      datacell{i} = [];
      dn = [];
   end
   % Check each USS for uncertain data consistency
   if ~strcmp(get_param(allsys{i},'unctype'),'Nominal')
      cn = setdiff(fn,dn);  % in FN, missing from DN
      if ~isempty(cn)
         diffsysdata{i} = cn;
      else %same field names in USYS and uncertain data
         try
            tmp = usubs(syscell{i},datacell{i});
         catch
            diffsysdata{i} = -1;
         end
      end
   end
end
dflag = cellfun('isempty',diffsysdata);

%Find if there are the same uncertain elements in several USS blocks
[fnunique,iiunq,jjunq] = unique(fnames);
nUnc = length(fnunique);

% Uncertainty and associated block paths
ucell = cell(nUnc,3);
for i=1:nUnc
   ctmp = cell(0,1);
   dtmp = zeros(0,1);
   for j=1:length(allsys)
      ind = fnidx(j):fnidx(j+1)-1;
      tmp = strcmp(fnunique{i},fnames(ind));
      if any(tmp)
         ctmp = [ctmp;allsys(j)];
         dtmp = [dtmp;j];
      end
   end
   ucell{i,1} = ctmp;
   ucell{i,2} = dtmp;
end

pathLeft = cell(0,1);
pathRight = cell(0,1);
for i=1:nUnc
   Nplaces = length(ucell{i,1});
   atomcell = cell(Nplaces,1);
   % patomcell = cell(Nplaces,1);
   for k=1:Nplaces
      atomcell{k} = pvget(syscell{ucell{i,2}(k)}.Uncertainty,fnunique{i});
      % patomcell{k} = find(strcmp(ucell{i,1}(k),allsys)); % equals ucell{i,2}(k)
   end
   first = atomcell{1};
   ucell{i,3} = mkdstr(first);
   ucheck = ones(length(ucell{i,1}),length(ucell{i,1}));
   ucheck2 = ones(length(ucell{i,1}),length(ucell{i,1}));
   TScheck = ones(length(ucell{i,1}),1);
   for k=1:length(ucell{i,1})
      TScheck(k) = TSList(ucell{i,2}(k));
      for kk=k+1:length(ucell{i,1})
         ucheck(k,kk) = isequal(atomcell{k},atomcell{kk});
%          ucheck2(k,kk) = isequal(mv{patomcell{k}}(UNCTYPEIDX),...
%             mv{patomcell{kk}}(UNCTYPEIDX));
         ucheck2(k,kk) = isequal(mv{ucell{i,2}(k)}(UNCTYPEIDX),...
            mv{ucell{i,2}(kk)}(UNCTYPEIDX));
      end
   end
   [iloc,jloc] = find(ucheck==0);
   if ~isempty(iloc)
      pathLeft{i,1} = ucell{i,1}(iloc);
      pathRight{i,1} = ucell{i,1}(jloc);
      cflag = 0; %inconsistent uncertainty definitions
      if isempty(silent)
         disp(['Uncertain atom ' '''' fnunique{i} '''' ' is defined differently in USS blocks: ']);
         for  k=1:length(iloc)
            disp([' ' pathLeft{i}{k} '  *NOT EQUAL TO*  ' pathRight{i}{k}]);
         end
      end
   else
      [iloc,jloc] = find(ucheck2==0);
      if ~isempty(iloc)
         pathLeft{i,1} = ucell{i,1}(iloc);
         pathRight{i,1} = ucell{i,1}(jloc);
         cflag = -1;  %inconsistent uncertainty Uncertainty Selection
         if isempty(silent)
            xtmp = ['Uncertain atom ' '''' fnunique{i} '''' '  Uncertainty Selection'];
            disp([xtmp ' is defined differently in USS blocks: ']);
            for  k=1:length(iloc)
               disp(['  ' pathLeft{i}{k} '  *NOT EQUAL TO*  ' pathRight{i}{k}]);
            end
         end
      else
         if isa(atomcell{1},'ultidyn')
            [TSU,TSI] = unique(TScheck);
            if length(TSU)>1
               cflag = -2;
               if isempty(silent)
                  xtmp = ['ULTIDYN ' '''' fnunique{i} '''' '  SampleTime Inconsistency'];
                  disp(xtmp)
                  for  k=1:length(TSU)
                     disp(['  SampleTime in ' allsys{TSI(k)} '  equals  ' num2str(TScheck(TSI(k)))]);
                  end
               end
            end
         end
      end
   end
end

allupaths = allsys;
allunames = fnunique;
upaths = ucell(:,1);
unames = infocell;
incompatible = [pathLeft pathRight];

sumcell = cell(0,1);
if isempty(incompatible)
   for i=1:size(ucell,1)
     sumcell = [sumcell;ucell{i,3}];
     for k=1:length(ucell{i,1})
        sumcell = [sumcell; {[' - ' ucell{i,1}{k}]}];
     end
   %  sumcell = [sumcell; {[' ']}];
   end
end
csumchar = char(sumcell);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GJB 21Nov06
function Value = LocalEvaluate(VarName,Model)
% Evaluates model variable in appropriate workspace.
if isempty(VarName)
   Value = [];
else
   try
      Value = evalin('base',VarName);
   catch
      try
         ModelWS = get_param(Model,'ModelWorkspace');
         Value = ModelWS.evalin(VarName);
      catch
         Value = [];
      end
   end
end