function [out] = usubs(mat,varargin)
%USUBS   Substitutes values for uncertain elements.
%   B = USUBS(A,NAME1,VALUE1,NAME2,VALUE2,...) instantiates the uncertain 
%   elements NAME1, NAME2,... of A to the values VALUE1, VALUE2,...
%   Set VALUE to 'NominalValue' or 'Random' to use the nominal value or
%   a random instance of a particular element.  USUBS works on uncertain 
%   matrices, models, and elements (see UMAT, USS, UFRD, UREAL,...).  
% 
%   If you assign a single value to each uncertain element, B has the 
%   same size as A.  However, you can also use this command in batch
%   mode by specifying multiple values at once.  Use double arrays for
%   scalar-valued elements, cell arrays for matrix-valued elements, and
%   LTI arrays for uncertain dynamics.  All VALUE arrays should have the
%   same size, say SZV, (or at least match along nonsingleton dimensions)
%   and the size of B is then [SIZE(A) SZV].
%
%   B = USUBS(A,S) instantiates the uncertain elements of A to the values 
%   specified in the structure or struct array S.  S typically comes from 
%   commands like USAMPLE, WCNORM, ROBUSTSTAB, etc.  If A is a single
%   uncertain matrix or system, then SIZE(B) is [SIZE(A) SIZE(S)].  If
%   A is an array of uncertain matrices or systems, with input/output
%   dimensions SZA and array dimensions ADA, then ADA and SIZE(S) must
%   match along non-singleton dimensions, and the resulting array B has
%   input/output size equal to SZA and array dimensions MAX(AdA,SIZE(S)).
%
%   Example 1:
%      % Create uncertain parameters a,b and build m = [a b]
%      a = ureal('a',5);  b = ureal('b',-3);
%      m = [a b];
%      % Instantiate a to the values 1,2,3,4
%      ma = usubs(m,'a',[1;2;3;4])  % 1x2x4 UMAT dependent on 'b'
%      % In the resulting UMAT, evaluate b at 10,20,30
%      mab = usubs(ma,'b',[10;20;30])  
%            % 1x2x4x3 DOUBLE (4x3 array of m values)
%       
%   Example 2:
%      % Evaluate m = [a b] at 4 (a,b) values
%      mab = usubs(m, 'a',[1;2;3;4], 'b',[10;11;12;13])
%            % 1x2x4 DOUBLE
%
%   Example 3: 
%      % Evaluate m = [a b;0 a*b] over a 3x4 grid of (a,b) values
%      m = [a b;0 a*b];  % 2-by-2 UMAT
%      aval = rand(3,1);  bval = rand(4,1);
%      % Build arrays of (a,b) values
%      [A,B] = ndgrid(aval,bval);
%      mab = usubs(m,'a',A,'b',B)    
%            % 2x2x3x4 DOUBLE (3x4 array of m values)
%
%   See also USAMPLE.

%   Copyright 2003-2005 The MathWorks, Inc.
%   $Revision: 1.1.6.7 $  $Date: 2008/12/04 23:16:25 $


if nargin==0
   disp('usage: Mout = usubs(m,atomid1,value1,atomid2,value2,...');
   return
end

try
   [blkid,val,grouping,instruct] = LOCALnvlist(varargin{:});
catch fred
   throw(fred);
end

umatflg = 0;
Ts = [];
if isa(mat,'uss') | isa(mat,'ufrd')
   Ts = utarbts(pvget(mat,'Ts'));  %XXXTS
   Ts = (pvget(mat,'Ts'));  %XXXTS
elseif isa(mat,'umat')
   umatflg = 1;
elseif isa(mat,'atom')
   umatflg = 1;
   mat = umat(mat);
end

szb = size(blkid);
szv = size(val);
szm = size(mat);
nAD = length(szm) - 2 - (length(szm)==4 && szm(4)==1);

if length(szb)==2 & length(szv)==2 & max(szb)==max(szv) & min(szb)==1 & min(szv)==1 
   delta = []; % if BVAL is an ATOM, DELTA will become UMAT
   out = mat;
   for k=1:max(szb)
      idx = gblkidx(out,blkid{k});
      if ~isempty(idx)
         AtomCell = pvget(out.Uncertainty,'AtomCell');
         thisatom = AtomCell{idx}; %replaced by BVAL, which may be anything
         szblk = size(thisatom);
         if isa(val{k},'char') & ndims(val{k})==2 & size(val{k},1)==1
            if strncmpi('NominalValue',val{k},length(val{k}))
               bval = pvget(thisatom,'NominalValue');
            elseif strncmpi('Random',val{k},length(val{k}))
               if isa(thisatom,'ultidyn')
                  if isempty(Ts)
                     disp('Random instances of ULTIDYN within UMATs are not allowed');
                     disp('Sample-time is unknown');
                     error('Cast 1st argument to USS - with appropriate Ts, and call again');
                  else
                     bval = pvget(thisatom,'Instance',[],Ts);
                  end
               else
                  bval = pvget(thisatom,'Instance');
               end
            else
               error('Invalid specifier');
            end
         else
            szvalk = size(val{k});
            if ~instruct(k)
               % Automatically Resize Stuff for Non-Struct Specification
               switch class(val{k})
                  case 'double'
                     szatom = size(thisatom);
                     nameatom = pvget(thisatom,'Name');
                     if isequal(szatom,[1 1])
                        bval = reshape(val{k},[1 1 ones(1,nAD) szvalk]);
                     else
                        try
                           bval = reshape(val{k},[szatom ones(1,nAD) szvalk(3:end) 1 1]);
                        catch
                           error(['Invalid dimensions for atom ''' nameatom ''' substitution']);
                        end
                     end
                  case {'ss' 'tf' 'zpk' 'frd' 'uss' 'ufrd'}
%                      if length(szvalk)==4 && szvalk(3)==1 && szvalk(4)>1
%                         bval = reshape(val{k},[ones(1,nAD) szvalk([4 3])]);
%                      else
                        bval = reshape(val{k},[ones(1,nAD) szvalk(3:end) 1 1]);
%                      end
                  case {'umat'}
                     bval = reshape(val{k},[szvalk(1:2) ones(1,nAD) szvalk(3:end) 1 1]);
                  case {'cell'}
                     try
                        tmp = cat(3,val{k}{:});
                     catch
                        error('Unable to concatenate Cell Array of Values');
                     end
                     sztmp = size(tmp);
                     switch class(tmp)
                        case 'double'
                           bval = reshape(tmp,[sztmp(1:2)  ones(1,nAD) szvalk]);
                         case {'ss' 'tf' 'zpk' 'frd' 'uss' 'umat' 'ufrd'}
                           bval = reshape(tmp,[ones(1,nAD) szvalk]);
                        otherwise 
                           error('Illegal Value.');
                     end
                  otherwise
                     bval = val{k};
               end
            else
               bval = val{k};
            end
         end
         szbval = size(bval);
         if isequal(szbval(1:2),szblk)
            Literal = pvget(out,'Literal');
            litval = Literal(idx);
            Copies = pvget(out,'Copies');
            copval = Copies(idx);
            [out] = blk2ioch(out,idx,'b','unchanged'); % move blocks to bottom, sequentially
            if copval>0
               if litval==0
                  if isa(bval,'atom')
                     bval = ndlft(bval);
                  end
                  % XXX: Setting all Literal of BVAL to 0 is not clear,
                  % Moreover, its only done for NDLFT and UMAT - what
                  switch class(bval)
                  case {'ndlft' 'umat' 'uss'}
                     bval = setlitr(bval,'all',0);
                  end
                  tbval = bval;
                  for i=2:copval
                     if isa(tbval,'double') & isa(bval,'double')
                        tbval = ndbdiagr(tbval,bval);
                     else
                        tbval = blkdiag(tbval,bval); % BLKDIAG does not call SIMPLIFY
                     end
                  end
                  bval = tbval; % correct number of copies
                  N = pvget(thisatom,'Normalizer');
                  S = N.S; T = N.T; TR = N.Tarb;
                  if ~isempty(TR) & ~LOCALisidlft(TR,szblk)
                     % if original atom had "bad" range/nominal
                     % data resulting in a singular S and hence
                     % an ill-defined T, perform first transformation
                     Text = LOCALtkron(TR,szblk(1),szblk(2),copval);
                     %bval = starp (bval,Text,szblk(1)*copval,szblk(2)*copval);
                     bval = lft(bval,Text,szblk(2)*copval,szblk(1)*copval);
                  end
                  if ~LOCALisidlft(T,szblk)
                     Text = LOCALtkron(T,szblk(1),szblk(2),copval);
                     %bval = starp (bval,Text,szblk(1)*copval,szblk(2)*copval);
                     bval = lft(bval,Text,szblk(2)*copval,szblk(1)*copval);
                  end
               else
                  tbval = bval;
                  for i=2:copval
                     if isa(tbval,'double') & isa(bval,'double')
                        tbval = ndbdiagr(tbval,bval);
                     else
%                        [class(tbval) '   ' class(bval)]
                        tbval = blkdiag(tbval,bval); % BLKDIAG does not call SIMPLIFY
                     end
                  end
                  bval = tbval;
               end
            else
               bval = [];
            end
            %Basic idea below is too build up DELTA as:
            %    delta = blkdiag(delta,bval);   This is correct stacking order,
            %based on how we BLK2IOCH the channels (see above).
            % Need to modify heavily, since LTI-Array EXDs are not handled
            % in the same manner, and since DOUBLE/BLKDIAG is not really sufficient
            if isempty(delta)
               delta = bval;
            elseif isa(delta,'double') & isa(bval,'double')
               delta = ndbdiagr(delta,bval);
            elseif  isa(delta,'lti') & isa(bval,'double')
               Tsdelta = delta.Ts;
               bval = uss(ss([],[],[],bval,Tsdelta)); %XXXTS
               delta = depromote(blkdiag(uss(delta),bval)); % back to SS
            elseif  isa(delta,'double') & isa(bval,'lti')
               Tsbval = bval.Ts;
               delta = uss(ss([],[],[],delta,Tsbval));
               %delta = blkdiag(delta,bval);
               delta = depromote(blkdiag(delta,uss(bval)));
            elseif  isa(delta,'lti') & isa(bval,'lti')
               %delta = append(delta,bval);
               delta = depromote(blkdiag(uss(delta),uss(bval)));
            else
               delta = blkdiag(delta,bval);
            end
         else
            error(['Invalid value for uncertain element ''' pvget(thisatom,'Name') '''']);
         end
      end
   end
   % Turn reduce option OFF, since the LFT call does do reduction
   try
      out = simplify(lft((out),(delta)),'off');
   catch
      rethrow(lasterror);
   end
elseif isempty(blkid) & isempty(val)
   out = mat;
else
   error('Invalid BLKID and/or VALUE')
end
if ~isa(out,'double') & ~isa(out,'ss')
   out = depromote(out);
end

function flag = LOCALisidlft(T,sz)
Tc = T-[zeros(sz(2),sz(1)) eye(sz(2));eye(sz(1)) zeros(sz(1),sz(2))];
flag = all(Tc(:)==0);

function tout = LOCALtkron(T,r,c,copies)
%function tout = LOCALtkron(T,r,c,copies)
% T = (c+r)x(r+c)x...
szt = size(T);
tout = zeros([copies*szt(1:2) szt(3:end)]);
r1 = 1:c; r1ext = 1:c*copies;
r2 = (c+1):(c+r); r2ext = (c*copies+1):(c+r)*copies;
c1 = 1:r; c1ext = 1:r*copies;
c2 = (r+1):(r+c); c2ext = (r*copies+1):(r+c)*copies;
for i=1:prod(szt(3:end))
   tout(r1ext,c1ext,i) = kron(eye(copies),T(r1,c1,i));
   tout(r1ext,c2ext,i) = kron(eye(copies),T(r1,c2,i));
   tout(r2ext,c1ext,i) = kron(eye(copies),T(r2,c1,i));
   tout(r2ext,c2ext,i) = kron(eye(copies),T(r2,c2,i));
end


function [names,values,grouping,instruct] = LOCALnvlist(varargin)

% Copyright 2003-2004 The MathWorks, Inc.

nin = nargin;
pt = 1;
cnt = 1;
names = cell(0,1);
values = cell(0,1);
grouping = zeros(0,1);
instruct = zeros(0,1);
while pt<=nin
   argname = varargin{pt};
   structflag = 0;
   nb2cell = 0;
   if isa(argname,'cell')
      % N-by-1 cell array of NAMES, just grab next as VALUE
      % 1-by-N cell array of NAMES, just grab next as VALUE
      sza = size(argname);
      if length(sza)==2
         if cellfun('isclass',argname,'char')
            if sza(1)==1 && sza(2)>=1
               argname = argname';
               argvalue = repmat({varargin{pt+1}},[sza(2) 1]);
               groupvalue = repmat(cnt,[sza(2) 1]);
               insvalue =  repmat(0,[sza(2) 1]);
               pt = pt + 2;
            elseif  sza(1)>=1 && sza(2)==1
               argvalue = repmat({varargin{pt+1}},[sza(1) 1]);
               groupvalue = repmat(cnt,[sza(1) 1]);
               insvalue = repmat(0,[sza(1) 1]);
               pt = pt + 2;
            else
               error('Invalid 1');
            end
         else
            error('Invalid 2');
         end
      else
         error('Invalid 3');
      end
   elseif isa(argname,'char')
      if ndims(argname)==2
         argname = strtrim(cellstr(argname));
      else
         error('Invalid 4')
      end
      argvalue = repmat({varargin{pt+1}},[length(argname) 1]);
      groupvalue = repmat(cnt,[length(argname) 1]);
      insvalue = repmat(0,[length(argname) 1]);
      pt = pt + 2;
   elseif isa(argname,'struct')
      [argname,argvalue,eflag] = struct2nv(argname);
      bloc = find(eflag>0);
      if length(bloc)>0
         error(['Unable to properly form data associated with ' argname{bloc(1)}]);
      end
      pt = pt + 1;
      groupvalue = repmat(cnt,[length(argname) 1]);
      insvalue = repmat(1,[length(argname) 1]);
   else
      error('Invalid 5')
   end
   names = [names;argname];
   values = [values;argvalue];
   grouping = [grouping;groupvalue];
   instruct = [instruct;insvalue];
   cnt = cnt + 1;
end
